@@ -1,7 +1,8 @@
 /*
- * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2014 Anders Jonsson <anders.jonsson@norsjovallen.se>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -82,16 +83,6 @@ enum {
 static guint config_assistant_signals[LAST_SIGNAL] = { 0, };
 static GObjectClass *parent_class = NULL;
 
-static void 
-data_source_info_free (GdaDsnInfo *info)
-{
-	g_free (info->provider); 
-	g_free (info->cnc_string); 
-	g_free (info->description);
-	g_free (info->auth_string);
-	g_free (info);
-}
-
 /*
  * Callbacks
  */
@@ -110,19 +101,36 @@ assistant_applied_cb (GtkAssistant *assist, G_GNUC_UNUSED gpointer data)
 	gboolean allok = TRUE;
 	GString *cnc_string = NULL;
 	GdauiDsnAssistant *assistant = (GdauiDsnAssistant *) assist;
+  GError *error = NULL;
 
 	g_return_if_fail (GDAUI_IS_DSN_ASSISTANT (assistant));
 
 	/* clear the internal dsn_info */
 	if (assistant->priv->dsn_info) {
-		data_source_info_free (assistant->priv->dsn_info);
+		gda_dsn_info_free (assistant->priv->dsn_info);
 		assistant->priv->dsn_info = NULL;
 	}
 
 	/* New database creation first */
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (assistant->priv->choose_toggle))) {
-		if (!gda_server_operation_is_valid (assistant->priv->create_db_op, NULL, NULL)) {
-			_gdaui_utility_show_error (NULL, _("Missing mandatory information, to create database"));
+    gdaui_server_operation_update_parameters (assistant->priv->newdb_params, &error);
+    gchar *msg = "No error details";
+    if (error != NULL) {
+      if (error->message != NULL) {
+        msg = error->message;
+      }
+			_gdaui_utility_show_error (NULL, _("Error at updating paramenters for CREATE DATABASE operation: '%s'"), msg);
+        g_error_free (error);
+      return;
+    }
+		if (!gda_server_operation_is_valid (assistant->priv->create_db_op, NULL, &error)) {
+      gchar *msg = "No error details";
+      if (error != NULL && error->message != NULL) {
+        msg = error->message;
+      }
+			_gdaui_utility_show_error (NULL, _("Missing mandatory information, to create database: '%s'"), msg);
+      if (error != NULL)
+        g_error_free (error);
 			gtk_assistant_set_current_page (assist, PAGE_CREATE_DB_INFO);
 			return;
 		}
@@ -147,7 +155,7 @@ assistant_applied_cb (GtkAssistant *assist, G_GNUC_UNUSED gpointer data)
 			prov_info = gda_config_get_provider_info (gdaui_provider_selector_get_provider 
 								  (GDAUI_PROVIDER_SELECTOR (assistant->priv->general_provider)));
 			g_return_if_fail (prov_info);
-			for (dsn_params = prov_info->dsn_params->holders; dsn_params; dsn_params = dsn_params->next) {
+			for (dsn_params = gda_set_get_holders (prov_info->dsn_params); dsn_params; dsn_params = dsn_params->next) {
 				GdaHolder *param = GDA_HOLDER (dsn_params->data);
 				const GValue *value;
 				
@@ -162,7 +170,7 @@ assistant_applied_cb (GtkAssistant *assist, G_GNUC_UNUSED gpointer data)
 				if (value && !gda_value_is_null ((GValue *) value)) {
 					gchar *str;
 
-					if (dsn_params == prov_info->dsn_params->holders)
+					if (dsn_params == gda_set_get_holders (prov_info->dsn_params))
 						cnc_string = g_string_new ("");
 					else
 						g_string_append (cnc_string, ";");
@@ -176,7 +184,7 @@ assistant_applied_cb (GtkAssistant *assist, G_GNUC_UNUSED gpointer data)
 
 	/* Data source declaration */
 	if (allok) {
-		assistant->priv->dsn_info = g_new0 (GdaDsnInfo, 1);
+		assistant->priv->dsn_info = gda_dsn_info_new ();
 		assistant->priv->dsn_info->name = g_strdup (gtk_entry_get_text (GTK_ENTRY (assistant->priv->general_name)));
 		assistant->priv->dsn_info->provider = g_strdup (
 					 gdaui_provider_selector_get_provider (
@@ -273,7 +281,20 @@ provider_changed_cb (G_GNUC_UNUSED GtkWidget *combo, GdauiDsnAssistant *assistan
 
 	/* dsn spec for the selected provider */
 	provider = gdaui_provider_selector_get_provider (GDAUI_PROVIDER_SELECTOR (assistant->priv->general_provider));
-	g_assert (provider);
+	if (provider == NULL) {
+    GtkWidget *parent = gtk_widget_get_toplevel(combo);
+    if (!GTK_IS_WINDOW (parent)) {
+      parent = NULL;
+    }
+    GtkWidget *msg = gtk_message_dialog_new (parent,
+                        GTK_DIALOG_MODAL,
+                        GTK_MESSAGE_ERROR,
+                        GTK_BUTTONS_CLOSE,
+                        _("No provider exists"));
+    gtk_dialog_run (GTK_DIALOG (msg));
+    gtk_widget_destroy (msg);
+    return;
+  }
 	if (!assistant->priv->provider_detail) {
 		assistant->priv->provider_detail = _gdaui_provider_spec_editor_new (provider);
 		gtk_box_pack_start (GTK_BOX (assistant->priv->provider_container),
@@ -385,15 +406,40 @@ forward_page_function (gint current_page, GdauiDsnAssistant *assistant)
 		GdaProviderInfo *pinfo;
 		const gchar *provider;
 		provider = gdaui_provider_selector_get_provider (GDAUI_PROVIDER_SELECTOR (assistant->priv->general_provider));
-		if (provider != NULL) {
+		if (provider == NULL) {
+      GtkWidget *parent = gtk_widget_get_toplevel(GTK_WIDGET(assistant));
+      if (!GTK_IS_WINDOW (parent)) {
+        parent = NULL;
+      }
+      GtkWidget *msg = gtk_message_dialog_new (parent,
+                          GTK_DIALOG_MODAL,
+                          GTK_MESSAGE_ERROR,
+                          GTK_BUTTONS_CLOSE,
+                          _("No providers exists"));
+      gtk_dialog_run (GTK_DIALOG (msg));
+      gtk_widget_destroy (msg);
+      return;
+    }
 		pinfo = gda_config_get_provider_info (provider);
-			if (pinfo != NULL) {
-				if (pinfo->auth_params && pinfo->auth_params->holders)
-					return PAGE_AUTH_INFO;
-				else
-					return PAGE_LAST;
-			}
-		}
+    if (pinfo == NULL) {
+      GtkWidget *parent = gtk_widget_get_toplevel(GTK_WIDGET(assistant));
+      if (!GTK_IS_WINDOW (parent)) {
+        parent = NULL;
+      }
+      GtkWidget *msg = gtk_message_dialog_new (parent,
+                          GTK_DIALOG_MODAL,
+                          GTK_MESSAGE_ERROR,
+                          GTK_BUTTONS_CLOSE,
+                          _("No provider's information exists"));
+      gtk_dialog_run (GTK_DIALOG (msg));
+      gtk_widget_destroy (msg);
+      return;
+    }
+
+		if (gda_set_get_holders (pinfo->auth_params))
+			return PAGE_AUTH_INFO;
+		else
+			return PAGE_LAST;
 	}
 	case PAGE_AUTH_INFO:
 		return PAGE_LAST;
@@ -472,7 +518,7 @@ gdaui_dsn_assistant_init (GdauiDsnAssistant *assistant,
 	label = gtk_label_new ("");
 	gtk_label_set_markup (GTK_LABEL (label), str);
 	g_free (str);
-	gtk_misc_set_alignment (GTK_MISC (label), 0., 0.);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
 	gtk_grid_attach (GTK_GRID (grid), label, 0, 1, 1, 1);
 
 	assistant->priv->general_name = gtk_entry_new ();
@@ -484,7 +530,7 @@ gdaui_dsn_assistant_init (GdauiDsnAssistant *assistant,
 
 	if (gda_config_can_modify_system_config ()) {
 		label = gtk_label_new (_("System wide data source:"));
-		gtk_misc_set_alignment (GTK_MISC (label), 0., 0.);
+		gtk_widget_set_halign (label, GTK_ALIGN_START);
 		gtk_grid_attach (GTK_GRID (grid), label, 0, 2, 1, 1);
 
 		assistant->priv->general_is_system = gtk_check_button_new ();
@@ -497,14 +543,14 @@ gdaui_dsn_assistant_init (GdauiDsnAssistant *assistant,
 	label = gtk_label_new ("");
 	gtk_label_set_markup (GTK_LABEL (label), str);
 	g_free (str);
-	gtk_misc_set_alignment (GTK_MISC (label), 0., 0.);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
 	gtk_grid_attach (GTK_GRID (grid), label, 0, 3, 1, 1);
 	
 	assistant->priv->general_provider = gdaui_provider_selector_new ();
 	gtk_grid_attach (GTK_GRID (grid), assistant->priv->general_provider, 1, 3, 1, 1);
 
 	label = gtk_label_new (_("Description:"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0., 0.);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
 	gtk_grid_attach (GTK_GRID (grid), label, 0, 4, 1, 1);
 	
 	assistant->priv->general_description = gtk_entry_new ();
@@ -560,7 +606,7 @@ gdaui_dsn_assistant_init (GdauiDsnAssistant *assistant,
 	gtk_label_set_markup (GTK_LABEL (label),
 			      _("The following fields represent the information needed "
 				"to create a new database "
-				"(mandatory fields are marked with a star)."
+				"(mandatory fields are marked with a star). "
 				"This information is database-specific, so check "
 				"the manual for more information."));
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
@@ -670,8 +716,7 @@ gdaui_dsn_assistant_finalize (GObject *object)
 	g_return_if_fail (GDAUI_IS_DSN_ASSISTANT (assistant));
 
 	/* free memory */
-	if (assistant->priv->dsn_info)
-		data_source_info_free (assistant->priv->dsn_info);
+	gda_dsn_info_free (assistant->priv->dsn_info);
 
 	if (assistant->priv->create_db_op)
 		g_object_unref (assistant->priv->create_db_op);
@@ -722,6 +767,7 @@ gdaui_dsn_assistant_new (void)
 	GdauiDsnAssistant *assistant;
 
 	assistant = g_object_new (GDAUI_TYPE_DSN_ASSISTANT, NULL);
+	gtk_window_set_default_size (GTK_WINDOW (assistant), 500, -1);
 	return GTK_WIDGET (assistant);
 }
 
