@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2007 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2008 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  *
@@ -39,6 +39,8 @@
 
 #include "rt-parser.h"
 
+#include <libgda/gda-debug-macros.h>
+
 struct _GdaReportEnginePrivate {
 	xmlDocPtr     doc; /* may be %NULL */
 	xmlNodePtr    spec;
@@ -155,7 +157,7 @@ gda_report_engine_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
-		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
+		static GMutex registering;
 		static GTypeInfo info = {
 			sizeof (GdaReportEngineClass),
 			(GBaseInitFunc) NULL,
@@ -168,10 +170,10 @@ gda_report_engine_get_type (void)
 			0
 		};
 		
-		g_static_mutex_lock (&registering);
+		g_mutex_lock (&registering);
 		if (type == 0)
 			type = g_type_register_static (G_TYPE_OBJECT, "GdaReportEngine", &info, 0);
-		g_static_mutex_unlock (&registering);
+		g_mutex_unlock (&registering);
 	}
 
 	return type;
@@ -1043,7 +1045,7 @@ assign_parameters_values (GdaReportEngine *engine, RunContext *context, GdaSet *
 {
 	if (plist) {
 		GSList *list;
-		for (list = plist->holders; list; list = list->next) {
+		for (list = gda_set_get_holders (plist); list; list = list->next) {
 			GdaHolder *source_param;
 			source_param = run_context_find_param (engine, context, 
 							       BAD_CAST gda_holder_get_id (GDA_HOLDER (list->data)));
@@ -1107,24 +1109,24 @@ evaluate_expression (GdaReportEngine *engine, RunContext *context, const gchar *
 	GdaDataModel *model;
 	GValue *retval;
 	gchar *sql;
-	static GStaticMutex init_mutex = G_STATIC_MUTEX_INIT;
+	static GMutex init_mutex;
 	GdaConnection *vcnc = NULL;
 	GdaSqlParser *parser;
 
 	/* create a virtual connection to execute the expression, if it's the first time */
-	g_static_mutex_lock (&init_mutex);
+	g_mutex_lock (&init_mutex);
 	if (!vcnc) {
 		static GdaVirtualProvider *provider = NULL;
 
 		if (!provider)
 			provider = gda_vprovider_data_model_new ();
-		vcnc = gda_virtual_connection_open (provider, error);
+		vcnc = gda_virtual_connection_open (provider, GDA_CONNECTION_OPTIONS_NONE, error);
 		if (! vcnc) {
-			g_static_mutex_unlock (&init_mutex);
+			g_mutex_unlock (&init_mutex);
 			return NULL;
 		}
 	}
-	g_static_mutex_unlock (&init_mutex);
+	g_mutex_unlock (&init_mutex);
 
 	/* parser */
 	parser = g_object_get_data (G_OBJECT (context->cnc), "__gda_parser");
@@ -1234,7 +1236,7 @@ value_to_node (GdaReportEngine *engine, G_GNUC_UNUSED RunContext *context, const
 			g_hash_table_insert (data_handlers, (gpointer) GDA_TYPE_USHORT, gda_handler_numerical_new ());
 			g_hash_table_insert (data_handlers, (gpointer) G_TYPE_STRING, gda_handler_string_new ());
 			g_hash_table_insert (data_handlers, (gpointer) GDA_TYPE_TIME, gda_handler_time_new ());
-			g_hash_table_insert (data_handlers, (gpointer) GDA_TYPE_TIMESTAMP, gda_handler_time_new ());
+			g_hash_table_insert (data_handlers, (gpointer) G_TYPE_DATE_TIME, gda_handler_time_new ());
 			g_hash_table_insert (data_handlers, (gpointer) G_TYPE_CHAR, gda_handler_numerical_new ());
 			g_hash_table_insert (data_handlers, (gpointer) G_TYPE_UCHAR, gda_handler_numerical_new ());
 			g_hash_table_insert (data_handlers, (gpointer) G_TYPE_ULONG, gda_handler_numerical_new ());
@@ -1307,7 +1309,7 @@ run_context_find_param (GdaReportEngine *engine, RunContext *context, const xmlC
 					retval = gda_set_get_holder (GDA_SET (ctx->iter), col_name + 1);
 				else if (*col_name == '#') {
 					gint nth = atoi (col_name + 1);
-					retval = g_slist_nth_data (GDA_SET (ctx->iter)->holders, nth);
+					retval = g_slist_nth_data (gda_set_get_holders (GDA_SET (ctx->iter)), nth);
 				}
 				else 
 					retval = g_object_get_data (G_OBJECT (ctx->model), (gchar*) name);
