@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -32,13 +32,16 @@
  */
 gboolean
 common_pict_load_data (PictOptions *options, const GValue *value, PictBinData *bindata, 
-		       const gchar **stock, GError **error)
+		       const gchar **out_icon_name, GError **error)
 {
 	gboolean allok = TRUE;
 
+	g_assert (out_icon_name);
+	*out_icon_name = NULL;
+
 	if (value) {
 		if (gda_value_is_null ((GValue *) value)) {
-			*stock = GTK_STOCK_MISSING_IMAGE;
+			*out_icon_name = "image-missing";
 			g_set_error (error, GDAUI_DATA_ENTRY_ERROR, GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
 				     "%s", _("No data"));
 			allok = FALSE;
@@ -50,27 +53,27 @@ common_pict_load_data (PictOptions *options, const GValue *value, PictBinData *b
 
 				blob = (GdaBlob *) gda_value_get_blob ((GValue *) value);
 				g_assert (blob);
-				bin = (GdaBinary *) blob;
-				if (blob->op &&
-				    (bin->binary_length != gda_blob_op_get_length (blob->op)))
-					gda_blob_op_read_all (blob->op, blob);
-				if (bin->binary_length > 0) {
-					bindata->data = g_new (guchar, bin->binary_length);
-					bindata->data_length = bin->binary_length;
-					memcpy (bindata->data, bin->data, bin->binary_length);
+				bin = gda_blob_get_binary (blob);
+				if (gda_blob_get_op (blob) &&
+				    (gda_binary_get_size (bin) != gda_blob_op_get_length (gda_blob_get_op (blob))))
+					gda_blob_op_read_all (gda_blob_get_op (blob), blob);
+				if (gda_binary_get_size (bin) > 0) {
+					bindata->data = g_new (guchar, gda_binary_get_size (bin));
+					bindata->data_length = gda_binary_get_size (bin);
+					memcpy (bindata->data, gda_binary_get_data (bin), gda_binary_get_size (bin));
 				}
 			}
 			else if (G_VALUE_TYPE ((GValue *) value) == GDA_TYPE_BINARY) {
 				GdaBinary *bin;
 
 				bin = (GdaBinary *) gda_value_get_binary ((GValue *) value);
-				if (bin && bin->binary_length > 0) {
-					bindata->data = g_new (guchar, bin->binary_length);
-					bindata->data_length = bin->binary_length;
-					memcpy (bindata->data, bin->data, bin->binary_length);
+				if (bin && gda_binary_get_size (bin) > 0) {
+					bindata->data = g_new (guchar, gda_binary_get_size (bin));
+					bindata->data_length = gda_binary_get_size (bin);
+					memcpy (bindata->data, gda_binary_get_data (bin), gda_binary_get_size (bin));
 				}
 				else {
-					*stock = GTK_STOCK_DIALOG_ERROR;
+					*out_icon_name = "dialog-error";
 					g_set_error (error, GDAUI_DATA_ENTRY_ERROR, GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
 						     "%s", _("No data"));
 					allok = FALSE;
@@ -101,14 +104,14 @@ common_pict_load_data (PictOptions *options, const GValue *value, PictBinData *b
 					}
 				}
 				else {
-					*stock = GTK_STOCK_MISSING_IMAGE;
+					*out_icon_name = "image-missing";
 					g_set_error (error, GDAUI_DATA_ENTRY_ERROR, GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
 						     "%s", _("Empty data"));
 					allok = FALSE;
 				}
 			}
 			else {
-				*stock = GTK_STOCK_DIALOG_ERROR;
+				*out_icon_name = "dialog-error";
 				g_set_error (error, GDAUI_DATA_ENTRY_ERROR, GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
 					     "%s", _("Unhandled type of data"));
 				allok = FALSE;
@@ -116,7 +119,7 @@ common_pict_load_data (PictOptions *options, const GValue *value, PictBinData *b
 		}
 	}
 	else {
-		*stock = GTK_STOCK_MISSING_IMAGE;
+		*out_icon_name = "image-missing";
 		g_set_error (error, GDAUI_DATA_ENTRY_ERROR, GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
 			     "%s", _("Empty data"));
 		allok = FALSE;
@@ -191,88 +194,44 @@ loader_size_prepared_cb (GdkPixbufLoader *loader, gint width, gint height, PictA
  */
 GdkPixbuf * 
 common_pict_make_pixbuf (PictOptions *options, PictBinData *bindata, PictAllocation *allocation, 
-			 const gchar **stock, GError **error)
+			 const gchar **out_icon_name, GError **error)
 {
 	GdkPixbuf *retpixbuf = NULL;
+	g_assert (out_icon_name);
+	*out_icon_name = NULL;
 
 	if (bindata->data) {
-		if (options->serialize) {
-			GdkPixdata pixdata;
-			GError *loc_error = NULL;
-			
-			if (!gdk_pixdata_deserialize (&pixdata, bindata->data_length, 
-						      bindata->data, &loc_error)) {
-				g_free (bindata->data);
-				bindata->data = NULL;
-				bindata->data_length = 0;
+		GdkPixbufLoader *loader;
+		GError *loc_error = NULL;
 
-				*stock = GTK_STOCK_DIALOG_ERROR;
-				g_set_error (error, GDAUI_DATA_ENTRY_ERROR, GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
-					     _("Error while deserializing data:\n%s"),
-					     loc_error && loc_error->message ? loc_error->message : _("No detail"));
-
-				g_error_free (loc_error);
-			}
-			else {
-				retpixbuf = gdk_pixbuf_from_pixdata (&pixdata, FALSE, &loc_error);
-				if (!retpixbuf) {
-					*stock = GTK_STOCK_DIALOG_ERROR;
-					g_set_error (error, GDAUI_DATA_ENTRY_ERROR, GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
-						     _("Error while interpreting data as an image:\n%s"),
-						     loc_error && loc_error->message ? loc_error->message : _("No detail"));
-					g_error_free (loc_error);
-				}
-				else {
-					/* scale resulting pixbuf */
-					GdkPixbuf *tmp;
-					gint width, height, w, h;
-					width = gdk_pixbuf_get_width (retpixbuf);
-					height = gdk_pixbuf_get_height (retpixbuf);
-					compute_reduced_size (width, height, allocation, &w, &h);
-					if ((w != width) || (h != height)) {
-						tmp = gdk_pixbuf_scale_simple (retpixbuf, w, h, 
-									       GDK_INTERP_BILINEAR);
-						if (tmp) {
-							g_object_unref (retpixbuf);
-							retpixbuf = tmp;
-						}
-					}
-				}
+		loader = gdk_pixbuf_loader_new ();
+		if (allocation)
+			g_signal_connect (G_OBJECT (loader), "size-prepared",
+					  G_CALLBACK (loader_size_prepared_cb), allocation);
+		if (gdk_pixbuf_loader_write (loader, bindata->data, bindata->data_length, &loc_error) &&
+		    gdk_pixbuf_loader_close (loader, &loc_error)) {
+			retpixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
+			if (!retpixbuf) {
+				if (loc_error)
+					g_propagate_error (error, loc_error);
+				*out_icon_name = "image-missing";
 			}
+			else
+				g_object_ref (retpixbuf);
 		}
 		else {
-			GdkPixbufLoader *loader;
-			GError *loc_error = NULL;
-			
-			loader = gdk_pixbuf_loader_new ();
-			if (allocation)
-				g_signal_connect (G_OBJECT (loader), "size-prepared",
-						  G_CALLBACK (loader_size_prepared_cb), allocation);
-			if (gdk_pixbuf_loader_write (loader, bindata->data, bindata->data_length, &loc_error) &&
-			    gdk_pixbuf_loader_close (loader, &loc_error)) {
-				retpixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
-				if (!retpixbuf) {
-					if (loc_error)
-						g_propagate_error (error, loc_error);
-					*stock = GTK_STOCK_MISSING_IMAGE;
-				}
-				else
-					g_object_ref (retpixbuf);
-			}
-			else {
-				gchar *notice_msg;
-				notice_msg = g_strdup_printf (_("Error while interpreting data as an image:\n%s"),
-							      loc_error && loc_error->message ? loc_error->message : _("No detail"));
-				*stock = GTK_STOCK_DIALOG_WARNING;
-				g_set_error_literal (error, loc_error ? loc_error->domain : GDAUI_DATA_ENTRY_ERROR,
-						     loc_error ? loc_error->code : GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
-						     notice_msg);
-				g_error_free (loc_error);
-				g_free (notice_msg);
-			}
-			
-			g_object_unref (loader);
+			gchar *notice_msg;
+			notice_msg = g_strdup_printf (_("Error while interpreting data as an image:\n%s"),
+						      loc_error && loc_error->message ? loc_error->message : _("No detail"));
+			*out_icon_name = "dialog-warning";
+			g_set_error_literal (error, loc_error ? loc_error->domain : GDAUI_DATA_ENTRY_ERROR,
+					     loc_error ? loc_error->code : GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
+					     notice_msg);
+			g_error_free (loc_error);
+			g_free (notice_msg);
 		}
+
+		g_object_unref (loader);
 	}
 
 	return retpixbuf;
@@ -288,11 +247,10 @@ typedef struct {
 	PictCallback  callback;
 	gpointer      data;
 } PictMenuData;
-static void file_load_cb (GtkWidget *button, PictMenuData *menudata);
-static void file_save_cb (GtkWidget *button, PictMenuData *menudata);
-static void copy_cb (GtkWidget *button, PictMenuData *menudata);
+static void file_load_cb (GtkMenuItem *mitem, PictMenuData *menudata);
+static void file_save_cb (GtkMenuItem *mitem, PictMenuData *menudata);
+static void copy_cb (GtkMenuItem *mitem, PictMenuData *menudata);
 
-/* Commented out because it's not used:
 static void
 menudata_free (PictMenuData *menudata)
 {
@@ -302,7 +260,6 @@ menudata_free (PictMenuData *menudata)
 	}
 	g_free (menudata);
 }
-*/
 
 void
 common_pict_create_menu (PictMenu *pictmenu, GtkWidget *attach_to, PictBinData *bindata, PictOptions *options,
@@ -320,7 +277,7 @@ common_pict_create_menu (PictMenu *pictmenu, GtkWidget *attach_to, PictBinData *
 	menudata->data = data;
 	
 	menu = gtk_menu_new ();
-	g_object_set_data_full (G_OBJECT (menu), "menudata", menudata, g_free);
+	g_object_set_data_full (G_OBJECT (menu), "menudata", menudata, (GDestroyNotify) menudata_free);
 	g_signal_connect (menu, "deactivate", 
 			  G_CALLBACK (gtk_widget_hide), NULL);
 	pictmenu->menu = menu;
@@ -336,6 +293,8 @@ common_pict_create_menu (PictMenu *pictmenu, GtkWidget *attach_to, PictBinData *
 	mitem = gtk_menu_item_new_with_mnemonic (_("_Load image from file"));
 	gtk_widget_show (mitem);
 	gtk_container_add (GTK_CONTAINER (menu), mitem);
+	if (attach_to)
+		g_object_set_data_full (G_OBJECT (mitem), "attach-to", g_object_ref (attach_to), g_object_unref);
 	g_signal_connect (mitem, "activate",
 			  G_CALLBACK (file_load_cb), menudata);
 	pictmenu->load_mitem = mitem;
@@ -343,6 +302,8 @@ common_pict_create_menu (PictMenu *pictmenu, GtkWidget *attach_to, PictBinData *
 	mitem = gtk_menu_item_new_with_mnemonic (_("_Save image"));
 	gtk_widget_show (mitem);
 	gtk_container_add (GTK_CONTAINER (menu), mitem);
+	if (attach_to)
+		g_object_set_data_full (G_OBJECT (mitem), "attach-to", g_object_ref (attach_to), g_object_unref);
 	g_signal_connect (mitem, "activate",
 			  G_CALLBACK (file_save_cb), menudata);
 	gtk_widget_set_sensitive (mitem, bindata->data ? TRUE : FALSE);
@@ -352,16 +313,20 @@ common_pict_create_menu (PictMenu *pictmenu, GtkWidget *attach_to, PictBinData *
 }
 
 static void
-file_load_cb (GtkWidget *button, PictMenuData *menudata)
+file_load_cb (GtkMenuItem *mitem, PictMenuData *menudata)
 {
 	GtkWidget *dlg;
 	GtkFileFilter *filter;
 
+	GtkWidget *attach_to;
+	attach_to = g_object_get_data (G_OBJECT (mitem), "attach-to");
+	if (!attach_to)
+		attach_to = GTK_WIDGET (mitem);
 	dlg = gtk_file_chooser_dialog_new (_("Select image to load"), 
-					   GTK_WINDOW (gtk_widget_get_toplevel (button)),
+					   GTK_WINDOW (gtk_widget_get_toplevel (attach_to)),
 					   GTK_FILE_CHOOSER_ACTION_OPEN, 
-					   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					   GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+					   _("_Cancel"), GTK_RESPONSE_CANCEL,
+					   _("_Open"), GTK_RESPONSE_ACCEPT,
 					   NULL);
 	filter = gtk_file_filter_new ();
 	gtk_file_filter_add_pixbuf_formats (filter);
@@ -380,32 +345,8 @@ file_load_cb (GtkWidget *button, PictMenuData *menudata)
 
 		if (g_file_get_contents (filename, &data, &length, &error)) {
 			g_free (menudata->bindata->data);
-			menudata->bindata->data = NULL;
-			menudata->bindata->data_length = 0;
-
-			if (menudata->options->serialize) {
-				GdkPixdata pixdata;
-				GdkPixbuf *pixbuf;
-				guint stream_length;
-
-				pixbuf = gdk_pixbuf_new_from_file (filename, &error);
-				if (pixbuf) {
-					gdk_pixdata_from_pixbuf (&pixdata, pixbuf, TRUE);
-					menudata->bindata->data = gdk_pixdata_serialize (&pixdata, &stream_length);
-					menudata->bindata->data_length = stream_length;
-
-					g_object_unref (pixbuf);
-					g_free (data);
-				}
-				else {
-					menudata->bindata->data = (guchar *) data;
-					menudata->bindata->data_length = length;
-				}
-			}
-			else {
-				menudata->bindata->data = (guchar *) data;
-				menudata->bindata->data_length = length;
-			}
+			menudata->bindata->data = (guchar *) data;
+			menudata->bindata->data_length = length;
 
 			/* call the callback */
 			if (menudata->callback)
@@ -415,15 +356,16 @@ file_load_cb (GtkWidget *button, PictMenuData *menudata)
 		}
 		else {
 			GtkWidget *msg;
-
-			msg = gtk_message_dialog_new_with_markup (GTK_WINDOW (gtk_widget_get_toplevel (button)), 
+			gchar *tmp;
+			tmp = g_strdup_printf (_("Could not load the contents of '%s'"), filename);
+			msg = gtk_message_dialog_new_with_markup (GTK_WINDOW (gtk_widget_get_toplevel (attach_to)),
 								  GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
 								  GTK_BUTTONS_CLOSE,
-								  _("Could not load the contents of '%s':\n %s"), 
-								  filename, 
+								  "<b>%s:</b>\n%s: %s",
+								  _("Error"), tmp,
 								  error && error->message ? error->message : _("No detail"));
-			if (error)
-				g_error_free (error);
+			g_free (tmp);
+			g_clear_error (&error);
 			gtk_widget_destroy (dlg);
 			dlg = NULL;
 
@@ -457,13 +399,18 @@ add_if_writable (GdkPixbufFormat *data, PictFormat *format)
 }
 
 static void
-file_save_cb (GtkWidget *button, PictMenuData *menudata)
+file_save_cb (GtkMenuItem *mitem, PictMenuData *menudata)
 {
 	GtkWidget *dlg;
 	GtkWidget *combo, *expander, *hbox, *label;
 	GSList *formats;
 	PictFormat pictformat;
 
+	GtkWidget *attach_to;
+	attach_to = g_object_get_data (G_OBJECT (mitem), "attach-to");
+	if (!attach_to)
+		attach_to = GTK_WIDGET (mitem);
+
 	/* determine writable formats */
 	expander = gtk_expander_new (_("Image format"));
 	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
@@ -485,10 +432,10 @@ file_save_cb (GtkWidget *button, PictMenuData *menudata)
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
 
 	dlg = gtk_file_chooser_dialog_new (_("Select a file to save the image to"), 
-					   GTK_WINDOW (gtk_widget_get_toplevel (button)),
+					   GTK_WINDOW (gtk_widget_get_toplevel (attach_to)),
 					   GTK_FILE_CHOOSER_ACTION_SAVE, 
-					   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					   GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+					   _("_Cancel"), GTK_RESPONSE_CANCEL,
+					   _("_Save"), GTK_RESPONSE_ACCEPT,
 					   NULL);
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg),
 					     gdaui_get_default_path ());
@@ -527,15 +474,16 @@ file_save_cb (GtkWidget *button, PictMenuData *menudata)
 
 		if (!allok) {
 			GtkWidget *msg;
-				
-			msg = gtk_message_dialog_new_with_markup (GTK_WINDOW (gtk_widget_get_toplevel (button)), 
+			gchar *tmp;
+			tmp = g_strdup_printf (_("Could not save the image to '%s'"), filename);
+			msg = gtk_message_dialog_new_with_markup (GTK_WINDOW (gtk_widget_get_toplevel (attach_to)),
 								  GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
 								  GTK_BUTTONS_CLOSE,
-								  _("Could not save the image to '%s':\n %s"), 
-								  filename, 
+								  "<b>%s:</b>\n%s: %s",
+								  _("Error"), tmp,
 								  error && error->message ? error->message : _("No detail"));
-			if (error)
-				g_error_free (error);
+			g_free (tmp);
+			g_clear_error (&error);
 			gtk_widget_destroy (dlg);
 			dlg = NULL;
 			
@@ -553,10 +501,10 @@ file_save_cb (GtkWidget *button, PictMenuData *menudata)
 }
 
 static void
-copy_cb (G_GNUC_UNUSED GtkWidget *button, PictMenuData *menudata)
+copy_cb (G_GNUC_UNUSED GtkMenuItem *mitem, PictMenuData *menudata)
 {
 	GtkClipboard *cp;
-	cp = gtk_clipboard_get (gdk_atom_intern_static_string ("CLIPBOARD"));
+	cp = gtk_clipboard_get_default (gdk_display_get_default ());
 	if (!cp)
 		return;
 
@@ -637,22 +585,22 @@ common_pict_add_cached_pixbuf (PictOptions *options, const GValue *value, GdkPix
 	if (!options->pixbuf_hash || !value)
 		return;
 	else if (GDA_VALUE_HOLDS_BINARY (value)) {
-		const GdaBinary *bin;
-		bin = gda_value_get_binary (value);
+		GdaBinary *bin;
+		bin = g_value_get_boxed (value);
 		hash = g_new (gint, 1);
-		*hash = compute_hash (bin->data, bin->binary_length);
+		*hash = compute_hash (gda_binary_get_data (bin), gda_binary_get_size (bin));
 		g_hash_table_insert (options->pixbuf_hash, hash, g_object_ref (pixbuf));
 	}
 	else if (GDA_VALUE_HOLDS_BLOB (value)) {
-		const GdaBinary *bin;
-		const GdaBlob *blob;
-		blob = gda_value_get_blob (value);
-		bin = (GdaBinary *) blob;
+		GdaBinary *bin;
+		GdaBlob *blob;
+		blob = g_value_get_boxed (value);
+		bin = (GdaBinary*) gda_blob_get_binary (blob);
 		if (bin) {
-			if (!bin->data && blob->op)
-				gda_blob_op_read_all (blob->op, (GdaBlob*) blob);
+			if (!gda_binary_get_data (bin) && gda_blob_get_op (blob))
+				gda_blob_op_read_all (gda_blob_get_op (blob), (GdaBlob*) blob);
 			hash = g_new (gint, 1);
-			*hash = compute_hash (bin->data, bin->binary_length);
+			*hash = compute_hash (gda_binary_get_data (bin), gda_binary_get_size (bin));
 			g_hash_table_insert (options->pixbuf_hash, hash, g_object_ref (pixbuf));
 		}
 	}
@@ -672,22 +620,22 @@ common_pict_fetch_cached_pixbuf (PictOptions *options, const GValue *value)
 	if (!value)
 		return NULL;
 	else if (GDA_VALUE_HOLDS_BINARY (value)) {
-		const GdaBinary *bin;
-		bin = gda_value_get_binary (value);
+		GdaBinary *bin;
+		bin = g_value_get_boxed (value);
 		if (bin) {
-			hash = compute_hash (bin->data, bin->binary_length);
+			hash = compute_hash (gda_binary_get_data (bin), gda_binary_get_size (bin));
 			pixbuf = g_hash_table_lookup (options->pixbuf_hash, &hash);
 		}
 	}
 	else if (GDA_VALUE_HOLDS_BLOB (value)) {
-		const GdaBinary *bin;
-		const GdaBlob *blob;
-		blob = gda_value_get_blob (value);
-		bin = (GdaBinary *) blob;
+		GdaBinary *bin;
+		GdaBlob *blob;
+		blob = g_value_get_boxed (value);
+		bin = (GdaBinary*) gda_blob_get_binary (blob);
 		if (bin) {
-			if (!bin->data && blob->op)
-				gda_blob_op_read_all (blob->op, (GdaBlob*) blob);
-			hash = compute_hash (bin->data, bin->binary_length);
+			if (!gda_binary_get_data (bin) && gda_blob_get_op (blob))
+				gda_blob_op_read_all (gda_blob_get_op (blob), (GdaBlob*) blob);
+			hash = compute_hash (gda_binary_get_data (bin), gda_binary_get_size (bin));
 			pixbuf = g_hash_table_lookup (options->pixbuf_hash, &hash);
 		}
 	}
@@ -722,11 +670,6 @@ common_pict_parse_options (PictOptions *options, const gchar *options_str)
 			if (!strcmp (str, "base64")) 
 				options->encoding = ENCODING_BASE64;
 		}
-		str = gda_quark_list_find (params, "SERIALIZE");
-		if (str) {
-			if ((*str == 't') || (*str == 'T'))
-				options->serialize = TRUE;
-		}
 		gda_quark_list_free (params);
 	}
 }
