@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -39,15 +39,7 @@
 #endif
 
 #ifdef HAVE_GTKSOURCEVIEW
-  #ifdef GTK_DISABLE_SINGLE_INCLUDES
-    #undef GTK_DISABLE_SINGLE_INCLUDES
-  #endif
-
-  #include <gtksourceview/gtksourceview.h>
-  #include <gtksourceview/gtksourcelanguagemanager.h>
-  #include <gtksourceview/gtksourcebuffer.h>
-  #include <gtksourceview/gtksourcestyleschememanager.h>
-  #include <gtksourceview/gtksourcestylescheme.h>
+  #include <gtksourceview/gtksource.h>
 #endif
 
 static GdauiDataEntry *plugin_entry_filesel_create_func (GdaDataHandler *handler, GType type, const gchar *options);
@@ -275,18 +267,6 @@ plugin_init (GError **error)
 	plugin->cell_create_func = plugin_cell_renderer_pict_create_func;
 	retlist = g_slist_append (retlist, plugin);
 
-	file = gda_gbr_get_file_path (GDA_LIB_DIR, LIBGDA_ABI_NAME, "plugins", "gdaui-entry-pict-spec.xml", NULL);
-	if (! g_file_test (file, G_FILE_TEST_EXISTS)) {
-		if (error && !*error)
-			g_set_error (error, GDAUI_DATA_ENTRY_ERROR, GDAUI_DATA_ENTRY_FILE_NOT_FOUND_ERROR,
-				     _("Missing spec. file '%s'"), file);
-        }
-	else {
-		gsize len;
-		g_file_get_contents (file, &(plugin->options_xml_spec), &len, error);
-	}
-	g_free (file);
-
 	/* picture - string encoded */
 	plugin = g_new0 (GdauiPlugin, 1);
 	plugin->plugin_name = "picture_as_string";
