@@ -1,7 +1,8 @@
 /*
- * Copyright (C) 2009 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2010 - 2011 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2013 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,10 +33,8 @@
 #include <libgda-ui/gdaui-data-selector.h>
 #include <libgda-ui/gdaui-raw-form.h>
 #include <libgda-ui/gdaui-easy.h>
-#include <libgda/binreloc/gda-binreloc.h>
 #include <libgda/gda-debug-macros.h>
 
-#define SPACING 3
 static void gdaui_basic_form_class_init (GdauiBasicFormClass * class);
 static void gdaui_basic_form_init (GdauiBasicForm *wid);
 static void gdaui_basic_form_dispose (GObject *object);
@@ -106,7 +105,6 @@ static void unpack_entries (GdauiBasicForm *form);
 static void destroy_entries (GdauiBasicForm *form);
 static gchar *create_text_label_for_sentry (SingleEntry *sentry, gchar **out_title);
 
-static void gdaui_basic_form_show_entry_actions (GdauiBasicForm *form, gboolean show_actions);
 static void gdaui_basic_form_set_entries_auto_default (GdauiBasicForm *form, gboolean auto_default);
 
 static void get_rid_of_set (GdaSet *paramlist, GdauiBasicForm *form);
@@ -118,6 +116,7 @@ static void paramlist_holder_type_set_cb (GdaSet *paramlist, GdaHolder *param,
 
 static void entry_contents_modified (GdauiDataEntry *entry, SingleEntry *sentry);
 static void entry_contents_activated (GdauiDataEntry *entry, GdauiBasicForm *form);
+static void sync_entry_attributes (GdaHolder *param, SingleEntry *sentry);
 static void parameter_changed_cb (GdaHolder *param, SingleEntry *sentry);
 
 static void mark_not_null_entry_labels (GdauiBasicForm *form, gboolean show_mark);
@@ -134,8 +133,6 @@ enum {
 	PROP_0,
 	PROP_XML_LAYOUT,
 	PROP_PARAMLIST,
-	PROP_HEADERS_SENSITIVE,
-	PROP_SHOW_ACTIONS,
 	PROP_ENTRIES_AUTO_DEFAULT,
 	PROP_CAN_VEXPAND
 };
@@ -160,7 +157,6 @@ struct _GdauiBasicFormPriv
 
 	GtkWidget  *top_container;
 
-	gboolean    show_actions;
 	gboolean    entries_auto_default;
 
 	GSList     *size_groups; /* list of SizeGroup pointers */
@@ -294,21 +290,6 @@ gdaui_basic_form_class_init (GdauiBasicFormClass *klass)
 					 g_param_spec_pointer ("paramlist",
 							       _("List of parameters to show in the form"), NULL,
                                                                G_PARAM_READABLE | G_PARAM_WRITABLE));
-	/**
-	 * GdauiBasicForm:headers-sensitive:
-	 *
-	 * Deprecated
-	 */
-	g_object_class_install_property (object_class, PROP_HEADERS_SENSITIVE,
-					 g_param_spec_boolean ("headers-sensitive",
-							       "",
-							       NULL, FALSE,
-							       G_PARAM_READABLE | G_PARAM_WRITABLE));
-	g_object_class_install_property (object_class, PROP_SHOW_ACTIONS,
-					 g_param_spec_boolean ("show-actions",
-							       _("Show Entry actions"),
-							       NULL, FALSE,
-							       G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (object_class, PROP_ENTRIES_AUTO_DEFAULT,
 					 g_param_spec_boolean ("entries-auto-default",
 							       _("Entries Auto-default"),
@@ -335,7 +316,6 @@ hidden_entry_mitem_toggled_cb (GtkCheckMenuItem *check, GdauiBasicForm *form)
 static void
 do_popup_menu (GdauiBasicForm *form, GdkEventButton *event)
 {
-	int button, event_time;
 	GtkWidget *menu, *submenu, *mitem;
 	GSList *list;
 	
@@ -362,20 +342,10 @@ do_popup_menu (GdauiBasicForm *form, GdkEventButton *event)
 				  G_CALLBACK (hidden_entry_mitem_toggled_cb), form);
 	}
 		
-	if (event) {
-		button = event->button;
-		event_time = event->time;
-	}
-	else {
-		button = 0;
-		event_time = gtk_get_current_event_time ();
-	}
-
 	/* allow listeners to add their custom menu items */
 	g_signal_emit (G_OBJECT (form), gdaui_basic_form_signals [POPULATE_POPUP], 0, GTK_MENU (menu));
 
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 
-			button, event_time);
+	gtk_menu_popup_at_pointer (GTK_MENU (menu), NULL);
 }
 
 static gboolean
@@ -406,7 +376,6 @@ gdaui_basic_form_init (GdauiBasicForm *wid)
 	wid->priv->place_holders = NULL;
 	wid->priv->top_container = NULL;
 
-	wid->priv->show_actions = FALSE;
 	wid->priv->entries_auto_default = FALSE;
 
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (wid), GTK_ORIENTATION_VERTICAL);
@@ -425,10 +394,10 @@ gdaui_basic_form_init (GdauiBasicForm *wid)
 	g_signal_connect (evbox, "button-press-event",
 			  G_CALLBACK (button_press_event_cb), wid);
 
-	wid->priv->red = .98;
-	wid->priv->green = .93;
-	wid->priv->blue = .25;
-	wid->priv->alpha = .50;
+	wid->priv->red = 0.;
+	wid->priv->green = 0.;
+	wid->priv->blue = 0.;
+	wid->priv->alpha = 0.;
 }
 
 /**
@@ -472,7 +441,6 @@ get_rid_of_set (GdaSet *paramlist, GdauiBasicForm *form)
 	/* unref the paramlist */
 	g_signal_handlers_disconnect_by_func (form->priv->set_info,
 					      G_CALLBACK (paramlist_public_data_changed_cb), form);
-
 	g_signal_handlers_disconnect_by_func (paramlist,
 					      G_CALLBACK (paramlist_param_attr_changed_cb), form);
 	g_signal_handlers_disconnect_by_func (paramlist,
@@ -500,9 +468,6 @@ paramlist_holder_type_set_cb (G_GNUC_UNUSED GdaSet *paramlist, GdaHolder *param,
 	sentry = get_single_entry_for_holder (form, param);
 	if (sentry) {
 		create_entry_widget (sentry);
-		gdaui_data_entry_set_attributes (GDAUI_DATA_ENTRY (sentry->entry),
-						 form->priv->show_actions ? GDA_VALUE_ATTR_ACTIONS_SHOWN : 0,
-						 GDA_VALUE_ATTR_ACTIONS_SHOWN);
 		pack_entry_widget (sentry);
 		gdaui_basic_form_entry_set_visible (form, param, !sentry->hidden);
 	}
@@ -563,9 +528,6 @@ paramlist_param_attr_changed_cb (G_GNUC_UNUSED GdaSet *paramlist, GdaHolder *par
 		if (sentry) {
 			/* recreate an entry widget */
 			create_entry_widget (sentry);
-			gdaui_data_entry_set_attributes (GDAUI_DATA_ENTRY (sentry->entry),
-							 form->priv->show_actions ? GDA_VALUE_ATTR_ACTIONS_SHOWN : 0,
-							 GDA_VALUE_ATTR_ACTIONS_SHOWN);
 			pack_entry_widget (sentry);
 			gdaui_basic_form_entry_set_visible (form, param, !sentry->hidden);
 		}
@@ -669,7 +631,7 @@ gdaui_basic_form_set_property (GObject *object,
 				g_return_if_fail (GDA_IS_SET (form->priv->set));
 
 				g_object_ref (form->priv->set);
-				form->priv->set_info = _gdaui_set_new (GDA_SET (form->priv->set));
+				form->priv->set_info = gdaui_set_new (GDA_SET (form->priv->set));
 
 				g_signal_connect (form->priv->set_info, "public-data-changed",
 						  G_CALLBACK (paramlist_public_data_changed_cb), form);
@@ -683,11 +645,6 @@ gdaui_basic_form_set_property (GObject *object,
 				g_signal_emit (G_OBJECT (form), gdaui_basic_form_signals[LAYOUT_CHANGED], 0);
 			}
 			break;
-		case PROP_HEADERS_SENSITIVE:
-			break;
-		case PROP_SHOW_ACTIONS:
-			gdaui_basic_form_show_entry_actions (form, g_value_get_boolean (value));
-			break;
 		case PROP_ENTRIES_AUTO_DEFAULT:
 			gdaui_basic_form_set_entries_auto_default (form, g_value_get_boolean (value));
 			break;
@@ -712,11 +669,6 @@ gdaui_basic_form_get_property (GObject *object,
 		case PROP_PARAMLIST:
 			g_value_set_pointer (value, form->priv->set);
 			break;
-		case PROP_HEADERS_SENSITIVE:
-			break;
-		case PROP_SHOW_ACTIONS:
-			g_value_set_boolean (value, form->priv->show_actions);
-			break;
 		case PROP_ENTRIES_AUTO_DEFAULT:
 			g_value_set_boolean (value, form->priv->entries_auto_default);
 			break;
@@ -932,7 +884,10 @@ create_entry_widget (SingleEntry *sentry)
 		const gchar *plugin = NULL;
 		const GValue *plugin_val;
 
-		g_assert (gda_set_group_get_n_nodes (sg) == 1); /* only 1 item in the list */
+		if (gda_set_group_get_n_nodes (sg) != 1) { /* only 1 item in the list */
+      g_warning (_("There are more entries than expected in group. Found %d"), gda_set_group_get_n_nodes (sg));
+      return;
+    }
 
 		param = GDA_HOLDER (gda_set_node_get_holder (gda_set_group_get_node (sg)));
 		sentry->single_param = param;
@@ -946,7 +901,7 @@ create_entry_widget (SingleEntry *sentry)
 		type = gda_holder_get_g_type (param);
 		value = val;
 		if (!value && default_val &&
-		    (G_VALUE_TYPE ((GValue *) default_val) == type))
+		    g_type_is_a (G_VALUE_TYPE ((GValue *) default_val), type))
 			value = default_val;
 
 		/* create entry */
@@ -959,6 +914,7 @@ create_entry_widget (SingleEntry *sentry)
 					   GDAUI_ATTRIBUTE_PLUGIN);
 		}
 		entry = GTK_WIDGET (gdaui_new_data_entry (type, plugin));
+		sentry->entry = GDAUI_DATA_ENTRY (entry);
 
 		/* set current value */
 		if (gda_holder_is_valid (param))
@@ -972,23 +928,11 @@ create_entry_widget (SingleEntry *sentry)
 		}
 		else if (!nnul ||
 			 (nnul && value &&
-			  (G_VALUE_TYPE ((GValue *) value) != GDA_TYPE_NULL)))
+			  !g_type_is_a (G_VALUE_TYPE ((GValue *) value), GDA_TYPE_NULL)))
 			gdaui_data_entry_set_reference_value (GDAUI_DATA_ENTRY (entry), value);
 
-		if (default_val) {
+		if (default_val)
 			gdaui_data_entry_set_default_value (GDAUI_DATA_ENTRY (entry), default_val);
-			gdaui_data_entry_set_attributes (GDAUI_DATA_ENTRY (entry),
-							 GDA_VALUE_ATTR_CAN_BE_DEFAULT,
-							 GDA_VALUE_ATTR_CAN_BE_DEFAULT);
-			if (gda_holder_value_is_default (param))
-				gdaui_data_entry_set_attributes (GDAUI_DATA_ENTRY (entry),
-							 GDA_VALUE_ATTR_IS_DEFAULT,
-							 GDA_VALUE_ATTR_IS_DEFAULT);
-		}
-
-		gdaui_data_entry_set_attributes (GDAUI_DATA_ENTRY (entry),
-						 nnul ? 0 : GDA_VALUE_ATTR_CAN_BE_NULL,
-						 GDA_VALUE_ATTR_CAN_BE_NULL);
 
 		/* connect to the parameter's changes */
 		sentry->single_signal.holder = g_object_ref (param);
@@ -1004,13 +948,15 @@ create_entry_widget (SingleEntry *sentry)
 		g_free (str);
 		g_object_ref_sink (sentry->label);
 		sentry->label_title = title;
-		gtk_misc_set_alignment (GTK_MISC (sentry->label), 0., 0.);
 		gtk_widget_show (sentry->label);
 			
 		g_object_get (G_OBJECT (param), "description", &title, NULL);
 		if (title && *title)
 			gtk_widget_set_tooltip_text (sentry->label, title);
 		g_free (title);
+
+		/* set up the data entry's attributes */
+		sync_entry_attributes (param, sentry);
 	}
 	else {
 		/* several parameters depending on the values of a GdaDataModel object */
@@ -1018,6 +964,7 @@ create_entry_widget (SingleEntry *sentry)
 		gboolean nullok = TRUE;
 			
 		entry = gdaui_entry_combo_new (sentry->form->priv->set_info, gdaui_set_group_get_source (group));
+		sentry->entry = GDAUI_DATA_ENTRY (entry);
 
 		/* connect to the parameter's changes */
 		sentry->group_signals = g_array_new (FALSE, FALSE, sizeof (SignalData));
@@ -1025,7 +972,7 @@ create_entry_widget (SingleEntry *sentry)
 			GdaHolder *param;
 
 			param = gda_set_node_get_holder (GDA_SET_NODE (plist->data));
-			if (gda_holder_get_not_null (param))
+			if (nullok && gda_holder_get_not_null (param))
 				nullok = FALSE;
 
 			SignalData sd;
@@ -1035,9 +982,6 @@ create_entry_widget (SingleEntry *sentry)
 							 sentry);
 			g_array_append_val (sentry->group_signals, sd);
 		}
-		gdaui_data_entry_set_attributes (GDAUI_DATA_ENTRY (entry),
-						 nullok ? GDA_VALUE_ATTR_CAN_BE_NULL : 0,
-						 GDA_VALUE_ATTR_CAN_BE_NULL);
 		sentry->not_null = !nullok;
 
 		/* label */
@@ -1050,21 +994,26 @@ create_entry_widget (SingleEntry *sentry)
 		g_free (str);
 		g_object_ref_sink (sentry->label);
 		sentry->label_title = title;
-		gtk_misc_set_alignment (GTK_MISC (sentry->label), 0., 0.);
 		gtk_widget_show (sentry->label);
 		ss = gda_set_group_get_source (gdaui_set_group_get_group (group));
 		title = g_object_get_data (G_OBJECT (gda_set_source_get_data_model (ss)), "descr");
 		if (title && *title)
 			gtk_widget_set_tooltip_text (sentry->label, title);
 
+		/* set up the data entry's attributes using the 1st parameter */
+		plist = gda_set_group_get_nodes (gdaui_set_group_get_group (group));
+		sync_entry_attributes (gda_set_node_get_holder (GDA_SET_NODE (plist->data)), sentry);
 	}
-	gtk_widget_set_hexpand (sentry->label, FALSE);
-	sentry->entry = GDAUI_DATA_ENTRY (entry);
 	g_object_ref_sink (sentry->entry);
+
+	gtk_widget_set_halign (sentry->label, GTK_ALIGN_END);
+	gtk_widget_set_hexpand (sentry->label, FALSE);
+	g_object_set (G_OBJECT (sentry->label), "xalign", 1., NULL);
 	gdaui_data_entry_set_editable (sentry->entry, editable);
-	gdaui_data_entry_set_unknown_color (sentry->entry, sentry->form->priv->red,
-					    sentry->form->priv->green, sentry->form->priv->blue,
-					    sentry->form->priv->alpha);
+	if (sentry->form->priv->alpha > 0.)
+		gdaui_data_entry_set_unknown_color (sentry->entry, sentry->form->priv->red,
+						    sentry->form->priv->green, sentry->form->priv->blue,
+						    sentry->form->priv->alpha);
 
 	GSList *list;
 	for (list = sentry->form->priv->size_groups; list; list = list->next) {
@@ -1119,8 +1068,6 @@ create_entries (GdauiBasicForm *form)
 		create_entry_widget (sentry);
 	}
  
-	/* Set the Show actions in the entries */
-	gdaui_basic_form_show_entry_actions (form, form->priv->show_actions);
 	/* Set the Auto entries default in the entries */
 	gdaui_basic_form_set_entries_auto_default (form, form->priv->entries_auto_default);
 
@@ -1182,10 +1129,16 @@ pack_entries_in_table (GdauiBasicForm *form)
 
 	/* creating a table for all the entries */
 	grid = gtk_grid_new ();
-	gtk_grid_set_row_spacing (GTK_GRID (grid), SPACING);
-	gtk_grid_set_column_spacing (GTK_GRID (grid), SPACING);
+	gtk_grid_set_row_spacing (GTK_GRID (grid), GDAUI_HIG_FORM_VSPACE);
+	gtk_grid_set_column_spacing (GTK_GRID (grid), GDAUI_HIG_FORM_HSPACE);
 	form->priv->top_container = grid;
 	gtk_box_pack_start (GTK_BOX (form->priv->mainbox), grid, TRUE, TRUE, 0);
+	g_object_set (G_OBJECT (grid),
+		      "margin-top", GDAUI_HIG_FORM_VBORDER,
+		      "margin-bottom", GDAUI_HIG_FORM_VBORDER,
+		      "margin-start", GDAUI_HIG_FORM_HBORDER,
+		      "margin-end", GDAUI_HIG_FORM_HBORDER, NULL);
+		      
 	for (list = form->priv->s_entries, i = 0;
 	     list;
 	     list = list->next, i++) {
@@ -1285,7 +1238,7 @@ load_xml_layout_children (GdauiBasicForm *form, xmlNodePtr parent_node)
 		if (wid) {
 			switch (ctype) {
 			case TOP_BOX:
-				gtk_box_pack_start (GTK_BOX (top), wid, TRUE, TRUE, SPACING);
+				gtk_box_pack_start (GTK_BOX (top), wid, TRUE, TRUE, GDAUI_HIG_FORM_VSPACE);
 				break;
 			case TOP_PANED:
 				if (pos == 0)
@@ -1351,7 +1304,9 @@ load_xml_layout_column (GdauiBasicForm *form, xmlNodePtr box_node)
 						}
 						sentry->label = gtk_label_new ((gchar*) label);
 						g_object_ref_sink (sentry->label);
-						gtk_misc_set_alignment (GTK_MISC (sentry->label), 0., 0.);
+						gtk_widget_set_halign (sentry->label, GTK_ALIGN_END);
+						gtk_widget_set_hexpand (sentry->label, FALSE);
+						g_object_set (G_OBJECT (sentry->label), "xalign", 1., NULL);
 
 						xmlFree (label);
 					}
@@ -1393,8 +1348,8 @@ load_xml_layout_column (GdauiBasicForm *form, xmlNodePtr box_node)
 			g_warning ("Unknown node type '%s', ignoring", (gchar*) child->name);	
 	}
 
-	gtk_grid_set_row_spacing (GTK_GRID (grid), SPACING);
-	gtk_grid_set_column_spacing (GTK_GRID (grid), SPACING);
+	gtk_grid_set_row_spacing (GTK_GRID (grid), GDAUI_HIG_FORM_VSPACE);
+	gtk_grid_set_column_spacing (GTK_GRID (grid), GDAUI_HIG_FORM_HSPACE);
 	return grid;
 }
 
@@ -1414,7 +1369,9 @@ load_xml_layout_section (GdauiBasicForm *form, xmlNodePtr section_node)
 		label = gtk_label_new ("");
 		gtk_label_set_markup (GTK_LABEL (label), str);
 		g_free (str);
-		gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
+		gtk_widget_set_halign (label, GTK_ALIGN_END);
+		gtk_widget_set_hexpand (label, FALSE);
+		g_object_set (G_OBJECT (label), "xalign", 1., NULL);
 
 		GtkWidget *vbox;
 		vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
@@ -1444,14 +1401,14 @@ mark_not_null_entry_labels (GdauiBasicForm *form, gboolean show_mark)
 
 	for (list = form->priv->s_entries; list; list = list->next) {
 		SingleEntry *sentry = (SingleEntry*) list->data;
-		gchar *str;
-
-		str = _gdaui_utility_markup_title (sentry->label_title, !show_mark || !sentry->not_null);
-		if (show_mark && sentry->not_null)
+		if (show_mark && sentry->not_null) {
+			gchar *str;
+			str = _gdaui_utility_markup_title (sentry->label_title, !show_mark || !sentry->not_null);
 			gtk_label_set_markup (GTK_LABEL (sentry->label), str);
+			g_free (str);
+		}
 		else
-			gtk_label_set_text (GTK_LABEL (sentry->label), str);
-		g_free (str);
+			gtk_label_set_text (GTK_LABEL (sentry->label), sentry->label_title);
 	}
 }
 
@@ -1562,6 +1519,64 @@ entry_contents_modified (GdauiDataEntry *entry, SingleEntry *sentry)
 	}
 }
 
+static void
+sync_entry_attributes (GdaHolder *param, SingleEntry *sentry)
+{
+	const GValue *value = gda_holder_get_value (param);
+	GdauiDataEntry *entry;
+	gboolean param_valid;
+	param_valid = gda_holder_is_valid (param);
+
+	entry = sentry->entry;
+	if (sentry->single_param) {
+		gdaui_data_entry_set_value (entry, param_valid ? value : NULL);
+		sentry->not_null = gda_holder_get_not_null (param);
+	}
+	else {
+		GSList *values = NULL;
+		GSList *list;
+		gboolean allnull = TRUE;
+		gboolean nullok = TRUE;
+
+		for (list = gda_set_group_get_nodes (gdaui_set_group_get_group (sentry->group));
+		     list; list = list->next) {
+			const GValue *pvalue;
+			pvalue = gda_holder_get_value (gda_set_node_get_holder (GDA_SET_NODE (list->data)));
+			param_valid = param_valid && gda_holder_is_valid (param);
+			values = g_slist_append (values, (GValue *) pvalue);
+			if (allnull && pvalue &&
+			    (G_VALUE_TYPE ((GValue *) pvalue) != GDA_TYPE_NULL))
+				allnull = FALSE;
+			if (nullok && gda_holder_get_not_null (param))
+				nullok = FALSE;
+		}
+
+		if (!allnull)
+			gdaui_entry_combo_set_values (GDAUI_ENTRY_COMBO (entry), values);
+		else
+			gdaui_entry_combo_set_values (GDAUI_ENTRY_COMBO (entry), NULL);
+
+		g_slist_free (values);
+		sentry->not_null = !nullok;
+	}
+
+	g_signal_emit (G_OBJECT (sentry->form), gdaui_basic_form_signals[HOLDER_CHANGED], 0,
+		       param, FALSE);
+
+	/* correctly update the NULLOK status */
+	GdaValueAttribute attr;
+	gboolean nullok;
+	attr = gdaui_data_entry_get_attributes (entry);
+	nullok = !sentry->not_null;
+	if (((attr & GDA_VALUE_ATTR_CAN_BE_NULL) && !nullok) ||
+	    (! (attr & GDA_VALUE_ATTR_CAN_BE_NULL) && nullok))
+		gdaui_data_entry_set_attributes (entry, nullok ? GDA_VALUE_ATTR_CAN_BE_NULL : 0,
+						 GDA_VALUE_ATTR_CAN_BE_NULL);
+
+	/* update the VALID atttibute */
+	gdaui_data_entry_set_attributes (entry, (!param_valid) ? GDA_VALUE_ATTR_DATA_NON_VALID : 0,
+					 GDA_VALUE_ATTR_DATA_NON_VALID);
+}
 
 /*
  * Called when a parameter changes
@@ -1571,22 +1586,20 @@ entry_contents_modified (GdauiDataEntry *entry, SingleEntry *sentry)
 static void
 parameter_changed_cb (GdaHolder *param, SingleEntry *sentry)
 {
-	const GValue *value = gda_holder_get_value (param);
 	GdauiDataEntry *entry;
-
 	entry = sentry->entry;
 	if (sentry->forward_param_updates) {
-		gboolean param_valid;
 		gboolean default_if_invalid = FALSE;
+		gboolean param_valid;
+		param_valid = gda_holder_is_valid (param);
 
 		/* There can be a feedback from the entry if the param is invalid and "set-default-if-invalid"
 		   exists and is TRUE */
-		param_valid = gda_holder_is_valid (param);
 		if (!param_valid)
 			if (g_object_class_find_property (G_OBJECT_GET_CLASS (entry),
 							  "set-default-if-invalid"))
 				g_object_get (G_OBJECT (entry),
-					      "set-default-if-invalid", &default_if_invalid, NULL);
+				"set-default-if-invalid", &default_if_invalid, NULL);
 
 		/* updating the corresponding entry */
 		if (! default_if_invalid) {
@@ -1595,58 +1608,14 @@ parameter_changed_cb (GdaHolder *param, SingleEntry *sentry)
 			g_signal_handler_block (G_OBJECT (entry),
 						sentry->entry_contents_activated_id);
 		}
-
-		if (sentry->single_param)
-			gdaui_data_entry_set_value (entry, param_valid ? value : NULL);
-		else {
-			GSList *values = NULL;
-			GSList *list;
-			gboolean allnull = TRUE;
-
-			for (list = gda_set_group_get_nodes (gdaui_set_group_get_group (sentry->group));
-			     list; list = list->next) {
-				const GValue *pvalue;
-				pvalue = gda_holder_get_value (gda_set_node_get_holder (GDA_SET_NODE (list->data)));
-				values = g_slist_append (values, (GValue *) pvalue);
-				if (allnull && pvalue &&
-				    (G_VALUE_TYPE ((GValue *) pvalue) != GDA_TYPE_NULL))
-					allnull = FALSE;
-			}
-
-			if (!allnull)
-				gdaui_entry_combo_set_values (GDAUI_ENTRY_COMBO (entry), values);
-			else
-				gdaui_entry_combo_set_values (GDAUI_ENTRY_COMBO (entry), NULL);
-
-			g_slist_free (values);
-		}
-
+		sync_entry_attributes (param, sentry);
 		if (! default_if_invalid) {
 			g_signal_handler_unblock (G_OBJECT (entry),
 						  sentry->entry_contents_modified_id);
 			g_signal_handler_unblock (G_OBJECT (entry),
 						  sentry->entry_contents_activated_id);
-		}		
-
-		gdaui_entry_shell_set_unknown (GDAUI_ENTRY_SHELL (entry),
-					       !gda_holder_is_valid (param));
-
-		g_signal_emit (G_OBJECT (sentry->form), gdaui_basic_form_signals[HOLDER_CHANGED], 0,
-			       param, FALSE);
+		}
 	}
-	else
-		gdaui_entry_shell_set_unknown (GDAUI_ENTRY_SHELL (entry),
-					       !gda_holder_is_valid (param));
-
-	/* correctly update the NULLOK status */
-	GdaValueAttribute attr;
-	gboolean nullok;
-	attr = gdaui_data_entry_get_attributes (entry);
-	nullok = !gda_holder_get_not_null (param);
-	if (((attr & GDA_VALUE_ATTR_CAN_BE_NULL) && !nullok) ||
-	    (! (attr & GDA_VALUE_ATTR_CAN_BE_NULL) && nullok))
-		gdaui_data_entry_set_attributes (entry, nullok ? GDA_VALUE_ATTR_CAN_BE_NULL : 0,
-						 GDA_VALUE_ATTR_CAN_BE_NULL);
 }
 
 /**
@@ -1771,33 +1740,6 @@ gdaui_basic_form_has_changed (GdauiBasicForm *form)
 	return FALSE;
 }
 
-/*
- * gdaui_basic_form_show_entry_actions
- * @form: a #GdauiBasicForm widget
- * @show_actions: a boolean
- *
- * Show or hide the actions button available at the end of each data entry
- * in the form
- */
-static void
-gdaui_basic_form_show_entry_actions (GdauiBasicForm *form, gboolean show_actions)
-{
-	GSList *list;
-	guint show;
-
-	g_return_if_fail (GDAUI_IS_BASIC_FORM (form));
-
-	show = show_actions ? GDA_VALUE_ATTR_ACTIONS_SHOWN : 0;
-	form->priv->show_actions = show_actions;
-
-	for (list = form->priv->s_entries; list; list = list->next) {
-		SingleEntry *sentry = (SingleEntry*) list->data;
-		gdaui_data_entry_set_attributes (GDAUI_DATA_ENTRY (sentry->entry), show,
-						 GDA_VALUE_ATTR_ACTIONS_SHOWN);
-		/* mark_not_null_entry_labels (form, show_actions); */
-	}
-}
-
 /**
  * gdaui_basic_form_reset:
  * @form: a #GdauiBasicForm widget
@@ -1929,7 +1871,7 @@ gdaui_basic_form_entry_grab_focus (GdauiBasicForm *form, GdaHolder *holder)
 
 	if (!entry && form->priv->set) {
 		GSList *list;
-		for (list = form->priv->set->holders; list; list = list->next) {
+		for (list = gda_set_get_holders (form->priv->set); list; list = list->next) {
 			GdaHolder *holder;
 			holder = GDA_HOLDER (list->data);
 			if (!gda_holder_is_valid (holder)) {
@@ -2156,10 +2098,8 @@ gdaui_basic_form_new_in_dialog (GdaSet *data_set, GtkWindow *parent,
 
 	dlg = gtk_dialog_new_with_buttons (rtitle, parent,
 					   GTK_DIALOG_MODAL,
-					   GTK_STOCK_CANCEL,
-					   GTK_RESPONSE_REJECT,
-					   GTK_STOCK_OK,
-					   GTK_RESPONSE_ACCEPT,
+					   _("_Cancel"), GTK_RESPONSE_REJECT,
+					   _("_Ok"), GTK_RESPONSE_ACCEPT,
 					   NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_ACCEPT);
 	if (header && *header) {
@@ -2167,12 +2107,14 @@ gdaui_basic_form_new_in_dialog (GdaSet *data_set, GtkWindow *parent,
 		gchar *str;
 
 		label = gtk_label_new (NULL);
-		gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
+		gtk_widget_set_halign (label, GTK_ALIGN_END);
+		gtk_widget_set_hexpand (label, FALSE);
+		g_object_set (G_OBJECT (label), "xalign", 1., NULL);
 		str = g_markup_printf_escaped ("<b>%s:</b>", header);
 		gtk_label_set_markup (GTK_LABEL (label), str);
 		g_free (str);
 		gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg))),
-				    label, FALSE, FALSE, SPACING);
+				    label, FALSE, FALSE, GDAUI_HIG_FORM_VSEP);
 		gtk_widget_show (label);
 	}
 
@@ -2180,7 +2122,7 @@ gdaui_basic_form_new_in_dialog (GdaSet *data_set, GtkWindow *parent,
 	can_expand = gtk_widget_compute_expand (GTK_WIDGET (form), GTK_ORIENTATION_VERTICAL);
 	gtk_container_set_border_width (GTK_CONTAINER (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg)))), 4);
 	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg))), form,
-			    can_expand, can_expand, 10);
+			    can_expand, can_expand, 0);
 
 	g_signal_connect (G_OBJECT (form), "holder-changed",
 			  G_CALLBACK (form_holder_changed), dlg);
@@ -2230,15 +2172,22 @@ gdaui_basic_form_set_layout_from_file (GdauiBasicForm *form, const gchar *file_n
 
         xmlDtdPtr dtd = NULL;
 
-	gchar *file = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, "dtd", "gdaui-layout.dtd", NULL);
-        if (g_file_test (file, G_FILE_TEST_EXISTS))
-                dtd = xmlParseDTD (NULL, BAD_CAST file);
-        if (dtd == NULL) {
-                g_warning (_("'%s' DTD not parsed successfully. "
-                             "XML data layout validation will not be "
-                             "performed (some errors may occur)"), file);
-        }
-        g_free (file);
+	GBytes *bytes;
+	bytes = g_resources_lookup_data ("/gdaui/gdaui-layout.dtd", G_RESOURCE_LOOKUP_FLAGS_NONE, NULL);
+	if (bytes) {
+		xmlParserInputBufferPtr	ibuffer;
+		gsize size;
+		const char *data;
+		data = (const char *) g_bytes_get_data (bytes, &size);
+		ibuffer = xmlParserInputBufferCreateMem (data, size, XML_CHAR_ENCODING_NONE);
+		dtd = xmlIOParseDTD (NULL, ibuffer, XML_CHAR_ENCODING_NONE);
+		/* No need to call xmlFreeParserInputBuffer (ibuffer), because xmlIOParseDTD() does it */
+		g_bytes_unref (bytes);
+	}
+
+        if (! dtd)
+                g_warning ("DTD not parsed successfully, please report error to "
+                           "http://bugzilla.gnome.org/ for the \"libgda\" product");
 
         /* Get the root element node */
         xmlNodePtr root_node = NULL;
@@ -2405,6 +2354,11 @@ gdaui_basic_form_set_unknown_color (GdauiBasicForm *form, gdouble red, gdouble g
 				    gdouble blue, gdouble alpha)
 {
 	g_return_if_fail (GDAUI_IS_BASIC_FORM (form));
+	g_return_if_fail ((red >= 0.) && (red <= 1.));
+	g_return_if_fail ((green >= 0.) && (green <= 1.));
+	g_return_if_fail ((blue >= 0.) && (blue <= 1.));
+	g_return_if_fail ((alpha >= 0.) && (alpha <= 1.));
+
 	form->priv->red = red;
 	form->priv->green = green;
 	form->priv->blue = blue;
@@ -2419,3 +2373,34 @@ gdaui_basic_form_set_unknown_color (GdauiBasicForm *form, gdouble red, gdouble g
 						    form->priv->alpha);
 	}
 }
+
+/**
+ * gdaui_basic_form_update_data_set:
+ * @form: a #GdauiBasicForm
+ *
+ * Updates values in all #GdaHolder in current #GdaSet
+ */
+void
+gdaui_basic_form_update_data_set (GdauiBasicForm *form, GError **error) {
+  g_return_if_fail (form != NULL);
+  GSList *list;
+  GdaSet *set;
+  set = gdaui_basic_form_get_data_set (form);
+  g_return_if_fail (set != NULL);
+  list = gda_set_get_holders (set);
+  for (; list != NULL; list = list->next) {
+    GtkWidget *widget;
+    GdaHolder *holder;
+    GValue *value;
+    if (list->data == NULL)
+      continue;
+    holder = GDA_HOLDER (list->data);
+    widget = gdaui_basic_form_get_entry_widget (form, holder);
+    if (!GDAUI_IS_DATA_ENTRY (widget)) {
+      g_warning ("No data entry for holder %s", gda_holder_get_id (holder));
+      continue;
+    }
+    value = gdaui_data_entry_get_value (GDAUI_DATA_ENTRY (widget));
+    gda_holder_set_value (holder, value, error);
+  }
+}
