@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2012 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2012 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -54,6 +55,10 @@ static void gdaui_combo_get_property (GObject *object,
 static void gdaui_combo_dispose      (GObject *object);
 static void gdaui_combo_finalize     (GObject *object);
 
+static void gdaui_combo_get_preferred_width (GtkWidget *widget,
+					     gint *minimum_size,
+					     gint *natural_size);
+
 /* GdauiDataSelector interface */
 static void              gdaui_combo_selector_init (GdauiDataSelectorIface *iface);
 static GdaDataModel     *combo_selector_get_model (GdauiDataSelector *iface);
@@ -72,7 +77,6 @@ enum {
 
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass *parent_class = NULL;
-static GtkCssProvider *css_provider = NULL;
 
 /*
  * GdauiCombo class implementation
@@ -121,6 +125,8 @@ gdaui_combo_class_init (GdauiComboClass *klass)
 	object_class->dispose = gdaui_combo_dispose;
 	object_class->finalize = gdaui_combo_finalize;
 
+	GTK_WIDGET_CLASS (klass)->get_preferred_width = gdaui_combo_get_preferred_width;
+
 	/* add class properties */
 	g_object_class_install_property (object_class, PROP_MODEL,
 					 g_param_spec_object ("model", _("The data model to display"), NULL, 
@@ -160,18 +166,6 @@ gdaui_combo_init (GdauiCombo *combo, G_GNUC_UNUSED GdauiComboClass *klass)
 	gtk_combo_box_set_wrap_width (GTK_COMBO_BOX (combo), 0);
 	combo->priv->changed_id = g_signal_connect (combo, "changed",
 						    G_CALLBACK (selection_changed_cb), NULL);
-
-	if (!css_provider) {
-		css_provider = gtk_css_provider_new ();
-		gtk_css_provider_load_from_data (css_provider,
-						 "#gdaui-combo-as-list {\n"
-						 "-GtkComboBox-appears-as-list : 1;\n"
-						 "-GtkComboBox-arrow-size : 5}",
-						 -1, NULL);
-	}
-	gtk_style_context_add_provider (gtk_widget_get_style_context ((GtkWidget*) combo),
-					GTK_STYLE_PROVIDER (css_provider),
-					GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 }
 
 static void
@@ -213,15 +207,21 @@ gdaui_combo_set_property (GObject *object,
 
 	switch (param_id) {
 	case PROP_MODEL :
-		gdaui_combo_set_model (combo,
+		gdaui_combo_set_data (combo,
 				       GDA_DATA_MODEL (g_value_get_object (value)),
 				       0, NULL);
 		break;
 	case PROP_AS_LIST: {
-		if (g_value_get_boolean (value))
-			gtk_widget_set_name ((GtkWidget*) combo, "gdaui-combo-as-list");
-		else
-			gtk_widget_set_name ((GtkWidget*) combo, NULL);
+		GtkStyleContext *style;
+		style = gtk_widget_get_style_context (GTK_WIDGET (combo));
+		if (g_value_get_boolean (value)) {
+			gtk_style_context_remove_class (style, "gdaui-combo-normal");
+			gtk_style_context_add_class (style, "gdaui-combo-as-list");
+		}
+		else {
+			gtk_style_context_remove_class (style, "gdaui-combo-as-list");
+			gtk_style_context_add_class (style, "gdaui-combo-normal");
+		}
 		break;
 	}
 	default :
@@ -256,6 +256,19 @@ gdaui_combo_get_property (GObject *object,
 	}
 }
 
+static void
+gdaui_combo_get_preferred_width (GtkWidget *widget,
+				 gint *minimum_size,
+				 gint *natural_size)
+{
+#define MINSIZE 50
+	GTK_WIDGET_CLASS (parent_class)->get_preferred_width (widget, minimum_size, natural_size);
+	if (minimum_size && (*minimum_size > MINSIZE))
+		*minimum_size = MINSIZE;
+	if (natural_size && (*natural_size > MINSIZE))
+		*natural_size = MINSIZE;
+}
+
 static void
 gdaui_combo_dispose (GObject *object)
 {
@@ -321,7 +334,7 @@ gdaui_combo_new ()
  * @n_cols: number of columns in the model to be shown
  * @cols_index: an array of columns to be shown, its size must be @n_cols
  *
- * Create a new GdauiCombo widget with a model. See gdaui_combo_set_model() for
+ * Create a new GdauiCombo widget with a model. See gdaui_combo_set_data() for
  * more information about the @n_cols and @cols_index usage.
  *
  * Returns: (transfer full): the newly-created widget.
@@ -336,7 +349,7 @@ gdaui_combo_new_with_model (GdaDataModel *model, gint n_cols, gint *cols_index)
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 
 	combo = GDAUI_COMBO (gdaui_combo_new ()); 
-	gdaui_combo_set_model (GDAUI_COMBO (combo), GDA_DATA_MODEL (model), n_cols, cols_index);
+	gdaui_combo_set_data (GDAUI_COMBO (combo), GDA_DATA_MODEL (model), n_cols, cols_index);
 
 	return GTK_WIDGET (combo);
 }
@@ -345,9 +358,9 @@ static void cell_layout_data_func (GtkCellLayout *cell_layout, GtkCellRenderer *
 				   GtkTreeModel *tree_model, GtkTreeIter *iter, GdauiCombo *combo);
 
 /**
- * gdaui_combo_set_model:
+ * gdaui_combo_set_data:
  * @combo: a #GdauiCombo widget.
- * @model: a #GdaDataModel object.
+ * @model: a #GdaDataModel object to get data from.
  * @n_cols: number of columns in the model to be shown
  * @cols_index: (array length=n_cols): an array of columns to be shown, its size must be @n_cols
  *
@@ -358,10 +371,10 @@ static void cell_layout_data_func (GtkCellLayout *cell_layout, GtkCellRenderer *
  *
  * if @n_cols is %0, then all the columns of @model will be displayed in @combo.
  *
- * Since: 4.2
+ * Since: 5.2
  */
 void
-gdaui_combo_set_model (GdauiCombo *combo, GdaDataModel *model, gint n_cols, gint *cols_index)
+gdaui_combo_set_data (GdauiCombo *combo, GdaDataModel *model, gint n_cols, gint *cols_index)
 {
 	gint ln_cols, model_ncols = -1;
 	gint *lcols_index;
