@@ -2,6 +2,7 @@
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2009 - 2012 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2012 - 2013 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -84,12 +85,6 @@ contents_valid_accumulator (G_GNUC_UNUSED GSignalInvocationHint *ihint,
         return thisvalue; /* stop signal if 'thisvalue' is FALSE */
 }
 
-static gboolean
-m_class_contents_valid (G_GNUC_UNUSED GdauiDataEntry *de, G_GNUC_UNUSED GError **error)
-{
-	return TRUE;
-}
-
 static void
 gdaui_data_entry_iface_init (gpointer g_class)
 {
@@ -120,27 +115,6 @@ gdaui_data_entry_iface_init (gpointer g_class)
 				      NULL, NULL,
 				      _gdaui_marshal_VOID__VOID,
 				      G_TYPE_NONE, 0);
-		/**
-		 * GdauiDataEntry::contents-valid: (skip)
-		 * @model: the #GdauiDataEntry
-		 * @error: a #GError to set erros to or NULL
-		 *
-		 * Gets emitted when data entered to the #GdaDataEntry widget must be validated.
-		 *
-		 * For bindings use #gdaui_data_entry_validate function.
-		 *
-		 * Returns: TRUE when the data was successfull validated, FALSE otherwise.
-		 *
-		 * Deprecated: 5.2.0
-		 */
-		gdaui_data_entry_signals[CONTENTS_VALID] =
-			g_signal_new ("contents-valid",
-				      GDAUI_TYPE_DATA_ENTRY,
-				      G_SIGNAL_RUN_LAST,
-				      G_STRUCT_OFFSET (GdauiDataEntryIface, contents_valid),
-				      contents_valid_accumulator, NULL,
-				      _gdaui_marshal_BOOLEAN__POINTER,
-				      G_TYPE_BOOLEAN, 1, G_TYPE_POINTER);
 		gdaui_data_entry_signals[EXPAND_CHANGED] =
 			g_signal_new ("expand-changed",
 				      GDAUI_TYPE_DATA_ENTRY,
@@ -150,7 +124,6 @@ gdaui_data_entry_iface_init (gpointer g_class)
 				      _gdaui_marshal_VOID__VOID,
 				      G_TYPE_NONE, 0);
 
-		((GdauiDataEntryIface*) g_class)->contents_valid = m_class_contents_valid;
 		initialized = TRUE;
 	}
 }
@@ -243,33 +216,6 @@ gdaui_data_entry_get_value (GdauiDataEntry *de)
 	return NULL;
 }
 
-/**
- * gdaui_data_entry_content_is_valid: (skip)
- * @de: a #GtkWidget object which implements the #GdauiDataEntry interface
- * @error: (allow-none): a place to store an error, or %NULL
- *
- * Tests the validity of @de's contents. The validity is a determined from:
- * <itemizedlist>
- *   <listitem><para>the @de widget itself if it is capable of doing it (depending on the implementation)</para></listitem>
- *   <listitem><para>the results of the "contents-valid" signal which can be connected from </para></listitem>
- * </itemizedlist>
- *
- * Returns: TRUE if @de's contents is valid
- *
- * Since: 4.2
- *
- * Deprecated: 5.2
- */
-gboolean
-gdaui_data_entry_content_is_valid (GdauiDataEntry *de, GError **error)
-{
-	gboolean is_valid;
-	g_return_val_if_fail (GDAUI_IS_DATA_ENTRY (de), FALSE);
-
-	g_signal_emit (de, gdaui_data_entry_signals [CONTENTS_VALID], 0, error, &is_valid);
-	return is_valid;
-}
-
 /**
  * gdaui_data_entry_validate:
  * @de: a #GtkWidget object which implements the #GdauiDataEntry interface
@@ -444,34 +390,6 @@ gdaui_data_entry_get_handler (GdauiDataEntry *de)
 	return NULL;
 }
 
-/**
- * gdaui_data_entry_can_expand:
- * @de: a #GtkWidget object which implements the #GdauiDataEntry interface
- * @horiz: %TRUE to query horizontal expansion requirements, or %FALSE for vertical
- *
- * Used for the layout of #GdaDataEntry widgets in containers: queries if @de requires
- * horizontal or vertical expansion, depending on @horiz
- *
- * Returns: %TRUE if the widget requires expansion
- *
- * Since: 4.2
- *
- * Deprecated: 5.2: use the GtkWidget::hexpand, GtkWidget::wexpand or GtkWidget::expand properties
- */
-gboolean
-gdaui_data_entry_can_expand (GdauiDataEntry *de, gboolean horiz)
-{
-	g_return_val_if_fail (GDAUI_IS_DATA_ENTRY (de), FALSE);
-
-	if (GDAUI_DATA_ENTRY_GET_IFACE (de)->can_expand)
-		g_warning ("The GdauiDataEntry::can_expand virtual method is not used anymore, "
-			   "please clean your code");
-	if (horiz)
-		return gtk_widget_get_hexpand ((GtkWidget*) de);
-	else
-		return gtk_widget_get_vexpand ((GtkWidget*) de);
-}
-
 /**
  * gdaui_data_entry_set_editable:
  * @de: a #GtkWidget object which implements the #GdauiDataEntry interface
