@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 2009 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2013 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -199,7 +200,7 @@ gdaui_data_selector_unselect_row (GdauiDataSelector *iface, gint row)
 /**
  * gdaui_data_selector_set_column_visible:
  * @iface: an object which implements the #GdauiDataSelector interface
- * @column: a column number, starting at %0, or -1 tp apply to all the columns
+ * @column: a column number, starting at %0, or -1 to apply to all the columns
  * @visible: required visibility of the data in the @column column
  *
  * Shows or hides the data at column @column
@@ -222,7 +223,7 @@ gdaui_data_selector_set_column_visible (GdauiDataSelector *iface, gint column, g
 		iter = gdaui_data_selector_get_data_set (iface);
 		if (!iter)
 			return;
-		ncols = g_slist_length (GDA_SET (iter)->holders);
+		ncols = g_slist_length (gda_set_get_holders (GDA_SET (iter)));
 		for (i = 0; i < ncols; i++)
 			(GDAUI_DATA_SELECTOR_GET_IFACE (iface)->set_column_visible) (iface, i, visible);
 	}
