@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 2009 - 2010 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -45,8 +46,8 @@ static void gdaui_form_get_property (GObject *object,
 static void            gdaui_form_widget_init         (GdauiDataProxyIface *iface);
 static GdaDataProxy   *gdaui_form_get_proxy           (GdauiDataProxy *iface);
 static void            gdaui_form_set_column_editable (GdauiDataProxy *iface, gint column, gboolean editable);
-static void            gdaui_form_show_column_actions (GdauiDataProxy *iface, gint column, gboolean show_actions);
-static GtkActionGroup *gdaui_form_get_actions_group   (GdauiDataProxy *iface);
+static gboolean        gdaui_form_supports_action       (GdauiDataProxy *iface, GdauiAction action);
+static void            gdaui_form_perform_action        (GdauiDataProxy *iface, GdauiAction action);
 static gboolean        gdaui_form_widget_set_write_mode (GdauiDataProxy *iface, GdauiDataProxyWriteMode mode);
 static GdauiDataProxyWriteMode gdaui_form_widget_get_write_mode (GdauiDataProxy *iface);
 
@@ -122,8 +123,8 @@ gdaui_form_widget_init (GdauiDataProxyIface *iface)
 {
 	iface->get_proxy = gdaui_form_get_proxy;
 	iface->set_column_editable = gdaui_form_set_column_editable;
-	iface->show_column_actions = gdaui_form_show_column_actions;
-	iface->get_actions_group = gdaui_form_get_actions_group;
+	iface->supports_action = gdaui_form_supports_action;
+	iface->perform_action = gdaui_form_perform_action;
 	iface->set_write_mode = gdaui_form_widget_set_write_mode;
 	iface->get_write_mode = gdaui_form_widget_get_write_mode;
 }
@@ -196,8 +197,15 @@ gdaui_form_init (GdauiForm *form)
 
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (form), GTK_ORIENTATION_VERTICAL);
 
+	GtkWidget *frame;
+	frame = gtk_frame_new (NULL);
+	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
+	gtk_box_pack_start (GTK_BOX (form), frame, TRUE, TRUE, 0);
+	gtk_widget_show (frame);
+
 	form->priv->raw_form = gdaui_raw_form_new (NULL);
-	gtk_box_pack_start (GTK_BOX (form), form->priv->raw_form, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (frame), form->priv->raw_form);
+
 	gtk_widget_show (form->priv->raw_form);
 	g_signal_connect (form->priv->raw_form, "layout-changed",
 			  G_CALLBACK (form_layout_changed_cb), form);
@@ -207,6 +215,9 @@ gdaui_form_init (GdauiForm *form)
 	form->priv->info = gdaui_data_proxy_info_new (GDAUI_DATA_PROXY (form->priv->raw_form),
 						      GDAUI_DATA_PROXY_INFO_CURRENT_ROW |
 						      GDAUI_DATA_PROXY_INFO_ROW_MOVE_BUTTONS);
+	gtk_widget_set_halign (form->priv->info, GTK_ALIGN_START);
+	gtk_style_context_add_class (gtk_widget_get_style_context (form->priv->info), "inline-toolbar");
+
 	gtk_box_pack_start (GTK_BOX (form), form->priv->info, FALSE, FALSE, 0);
 	gtk_widget_show (form->priv->info);
 
@@ -339,17 +350,16 @@ gdaui_form_set_column_editable (GdauiDataProxy *iface, gint column, gboolean edi
 					      column, editable);
 }
 
-static void
-gdaui_form_show_column_actions (GdauiDataProxy *iface, gint column, gboolean show_actions)
+static gboolean
+gdaui_form_supports_action (GdauiDataProxy *iface, GdauiAction action)
 {
-	gdaui_data_proxy_column_show_actions ((GdauiDataProxy*) GDAUI_FORM (iface)->priv->raw_form,
-					      column, show_actions);
+	return gdaui_data_proxy_supports_action ((GdauiDataProxy*) GDAUI_FORM (iface)->priv->raw_form, action);
 }
 
-static GtkActionGroup *
-gdaui_form_get_actions_group (GdauiDataProxy *iface)
+static void
+gdaui_form_perform_action (GdauiDataProxy *iface, GdauiAction action)
 {
-	return gdaui_data_proxy_get_actions_group ((GdauiDataProxy*) GDAUI_FORM (iface)->priv->raw_form);
+	gdaui_data_proxy_perform_action ((GdauiDataProxy*) GDAUI_FORM (iface)->priv->raw_form, action);
 }
 
 static gboolean
