@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
@@ -54,9 +54,7 @@ GHashTable *gdaui_plugins_hash = NULL; /* key = plugin name, value = GdauiPlugin
  * <itemizedlist>
  * <listitem><para>Note 1: gtk_init() is not called by this function and should also
  *   be called (see Note 2 about the importance of the calling order)</para></listitem>
- * <listitem><para>Note 2: unless you call setlocale() first in your program, calling gtk_init()
- *    before gdaui_init() will ensure that all information loaded from config files will
- *    be loaded in the correct locale (as gtk_init() calls setlocale()).</para></listitem>
+ * <listitem><para>Note 2: you should always call gtk_init() before gdaui_init().</para></listitem>
  * <listitem><para>Note 3: this funtion also calls gda_init() so it should not be called
  *    again</para></listitem>
  * </itemizedlist>
@@ -73,16 +71,9 @@ gdaui_init (void)
 		return;
 	}
 
-	/*
-	gchar *str;
-	gdaui_gbr_init ();
-	str = gdaui_gbr_get_locale_dir_path ();
-	bindtextdomain (GETTEXT_PACKAGE, str);
-	g_free (str);
-	*/
-
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
+	_gdaui_register_resource ();
 	gda_init ();
 	if (! gdaui_plugins_hash)
 		gdaui_plugins_hash = init_plugins_hash ();
@@ -90,7 +81,6 @@ gdaui_init (void)
 	/* initialize CSS */
 	GBytes *css_data;
 	GError *error = NULL;
-	_gdaui_register_resource ();
 	css_data = g_resources_lookup_data ("/gdaui/gdaui.css", G_RESOURCE_LOOKUP_FLAGS_NONE, &error);
 	if (css_data) {
 		GtkCssProvider *css_provider;
@@ -114,7 +104,6 @@ gdaui_init (void)
 			   error && error->message ? error->message : _("No detail"));
 		g_clear_error (&error);
 	}
-	_gdaui_unregister_resource ();
 
 	initialized = TRUE;
 }
@@ -206,7 +195,7 @@ gdaui_new_data_entry (GType type, const gchar *plugin_name)
 			entry = (GdauiDataEntry *) gdaui_entry_none_new (type);
 		else if	(type == GDA_TYPE_TIME)
 			entry = (GdauiDataEntry *) gdaui_entry_time_new (dh);
-		else if (type == GDA_TYPE_TIMESTAMP)
+		else if (type == G_TYPE_DATE_TIME)
 			entry = (GdauiDataEntry *) gdaui_entry_timestamp_new (dh);
 		else if (type == G_TYPE_DATE)
 			entry = (GdauiDataEntry *) gdaui_entry_date_new (dh);
@@ -347,7 +336,6 @@ init_plugins_hash (void)
 {
 	GHashTable *hash;
 	GdauiPlugin *plugin;
-	gchar *file;
 
 	hash = g_hash_table_new (g_str_hash, g_str_equal); /* key strings are not handled in the hash table */
 
@@ -399,58 +387,76 @@ init_plugins_hash (void)
 	plugin->entry_create_func = entry_string_create_func;
 	plugin->cell_create_func = cell_textual_create_func;
 	g_hash_table_insert (hash, plugin->plugin_name, plugin);
-	file = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, "ui", "gdaui-entry-string.xml", NULL);
-	if (! g_file_test (file, G_FILE_TEST_EXISTS)) {
-		g_message ("Could not find file '%s': '%s' data entry will not report any possible option",
-			   file, plugin->plugin_name);
-        }
-	else {
-		gsize len;
-		g_file_get_contents (file, &(plugin->options_xml_spec), &len, NULL);
+	GBytes *bytes;
+	bytes = g_resources_lookup_data ("/gdaui/data-entries/gdaui-entry-string.xml", G_RESOURCE_LOOKUP_FLAGS_NONE, NULL);
+	if (bytes) {
+		const char *data;
+		data = (const char *) g_bytes_get_data (bytes, NULL);
+		plugin->options_xml_spec = g_strdup (data);
+		g_bytes_unref (bytes);
 	}
-	g_free (file);
+	else
+		g_message ("Could not load data entry specifications for the '%s' plugin, please report error to "
+                           "http://bugzilla.gnome.org/ for the \"libgda\" product", plugin->plugin_name);
+
+	plugin = g_new0 (GdauiPlugin, 1);
+	plugin->plugin_name = "integer";
+	plugin->plugin_descr = "Numeric integer entry";
+	plugin->plugin_file = NULL;
+	plugin->nb_g_types = 10;
+	plugin->valid_g_types = g_new (GType, plugin->nb_g_types);
+	plugin->valid_g_types [0] = G_TYPE_INT64;
+	plugin->valid_g_types [1] = G_TYPE_UINT64;
+	plugin->valid_g_types [3] = G_TYPE_INT;
+	plugin->valid_g_types [4] = GDA_TYPE_SHORT;
+	plugin->valid_g_types [5] = GDA_TYPE_USHORT;
+	plugin->valid_g_types [6] = G_TYPE_CHAR;
+	plugin->valid_g_types [7] = G_TYPE_UCHAR;
+	plugin->valid_g_types [8] = G_TYPE_ULONG;
+	plugin->valid_g_types [9] = G_TYPE_UINT;
+	plugin->options_xml_spec = NULL;
+	plugin->entry_create_func = entry_number_create_func;
+	plugin->cell_create_func = cell_textual_create_func;
+	g_hash_table_insert (hash, plugin->plugin_name, plugin);
+
+	bytes = g_resources_lookup_data ("/gdaui/data-entries/gdaui-entry-integer.xml", G_RESOURCE_LOOKUP_FLAGS_NONE, NULL);
+	if (bytes) {
+		const char *data;
+		data = (const char *) g_bytes_get_data (bytes, NULL);
+		plugin->options_xml_spec = g_strdup (data);
+		g_bytes_unref (bytes);
+	}
+	else
+		g_message ("Could not load data entry specifications for the '%s' plugin, please report error to "
+                           "http://bugzilla.gnome.org/ for the \"libgda\" product", plugin->plugin_name);
 
 	plugin = g_new0 (GdauiPlugin, 1);
 	plugin->plugin_name = "number";
 	plugin->plugin_descr = "Numeric entry";
 	plugin->plugin_file = NULL;
-	plugin->nb_g_types = 12;
+	plugin->nb_g_types = 3;
 	plugin->valid_g_types = g_new (GType, plugin->nb_g_types);
-	plugin->valid_g_types [0] = G_TYPE_INT64;
-	plugin->valid_g_types [1] = G_TYPE_UINT64;
+	plugin->valid_g_types [0] = GDA_TYPE_NUMERIC;
+	plugin->valid_g_types [1] = G_TYPE_FLOAT;
 	plugin->valid_g_types [2] = G_TYPE_DOUBLE;
-	plugin->valid_g_types [3] = G_TYPE_INT;
-	plugin->valid_g_types [4] = GDA_TYPE_NUMERIC;
-	plugin->valid_g_types [5] = G_TYPE_FLOAT;
-	plugin->valid_g_types [6] = GDA_TYPE_SHORT;
-	plugin->valid_g_types [7] = GDA_TYPE_USHORT;
-	plugin->valid_g_types [8] = G_TYPE_CHAR;
-	plugin->valid_g_types [9] = G_TYPE_UCHAR;
-	plugin->valid_g_types [10] = G_TYPE_ULONG;
-	plugin->valid_g_types [11] = G_TYPE_UINT;
 	plugin->options_xml_spec = NULL;
 	plugin->entry_create_func = entry_number_create_func;
 	plugin->cell_create_func = cell_textual_create_func;
 	g_hash_table_insert (hash, plugin->plugin_name, plugin);
-	file = gda_gbr_get_file_path (GDA_DATA_DIR, LIBGDA_ABI_NAME, "ui", "gdaui-entry-number.xml", NULL);
-	xmlChar *xml_spec = get_spec_with_isocodes (file);
-	if (xml_spec) {
-		plugin->options_xml_spec = g_strdup ((gchar*) xml_spec);
+
+	bytes = g_resources_lookup_data ("/gdaui/data-entries/gdaui-entry-number.xml", G_RESOURCE_LOOKUP_FLAGS_NONE, NULL);
+	if (bytes) {
+		const char *data;
+		data = (const char *) g_bytes_get_data (bytes, NULL);
+		xmlChar *xml_spec;
+		xml_spec = get_spec_with_isocodes (data);
+		plugin->options_xml_spec = g_strdup ((gchar *) xml_spec);
+		g_bytes_unref (bytes);
 		xmlFree (xml_spec);
 	}
-	g_free (file);
-
-	plugin = g_new0 (GdauiPlugin, 1);
-	plugin->plugin_name = "textual";
-	plugin->plugin_descr = "Textual entry";
-	plugin->plugin_file = NULL;
-	plugin->nb_g_types = 1;
-	plugin->valid_g_types = g_new (GType, plugin->nb_g_types);
-	plugin->valid_g_types [0] = G_TYPE_STRING;
-	plugin->options_xml_spec = NULL;
-	plugin->entry_create_func = entry_string_create_func;
-	plugin->cell_create_func = cell_textual_create_func;
-	g_hash_table_insert (hash, plugin->plugin_name, plugin);
+	else
+		g_message ("Could not load data entry specifications for the '%s' plugin, please report error to "
+                           "http://bugzilla.gnome.org/ for the \"libgda\" product", plugin->plugin_name);
 
 	plugin = g_new0 (GdauiPlugin, 1);
 	plugin->plugin_name = "time";
@@ -470,7 +476,7 @@ init_plugins_hash (void)
 	plugin->plugin_file = NULL;
 	plugin->nb_g_types = 1;
 	plugin->valid_g_types = g_new (GType, plugin->nb_g_types);
-	plugin->valid_g_types [0] = GDA_TYPE_TIMESTAMP;	
+	plugin->valid_g_types [0] = G_TYPE_DATE_TIME;
 	plugin->options_xml_spec = NULL;
 	plugin->entry_create_func = entry_timestamp_create_func;
 	plugin->cell_create_func = NULL;
@@ -677,7 +683,7 @@ find_child_node_from_name (xmlNodePtr parent, const gchar *name, const gchar *at
 }
 
 static xmlChar *
-get_spec_with_isocodes (const gchar *file)
+get_spec_with_isocodes (const gchar *data_spec)
 {
 	xmlDocPtr spec, isocodes = NULL;
 	xmlChar *retval = NULL;
@@ -704,9 +710,10 @@ get_spec_with_isocodes (const gchar *file)
 	/* 
 	 * Load spec string 
 	 */
-	spec = xmlParseFile (file);
+	spec = xmlParseDoc (BAD_CAST data_spec);
 	if (!spec) {
-		g_warning ("Can't load '%s' file", file);
+		g_warning ("Can't parse XML data, please report error to "
+                           "http://bugzilla.gnome.org/ for the \"libgda\" product");
 		goto cleanup;
 	}
 
@@ -809,3 +816,100 @@ gdaui_set_default_path (const gchar *path)
 	if (path)
 		gdaui_path = g_strdup (path);
 }
+
+static guint
+nocase_hash (gconstpointer key)
+{
+	gchar *cmp;
+	cmp = g_utf8_casefold ((const gchar*) key, -1);
+	guint retval;
+	retval = g_str_hash (cmp);
+	g_free (cmp);
+	return retval;
+}
+
+static gboolean
+nocase_equal (gconstpointer a, gconstpointer b)
+{
+	gchar *ca, *cb;
+	ca = g_utf8_casefold ((const gchar*) a, -1);
+	cb = g_utf8_casefold ((const gchar*) b, -1);
+
+	gboolean retval;
+	retval = g_str_equal (ca, cb);
+	g_free (ca);
+	g_free (cb);
+	return retval;
+}
+
+/**
+ * gdaui_get_icon_for_db_engine:
+ * @engine: a database engine (case non sensitive), like "PostgreSQL", "MySQL", etc.
+ *
+ * Get an icon for the database engine.
+ *
+ * Returns: (transfer none): a #GdkPixbuf, or %NULL if none found
+ *
+ * Since: 6.0
+ */
+GdkPixbuf *
+gdaui_get_icon_for_db_engine (const gchar *engine)
+{
+	g_return_val_if_fail (engine && *engine, NULL);
+	static GHashTable *icons_hash = NULL;
+	GdkPixbuf *pix;
+
+	if (! icons_hash) {
+		icons_hash = g_hash_table_new_full (nocase_hash, nocase_equal, g_free, g_object_unref);
+		/* add icons from resources */
+		gchar **dbs;
+#define RESOURCES_PREFIX "/gdaui/db-engines/data"
+		dbs = g_resources_enumerate_children (RESOURCES_PREFIX, G_RESOURCE_LOOKUP_FLAGS_NONE, NULL);
+		if (dbs) {
+			guint i;
+			for (i = 0; dbs[i]; i++) {
+				/* dbs[i] is for example "Firebird.png", we need to grab the key from that */
+				gchar *path;
+				path = g_strdup_printf ("%s/%s", RESOURCES_PREFIX, dbs[i]);
+				pix = gdk_pixbuf_new_from_resource (path, NULL);
+				g_free (path);
+				if (pix) {
+					gchar *tmp;
+					gint len;
+					len = strlen (dbs[i]) - 4;
+					g_assert (len > 0);
+					tmp = dbs[i] + len;
+					g_assert (! strcmp (tmp, ".png"));
+					if (tmp > dbs[i]) {
+						gchar *key, *tmp;
+						key = g_strdup (dbs[i]);
+						for (tmp = key + len;
+						     (tmp > key + 4) || ((len <= 4) && (tmp == key + len));
+						     tmp --) {
+							*tmp = 0;
+							if (g_hash_table_lookup (icons_hash, key)) {
+								g_hash_table_remove (icons_hash, key);
+								break;
+							}
+							/*g_print ("Inserted '%s'\n", key);*/
+							g_hash_table_insert (icons_hash, g_strdup (key), g_object_ref (pix));
+						}
+						g_free (key);
+					}
+					g_object_unref (pix);
+				}
+				else
+					g_warning ("Could not load GdkPixbuf for resource %s", dbs[i]);
+			}
+			g_strfreev (dbs);
+		}
+		else
+			g_warning ("Could not locate any DB engine resource");
+	}
+
+	gchar *ekey;
+	ekey = g_utf8_casefold (engine, -1);
+	pix = g_hash_table_lookup (icons_hash, ekey);
+	g_free (ekey);
+	return pix;
+}
