@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2010 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 - 2014 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,6 +26,7 @@
 #define MAX_BULLETS 2
 //gchar * bullet_strings[] = {"•", "◦"};
 #include <libgda-ui.h>
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 
 GdkPixbuf *bullet_pix[MAX_BULLETS] = {NULL};
 gchar     *lists_tokens[MAX_BULLETS] = {"- ", " - "};
@@ -69,7 +71,6 @@ typedef struct {
 struct _GdauiRtEditorPriv
 {
 	GtkTextView    *textview;
-	GtkAdjustment  *vadj;
 	gdouble         vadj_value;
 	GtkTextBuffer  *textbuffer;
 	GtkWidget      *toolbar;
@@ -89,7 +90,6 @@ struct _GdauiRtEditorPriv
 
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass *parent_class = NULL;
-static GtkCssProvider *css_provider = NULL;
 static gchar *help_str=N_("\"\"\"= Title level 1 =\n"
 			  "== Title level 2 ==\n"
 			  "\"\"\"= Title level 1 =\n"
@@ -281,12 +281,13 @@ gdaui_rt_editor_class_init (GdauiRtEditorClass *klass)
 static void
 text_view_realized_cb (GtkWidget *tv, GdauiRtEditor *rte)
 {
-	GdkWindow *win;
-	GtkStyle *style;
 	if (rte->priv->no_background && ! gtk_text_view_get_editable (GTK_TEXT_VIEW (tv))) {
+		GdkWindow *win;
+		GtkStyleContext* style_context = gtk_widget_get_style_context (tv);
+                GdkRGBA color;
 		win = gtk_text_view_get_window (GTK_TEXT_VIEW (tv), GTK_TEXT_WINDOW_TEXT);
-		style = gtk_widget_get_style (tv);
-		gdk_window_set_background (win, &(style->bg [GTK_STATE_NORMAL]));
+                gtk_style_context_get_background_color (style_context, GTK_STATE_FLAG_NORMAL, &color);
+		gdk_window_set_background_rgba (win, &color);
 	}
 }
 
@@ -301,14 +302,12 @@ static void
 gdaui_rt_editor_init (GdauiRtEditor *rte)
 {
 	GtkWidget *sw, *textview, *toolbar;
-	GtkAdjustment *adj;
 
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (rte), GTK_ORIENTATION_VERTICAL);
 
 	sw = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
 					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (sw));
 	gtk_box_pack_end (GTK_BOX (rte), sw, TRUE, TRUE, 0);
 
 	textview = gtk_text_view_new ();
@@ -321,7 +320,6 @@ gdaui_rt_editor_init (GdauiRtEditor *rte)
 
 	rte->priv = g_new0 (GdauiRtEditorPriv, 1);
 	rte->priv->sw = sw;
-	rte->priv->vadj = adj;
 	rte->priv->vadj_value = 0.;
 	rte->priv->saved_for_help = NULL;
 	rte->priv->enable_changed_signal = TRUE;
@@ -406,18 +404,6 @@ gdaui_rt_editor_init (GdauiRtEditor *rte)
 
 	/* toolbar */
 	toolbar = gtk_ui_manager_get_widget (rte->priv->uimanager, "/ToolBar");
-
-	if (!css_provider) {
-		css_provider = gtk_css_provider_new ();
-		gtk_css_provider_load_from_data (css_provider,
-						 "* {\n"
-						 "-GtkToolbar-shadow-type : none}",
-						 -1, NULL);
-	}
-	gtk_style_context_add_provider (gtk_widget_get_style_context ((GtkWidget*) toolbar),
-					GTK_STYLE_PROVIDER (css_provider),
-					GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-
 	gtk_toolbar_set_icon_size (GTK_TOOLBAR (toolbar), GTK_ICON_SIZE_MENU);
 	rte->priv->toolbar = toolbar;
 	gtk_box_pack_end (GTK_BOX (rte), toolbar, FALSE, FALSE, 0);
@@ -704,8 +690,8 @@ add_image_cb (G_GNUC_UNUSED GtkAction *action, GdauiRtEditor *rte)
         dlg = gtk_file_chooser_dialog_new (_("Select image to load"),
                                            GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) rte)),
                                            GTK_FILE_CHOOSER_ACTION_OPEN,
-                                           GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                           GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+                                           _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                           _("_Open"), GTK_RESPONSE_ACCEPT,
                                            NULL);
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg),
 					     gdaui_get_default_path ());
@@ -738,13 +724,15 @@ add_image_cb (G_GNUC_UNUSED GtkAction *action, GdauiRtEditor *rte)
 		}
 		else {
 			GtkWidget *msg;
-
-                        msg = gtk_message_dialog_new_with_markup (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) rte)),
-                                                                  GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
-                                                                  GTK_BUTTONS_CLOSE,
-                                                                  _("Could not load the contents of '%s':\n %s"),
-                                                                  filename,
-                                                                  error && error->message ? error->message : _("No detail"));
+			gchar *tmp;
+			tmp = g_strdup_printf (_("Could not load the contents of '%s'"), filename);
+			msg = gtk_message_dialog_new_with_markup (GTK_WINDOW (gtk_widget_get_toplevel (dlg)),
+								  GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
+								  GTK_BUTTONS_CLOSE,
+								  "<b>%s:</b>\n%s: %s",
+								  _("Error"), tmp,
+								  error && error->message ? error->message : _("No detail"));
+			g_free (tmp);
 			g_clear_error (&error);
                         gtk_widget_destroy (dlg);
                         dlg = NULL;
@@ -1275,7 +1263,7 @@ spaces_since_start_of_line (GtkTextIter *iter)
 }
 
 /**
- * get_markup_token
+ * get_markup_token:
  * @iter: starting position
  * @out_nb_spaces_before: a place to set the value returned by spaces_since_start_of_line() if called
  * @out_end: (allow-none): place to put the last used position, or %NULL
@@ -1483,7 +1471,7 @@ get_markup_token (GtkTextIter *iter, gint *out_nb_spaces_before, GtkTextIter *ou
 }
 
 /**
- * get_char_at_iter
+ * get_char_at_iter:
  * @iter: an iter
  * @move_forward_first: %TRUE if @iter should be moved forward first
  *
@@ -1552,7 +1540,7 @@ static guint8 *serialize_as_txt2tag (GtkTextBuffer     *register_buffer,
 				     GdauiRtEditor     *editor);
 
 /**
- * gdaui_rt_editor_get_contents
+ * gdaui_rt_editor_get_contents:
  * @editor: a #GdauiRtEditor
  *
  * Get the contents of @editor, using the markup syntax
@@ -1908,7 +1896,7 @@ serialize_as_txt2tag (G_GNUC_UNUSED  GtkTextBuffer *register_buffer,
 
 
 /**
- * gdaui_rt_editor_set_contents
+ * gdaui_rt_editor_set_contents:
  * @editor: a #GdauiRtEditor
  * @markup: the text to set in @editor, using the markup syntax (must be valid UTF-8)
  * @length: length of text in bytes.
@@ -1928,7 +1916,7 @@ gdaui_rt_editor_set_contents (GdauiRtEditor *editor, const gchar *markup, gint l
 }
 
 /**
- * gdaui_rt_editor_set_editable
+ * gdaui_rt_editor_set_editable:
  * @editor: a #GdauiRtEditor
  * @editable: whether it's editable
  *
@@ -1991,14 +1979,20 @@ show_hide_toolbar (GdauiRtEditor *editor)
 
 	if (doshow) {
 		gtk_widget_show (editor->priv->toolbar);
-		if (editor->priv->vadj_value != 0.) {
+		if (editor->priv->sw && (editor->priv->vadj_value != 0.)) {
+			GtkAdjustment *adj;	
+			adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (editor->priv->sw));
 			while (gtk_events_pending ())
 				gtk_main_iteration ();
-			gtk_adjustment_set_value (editor->priv->vadj, editor->priv->vadj_value);
+			gtk_adjustment_set_value (adj, editor->priv->vadj_value);
 		}
 	}
 	else {
-		editor->priv->vadj_value = gtk_adjustment_get_value (editor->priv->vadj);
+		if (editor->priv->sw) {
+			GtkAdjustment *adj;
+			adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (editor->priv->sw));
+			editor->priv->vadj_value = gtk_adjustment_get_value (adj);
+		}
 		gtk_widget_hide (editor->priv->toolbar);
 	}
 
