@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
@@ -67,10 +67,9 @@ struct _GdauiServerOperationPriv
 {
 	GdaServerOperation     *op;
 	GSList                 *widget_data; /* list of WidgetData structures */
-#ifdef HAVE_LIBGLADE
-	GladeXML               *glade;
-#endif
+	GtkBuilder             *builder;
 	gboolean                opt_header;
+	GtkWidget              *widget; /* Widget used to change operation parameters */
 };
 
 WidgetData *widget_data_new (WidgetData *parent, const gchar *path_name);
@@ -213,10 +212,9 @@ gdaui_server_operation_init (GdauiServerOperation * wid)
 	wid->priv = g_new0 (GdauiServerOperationPriv, 1);
 	wid->priv->op = NULL;
 	wid->priv->widget_data = NULL;
-#ifdef HAVE_LIBGLADE
-	wid->priv->glade = NULL;
-#endif
+	wid->priv->builder = NULL;
 	wid->priv->opt_header = FALSE;
+	wid->priv->widget = NULL;
 
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (wid), GTK_ORIENTATION_VERTICAL);
 }
@@ -273,10 +271,8 @@ gdaui_server_operation_dispose (GObject *object)
 			form->priv->widget_data = NULL;
 		}
 
-#ifdef HAVE_LIBGLADE
-		if (form->priv->glade)
-			g_object_unref (form->priv->glade);
-#endif
+		if (form->priv->builder)
+			g_object_unref (form->priv->builder);
 
 		/* the private area itself */
 		g_free (form->priv);
@@ -380,7 +376,7 @@ sequence_grid_attach_widget (GdauiServerOperation *form, GtkWidget *grid, GtkWid
 	gtk_widget_show (wid);
 
 	/* "-" button */
-	image = gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU);
+	image = gtk_image_new_from_icon_name ("list-remove-symbolic", GTK_ICON_SIZE_MENU);
 	wid = gtk_button_new ();
 	gtk_button_set_image (GTK_BUTTON (wid), image);
 	gtk_grid_attach (GTK_GRID (grid), wid, 1, index, 1, 1);
@@ -393,8 +389,6 @@ sequence_grid_attach_widget (GdauiServerOperation *form, GtkWidget *grid, GtkWid
 		gtk_widget_set_sensitive (wid, FALSE);
 }
 
-static GtkWidget *create_table_fields_array_create_widget (GdauiServerOperation *form, const gchar *path,
-							   gchar **section_str, GSList **label_widgets);
 static GtkWidget *
 fill_create_widget (GdauiServerOperation *form, const gchar *path, gchar **section_str, GSList **label_widgets)
 {
@@ -402,18 +396,16 @@ fill_create_widget (GdauiServerOperation *form, const gchar *path, gchar **secti
 	GtkWidget *plwid = NULL;
 
 	info_node = gda_server_operation_get_node_info (form->priv->op, path);
-	g_assert (info_node);
+	if (info_node == NULL) {
+		g_warning (_("Mal formed Server Operation. No Node at path: '%s'"), path);
+		return NULL;
+	}
 
 	if (label_widgets)
 		*label_widgets = NULL;
 	if (section_str)
 		*section_str = NULL;
 
-	/* very custom widget rendering goes here */
-	if ((gda_server_operation_get_op_type (form->priv->op) == GDA_SERVER_OPERATION_CREATE_TABLE) &&
-	    !strcmp (path, "/FIELDS_A"))
-		return create_table_fields_array_create_widget (form, path, section_str, label_widgets);
-
 	/* generic widget rendering */
 	switch (info_node->type) {
 	case GDA_SERVER_OPERATION_NODE_PARAMLIST: {
@@ -422,7 +414,6 @@ fill_create_widget (GdauiServerOperation *form, const gchar *path, gchar **secti
 		plist = info_node->plist;
 		plwid = gdaui_basic_form_new (plist);
 		gdaui_basic_form_set_unknown_color (GDAUI_BASIC_FORM (plwid), 0., 0., 0., 0.);
-		g_object_set ((GObject*) plwid, "show-actions", FALSE, NULL);
 
 		if (section_str) {
 			const gchar *name;
@@ -435,7 +426,7 @@ fill_create_widget (GdauiServerOperation *form, const gchar *path, gchar **secti
 		if (label_widgets) {
 			GSList *params;
 
-			params = plist->holders;
+			params = gda_set_get_holders (plist);
 			while (params) {
 				GtkWidget *label_entry;
 
@@ -447,6 +438,7 @@ fill_create_widget (GdauiServerOperation *form, const gchar *path, gchar **secti
 			}
 			*label_widgets = g_slist_reverse (*label_widgets);
 		}
+		form->priv->widget = plwid;
 		break;
 	}
 	case GDA_SERVER_OPERATION_NODE_DATA_MODEL: {
@@ -462,9 +454,10 @@ fill_create_widget (GdauiServerOperation *form, const gchar *path, gchar **secti
 
 		model = info_node->model;
 		grid = gdaui_raw_grid_new (model);
-		gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (plwid), grid);
-		gtk_viewport_set_shadow_type (GTK_VIEWPORT (gtk_bin_get_child (GTK_BIN (plwid))),
-					      GTK_SHADOW_NONE);
+		gtk_container_add (GTK_CONTAINER (plwid), grid);
+		if (GTK_IS_VIEWPORT (gtk_bin_get_child (GTK_BIN (plwid))))
+			gtk_viewport_set_shadow_type (GTK_VIEWPORT (gtk_bin_get_child (GTK_BIN (plwid))),
+						      GTK_SHADOW_NONE);
 		gdaui_data_proxy_set_write_mode (GDAUI_DATA_PROXY (grid),
 						 GDAUI_DATA_PROXY_WRITE_ON_ROW_CHANGE);
 		gtk_widget_show (grid);
@@ -474,6 +467,24 @@ fill_create_widget (GdauiServerOperation *form, const gchar *path, gchar **secti
 		winfo = gdaui_data_proxy_info_new (GDAUI_DATA_PROXY (grid),
 						   GDAUI_DATA_PROXY_INFO_ROW_MODIFY_BUTTONS);
 
+		GtkToolItem *item;
+		item = gdaui_data_proxy_info_get_item (GDAUI_DATA_PROXY_INFO (winfo), GDAUI_ACTION_NEW_DATA);
+		if (item)
+			gtk_widget_set_tooltip_text (GTK_WIDGET (item), _("Add a new field"));
+		item = gdaui_data_proxy_info_get_item (GDAUI_DATA_PROXY_INFO (winfo), GDAUI_ACTION_DELETE_SELECTED_DATA);
+		if (item)
+			gtk_widget_set_tooltip_text (GTK_WIDGET (item), _("Remove selected field"));
+		item = gdaui_data_proxy_info_get_item (GDAUI_DATA_PROXY_INFO (winfo), GDAUI_ACTION_WRITE_MODIFIED_DATA);
+		if (item) {
+			gtk_widget_hide (GTK_WIDGET (item));
+			gtk_widget_set_no_show_all (GTK_WIDGET (item), TRUE);
+		}
+		item = gdaui_data_proxy_info_get_item (GDAUI_DATA_PROXY_INFO (winfo), GDAUI_ACTION_RESET_DATA);
+		if (item) {
+			gtk_widget_hide (GTK_WIDGET (item));
+			gtk_widget_set_no_show_all (GTK_WIDGET (item), TRUE);
+		}
+
 		box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 		gtk_box_pack_start (GTK_BOX (box), plwid, TRUE, TRUE, 0);
 		gtk_widget_show (plwid);
@@ -502,7 +513,7 @@ fill_create_widget (GdauiServerOperation *form, const gchar *path, gchar **secti
 				label_entry = gtk_label_new (str);
 			}
 			g_free (str);
-			gtk_misc_set_alignment (GTK_MISC (label_entry), 0., 0.);
+			gtk_widget_set_halign (label_entry, GTK_ALIGN_START);
 
 			gtk_widget_show (label_entry);
 			str = (gchar *) g_object_get_data (G_OBJECT (model), "descr");
@@ -526,7 +537,7 @@ fill_create_widget (GdauiServerOperation *form, const gchar *path, gchar **secti
 		g_slist_free (list);
 		plwid = gdaui_basic_form_new (plist);
 		gdaui_basic_form_set_unknown_color (GDAUI_BASIC_FORM (plwid), 0., 0., 0., 0.);
-		g_object_set ((GObject*) plwid, "show-actions", FALSE, NULL);
+
 		/* we don't need plist anymore */
 		g_object_unref (plist);
 
@@ -539,6 +550,7 @@ fill_create_widget (GdauiServerOperation *form, const gchar *path, gchar **secti
 			label_entry = gdaui_basic_form_get_label_widget (GDAUI_BASIC_FORM (plwid), param);
 			*label_widgets = g_slist_prepend (*label_widgets, label_entry);
 		}
+		form->priv->widget = plwid;
 		break;
 	}
 	case GDA_SERVER_OPERATION_NODE_SEQUENCE: {
@@ -565,7 +577,7 @@ fill_create_widget (GdauiServerOperation *form, const gchar *path, gchar **secti
 		size = gda_server_operation_get_sequence_size (form->priv->op, path);
 		grid = gtk_grid_new ();
 		gtk_grid_set_row_spacing (GTK_GRID (grid), 10);
-		gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (plwid), grid);
+		gtk_container_add (GTK_CONTAINER (plwid), grid);
 		gtk_viewport_set_shadow_type (GTK_VIEWPORT (gtk_bin_get_child (GTK_BIN (plwid))),
 					      GTK_SHADOW_NONE);
 		gtk_widget_show (grid);
@@ -594,11 +606,11 @@ fill_create_widget (GdauiServerOperation *form, const gchar *path, gchar **secti
 		if (size < max) {
 			/* last row is for new entries */
 			wid = gtk_label_new (_("Add"));
-			gtk_misc_set_alignment (GTK_MISC (wid), .0, -1);
+			gtk_widget_set_halign (wid, GTK_ALIGN_START);
 			gtk_grid_attach (GTK_GRID (grid), wid, 0, size, 1, 1);
 			gtk_widget_show (wid);
 
-			image = gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_MENU);
+			image = gtk_image_new_from_icon_name ("list-add-symbolic", GTK_ICON_SIZE_MENU);
 			wid = gtk_button_new ();
 			gtk_button_set_image (GTK_BUTTON (wid), image);
 			gtk_grid_attach (GTK_GRID (grid), wid, 1, size, 1, 1);
@@ -688,59 +700,52 @@ gdaui_server_operation_fill (GdauiServerOperation *form)
 {
 	gint i;
 	gchar **topnodes;
-#ifdef HAVE_LIBGLADE
-	gchar *glade_file;
-#endif
 
 	/* parameters list management */
-	if (!form->priv->op)
+	if (form->priv->op == NULL)
 		/* nothing to do */
 		return;
 
-	/* load Glade file for specific GUI if it exists */
-#ifdef HAVE_LIBGLADE
-	glade_file = gdaui_gbr_get_data_dir_path ("server_operation.glade");
-	form->priv->glade = glade_xml_new (glade_file,
-					   gda_server_operation_op_type_to_string (gda_server_operation_get_op_type (form->priv->op)),
-					   NULL);
-	g_free (glade_file);
-	if (form->priv->glade) {
-		GtkWidget *mainw;
-		mainw = glade_xml_get_widget (form->priv->glade,
-					      gda_server_operation_op_type_to_string (gda_server_operation_get_op_type (form->priv->op)));
+	/* load specific GUI */
+	if (form->priv->builder == NULL) {
+		form->priv->builder = gtk_builder_new ();
+		if (! gtk_builder_add_from_resource (form->priv->builder, "/gdaui/glade/data/server_operation.xml", NULL)) {
+			g_message ("Could not load GdaServerOperation UI data, please report error to "
+				   "https://gitlab.gnome.org/GNOME/libgda/issues");
+			g_object_unref (form->priv->builder);
+			form->priv->builder = NULL;
+		}
+	}
+
+	GtkWidget *mainw = NULL;
+	if (form->priv->builder) {
+		mainw = (GtkWidget*) gtk_builder_get_object (form->priv->builder,
+							     gda_server_operation_op_type_to_string
+							     (gda_server_operation_get_op_type (form->priv->op)));
 		if (mainw) {
+			gtk_widget_unparent (mainw);
 			gtk_box_pack_start (GTK_BOX (form), mainw, TRUE, TRUE, 0);
 			gtk_widget_show (mainw);
 		}
-		else {
-			g_object_unref (form->priv->glade);
-			form->priv->glade = NULL;
-		}
 	}
-#endif
 
 	/* user visible widgets */
 	topnodes = gda_server_operation_get_root_nodes (form->priv->op);
-	i = 0;
-	while (topnodes[i]) {
+	for (i = 0; topnodes[i]; i++) {
 		GtkWidget *plwid;
 		gchar *section_str;
 		GtkWidget *container = NULL;
 
-#ifdef HAVE_LIBGLADE
-		if (form->priv->glade) {
-			container = glade_xml_get_widget (form->priv->glade, topnodes[i]);
-			if (!container) {
-				i++;
+		if (mainw) {
+			container = (GtkWidget*) gtk_builder_get_object (form->priv->builder, topnodes[i]);
+			if (!container)
 				continue;
-			}
 		}
-#endif
 		if (!container)
 			container = (GtkWidget *) form;
 
 		plwid = fill_create_widget (form, topnodes[i], &section_str, NULL);
-		if (plwid) {
+		if (plwid != NULL) {
 			GdaServerOperationNodeStatus status;
 			GtkWidget *label = NULL, *hbox = NULL;
 
@@ -748,7 +753,7 @@ gdaui_server_operation_fill (GdauiServerOperation *form)
 				GtkWidget *lab;
 				label = gtk_label_new ("");
 				gtk_widget_show (label);
-				gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
+				gtk_widget_set_halign (label, GTK_ALIGN_START);
 				gtk_label_set_markup (GTK_LABEL (label), section_str);
 				g_free (section_str);
 
@@ -774,7 +779,7 @@ gdaui_server_operation_fill (GdauiServerOperation *form)
 					gchar *str;
 					label = gtk_label_new ("");
 					gtk_widget_show (label);
-					gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
+					gtk_widget_set_halign (label, GTK_ALIGN_START);
 					str = g_strdup_printf ("<b>%s:</b>", _("Options"));
 					gtk_label_set_markup (GTK_LABEL (label), str);
 					g_free (str);
@@ -805,21 +810,18 @@ gdaui_server_operation_fill (GdauiServerOperation *form)
 				break;
 			}
 		}
-
-		i++;
 	}
 
 	/* destroying unused widgets in the Glade description */
-#ifdef HAVE_LIBGLADE
-	if (form->priv->glade) {
-		GList *widgets, *list;
+	if (mainw) {
+		GSList *widgets, *list;
 
-		widgets = glade_xml_get_widget_prefix (form->priv->glade, "/");
+		widgets = gtk_builder_get_objects (form->priv->builder);
 		for (list = widgets; list; list = list->next) {
 			const gchar *name;
 
-			name = glade_get_widget_name ((GtkWidget *) (list->data));
-			if (!gda_server_operation_get_node_info (form->priv->op, name)) {
+			name = gtk_buildable_get_name ((GtkBuildable *) (list->data));
+			if ((name[0] == '/') && !gda_server_operation_get_node_info (form->priv->op, name)) {
 				GtkWidget *parent;
 
 				/* dirty hack to remove a notebook page */
@@ -836,15 +838,13 @@ gdaui_server_operation_fill (GdauiServerOperation *form)
 					gtk_notebook_remove_page (GTK_NOTEBOOK (parent), pageno);
 				}
 				else
-					gtk_widget_destroy ((GtkWidget *) (list->data));
+					gtk_widget_hide ((GtkWidget *) (list->data));
 			}
 		}
-		g_list_free (widgets);
+		g_slist_free (widgets);
 	}
-#endif
 
 	g_strfreev (topnodes);
-
 }
 
 /*
@@ -955,6 +955,8 @@ sequence_item_added_cb (GdaServerOperation *op, const gchar *seq_path, gint item
 	/* add widget corresponding to the new sequence item */
 	str = g_strdup_printf ("%s/%d", seq_path, item_index);
 	wid = fill_create_widget (form, str, NULL, NULL);
+	if (wid == NULL)
+		return;
 	sequence_grid_attach_widget (form, grid, wid, seq_path, item_index);
 	g_free (str);
 }
@@ -1094,10 +1096,8 @@ gdaui_server_operation_new_in_dialog (GdaServerOperation *op, GtkWindow *parent,
 
 	dlg = gtk_dialog_new_with_buttons (rtitle, parent,
 					   GTK_DIALOG_MODAL,
-					   GTK_STOCK_OK,
-					   GTK_RESPONSE_ACCEPT,
-					   GTK_STOCK_CANCEL,
-					   GTK_RESPONSE_REJECT,
+					   _("_Ok"), GTK_RESPONSE_ACCEPT,
+					   _("_Cancel"), GTK_RESPONSE_REJECT,
 					   NULL);
 	dcontents = gtk_dialog_get_content_area (GTK_DIALOG (dlg));
 
@@ -1105,7 +1105,7 @@ gdaui_server_operation_new_in_dialog (GdaServerOperation *op, GtkWindow *parent,
 		GtkWidget *label;
 
 		label = gtk_label_new (NULL);
-		gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
+		gtk_widget_set_halign (label, GTK_ALIGN_START);
 		gtk_label_set_markup (GTK_LABEL (label), header);
 		gtk_box_pack_start (GTK_BOX (dcontents), label, FALSE, FALSE, 5);
 
@@ -1120,112 +1120,129 @@ gdaui_server_operation_new_in_dialog (GdaServerOperation *op, GtkWindow *parent,
 }
 
 
+/**
+ */
+void
+gdaui_server_operation_update_parameters (GdauiServerOperation *op, GError** error) {
+	GdauiBasicForm *form;
+	g_return_if_fail (op != NULL);
+	g_return_if_fail (op->priv->widget != NULL);
+	g_return_if_fail (GDAUI_IS_BASIC_FORM (op->priv->widget));
+	form = GDAUI_BASIC_FORM (op->priv->widget);
+	gdaui_basic_form_update_data_set (form, error);
+}
+
 /*
  * CREATE_TABLE "/FIELDS_A" Custom widgets rendering
  */
 static void create_table_grid_fields_iter_row_changed_cb (GdaDataModelIter *grid_iter, gint row,
 							  GdaDataModelIter *form_iter);
-static void create_table_proxy_row_inserted_cb (GdaDataProxy *proxy, gint row, GdauiServerOperation *form);
-static GtkWidget *
-create_table_fields_array_create_widget (GdauiServerOperation *form, const gchar *path,
-					 G_GNUC_UNUSED gchar **section_str,
-					 G_GNUC_UNUSED GSList **label_widgets)
-{
-	GdaServerOperationNode *info_node;
-	GtkWidget *hlayout, *sw, *box, *label;
-	GtkWidget *grid_fields, *form_props, *winfo;
-	GdaDataProxy *proxy;
-	gint name_col, col, nbcols;
-	GdaDataModelIter *grid_iter, *form_iter;
+// FIXME: Removed to avoid warning
+//static void create_table_proxy_row_inserted_cb (GdaDataProxy *proxy, gint row, GdauiServerOperation *form);
+// FIXME: Removed to avoid warning
+/* static GtkWidget * */
+/* create_table_fields_array_create_widget (GdauiServerOperation *form, const gchar *path, */
+/* 					 G_GNUC_UNUSED gchar **section_str, */
+/* 					 G_GNUC_UNUSED GSList **label_widgets) */
+/* { */
+/* 	GdaServerOperationNode *info_node; */
+/* 	GtkWidget *hlayout, *sw, *box, *label; */
+/* 	GtkWidget *grid_fields, *form_props, *winfo; */
+/* 	GdaDataProxy *proxy; */
+/* 	gint name_col, col, nbcols; */
+/* 	GdaDataModelIter *grid_iter, *form_iter; */
 
-	info_node = gda_server_operation_get_node_info (form->priv->op, path);
-	g_assert (info_node->type == GDA_SERVER_OPERATION_NODE_DATA_MODEL);
+/* 	info_node = gda_server_operation_get_node_info (form->priv->op, path); */
+/* 	g_assert (info_node->type == GDA_SERVER_OPERATION_NODE_DATA_MODEL); */
 
-	hlayout = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
+/* 	hlayout = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL); */
 
 	/* form for field properties */
-	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-	gtk_paned_pack2 (GTK_PANED (hlayout), box, TRUE, TRUE);
+/* 	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0); */
+/* 	gtk_paned_pack2 (GTK_PANED (hlayout), box, TRUE, TRUE); */
 
-	label = gtk_label_new (_("<b>Field properties:</b>"));
-	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
-	gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
+/* 	label = gtk_label_new (_("<b>Field properties:</b>")); */
+/* 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE); */
+/* 	gtk_widget_set_halign (label, GTK_ALIGN_START); */
+/* 	gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0); */
 
-	form_props = gdaui_raw_form_new (GDA_DATA_MODEL (info_node->model));
-	proxy = gdaui_data_proxy_get_proxy (GDAUI_DATA_PROXY (form_props));
-	gdaui_data_proxy_set_write_mode (GDAUI_DATA_PROXY (form_props),
-					 GDAUI_DATA_PROXY_WRITE_ON_VALUE_CHANGE);
-	gtk_box_pack_start (GTK_BOX (box), form_props, TRUE, TRUE, 0);
-	g_signal_connect (proxy, "row-inserted",
-			  G_CALLBACK (create_table_proxy_row_inserted_cb), form);
+/* 	form_props = gdaui_raw_form_new (GDA_DATA_MODEL (info_node->model)); */
+/* 	proxy = gdaui_data_proxy_get_proxy (GDAUI_DATA_PROXY (form_props)); */
+/* 	gdaui_data_proxy_set_write_mode (GDAUI_DATA_PROXY (form_props), */
+/* 					 GDAUI_DATA_PROXY_WRITE_ON_VALUE_CHANGE); */
+/* 	gtk_box_pack_start (GTK_BOX (box), form_props, TRUE, TRUE, 0); */
+/* 	g_signal_connect (proxy, "row-inserted", */
+/* 			  G_CALLBACK (create_table_proxy_row_inserted_cb), form); */
 
-	gtk_widget_show_all (box);
+/* 	gtk_widget_show_all (box); */
 
 	/* grid for field names */
-	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-	gtk_paned_pack1 (GTK_PANED (hlayout), box, TRUE, TRUE);
+/* 	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0); */
+/* 	gtk_paned_pack1 (GTK_PANED (hlayout), box, TRUE, TRUE); */
 
-	label = gtk_label_new (_("<b>Fields:</b>"));
-	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
-	gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
+/* 	label = gtk_label_new (_("<b>Fields:</b>")); */
+/* 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE); */
+/* 	gtk_widget_set_halign (label, GTK_ALIGN_START); */
+/* 	gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0); */
 
-	sw = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
-					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_NONE);
+/* 	sw = gtk_scrolled_window_new (NULL, NULL); */
+/* 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), */
+/* 					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC); */
+/* 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_NONE); */
 
-	grid_fields = gdaui_raw_grid_new (GDA_DATA_MODEL (proxy));
-	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (grid_fields), FALSE);
-	g_object_set (G_OBJECT (grid_fields), "info-cell-visible", FALSE, NULL);
+/* 	grid_fields = gdaui_raw_grid_new (GDA_DATA_MODEL (proxy)); */
+/* 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (grid_fields), FALSE); */
+/* 	g_object_set (G_OBJECT (grid_fields), "info-cell-visible", FALSE, NULL); */
 
-	name_col = 0;
-	nbcols = gda_data_proxy_get_proxied_model_n_cols (proxy);
-	g_assert (name_col < nbcols);
-	for (col = 0; col < name_col; col++)
-		gdaui_data_selector_set_column_visible (GDAUI_DATA_SELECTOR (grid_fields), col, FALSE);
-	for (col = name_col + 1; col < nbcols; col++)
-		gdaui_data_selector_set_column_visible (GDAUI_DATA_SELECTOR (grid_fields), col, FALSE);
+/* 	name_col = 0; */
+/* 	nbcols = gda_data_proxy_get_proxied_model_n_cols (proxy); */
+/* 	g_assert (name_col < nbcols); */
+/* 	for (col = 0; col < name_col; col++) */
+/* 		gdaui_data_selector_set_column_visible (GDAUI_DATA_SELECTOR (grid_fields), col, FALSE); */
+/* 	for (col = name_col + 1; col < nbcols; col++) */
+/* 		gdaui_data_selector_set_column_visible (GDAUI_DATA_SELECTOR (grid_fields), col, FALSE); */
 
-	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sw), grid_fields);
-	gtk_viewport_set_shadow_type (GTK_VIEWPORT (gtk_bin_get_child (GTK_BIN (sw))),
-				      GTK_SHADOW_NONE);
-	gtk_box_pack_start (GTK_BOX (box), sw, TRUE, TRUE, 0);
+/* 	gtk_container_add (GTK_CONTAINER (sw), grid_fields); */
+/* 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_NONE); */
+/* 	gtk_box_pack_start (GTK_BOX (box), sw, TRUE, TRUE, 0); */
 
 	/* buttons to add/remove fields */
-	winfo = gdaui_data_proxy_info_new (GDAUI_DATA_PROXY (form_props),
-					   GDAUI_DATA_PROXY_INFO_ROW_MODIFY_BUTTONS);
-	gtk_box_pack_start (GTK_BOX (box), winfo, FALSE, FALSE, 0);
+/* 	winfo = gdaui_data_proxy_info_new (GDAUI_DATA_PROXY (form_props), */
+/* 					   GDAUI_DATA_PROXY_INFO_ROW_MODIFY_BUTTONS); */
+/* 	gtk_box_pack_start (GTK_BOX (box), winfo, FALSE, FALSE, 0); */
 
-	gtk_widget_show_all (box);
+/* 	gtk_widget_show_all (box); */
 
 	/* keep the selections in sync */
-	grid_iter = gdaui_data_selector_get_data_set (GDAUI_DATA_SELECTOR (grid_fields));
-	form_iter = gdaui_data_selector_get_data_set (GDAUI_DATA_SELECTOR (form_props));
-	g_signal_connect (grid_iter, "row-changed",
-			  G_CALLBACK (create_table_grid_fields_iter_row_changed_cb), form_iter);
-	g_signal_connect (form_iter, "row-changed",
-			  G_CALLBACK (create_table_grid_fields_iter_row_changed_cb), grid_iter);
+/* 	grid_iter = gdaui_data_selector_get_data_set (GDAUI_DATA_SELECTOR (grid_fields)); */
+/* 	form_iter = gdaui_data_selector_get_data_set (GDAUI_DATA_SELECTOR (form_props)); */
+/* 	g_signal_connect (grid_iter, "row-changed", */
+/* 			  G_CALLBACK (create_table_grid_fields_iter_row_changed_cb), form_iter); */
+/* 	g_signal_connect (form_iter, "row-changed", */
+/* 			  G_CALLBACK (create_table_grid_fields_iter_row_changed_cb), grid_iter); */
 
-	gtk_widget_set_vexpand (hlayout, TRUE);
+/* 	gtk_widget_set_vexpand (hlayout, TRUE); */
 
-	{
-		GtkActionGroup *group;
-		GtkAction *action;
-		group = gdaui_data_proxy_get_actions_group (GDAUI_DATA_PROXY (form_props));
-		action = gtk_action_group_get_action (group, "ActionNew");
-		g_object_set (G_OBJECT (action), "tooltip", _("Add a new field"), NULL);
-		action = gtk_action_group_get_action (group, "ActionDelete");
-		g_object_set (G_OBJECT (action), "tooltip", _("Remove selected field"), NULL);
-		action = gtk_action_group_get_action (group, "ActionCommit");
-		gtk_action_set_visible (action, FALSE);
-		action = gtk_action_group_get_action (group, "ActionReset");
-		gtk_action_set_visible (action, FALSE);
-	}
+/* 	GtkToolItem *item; */
+/* 	item = gdaui_data_proxy_info_get_item (GDAUI_DATA_PROXY_INFO (winfo), GDAUI_ACTION_NEW_DATA); */
+/* 	if (item) */
+/* 		gtk_widget_set_tooltip_text (GTK_WIDGET (item), _("Add a new field")); */
+/* 	item = gdaui_data_proxy_info_get_item (GDAUI_DATA_PROXY_INFO (winfo), GDAUI_ACTION_DELETE_SELECTED_DATA); */
+/* 	if (item) */
+/* 		gtk_widget_set_tooltip_text (GTK_WIDGET (item), _("Remove selected field")); */
+/* 	item = gdaui_data_proxy_info_get_item (GDAUI_DATA_PROXY_INFO (winfo), GDAUI_ACTION_WRITE_MODIFIED_DATA); */
+/* 	if (item) { */
+/* 		gtk_widget_hide (GTK_WIDGET (item)); */
+/* 		g_object_set ((GObject*) item, "no-show-all", TRUE, NULL); */
+/* 	} */
+/* 	item = gdaui_data_proxy_info_get_item (GDAUI_DATA_PROXY_INFO (winfo), GDAUI_ACTION_RESET_DATA); */
+/* 	if (item) { */
+/* 		gtk_widget_hide (GTK_WIDGET (item)); */
+/* 		g_object_set ((GObject*) item, "no-show-all", TRUE, NULL); */
+/* 	} */
 
-	return hlayout;
-}
+/* 	return hlayout; */
+/* } */
 
 static void
 create_table_grid_fields_iter_row_changed_cb (GdaDataModelIter *iter1, gint row, GdaDataModelIter *iter2)
@@ -1237,27 +1254,28 @@ create_table_grid_fields_iter_row_changed_cb (GdaDataModelIter *iter1, gint row,
 					   G_CALLBACK (create_table_grid_fields_iter_row_changed_cb), iter1);
 }
 
-static void
-create_table_proxy_row_inserted_cb (GdaDataProxy *proxy, gint row, GdauiServerOperation *form)
-{
-	GdaDataModelIter *iter;
-	GdaHolder *holder;
-	GdaServerProvider *prov;
-	GdaConnection *cnc;
-	const gchar *type = NULL;
+/* static void */
+/* create_table_proxy_row_inserted_cb (GdaDataProxy *proxy, gint row, GdauiServerOperation *form) */
+/* { */
+/* 	GdaDataModelIter *iter; */
+/* 	GdaHolder *holder; */
+/* 	GdaServerProvider *prov; */
+/* 	GdaConnection *cnc; */
+/* 	const gchar *type = NULL; */
 
-	iter = gda_data_model_create_iter (GDA_DATA_MODEL (proxy));
-	gda_data_model_iter_move_to_row (iter, row);
-	holder = gda_set_get_nth_holder (GDA_SET (iter), 0);
-	gda_holder_set_value_str (holder, NULL, "fieldname", NULL);
+/* 	iter = gda_data_model_create_iter (GDA_DATA_MODEL (proxy)); */
+/* 	gda_data_model_iter_move_to_row (iter, row); */
+/* 	holder = gda_set_get_nth_holder (GDA_SET (iter), 0); */
+/* 	gda_holder_set_value_str (holder, NULL, "fieldname", NULL); */
+
+/* 	g_object_get (form->priv->op, "connection", &cnc, "provider", &prov, NULL); */
+/* 	if (prov) */
+/* 		type = gda_server_provider_get_default_dbms_type (prov, cnc, G_TYPE_STRING); */
+/* 	holder = gda_set_get_nth_holder (GDA_SET (iter), 1); */
+/* 	gda_holder_set_value_str (holder, NULL, type ? type : "varchar", NULL); */
+/* 	if (cnc) */
+/* 		g_object_unref (cnc); */
+/* 	if (prov) */
+/* 		g_object_unref (prov); */
+/* } */
 
-	g_object_get (form->priv->op, "connection", &cnc, "provider", &prov, NULL);
-	if (prov)
-		type = gda_server_provider_get_default_dbms_type (prov, cnc, G_TYPE_STRING);
-	holder = gda_set_get_nth_holder (GDA_SET (iter), 1);
-	gda_holder_set_value_str (holder, NULL, type ? type : "varchar", NULL);
-	if (cnc)
-		g_object_unref (cnc);
-	if (prov)
-		g_object_unref (prov);
-}
