@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -23,9 +23,6 @@
 
 #include <gtk/gtk.h>
 #include <libgda/libgda.h>
-#ifdef HAVE_LIBGLADE
-#include <glade/glade.h>
-#endif
 
 G_BEGIN_DECLS
 
@@ -66,10 +63,11 @@ struct _GdauiServerOperationClass
  * SQLite database.
  */
 
-GType             gdaui_server_operation_get_type      (void) G_GNUC_CONST;
-GtkWidget        *gdaui_server_operation_new           (GdaServerOperation *op);
-GtkWidget        *gdaui_server_operation_new_in_dialog (GdaServerOperation *op, GtkWindow *parent,
+GType             gdaui_server_operation_get_type         (void) G_GNUC_CONST;
+GtkWidget        *gdaui_server_operation_new              (GdaServerOperation *op);
+GtkWidget        *gdaui_server_operation_new_in_dialog    (GdaServerOperation *op, GtkWindow *parent,
 							   const gchar *title, const gchar *header);
+void              gdaui_server_operation_update_parameters (GdauiServerOperation *op, GError **error);
 
 G_END_DECLS
 
