@@ -433,7 +433,7 @@ gint*
 gdaui_set_source_get_ref_columns  (GdauiSetSource *s)
 {
 	g_return_val_if_fail (s, NULL);
-	return s->ref_cols_index;
+	return s->shown_cols_index;
 }
 
 /**
@@ -505,11 +505,6 @@ enum {
 
 static gint gdaui_set_signals[LAST_SIGNAL] = { 0, 0 };
 
-GType
-_gdaui_set_get_type (void) {
-	return gdaui_set_get_type ();
-}
-
 GType
 gdaui_set_get_type (void)
 {
@@ -561,6 +556,7 @@ gdaui_set_class_init (GdauiSetClass *class)
 	/**
          * GdauiSet::source-model-changed:
          * @set: the #GdauiSet
+         * @source: (type GdauiSetSource): the #GdauiSetSource
          * 
          * Gets emitted when the data model used in @set's #GdauiSetSource has changed
 	 *
@@ -594,22 +590,6 @@ gdaui_set_init (GdauiSet *set)
 	set->priv->set = NULL;
 }
 
-/*
- * _gdaui_set_new
- * @set: a #GdaSet
- *
- * Creates a new #GdauiSet which wraps @set's properties
- *
- * Returns: the new widget
- * 
- * Deprecated: Since 5.2
- */
-GdauiSet *
-_gdaui_set_new (GdaSet *set)
-{
-	return gdaui_set_new (set);
-}
-
 /**
  * gdaui_set_new:
  * @set: a #GdaSet
@@ -736,7 +716,7 @@ compute_public_data (GdauiSet *set)
 	
 	/* scan GdaSetSource list */
 	hash = g_hash_table_new (NULL, NULL);
-	for (list = aset->sources_list; list; list = list->next) {
+	for (list = gda_set_get_sources (aset); list; list = list->next) {
 		GdauiSetSource *dsource;
 		dsource = gdaui_set_source_new (GDA_SET_SOURCE (list->data));
 		set->sources_list = g_slist_prepend (set->sources_list, dsource);
@@ -748,9 +728,12 @@ compute_public_data (GdauiSet *set)
 	set->sources_list = g_slist_reverse (set->sources_list);
 
 	/* scan GdaSetGroup list */
-	for (list = aset->groups_list; list; list = list->next) {
+	for (list = gda_set_get_groups (aset); list; list = list->next) {
 		GdauiSetGroup *dgroup;
-		g_assert (list->data);
+		if (list->data == NULL) {
+      g_warning (_("No group exists for Set"));
+      continue;
+    }
 		dgroup = gdaui_set_group_new (GDA_SET_GROUP (list->data));
 		gdaui_set_group_set_source (dgroup, 
 		                            g_hash_table_lookup (hash, 
@@ -780,7 +763,7 @@ update_public_data (GdauiSet *set)
 	/* scan GdaSetSource list */
 	elist = set->sources_list;
 	set->sources_list = NULL;
-	for (list = aset->sources_list; list; list = list->next) {
+	for (list = gda_set_get_sources (aset); list; list = list->next) {
 		GdauiSetSource *dsource;
 		dsource = g_hash_table_lookup (shash, list->data);
 		if (dsource) {
@@ -816,7 +799,7 @@ update_public_data (GdauiSet *set)
 	/* scan GdaSetGroup list */
 	elist = set->groups_list;
 	set->groups_list = NULL;
-	for (list = aset->groups_list; list; list = list->next) {
+	for (list = gda_set_get_groups (aset); list; list = list->next) {
 		GdauiSetGroup *dgroup;
 		dgroup = g_hash_table_lookup (ghash, list->data);
 		if (dgroup) {
@@ -957,20 +940,6 @@ gdaui_set_get_property (GObject *object,
 	}	
 }
 
-/**
- * _gdaui_set_get_group:
- * @dbset:
- * @holder:
- * 
- * Returns: A new #GdauiSetGroup struct
- * Deprecated: Since 5.2
- **/
-GdauiSetGroup  *
-_gdaui_set_get_group (GdauiSet *dbset, GdaHolder *holder) 
-{
-	return gdaui_set_get_group (dbset, holder);
-}
-
 /**
  * gdaui_set_get_group:
  * @dbset:
