@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 2009 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2012 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -390,10 +391,10 @@ gdaui_tree_store_get_property (GObject *object,
 }
 
 /**
- * gdaui_tree_store_new: (skip)
+ * gdaui_tree_store_new:
  * @tree: a #GdaTree object
  * @n_columns: number of columns in the tree store
- * @...: couples of (GType, attribute name) for each column, from first to last 
+ * @...: couples of (GType, attribute name) for each column, from first to last
  *
  * Creates a #GtkTreeModel interface with a #GdaTree, mapping columns to attributes' values.
  *
@@ -404,7 +405,7 @@ gdaui_tree_store_get_property (GObject *object,
  * Note that the GType has to correspond to the type of value associated with the attribute name (no type
  * conversion is done), and a warning will be displayed in case of type mismatch.
  *
- * Returns: the new object, or %NULL if an attribute's name was NULL or an empty string
+ * Returns: (transfer full): the new object, or %NULL if an attribute's name was NULL or an empty string
  *
  * Since: 4.2
  */
@@ -519,27 +520,6 @@ gdaui_tree_store_get_node (GdauiTreeStore *store, GtkTreeIter *iter)
 	return node;
 }
 
-/**
- * gdaui_tree_store_get_iter: (skip)
- * @store: a #GdauiTreeStore object
- * @iter: a pointer to a #GtkTreeIter
- * @node: a #GdaTreeNode in @store
- *
- * Sets @iter to represent @node in the tree.
- *
- * Returns: %TRUE if no error occurred and @iter is valid
- *
- * Since: 4.2.8
- *
- * Deprecated: 5.2
- */
-gboolean
-gdaui_tree_store_get_iter (GdauiTreeStore *store, GtkTreeIter *iter, GdaTreeNode *node)
-{
-	return gdaui_tree_store_get_iter_from_node (store, iter, node);
-}
-
-
 /**
  * gdaui_tree_store_get_iter_from_node:
  * @store: a #GdauiTreeStore object
