@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2007 - 2013 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2008 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  *
@@ -27,7 +27,7 @@
 #include "libgda-xslt.h"
 #include "sql_backend.h"
 #include <libxml/xpathInternals.h>
-
+#include <libgda/gda-debug-macros.h>
 
 #include <libgda/handlers/gda-handler-boolean.h>
 #include <libgda/handlers/gda-handler-numerical.h>
@@ -545,7 +545,7 @@ gda_xslt_bk_internal_query (GdaXsltExCont * exec, GdaXsltIntCont * pdata,
 		return -1;
 
 	if (params != NULL) {
-		plist = params->holders;
+		plist = gda_set_get_holders (params);
 		while (plist && ret == 0) {
 			ret = _gda_xslt_holder_set_value (GDA_HOLDER (plist->data), ctxt);
 			plist = g_slist_next (plist);
@@ -638,7 +638,7 @@ value_to_xmlchar (const GValue * value)
 				     (gpointer) GDA_TYPE_TIME,
 				     gda_handler_time_new ());
 		g_hash_table_insert (data_handlers,
-				     (gpointer) GDA_TYPE_TIMESTAMP,
+				     (gpointer) G_TYPE_DATE_TIME,
 				     gda_handler_time_new ());
 		g_hash_table_insert (data_handlers, (gpointer) G_TYPE_CHAR,
 				     gda_handler_numerical_new ());
