@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2007 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -21,6 +21,7 @@
 #include <string.h>
 #define __GDA_INTERNAL__
 #include "dir-blob-op.h"
+#include <libgda/gda-blob-op-impl.h>
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -33,7 +34,7 @@ struct _GdaDirBlobOpPrivate {
 
 static void gda_dir_blob_op_class_init (GdaDirBlobOpClass *klass);
 static void gda_dir_blob_op_init       (GdaDirBlobOp *blob,
-					  GdaDirBlobOpClass *klass);
+					GdaDirBlobOpClass *klass);
 static void gda_dir_blob_op_finalize   (GObject *object);
 
 static glong gda_dir_blob_op_get_length (GdaBlobOp *op);
@@ -74,7 +75,7 @@ _gda_dir_blob_op_get_type (void)
 
 static void
 gda_dir_blob_op_init (GdaDirBlobOp *op,
-			   G_GNUC_UNUSED GdaDirBlobOpClass *klass)
+		      G_GNUC_UNUSED GdaDirBlobOpClass *klass)
 {
 	g_return_if_fail (GDA_IS_DIR_BLOB_OP (op));
 
@@ -91,9 +92,9 @@ gda_dir_blob_op_class_init (GdaDirBlobOpClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_dir_blob_op_finalize;
-	blob_class->get_length = gda_dir_blob_op_get_length;
-	blob_class->read = gda_dir_blob_op_read;
-	blob_class->write = gda_dir_blob_op_write;
+	GDA_BLOB_OP_FUNCTIONS (blob_class->functions)->get_length = gda_dir_blob_op_get_length;
+	GDA_BLOB_OP_FUNCTIONS (blob_class->functions)->read = gda_dir_blob_op_read;
+	GDA_BLOB_OP_FUNCTIONS (blob_class->functions)->write = gda_dir_blob_op_write;
 }
 
 
@@ -179,6 +180,7 @@ gda_dir_blob_op_read (GdaBlobOp *op, GdaBlob *blob, glong offset, glong size)
 	GdaBinary *bin;
 	FILE *file;
 	size_t nread;
+	guchar *buffer;
 
 	g_return_val_if_fail (GDA_IS_DIR_BLOB_OP (op), -1);
 	dirop = GDA_DIR_BLOB_OP (op);
@@ -198,14 +200,11 @@ gda_dir_blob_op_read (GdaBlobOp *op, GdaBlob *blob, glong offset, glong size)
 		return -1;
 	}
 	
-	bin = (GdaBinary *) blob;
-	if (bin->data) {
-		g_free (bin->data);
-		bin->data = NULL;
-	}
-	bin->data = g_new0 (guchar, size);
-	nread = fread ((char *) (bin->data), 1, size, file);
-	bin->binary_length = nread;
+	bin = gda_blob_get_binary (blob);
+	gda_binary_reset_data (bin);
+	buffer = g_new0 (guchar, size);
+	nread = fread ((char *) (buffer), 1, size, file);
+	gda_binary_take_data (bin, buffer, nread);
 	fclose (file);
 
 	return nread;
@@ -239,25 +238,25 @@ gda_dir_blob_op_write (GdaBlobOp *op, GdaBlob *blob, glong offset)
 		}
 	}
 
-	if (blob->op && (blob->op != op)) {
+	if (gda_blob_get_op (blob) && (gda_blob_get_op (blob) != op)) {
 		/* use data through blob->op */
-                #define buf_size 16384
+#define buf_size 16384
 		gint nread = 0;
-		GdaBlob *tmpblob = g_new0 (GdaBlob, 1);
-		gda_blob_set_op (tmpblob, blob->op);
+		GdaBlob *tmpblob = gda_blob_new ();
+		gda_blob_set_op (tmpblob, gda_blob_get_op (blob));
 
 		nbwritten = 0;
 
-		for (nread = gda_blob_op_read (tmpblob->op, tmpblob, 0, buf_size);
+		for (nread = gda_blob_op_read (gda_blob_get_op (tmpblob), tmpblob, 0, buf_size);
 		     nread > 0;
-		     nread = gda_blob_op_read (tmpblob->op, tmpblob, nbwritten, buf_size)) {
-			GdaBinary *bin = (GdaBinary *) tmpblob;
+		     nread = gda_blob_op_read (gda_blob_get_op (tmpblob), tmpblob, nbwritten, buf_size)) {
+			GdaBinary *bin = gda_blob_get_binary (tmpblob);
 			glong tmp_written;
-			tmp_written = fwrite ((char *) (bin->data), sizeof (guchar), bin->binary_length, file);
-			if (tmp_written < bin->binary_length) {
+			tmp_written = fwrite ((char *) gda_binary_get_data (bin), sizeof (guchar), gda_binary_get_size (bin), file);
+			if (tmp_written < gda_binary_get_size (bin)) {
 				/* error writing stream */
 				fclose (file);
-				gda_blob_free ((gpointer) tmpblob);
+				gda_blob_free (tmpblob);
 				return -1;
 			}
 			nbwritten += tmp_written;
@@ -265,12 +264,13 @@ gda_dir_blob_op_write (GdaBlobOp *op, GdaBlob *blob, glong offset)
 				/* nothing more to read */
 				break;
 		}
+
 		fclose (file);
-		gda_blob_free ((gpointer) tmpblob);
+		gda_blob_free (tmpblob);
 	}
 	else {
 		bin = (GdaBinary *) blob;
-		nbwritten = fwrite ((char *) (bin->data), 1, bin->binary_length, file);
+		nbwritten = fwrite ((char *) (gda_binary_get_data (bin)), 1, gda_binary_get_size (bin), file);
 		fclose (file);
 	}
 
