@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2000 - 2001 Reinhard Müller <reinhard@src.gnome.org>
  * Copyright (C) 2000 - 2003 Rodrigo Moya <rodrigo@gnome-db.org>
- * Copyright (C) 2001 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2001 - 2013 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2002 - 2003 Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
@@ -427,7 +427,7 @@ gda_batch_get_parameters (GdaBatch *batch, GdaSet **out_params, GError **error)
 			return FALSE;
 		}
 
-		if (tmpset && tmpset->holders) {
+		if (tmpset && gda_set_get_holders (tmpset)) {
 			if (!set) {
 				set = tmpset;
 				tmpset = NULL;
@@ -435,7 +435,7 @@ gda_batch_get_parameters (GdaBatch *batch, GdaSet **out_params, GError **error)
 			else {
 				/* merge @set and @tmp_set */
 				GSList *holders;
-				for (holders = tmpset->holders; holders; holders = holders->next) {
+				for (holders = gda_set_get_holders (tmpset); holders; holders = holders->next) {
 					GdaHolder *holder = (GdaHolder *) holders->data;
 					if (! gda_set_add_holder (set, holder)) {
 						GdaHolder *eholder = gda_set_get_holder (set, gda_holder_get_id (holder));
