@@ -2,7 +2,7 @@
  * Copyright (C) 2000 Akira Tagoh <tagoh@src.gnome.org>
  * Copyright (C) 2000 Reinhard Müller <reinhard@src.gnome.org>
  * Copyright (C) 2000 - 2005 Rodrigo Moya <rodrigo@gnome-db.org>
- * Copyright (C) 2001 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2001 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2002 - 2003 Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
  * Copyright (C) 2002 Zbigniew Chyla <cyba@gnome.pl>
  * Copyright (C) 2003 Akira TAGOH <tagoh@gnome-db.org>
@@ -15,6 +15,8 @@
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
+ * Copyright (C) 2013 Daniel Espinosa <esodan@gmail.com>
+ * Copyright (C) 2013 Miguel Angel Cabrera Moya <madmac2501@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -59,9 +61,25 @@
 
 #ifdef HAVE_LIBSECRET
   #include <libsecret/secret.h>
+  const SecretSchema *_gda_secret_get_schema (void) G_GNUC_CONST;
+  #define GDA_SECRET_SCHEMA _gda_secret_get_schema ()
+  #define SECRET_LABEL "DSN"
+  const SecretSchema *
+  _gda_secret_get_schema (void)
+  {
+	  static const SecretSchema the_schema = {
+		  "org.gnome-db.DSN", SECRET_SCHEMA_NONE,
+		  {
+			  {  "DSN", SECRET_SCHEMA_ATTRIBUTE_STRING },
+			  {  "NULL", 0 },
+		  }
+	  };
+	  return &the_schema;
+  }
 #else
   #ifdef HAVE_GNOME_KEYRING
   #include <gnome-keyring.h>
+  #define SECRET_LABEL "server"
   #endif
 #endif
 
@@ -74,7 +92,7 @@ gda_dsn_info_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
-        if (type == 0)
+		if (type == 0)
 			type = g_boxed_type_register_static ("GdaDsnInfo",
 							     (GBoxedCopyFunc) gda_dsn_info_copy,
 							     (GBoxedFreeFunc) gda_dsn_info_free);
@@ -86,7 +104,7 @@ gda_dsn_info_get_type (void)
 /**
  * gda_dsn_info_new:
  *
- * Creates a new #GdaDsnInfo struct.
+ * Creates a new empty #GdaDsnInfo struct.
  *
  * Returns: (transfer full): a new #GdaDsnInfo struct.
  *
@@ -116,17 +134,17 @@ gda_dsn_info_new (void)
  * Since: 5.2
  */
 GdaDsnInfo *
-gda_dsn_info_copy (GdaDsnInfo *source)
+gda_dsn_info_copy (const GdaDsnInfo *source)
 {
 	GdaDsnInfo *n;
 	g_return_val_if_fail (source, NULL);
 	n = gda_dsn_info_new ();
-	n->name = source->name;
-	n->provider = source->provider;
-	n->description = source->description;
-	n->cnc_string = source->cnc_string;
-	n->auth_string = source->auth_string;
-	n->is_system = source->is_system;;
+	n->name = source->name ? g_strdup (source->name) : NULL;
+	n->provider = source->provider ? g_strdup (source->provider) : NULL;
+	n->description = source->description ? g_strdup (source->description) : NULL;
+	n->cnc_string = source->cnc_string ? g_strdup (source->cnc_string) : NULL;
+	n->auth_string = source->auth_string ? g_strdup (source->auth_string) : NULL;
+	n->is_system = source->is_system;
 	return n;
 }
 
@@ -141,10 +159,106 @@ gda_dsn_info_copy (GdaDsnInfo *source)
 void
 gda_dsn_info_free (GdaDsnInfo *dsn)
 {
-	g_return_if_fail(dsn);
-	g_free (dsn);
+	if (dsn) {
+		g_free (dsn->name);
+		g_free (dsn->provider);
+		g_free (dsn->description);
+		g_free (dsn->cnc_string);
+		g_free (dsn->auth_string);
+		g_free (dsn);
+	}
 }
 
+static
+gboolean string_equal (const gchar *s1, const gchar *s2)
+{
+	if (s1) {
+		if (s2)
+			return !strcmp (s1, s2);
+		else
+			return FALSE;
+	}
+	else
+		return s2 ? FALSE : TRUE;
+}
+
+static gchar *
+make_cmp_string (const gchar *key_values_string)
+{
+	if (!key_values_string)
+		return NULL;
+
+	GdaQuarkList *ql;
+	GSList *list, *sorted_list = NULL;
+	GString *string = NULL;
+	ql = gda_quark_list_new_from_string (key_values_string);
+	for (list = sorted_list; list; list = list->next) {
+		const gchar *value;
+		gchar *evalue;
+
+		if (!string)
+			string = g_string_new ("");
+		else
+			g_string_append_c (string, ',');
+
+		value = gda_quark_list_find (ql, (gchar *) list->data);
+		evalue = gda_rfc1738_encode (value);
+		g_string_append_printf (string, ",%s=%s", (gchar *) list->data, evalue);
+		g_free (evalue);
+	}
+	gda_quark_list_free (ql);
+	if (string)
+		return g_string_free (string, FALSE);
+	else
+		return NULL;
+}
+
+/**
+ * gda_dsn_info_equal:
+ * @dsn1: (allow-none): a #GdaDsnInfo
+ * @dsn2: (allow-none): a #GdaDsnInfo
+ *
+ * Compares @dsn1 and @dsn2.
+ *
+ * If both @dsn1 and @dsn2 are %NULL, then the function returns %TRUE.
+ * If only one of @dsn1 or @dsn2 is %NULL, then the function return %FALSE.
+ *
+ * Returns: %TRUE if they are equal.
+ */
+gboolean
+gda_dsn_info_equal (const GdaDsnInfo *dsn1, const GdaDsnInfo *dsn2)
+{
+	if (dsn1) {
+		if (dsn2) {
+			if ((dsn1->is_system == dsn2->is_system) &&
+			    string_equal (dsn1->name, dsn2->name) &&
+			    string_equal (dsn1->provider, dsn2->provider) &&
+			    string_equal (dsn1->description, dsn2->description)) {
+				gboolean eq;
+				gchar *s1, *s2;
+				s1 = make_cmp_string (dsn1->cnc_string);
+				s2 = make_cmp_string (dsn2->cnc_string);
+				eq = string_equal (s1, s2);
+				g_free (s1);
+				g_free (s2);
+				if (eq) {
+					s1 = make_cmp_string (dsn1->auth_string);
+					s2 = make_cmp_string (dsn2->auth_string);
+					eq = string_equal (s1, s2);
+					g_free (s1);
+					g_free (s2);
+				}
+				return eq;
+			}
+			else
+				return FALSE;
+		}
+		else
+			return FALSE;
+	}
+	else
+		return dsn2 ? FALSE : TRUE;
+}
 
 typedef struct {
 	GdaProviderInfo    pinfo;
@@ -187,7 +301,6 @@ static gboolean sync_keyring = FALSE;
 #endif
 
 static gint data_source_info_compare (GdaDsnInfo *infoa, GdaDsnInfo *infob);
-static void data_source_info_free (GdaDsnInfo *info);
 static void internal_provider_free (InternalProvider *ip);
 static void load_config_file (const gchar *file, gboolean is_system);
 static void save_config_file (gboolean is_system);
@@ -330,7 +443,7 @@ gda_config_class_init (GdaConfigClass *klass)
 	 *
 	 * File to use for system-wide DSN list. When changed, the whole list of DSN will be reloaded.
 	 */
-	/* To translators: DSN stands for Data Source Name, it's a named connection string defined in $PREFIX/etc/libgda-5.0/config */
+	/* To translators: DSN stands for Data Source Name, it's a named connection string defined in $PREFIX/etc/libgda-6.0/config */
 	g_object_class_install_property (object_class, PROP_SYSTEM_FILE,
                                          g_param_spec_string ("system-filename", NULL,
 							      "File to use for system-wide DSN list", 
@@ -520,9 +633,9 @@ load_config_file (const gchar *file, gboolean is_system)
 				if (sync_keyring) {
 					GError *error = NULL;
 					gchar *auth = NULL;
-					auth = secret_password_lookup_sync (SECRET_SCHEMA_COMPAT_NETWORK,
+					auth = secret_password_lookup_sync (GDA_SECRET_SCHEMA,
 									    NULL, &error,
-									    "server", info->name, NULL);
+									    SECRET_LABEL, info->name, NULL);
 					if (auth) {
 						/*g_print ("Loaded sync. auth info for '%s': %s\n", info->name, auth);*/
 						info->auth_string = g_strdup (auth);
@@ -535,10 +648,10 @@ load_config_file (const gchar *file, gboolean is_system)
 					}
 				}
 				else {
-					secret_password_lookup (SECRET_SCHEMA_COMPAT_NETWORK,
+					secret_password_lookup (GDA_SECRET_SCHEMA,
 								NULL, (GAsyncReadyCallback) secret_password_found_cb,
 								g_strdup (info->name),
-								"server", info->name, NULL);
+								SECRET_LABEL, info->name, NULL);
 				}
 			}
 #else
@@ -548,7 +661,7 @@ load_config_file (const gchar *file, gboolean is_system)
 					GnomeKeyringResult res;
 					gchar *auth = NULL;
 					res = gnome_keyring_find_password_sync (GNOME_KEYRING_NETWORK_PASSWORD, &auth,
-										"server", info->name, NULL);
+										SECRET_LABEL, info->name, NULL);
 					if (res == GNOME_KEYRING_RESULT_OK) {
 						/*g_print ("Loaded sync. auth info for '%s': %s\n", info->name, auth);*/
 						info->auth_string = g_strdup (auth);
@@ -564,7 +677,7 @@ load_config_file (const gchar *file, gboolean is_system)
 					gnome_keyring_find_password (GNOME_KEYRING_NETWORK_PASSWORD,
 								     (GnomeKeyringOperationGetStringCallback) password_found_cb,
 								     tmp, g_free,
-								     "server", tmp, NULL);
+								     SECRET_LABEL, tmp, NULL);
 				}
 			}
   #endif
@@ -868,7 +981,7 @@ gda_config_dispose (GObject *object)
 		g_free (conf->priv->system_file);
 
 		if (conf->priv->dsn_list) {
-			g_slist_foreach (conf->priv->dsn_list, (GFunc) data_source_info_free, NULL);
+			g_slist_foreach (conf->priv->dsn_list, (GFunc) gda_dsn_info_free, NULL);
 			g_slist_free (conf->priv->dsn_list);
 		}
 		if (conf->priv->prov_list) {
@@ -1164,15 +1277,15 @@ gda_config_define_dsn (const GdaDsnInfo *info, GError **error)
 	if (! info->is_system && info->auth_string) {
 		/* save to keyring */
 		gchar *tmp;
-		tmp = g_strdup_printf (_("Authentication for the '%s' DSN"), info->name);
+		tmp = g_strdup_printf (_("Gnome-DB: authentication for the '%s' DSN"), info->name);
 		if (sync_keyring) {
 			gboolean res;
 			GError *error = NULL;
-			res = secret_password_store_sync (SECRET_SCHEMA_COMPAT_NETWORK,
+			res = secret_password_store_sync (GDA_SECRET_SCHEMA,
 							  SECRET_COLLECTION_DEFAULT,
 							  tmp, info->auth_string,
 							  NULL, &error,
-							  "server", info->name, NULL);
+							  SECRET_LABEL, info->name, NULL);
 			if (!res) {
 				gda_log_error (_("Couldn't save authentication information for DSN '%s': %s"),
 					       info->name,
@@ -1181,13 +1294,13 @@ gda_config_define_dsn (const GdaDsnInfo *info, GError **error)
 			}
 		}
 		else {
-			secret_password_store (SECRET_SCHEMA_COMPAT_NETWORK,
+			secret_password_store (GDA_SECRET_SCHEMA,
 					       SECRET_COLLECTION_DEFAULT,
 					       tmp, info->auth_string,
 					       NULL,
 					       (GAsyncReadyCallback) secret_password_stored_cb,
 					       g_strdup (info->name),
-					       "server", info->name, NULL);
+					       SECRET_LABEL, info->name, NULL);
 		}
 		g_free (tmp);
 	}
@@ -1196,12 +1309,12 @@ gda_config_define_dsn (const GdaDsnInfo *info, GError **error)
 	if (! info->is_system && info->auth_string) {
 		/* save to keyring */
 		gchar *tmp;
-		tmp = g_strdup_printf (_("Authentication for the '%s' DSN"), info->name);
+		tmp = g_strdup_printf (_("Gnome-DB: authentication for the '%s' DSN"), info->name);
 		if (sync_keyring) {
 			GnomeKeyringResult res;
 			res = gnome_keyring_store_password_sync (GNOME_KEYRING_NETWORK_PASSWORD, GNOME_KEYRING_DEFAULT,
 								 tmp, info->auth_string,
-								 "server", info->name, NULL);
+								 SECRET_LABEL, info->name, NULL);
 			password_stored_cb (res, info->name);
 		}
 		else {
@@ -1211,7 +1324,7 @@ gda_config_define_dsn (const GdaDsnInfo *info, GError **error)
 						      GNOME_KEYRING_DEFAULT,
 						      tmp, info->auth_string,
 						      (GnomeKeyringOperationDoneCallback) password_stored_cb, tmp1, g_free,
-						      "server", info->name, NULL);
+						      SECRET_LABEL, info->name, NULL);
 		}
 		g_free (tmp);
 	}
@@ -1299,16 +1412,15 @@ gda_config_remove_dsn (const gchar *dsn_name, GError **error)
 	unique_instance->priv->dsn_list = g_slist_remove (unique_instance->priv->dsn_list, info);
 	if (unique_instance->priv->emit_signals)
 		g_signal_emit (unique_instance, gda_config_signals[DSN_REMOVED], 0, info);
-	data_source_info_free (info);
 
 #ifdef HAVE_LIBSECRET
 	if (! info->is_system) {
 		/* remove from keyring */
 		if (sync_keyring) {
 			GError *error = NULL;
-			if (! secret_password_clear_sync (SECRET_SCHEMA_COMPAT_NETWORK,
+			if (! secret_password_clear_sync (GDA_SECRET_SCHEMA,
 							  NULL, &error,
-							  "server", info->name, NULL)) {
+							  SECRET_LABEL, info->name, NULL)) {
 				gda_log_error (_("Couldn't delete authentication information for DSN '%s': %s"),
 					       info->name,
 					       error && error->message ? error->message : _("No detail"));
@@ -1316,10 +1428,10 @@ gda_config_remove_dsn (const gchar *dsn_name, GError **error)
 			}
 		}
 		else {
-			secret_password_clear (SECRET_SCHEMA_COMPAT_NETWORK,
+			secret_password_clear (GDA_SECRET_SCHEMA,
 					       NULL, (GAsyncReadyCallback) secret_password_deleted_cb,
 					       g_strdup (info->name),
-					       "server", info->name, NULL);
+					       SECRET_LABEL, info->name, NULL);
 		}
 	}
 #else
@@ -1329,7 +1441,7 @@ gda_config_remove_dsn (const gchar *dsn_name, GError **error)
 		if (sync_keyring) {
 			GnomeKeyringResult res;
 			res = gnome_keyring_delete_password_sync (GNOME_KEYRING_NETWORK_PASSWORD, GNOME_KEYRING_DEFAULT,
-								  "server", info->name, NULL);
+								  SECRET_LABEL, info->name, NULL);
 			password_deleted_cb (res, info->name);
 		}
 		else {
@@ -1338,11 +1450,12 @@ gda_config_remove_dsn (const gchar *dsn_name, GError **error)
 			gnome_keyring_delete_password (GNOME_KEYRING_NETWORK_PASSWORD,
 						       (GnomeKeyringOperationDoneCallback) password_deleted_cb,
 						       tmp, g_free,
-						       "server", info->name, NULL);
+						       SECRET_LABEL, info->name, NULL);
 		}
 	}
   #endif
 #endif
+	gda_dsn_info_free (info);
 
 	if (save_system)
 		save_config_file (TRUE);
@@ -1376,7 +1489,7 @@ gda_config_dsn_needs_authentication (const gchar *dsn_name)
 		gda_log_message (_("Provider '%s' not found"), info->provider);
 		return FALSE;
 	}
-	if (pinfo->auth_params && pinfo->auth_params->holders)
+	if (pinfo->auth_params && gda_set_get_holders (pinfo->auth_params))
 		return TRUE;
 	else
 		return FALSE;
@@ -1668,12 +1781,12 @@ gda_config_list_providers (void)
 		if (info->dsn_params) {
 			GSList *params;
 			GString *string = g_string_new ("");
-			for (params = info->dsn_params->holders;
+			for (params = gda_set_get_holders (info->dsn_params);
 			     params; params = params->next) {
 				const gchar *id;
 
 				id = gda_holder_get_id (GDA_HOLDER (params->data));
-				if (params != info->dsn_params->holders)
+				if (params != gda_set_get_holders (info->dsn_params))
 					g_string_append (string, ",\n");
 				g_string_append (string, id);
 			}
@@ -1686,12 +1799,12 @@ gda_config_list_providers (void)
 		if (info->auth_params) {
 			GSList *params;
 			GString *string = g_string_new ("");
-			for (params = info->auth_params->holders;
+			for (params = gda_set_get_holders (info->auth_params);
 			     params; params = params->next) {
 				const gchar *id;
 
 				id = gda_holder_get_id (GDA_HOLDER (params->data));
-				if (params != info->auth_params->holders)
+				if (params != gda_set_get_holders (info->auth_params))
 					g_string_append (string, ",\n");
 				g_string_append (string, id);
 			}
@@ -1754,8 +1867,10 @@ load_all_providers (void)
 static InternalProvider *
 create_internal_provider (const gchar *path,
 			  const gchar *prov_name, const gchar *prov_descr,
-			  gchar *dsn_spec, gchar *auth_spec)
+			  gchar *dsn_spec, gchar *auth_spec, const gchar *icon_id)
 {
+	g_return_val_if_fail (prov_name, NULL);
+
 	InternalProvider *ip;
 	GdaProviderInfo *info;
 
@@ -1773,10 +1888,13 @@ create_internal_provider (const gchar *path,
 		GError *error = NULL;
 		info->dsn_params = gda_set_new_from_spec_string (dsn_spec, &error);
 		if (!info->dsn_params) {
-			gda_log_message ("Invalid format for provider '%s' DSN spec : %s",
+			g_warning ("Invalid format for provider '%s' DSN spec : %s",
 					 info->id,
 					 error ? error->message : "Unknown error");
 			g_clear_error (&error);
+#ifdef GDA_DEBUG
+			g_print ("Dump DSN spec:\n%s\n", dsn_spec);
+#endif
 
 			/* there may be traces of the provider installed but some parts are missing,
 			   forget about that provider... */
@@ -1796,7 +1914,7 @@ create_internal_provider (const gchar *path,
 
 		info->auth_params = gda_set_new_from_spec_string (auth_spec, &error);
 		if (!info->auth_params) {
-			gda_log_message ("Invalid format for provider '%s' AUTH spec : %s",
+			g_warning ("Invalid format for provider '%s' AUTH spec : %s",
 					 info->id,
 				   error ? error->message : "Unknown error");
 			if (error)
@@ -1828,6 +1946,9 @@ create_internal_provider (const gchar *path,
                 gda_holder_set_attribute_static (h, GDAUI_ATTRIBUTE_PLUGIN, value);
                 gda_value_free (value);
 	}
+
+	info->icon_id = icon_id ? g_strdup (icon_id) : g_strdup (prov_name);
+
 	return ip;
 }
 
@@ -1839,7 +1960,7 @@ load_providers_from_dir (const gchar *dirname, gboolean recurs)
 	const gchar *name;
 
 	/* read the plugin directory */
-#ifdef GDA_DEBUG_NO
+#ifdef GDA_DEBUG
 	g_print ("Loading providers in %s\n", dirname);
 #endif
 	dir = g_dir_open (dirname, 0, &err);
@@ -1861,12 +1982,14 @@ load_providers_from_dir (const gchar *dirname, gboolean recurs)
 		const gchar * (* plugin_get_description) (void);
 		gchar * (* plugin_get_dsn_spec) (void);
 		gchar * (* plugin_get_auth_spec) (void);
+		const gchar * (*plugin_get_icon_id) (void);
 		
 		/* methods for shared libraries which provide several types of providers (ODBC, JDBC, ...) */
 		const gchar ** (* plugin_get_sub_names) (void);
 		const gchar * (* plugin_get_sub_description) (const gchar *name);
 		gchar * (* plugin_get_sub_dsn_spec) (const gchar *name);
 		gchar * (* plugin_get_sub_auth_spec) (const gchar *name);
+		const gchar * (*plugin_get_sub_icon_id) (const gchar *name);
 
 		if (recurs) {
 			gchar *cname;
@@ -1875,7 +1998,6 @@ load_providers_from_dir (const gchar *dirname, gboolean recurs)
 				load_providers_from_dir (cname, TRUE);
 			g_free (cname);
 		}
-
 		if (!g_str_has_suffix (name, "." G_MODULE_SUFFIX))
 			continue;
 #ifdef G_WITH_CYGWIN
@@ -1885,12 +2007,17 @@ load_providers_from_dir (const gchar *dirname, gboolean recurs)
 #endif
 			continue;
 
-		path = g_build_path (G_DIR_SEPARATOR_S, dirname,
-				     name, NULL);
+#ifdef GDA_DEBUG
+		g_print ("File's name checking for provider: %s\n", name);
+#endif
+		path = g_build_path (G_DIR_SEPARATOR_S, dirname, name, NULL);
 		handle = g_module_open (path, G_MODULE_BIND_LAZY);
 		if (!handle) {
 			if (g_getenv ("GDA_SHOW_PROVIDER_LOADING_ERROR"))
 				gda_log_message (_("Error loading provider '%s': %s"), path, g_module_error ());
+#ifdef GDA_DEBUG
+		g_print ("Error loading provider's module: %s : %s\n", path, g_module_error ());
+#endif
 			g_free (path);
 			continue;
 		}
@@ -1911,6 +2038,8 @@ load_providers_from_dir (const gchar *dirname, gboolean recurs)
 				 (gpointer *) &plugin_get_dsn_spec);
 		g_module_symbol (handle, "plugin_get_auth_spec",
 				 (gpointer *) &plugin_get_auth_spec);
+		g_module_symbol (handle, "plugin_get_icon_id",
+				 (gpointer *) &plugin_get_icon_id);
 		g_module_symbol (handle, "plugin_get_sub_names",
 				 (gpointer *) &plugin_get_sub_names);
 		g_module_symbol (handle, "plugin_get_sub_description",
@@ -1919,6 +2048,8 @@ load_providers_from_dir (const gchar *dirname, gboolean recurs)
 				 (gpointer *) &plugin_get_sub_dsn_spec);
 		g_module_symbol (handle, "plugin_get_sub_auth_spec",
 				 (gpointer *) &plugin_get_sub_auth_spec);
+		g_module_symbol (handle, "plugin_get_sub_icon_id",
+				 (gpointer *) &plugin_get_sub_icon_id);
 
 		if (plugin_get_sub_names) {
 			const gchar **subnames = plugin_get_sub_names ();
@@ -1932,14 +2063,21 @@ load_providers_from_dir (const gchar *dirname, gboolean recurs)
 							       plugin_get_sub_dsn_spec ? 
 							       plugin_get_sub_dsn_spec (*ptr) : NULL,
 							       plugin_get_sub_auth_spec ?
-							       plugin_get_sub_auth_spec (*ptr) : NULL);
+							       plugin_get_sub_auth_spec (*ptr) : NULL,
+							       plugin_get_sub_icon_id ?
+							       plugin_get_sub_icon_id (*ptr) : NULL);
 				if (ip) {
 					unique_instance->priv->prov_list =
 						g_slist_prepend (unique_instance->priv->prov_list, ip);
-#ifdef GDA_DEBUG_NO
+#ifdef GDA_DEBUG
 					g_print ("Loaded '%s' sub-provider\n", ((GdaProviderInfo*) ip)->id);
 #endif
 				}
+#ifdef GDA_DEBUG
+				else {
+					g_print ("Error Loading sub-provider\n");
+				}
+#endif
 			}
 		}
 		else {
@@ -1948,14 +2086,20 @@ load_providers_from_dir (const gchar *dirname, gboolean recurs)
 						       plugin_get_name ? plugin_get_name () : name,
 						       plugin_get_description ? plugin_get_description () : NULL,
 						       plugin_get_dsn_spec ? plugin_get_dsn_spec () : NULL,
-						       plugin_get_auth_spec ? plugin_get_auth_spec () : NULL);
+						       plugin_get_auth_spec ? plugin_get_auth_spec () : NULL,
+						       plugin_get_icon_id ? plugin_get_icon_id () : NULL);
 			if (ip) {
 				unique_instance->priv->prov_list =
 					g_slist_prepend (unique_instance->priv->prov_list, ip);
-#ifdef GDA_DEBUG_NO
+#ifdef GDA_DEBUG
 				g_print ("Loaded '%s' provider\n", ((GdaProviderInfo*) ip)->id);
 #endif
 			}
+#ifdef GDA_DEBUG
+				else {
+					g_print ("Error Loading provider\n");
+				}
+#endif
 		}
 		g_free (path);
 		g_module_close (handle);
@@ -1991,16 +2135,6 @@ data_source_info_compare (GdaDsnInfo *infoa, GdaDsnInfo *infob)
 		return -1;
 }
 
-static void 
-data_source_info_free (GdaDsnInfo *info)
-{
-	g_free (info->provider); 
-	g_free (info->cnc_string); 
-	g_free (info->description);
-	g_free (info->auth_string);
-	g_free (info);
-}
-
 static void
 internal_provider_free (InternalProvider *ip)
 {
@@ -2094,7 +2228,7 @@ reload_dsn_configuration (void)
 #endif
 			    (oinfo->is_system == ninfo->is_system)) {
 				/* no change for this DSN */
-				data_source_info_free (ninfo);
+				gda_dsn_info_free (ninfo);
 			}
 			else {
 				GdaDsnInfo tmp;
@@ -2103,7 +2237,7 @@ reload_dsn_configuration (void)
 				*ninfo = tmp;
 				if (unique_instance->priv->emit_signals)
 					g_signal_emit (unique_instance, gda_config_signals[DSN_CHANGED], 0, oinfo);
-				data_source_info_free (ninfo);
+				gda_dsn_info_free (ninfo);
 			}
 			g_hash_table_insert (hash, oinfo->name, (gpointer) 0x1);
 		}
@@ -2123,7 +2257,7 @@ reload_dsn_configuration (void)
 		unique_instance->priv->dsn_list = g_slist_remove (unique_instance->priv->dsn_list, info);
 		if (unique_instance->priv->emit_signals)
 			g_signal_emit (unique_instance, gda_config_signals[DSN_REMOVED], 0, info);
-		data_source_info_free (info);
+		gda_dsn_info_free (info);
 	}
 	g_hash_table_destroy (hash);
 
