@@ -1,11 +1,12 @@
 /*
  * Copyright (C) 2000 Reinhard Müller <reinhard@src.gnome.org>
  * Copyright (C) 2000 - 2005 Rodrigo Moya <rodrigo@gnome-db.org>
- * Copyright (C) 2001 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2001 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2002 Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
  * Copyright (C) 2003 Laurent Sansonetti <laurent@datarescue.be>
  * Copyright (C) 2003 - 2007 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2005 Andrew Hill <andru@src.gnome.org>
+ * Copyright (C) 2013 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -79,6 +80,14 @@ struct _GdaDsnInfo {
 	gpointer _gda_reserved4;
 };
 
+#define GDA_TYPE_DSN_INFO (gda_dsn_info_get_type ())
+
+GType            gda_dsn_info_get_type  (void) G_GNUC_CONST;
+GdaDsnInfo*      gda_dsn_info_new       (void);
+GdaDsnInfo*      gda_dsn_info_copy      (const GdaDsnInfo *source);
+void             gda_dsn_info_free      (GdaDsnInfo *dsn);
+gboolean         gda_dsn_info_equal     (const GdaDsnInfo *dsn1, const GdaDsnInfo *dsn2);
+
 /**
  * GdaProviderInfo:
  * @id: the unique identifier of the database provider
@@ -95,6 +104,7 @@ struct _GdaProviderInfo {
         gchar             *description;
         GdaSet            *dsn_params;  /* Specs to create a DSN */
 	GdaSet            *auth_params; /* Specs to authenticate a client */
+	gchar             *icon_id; /* use gdaui_get_icon_for_db_engine(icon_id) to get the actual GdkPixbuf */
 
 	/*< private >*/
 	/* Padding for future expansion */
@@ -126,6 +136,50 @@ struct _GdaConfigClass {
 	void (*_gda_reserved4) (void);
 };
 
+/**
+ * SECTION:gda-config
+ * @short_description: Access/Management of libgda configuration
+ * @title: Configuration
+ * @stability: Stable
+ * @see_also:
+ *
+ * The functions in this section allow applications an easy access to libgda's
+ * configuration (the list of data sources and database providers).
+ *
+ * As soon as a <link linkend="GdaConfig">GdaConfig</link> is needed (for example when requesting information
+ * about a data source or about a server provider), a single instance object is created,
+ * and no other will need to be created. A pointer to this object can be obtained with
+ * <link linkend="gda-config-get">gda_config_get()</link>. Of course one can (right after having called
+ * <link linkend="gda-init">gda_init()</link>) force the creation of a GdaConfig object with some
+ * specific properties set, using a simple call like:
+ * <programlisting>
+ *g_object_new (GDA_TYPE_CONFIG, "user-filename", "my_file", NULL);
+ * </programlisting>
+ * Please note that after that call, the caller has a reference to the newly created object, and should technically
+ * call <link linkend="g-object-unref">g_object_unref()</link> when finished using it. It is safe to do this
+ * but also pointless since that object should not be destroyed (as no other will be created) as &LIBGDA; also
+ * keeps a reference for itself.
+ * 
+ *Data sources are defined in a per-user configuration file which is by default <filename>${HOME}/.libgda/config</filename> and
+ * in a system wide configuration file which is by default <filename>${prefix}/etc/libgda-4.0/config</filename>. Those
+ * filenames can be modified by setting the <link linkend="GdaConfig--user-file">user-file</link> and
+ * <link linkend="GdaConfig--system-file">system-file</link> properties for the single <link linkend="GdaConfig">GdaConfig</link>
+ * instance. Note that setting either of these properties to <literal>NULL</literal> will disable using the corresponding
+ * configuration file (DSN will exist only in memory and their definition will be lost when the application finishes).
+ *
+ * The #GdaConfig object implements its own locking mechanism so it is thread-safe.
+ *
+ * Note about localization: when the #GdaConfig loads configuration files, it filters the
+ * contents based on the current locale, so for example if your current locale is "de" then
+ * all the loaded strings (for the ones which are translated) will be in the German language.
+ * Changing the locale afterwards will have no effect on the #GdaConfig and the already loaded
+ * configuration.
+ * The consequence is that you should first call setlocale() youself in your code before using
+ * a #GdaConfig object. As a side note you should also call gtk_init() before gdaui_init() because
+ * gtk_init() calls setlocale().
+ */
+
+
 GType              gda_config_get_type                 (void) G_GNUC_CONST;
 GdaConfig*         gda_config_get                      (void);
 
