@@ -3,6 +3,7 @@
  * Copyright (C) 2007 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2015 Corentin Noël <corentin@elementary.io>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -150,7 +151,7 @@ gda_data_handler_get_str_from_value (GdaDataHandler *dh, const GValue *value)
 /**
  * gda_data_handler_get_value_from_sql:
  * @dh: an object which implements the #GdaDataHandler interface
- * @sql: (allow-none) (transfer none): an SQL string, or %NULL
+ * @sql: (nullable) (transfer none): an SQL string, or %NULL
  * @type: a GType
  *
  * Creates a new GValue which represents the SQL value given as argument. This is
@@ -328,7 +329,6 @@ gda_data_handler_get_default (GType for_type)
                 g_hash_table_insert (hash, (gpointer) GDA_TYPE_USHORT, gda_handler_numerical_new ());
                 g_hash_table_insert (hash, (gpointer) G_TYPE_STRING, gda_handler_string_new ());
                 g_hash_table_insert (hash, (gpointer) GDA_TYPE_TIME, gda_handler_time_new ());
-                g_hash_table_insert (hash, (gpointer) GDA_TYPE_TIMESTAMP, gda_handler_time_new ());
                 g_hash_table_insert (hash, (gpointer) G_TYPE_DATE_TIME, gda_handler_time_new ());
                 g_hash_table_insert (hash, (gpointer) G_TYPE_CHAR, gda_handler_numerical_new ());
                 g_hash_table_insert (hash, (gpointer) G_TYPE_UCHAR, gda_handler_numerical_new ());
