@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
- * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2013 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -34,7 +34,7 @@
 extern const unsigned char UpperToLower[];
 #define charMap(X) UpperToLower[(unsigned char)(X)]
 extern int casecmp(const char *zLeft, const char *zRight, int N);
-#include "keywords_hash.c" /* this one is dynamically generated */
+#include "keywords_hash.code" /* this one is dynamically generated */
 
 /*
  * Each value in @values may be %NULL, a valid pointer, or 0x1
@@ -170,9 +170,6 @@ gda_data_meta_wrapper_class_init (GdaDataMetaWrapperClass *klass)
 	object_class->dispose = gda_data_meta_wrapper_dispose;
 	object_class->finalize = gda_data_meta_wrapper_finalize;
 
-#ifdef GDA_DEBUG
-	test_keywords ();
-#endif
 }
 
 static void
