@@ -1,9 +1,10 @@
 /*
- * Copyright (C) 2007 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2007 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2008 - 2011 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
+ * Copyright (C) 2013 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,10 +33,6 @@
 #include <gio/gio.h>
 #endif
 
-/* Use the RSA reference implementation included in the RFC-1321, http://www.freesoft.org/CIE/RFC/1321/ */
-#include "global.h"
-#include "md5.h"
-
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -529,22 +526,10 @@ update_file_md5sum (FileRow *row, const gchar *complete_filename)
 	}
 #endif /* !G_OS_WIN32 */
 
-	/* MD5 computation */
-	MD5_CTX context;
-	unsigned char digest[16]; /* Flawfinder: ignore */
-	GString *md5str;
-	gint i;
-
-	MD5Init (&context);
-	MD5Update (&context, map, length);
-	MD5Final (digest, &context);
-
-	md5str = g_string_new ("");
-	for (i = 0; i < 16; i++)
-		g_string_append_printf (md5str, "%02x", digest[i]);
+	gchar *md5str;
+	md5str = g_compute_checksum_for_data (G_CHECKSUM_MD5, map, length);
 	value = gda_value_new (G_TYPE_STRING);
-	g_value_take_string (value, md5str->str);
-	g_string_free (md5str, FALSE);
+	g_value_take_string (value, md5str);
 
 #ifndef G_OS_WIN32
 	munmap (map, length);
@@ -847,7 +832,7 @@ gda_data_model_dir_get_value_at (GdaDataModel *model, gint col, gint row, GError
 				GdaBlobOp *op;
 				gchar *filename;
 
-				blob = g_new0 (GdaBlob, 1);
+				blob = gda_blob_new ();
 
 				/* file mapping in mem */
 				filename = compute_filename (imodel, frow);
@@ -1044,8 +1029,8 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 						if (frow->data_value) {
 							GdaBlob *blob;
 							blob = (GdaBlob *) gda_value_get_blob (frow->data_value);
-							if (blob && blob->op)
-								_gda_dir_blob_set_filename (GDA_DIR_BLOB_OP (blob->op),
+							if (blob && gda_blob_get_op (blob))
+								_gda_dir_blob_set_filename (GDA_DIR_BLOB_OP (gda_blob_get_op (blob)),
 											   new_filename);
 						}
 					}
@@ -1120,8 +1105,8 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 				if (frow->data_value) {
 					GdaBlob *blob;
 					blob = (GdaBlob *) gda_value_get_blob (frow->data_value);
-					if (blob && blob->op)
-						_gda_dir_blob_set_filename (GDA_DIR_BLOB_OP (blob->op),
+					if (blob && gda_blob_get_op (blob))
+						_gda_dir_blob_set_filename (GDA_DIR_BLOB_OP (gda_blob_get_op (blob)),
 									   new_filename);
 				}
 			}
@@ -1140,7 +1125,7 @@ gda_data_model_dir_set_values (GdaDataModel *model, gint row, GList *values, GEr
 			}
 			else if (gda_value_is_null (value)) {
 				/* create a new empty blob */
-				blob = g_new0 (GdaBlob, 1);
+				blob = gda_blob_new ();
 			}
 
 			if (blob) {
@@ -1254,11 +1239,11 @@ gda_data_model_dir_append_values (GdaDataModel *model, const GList *values, GErr
 			gboolean bin_to_free = FALSE;
 			complete_filename = g_build_filename (dirname, filename, NULL);
 			if (!bin_data) {
-				bin_data = g_new0 (GdaBinary, 1);
+				bin_data = gda_binary_new ();
 				bin_to_free = TRUE;
 			}
-			if (g_file_set_contents (complete_filename, (gchar *) bin_data->data,
-						 bin_data->binary_length, NULL)) {
+			if (g_file_set_contents (complete_filename, (gchar *) gda_binary_get_data (bin_data),
+						 gda_binary_get_size (bin_data), NULL)) {
 				FileRow *row;
 
 				row = file_row_new ();
