@@ -1,10 +1,12 @@
 /*
- * Copyright (C) 2006 - 2011 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2006 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2006 - 2014 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2006 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2007 Brecht Sanders <brecht@edustria.be>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
+ * Copyright (C) 2013 Daniel Espinosa <esodan@gmail.com>
+ * Copyright (C) 2013 Miguel Angel Cabrera Moya <madmac2501@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -747,7 +749,6 @@ gda_data_model_import_set_property (GObject *object,
 				    GParamSpec *pspec)
 {
 	GdaDataModelImport *model;
-	const gchar *string;
 
 	model = GDA_DATA_MODEL_IMPORT (object);
 	if (model->priv) {
@@ -1584,7 +1585,6 @@ init_node_import (GdaDataModelImport *model)
 	GSList *list;
 	gint pos;
 	gchar *str;
-	GError *error = NULL;
 
 	node = model->priv->extract.node.node;
 	if (!node)
@@ -1656,7 +1656,7 @@ init_node_import (GdaDataModelImport *model)
 				xmlFree (str);
 			}
 			str = (gchar*)xmlGetProp (cur, (xmlChar*)"nullok");
-			spec->nullok = TRUE;
+			spec->nullok = FALSE;
 			if (str) {
 				spec->nullok = ((*str == 't') || (*str == 'T')) ? TRUE : FALSE;
 				xmlFree (str);
@@ -1717,8 +1717,11 @@ init_node_import (GdaDataModelImport *model)
 	clean_field_specs (fields);
 	model->priv->columns = g_slist_reverse (model->priv->columns);
 
-	if (cur && ! gda_data_model_add_data_from_xml_node (ramodel, cur, &error))
+	GError *error = NULL;
+	if (cur && ! gda_data_model_add_data_from_xml_node (ramodel, cur, &error)) {
 		add_error (model, error && error->message ? error->message : _("No detail"));
+		g_clear_error (&error);
+	}
 }
 
 
@@ -2009,7 +2012,7 @@ gda_data_model_import_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
 
 		g_object_get (G_OBJECT (iter), "update-model", &update_model, NULL);
 		g_object_set (G_OBJECT (iter), "update-model", FALSE, NULL);
-		for (plist = ((GdaSet *) iter)->holders, vlist = next_values;
+		for (plist = gda_set_get_holders (((GdaSet *) iter)), vlist = next_values;
 		     plist && vlist;
 		     plist = plist->next, vlist = vlist->next) {
 			GError *lerror = NULL;
@@ -2080,7 +2083,7 @@ gda_data_model_import_iter_prev (GdaDataModel *model, GdaDataModelIter *iter)
 
 		g_object_get (G_OBJECT (iter), "update-model", &update_model, NULL);
 		g_object_set (G_OBJECT (iter), "update-model", FALSE, NULL);
-		for (plist = ((GdaSet *) iter)->holders, vlist = imodel->priv->cursor_values;
+		for (plist = gda_set_get_holders (((GdaSet *) iter)), vlist = imodel->priv->cursor_values;
 		     plist && vlist;
 		     plist = plist->next, vlist = vlist->next) {
 			GError *lerror = NULL;
