@@ -1,8 +1,9 @@
 /*
  * Copyright (C) 2006 - 2008 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2006 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2006 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2007 Leonardo Boshell <lb@kmc.com.co>
  * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2015 Corentin Noël <corentin@elementary.io>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -332,7 +333,7 @@ model_reset_cb (GdaDataModel *model, GdaDataModelIter *iter)
 	if (GDA_IS_DATA_PROXY (model))
 		nbcols = nbcols / 2;
 
-	for (i = 0, list = ((GdaSet*) iter)->holders;
+	for (i = 0, list = gda_set_get_holders (((GdaSet*) iter));
 	     (i < nbcols) && list;
 	     i++, list = list->next) {
 		GdaColumn *col;
@@ -399,7 +400,7 @@ validate_holder_change_cb (GdaSet *paramlist, GdaHolder *param, const GValue *ne
 		col = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (param), "model_col")) - 1;
 		if (col < 0) 
 			g_set_error (&error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_COLUMN_OUT_OF_RANGE_ERROR,
-				     _("Column %d out of range (0-%d)"), col, g_slist_length (paramlist->holders) - 1);
+				     _("Column %d out of range (0-%d)"), col, g_slist_length (gda_set_get_holders (paramlist)) - 1);
 		else if (GDA_DATA_MODEL_GET_CLASS ((GdaDataModel *) iter->priv->data_model)->i_iter_set_value) {
 			if (! (GDA_DATA_MODEL_GET_CLASS ((GdaDataModel *) iter->priv->data_model)->i_iter_set_value) 
 			    ((GdaDataModel *) iter->priv->data_model, iter, col, nvalue, &error)) {
@@ -690,7 +691,7 @@ set_param_attributes (GdaHolder *holder, GdaValueAttribute flags)
 }
 
 /**
- * gda_data_model_iter_move_to_row_default: (skip)
+ * gda_data_model_iter_move_to_row_default:
  * @model: a #GdaDataModel
  * @iter: a #GdaDataModelIter iterating in @model
  * @row: the requested row
@@ -731,7 +732,7 @@ gda_data_model_iter_move_to_row_default (GdaDataModel *model, GdaDataModelIter *
 	/* actual sync. */
 	g_object_get (G_OBJECT (iter), "update-model", &update_model, NULL);
 	g_object_set (G_OBJECT (iter), "update-model", FALSE, NULL);
-	for (col = 0, list = ((GdaSet *) iter)->holders; list; col++, list = list->next) {
+	for (col = 0, list = gda_set_get_holders ((GdaSet *) iter); list; col++, list = list->next) {
 		const GValue *cvalue;
 		GError *lerror = NULL;
 		cvalue = gda_data_model_get_value_at (model, col, row, &lerror);
@@ -788,7 +789,7 @@ gda_data_model_iter_move_next (GdaDataModelIter *iter)
 }
 
 /**
- * gda_data_model_iter_move_next_default: (skip)
+ * gda_data_model_iter_move_next_default:
  * @model: a #GdaDataModel
  * @iter: a #GdaDataModelIter iterating in @model
  * 
@@ -829,7 +830,7 @@ gda_data_model_iter_move_next_default (GdaDataModel *model, GdaDataModelIter *it
 	/* actual sync. */
 	g_object_get (G_OBJECT (iter), "update-model", &update_model, NULL);
 	g_object_set (G_OBJECT (iter), "update-model", FALSE, NULL);
-	for (col = 0, list = ((GdaSet *) iter)->holders; list; col++, list = list->next) {
+	for (col = 0, list = gda_set_get_holders ((GdaSet *) iter); list; col++, list = list->next) {
 		const GValue *cvalue;
 		GError *lerror = NULL;
 		cvalue = gda_data_model_get_value_at (model, col, row, &lerror);
@@ -887,7 +888,7 @@ gda_data_model_iter_move_prev (GdaDataModelIter *iter)
 }
 
 /**
- * gda_data_model_iter_move_prev_default: (skip)
+ * gda_data_model_iter_move_prev_default:
  * @model: a #GdaDataModel
  * @iter: a #GdaDataModelIter iterating in @model
  * 
@@ -928,7 +929,7 @@ gda_data_model_iter_move_prev_default (GdaDataModel *model, GdaDataModelIter *it
 	/* actual sync. */
 	g_object_get (G_OBJECT (iter), "update-model", &update_model, NULL);
 	g_object_set (G_OBJECT (iter), "update-model", FALSE, NULL);
-	for (col = 0, list = ((GdaSet *) iter)->holders; list; col++, list = list->next) {
+	for (col = 0, list = gda_set_get_holders ((GdaSet *) iter); list; col++, list = list->next) {
 		const GValue *cvalue;
 		GError *lerror = NULL;
 		cvalue = gda_data_model_get_value_at (model, col, row, &lerror);
@@ -979,7 +980,7 @@ gda_data_model_iter_invalidate_contents (GdaDataModelIter *iter)
 	g_return_if_fail (iter->priv);
 
 	iter->priv->keep_param_changes = TRUE;
-	for (list = GDA_SET (iter)->holders; list; list = list->next)
+	for (list = gda_set_get_holders (GDA_SET (iter)); list; list = list->next)
 		gda_holder_force_invalid (GDA_HOLDER (list->data));
 	iter->priv->keep_param_changes = FALSE;
 }
@@ -1001,29 +1002,6 @@ gda_data_model_iter_is_valid (GdaDataModelIter *iter)
 	return iter->priv->row >= 0 ? TRUE : FALSE;
 }
 
-/**
- * gda_data_model_iter_get_column_for_param:
- * @iter: a #GdaDataModelIter object
- * @param: a #GdaHolder object, listed in @iter
- *
- * Get the column number in the #GdaDataModel for which @iter is an iterator as
- * represented by the @param parameter
- *
- * Returns: the column number, or @param is not valid
- *
- * Deprecated: 5.2: not very useful
- */
-gint
-gda_data_model_iter_get_column_for_param (GdaDataModelIter *iter, GdaHolder *param)
-{
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), -1);
-	g_return_val_if_fail (iter->priv, -1);
-	g_return_val_if_fail (GDA_IS_HOLDER (param), -1);
-	g_return_val_if_fail (g_slist_find (((GdaSet *) iter)->holders, param), -1);
-
-	return g_slist_index (((GdaSet *) iter)->holders, param);
-}
-
 /**
  * gda_data_model_iter_get_holder_for_field:
  * @iter: a #GdaDataModelIter object
@@ -1050,7 +1028,7 @@ gda_data_model_iter_get_holder_for_field (GdaDataModelIter *iter, gint col)
  *
  * Get the value stored at the column @col in @iter. The returned value must not be modified.
  *
- * Returns: (transfer none): the #GValue, or %NULL if the value could not be fetched
+ * Returns: (allow-none) (transfer none): the #GValue, or %NULL if the value could not be fetched
  */
 const GValue *
 gda_data_model_iter_get_value_at (GdaDataModelIter *iter, gint col)
@@ -1060,9 +1038,13 @@ gda_data_model_iter_get_value_at (GdaDataModelIter *iter, gint col)
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), NULL);
 	g_return_val_if_fail (iter->priv, NULL);
 
-	param = (GdaHolder *) g_slist_nth_data (((GdaSet *) iter)->holders, col);
-	if (param)
-		return gda_holder_get_value (param);
+	param = (GdaHolder *) g_slist_nth_data (gda_set_get_holders ((GdaSet *) iter), col);
+	if (param) {
+		if (gda_holder_is_valid (param))
+			return gda_holder_get_value (param);
+		else
+			return NULL;
+	}
 	else
 		return NULL;
 }
@@ -1075,7 +1057,7 @@ gda_data_model_iter_get_value_at (GdaDataModelIter *iter, gint col)
  *
  * Get the value stored at the column @col in @iter. The returned value must not be modified.
  *
- * Returns: (transfer none): the #GValue, or %NULL if the value could not be fetched
+ * Returns: (allow-none) (transfer none): the #GValue, or %NULL if the value could not be fetched
  *
  * Since: 4.2.10
  */
@@ -1087,11 +1069,12 @@ gda_data_model_iter_get_value_at_e (GdaDataModelIter *iter, gint col, GError **e
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), NULL);
 	g_return_val_if_fail (iter->priv, NULL);
 
-	param = (GdaHolder *) g_slist_nth_data (((GdaSet *) iter)->holders, col);
+	param = (GdaHolder *) g_slist_nth_data (gda_set_get_holders ((GdaSet *) iter), col);
 	if (param) {
-		if (error)
-			gda_holder_is_valid_e (param, error);
-		return gda_holder_get_value (param);
+		if (gda_holder_is_valid_e (param, error))
+			return gda_holder_get_value (param);
+		else
+			return NULL;
 	}
 	else
 		return NULL;
@@ -1121,7 +1104,7 @@ gda_data_model_iter_set_value_at (GdaDataModelIter *iter, gint col, const GValue
 	if (!holder) {
 		g_set_error (error, GDA_DATA_MODEL_ITER_ERROR, GDA_DATA_MODEL_ITER_COLUMN_OUT_OF_RANGE_ERROR,
 			     _("Column %d out of range (0-%d)"), col, 
-			     g_slist_length (((GdaSet *) iter)->holders) - 1);
+			     g_slist_length (gda_set_get_holders ((GdaSet *) iter)) - 1);
 		return FALSE;
 	}
 	return gda_holder_set_value (holder, value, error);
@@ -1134,7 +1117,7 @@ gda_data_model_iter_set_value_at (GdaDataModelIter *iter, gint col, const GValue
  *
  * Get the value stored at the column @field_name in @iter
  *
- * Returns: (transfer none): the #GValue, or %NULL
+ * Returns: (allow-none) (transfer none): the #GValue, or %NULL
  */
 const GValue *
 gda_data_model_iter_get_value_for_field (GdaDataModelIter *iter, const gchar *field_name)
@@ -1145,8 +1128,12 @@ gda_data_model_iter_get_value_for_field (GdaDataModelIter *iter, const gchar *fi
 	g_return_val_if_fail (iter->priv, NULL);
 
 	param = gda_set_get_holder ((GdaSet *) iter, field_name);
-	if (param)
-		return gda_holder_get_value (param);
+	if (param) {
+		if (gda_holder_is_valid (param))
+			return gda_holder_get_value (param);
+		else
+			return NULL;
+	}
 	else
 		return NULL;
 }
