@@ -11,13 +11,15 @@
  * Copyright (C) 2005 - 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2005 Dan Winship <danw@src.gnome.org>
  * Copyright (C) 2005 Stanislav Brabec <sbrabec@suse.de>
- * Copyright (C) 2005 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2005 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2006 - 2011 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2007 Leonardo Boshell <lb@kmc.com.co>
  * Copyright (C) 2008 Phil Longstaff <plongstaff@rogers.com>
  * Copyright (C) 2008 Przemysław Grzegorczyk <pgrzegorczyk@gmail.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
- * Copyright (C) 2011 Daniel Espinosa <despinosa@src.gnome.org>
+ * Copyright (C) 2011 - 2013 Daniel Espinosa <esodan@gmail.com>
+ * Copyright (C) 2013 Miguel Angel Cabrera Moya <madmac2501@gmail.com>
+ * Copyright (C) 2015 Corentin Noël <corentin@elementary.io>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -398,30 +400,11 @@ gda_data_model_thaw (GdaDataModel *model)
 		(GDA_DATA_MODEL_GET_CLASS (model)->i_set_notify) (model, TRUE);
 }
 
-/**
- * gda_data_model_set_notify: (virtual i_set_notify)
- * @model: a #GdaDataModel object.
- * @do_notify_changes: Set to TRUE if you require notifications.
- *
- * Enable or disable notifications changes on the given data model.
- * 
- * Deprecated: 5.2: use gda_data_model_freeze() and gda_data_model_thaw() instead
- *
- */
-void
-gda_data_model_set_notify (GdaDataModel *model, gboolean do_notify_changes)
-{
-	g_return_if_fail (GDA_IS_DATA_MODEL (model));
-	if (GDA_DATA_MODEL_GET_CLASS (model)->i_set_notify)
-		(GDA_DATA_MODEL_GET_CLASS (model)->i_set_notify) (model, do_notify_changes);
-}
-
 /**
  * gda_data_model_get_notify: (virtual i_get_notify)
  * @model: a #GdaDataModel object.
  *
  * Returns the status of notifications changes on the given data model.
- * 
  */
 gboolean
 gda_data_model_get_notify (GdaDataModel *model)
@@ -441,8 +424,6 @@ gda_data_model_get_notify (GdaDataModel *model)
  * Get the attributes of @model such as how to access the data it contains if it's modifiable, etc.
  *
  * Returns: (transfer none): an ORed value of #GdaDataModelAccessFlags flags
- *
- *
  */
 GdaDataModelAccessFlags
 gda_data_model_get_access_flags (GdaDataModel *model)
@@ -463,7 +444,6 @@ gda_data_model_get_access_flags (GdaDataModel *model)
  * @model: a #GdaDataModel object.
  *
  * Returns: the number of rows in the given data model, or -1 if the number of rows is not known
- *
  */
 gint
 gda_data_model_get_n_rows (GdaDataModel *model)
@@ -481,7 +461,6 @@ gda_data_model_get_n_rows (GdaDataModel *model)
  * @model: a #GdaDataModel object.
  *
  * Returns: the number of columns in the given data model, or -1 if unknown.
- *
  */
 gint
 gda_data_model_get_n_columns (GdaDataModel *model)
@@ -509,8 +488,7 @@ gda_data_model_get_n_columns (GdaDataModel *model)
  * WARNING: the returned #GdaColumn object belongs to the @model model and
  * and should not be destroyed; any modification will affect the whole data model.
  *
- * Returns: (transfer none) (nullable): the description of the column.
- *
+ * Returns: (transfer none) (allow-none): the description of the column.
  */
 GdaColumn *
 gda_data_model_describe_column (GdaDataModel *model, gint col)
@@ -681,10 +659,8 @@ gda_data_model_set_column_title (GdaDataModel *model, gint col, const gchar *tit
  * Upon errors %NULL will be returned and @error will be assigned a
  * #GError from the #GDA_DATA_MODEL_ERROR domain.
  *
- * Returns: (nullable) (transfer none): a #GValue containing the value stored in the given
+ * Returns: (allow-none) (transfer none): a #GValue containing the value stored in the given
  * position, or %NULL on error (out-of-bound position, etc).
- *
- *
  */
 const GValue *
 gda_data_model_get_value_at (GdaDataModel *model, gint col, gint row, GError **error)
@@ -722,7 +698,7 @@ gda_data_model_get_value_at (GdaDataModel *model, gint col, gint row, GError **e
  * Upon errors %NULL will be returned and @error will be assigned a
  * #GError from the #GDA_DATA_MODEL_ERROR domain.
  *
- * Returns: (nullable) (transfer none): a #GValue containing the value stored in the given
+ * Returns: (allow-none) (transfer none): a #GValue containing the value stored in the given
  * position, or %NULL on error (out-of-bound position, wrong data type, etc).
  */
 const GValue *
@@ -769,7 +745,6 @@ gda_data_model_get_typed_value_at (GdaDataModel *model, gint col, gint row, GTyp
  * if a row was added to @model.
  *
  * Returns: (transfer none): the attributes as an ORed value of #GdaValueAttribute
- *
  */
 GdaValueAttribute
 gda_data_model_get_attributes_at (GdaDataModel *model, gint col, gint row)
@@ -802,7 +777,6 @@ gda_data_model_get_attributes_at (GdaDataModel *model, gint col, gint row)
  * #GError from the #GDA_DATA_MODEL_ERROR domain.
  *
  * Returns: TRUE if the value in the data model has been updated and no error occurred
- *
  */
 gboolean
 gda_data_model_set_value_at (GdaDataModel *model, gint col, gint row, const GValue *value, GError **error)
@@ -823,7 +797,7 @@ gda_data_model_set_value_at (GdaDataModel *model, gint col, gint row, const GVal
  * gda_data_model_set_values: (virtual i_set_values)
  * @model: a #GdaDataModel object.
  * @row: row number.
- * @values: (element-type GObject.Value) (transfer none) (nullable): a list of #GValue (or %NULL), one for at most the number of columns of @model
+ * @values: (element-type GObject.Value) (transfer none) (allow-none): a list of #GValue (or %NULL), one for at most the number of columns of @model
  * @error: a place to store errors, or %NULL
  *
  * In a similar way to gda_data_model_set_value_at(), this method modifies a data model's contents
@@ -836,7 +810,6 @@ gda_data_model_set_value_at (GdaDataModel *model, gint col, gint row, const GVal
  * #GError from the #GDA_DATA_MODEL_ERROR domain.
  *
  * Returns: %TRUE if the value in the data model has been updated and no error occurred
- *
  */
 gboolean
 gda_data_model_set_values (GdaDataModel *model, gint row, GList *values, GError **error)
@@ -905,7 +878,6 @@ gda_data_model_set_values (GdaDataModel *model, gint row, GList *values, GError
  * <programlisting><![CDATA[iter = g_object_new (GDA_TYPE_DATA_MODEL_ITER, "data-model", proxy, NULL);]]></programlisting>
  *
  * Returns: (transfer full): a #GdaDataModelIter object, or %NULL if an error occurred
- *
  */
 GdaDataModelIter *
 gda_data_model_create_iter (GdaDataModel *model)
@@ -919,93 +891,10 @@ gda_data_model_create_iter (GdaDataModel *model)
 				      "data-model", model, NULL);
 }
 
-/**
- * gda_data_model_iter_at_row: (virtual i_iter_at_row)
- * @model: a #GdaDataModel object.
- * @iter: a #GdaDataModelIter object.
- * @row: a row to point to with @iter
- *
- * Moves @iter to the row number given by @row.
- * 
- * Deprecated: 5.2: use gda_data_model_iter_move_to_row() instead
- *
- */
-gboolean
-gda_data_model_iter_at_row (GdaDataModel *model, GdaDataModelIter *iter, gint row)
-{
-	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (model), FALSE);
-
-	return gda_data_model_iter_move_to_row (iter, row);
-}
-
-/**
- * gda_data_model_iter_next: ( virtual i_iter_next)
- * @model: a #GdaDataModel object.
- * @iter: a #GdaDataModelIter object.
- *
- * Moves @iter to the next row in @model.
- * 
- * Deprecated: 5.2: use gda_data_model_iter_move_next() instead
- *
- */
-gboolean
-gda_data_model_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
-{
-	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (model), FALSE);
-
-	return gda_data_model_iter_move_next (iter);
-}
-
-/**
- * gda_data_model_iter_prev: (virtual i_iter_prev)
- * @model: a #GdaDataModel object.
- * @iter: a #GdaDataModelIter object.
- *
- * Moves @iter to the next row in @model.
- *
- * Deprecated: 5.2: use gda_data_model_iter_move_prev() instead
- * 
- */
-gboolean 
-gda_data_model_iter_prev (GdaDataModel *model, GdaDataModelIter *iter)
-{
-	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (model), FALSE);
-
-	return gda_data_model_iter_move_prev (iter);
-}
-
-
-/**
- * gda_data_model_iter_set_value: (virtual i_iter_set_value)
- * @model: a #GdaDataModel object.
- * @iter: a #GdaDataModelIter object.
- * @col: the number of column to set value to
- * @value: the to use to set on
- * @error: a place to set errors
- *
- * Set @value to the given @column and row pointed by @iter in the given @model.
- * 
- * Deprecated: 5.2: use gda_data_model_iter_set_value_at() instead
- *
- */
-gboolean 
-gda_data_model_iter_set_value (GdaDataModel *model, GdaDataModelIter *iter, gint col,
-			       const GValue *value, GError **error)
-{
-	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (model), FALSE);
-	
-	return gda_data_model_iter_set_value_at (iter, col, value, error);
-}
-
-
 /**
  * gda_data_model_append_values: (virtual i_append_values)
  * @model: a #GdaDataModel object.
- * @values: (element-type GObject.Value) (nullable): #GList of #GValue* representing the row to add.  The
+ * @values: (element-type GObject.Value) (allow-none): #GList of #GValue* representing the row to add.  The
  *          length must match model's column count.  These #GValue
  *	    are value-copied (the user is still responsible for freeing them).
  * @error: a place to store errors, or %NULL
@@ -1017,7 +906,6 @@ gda_data_model_iter_set_value (GdaDataModel *model, GdaDataModelIter *iter, gint
  * #GError from the #GDA_DATA_MODEL_ERROR domain.
  *
  * Returns: the number of the added row, or -1 if an error occurred
- *
  */
 gint
 gda_data_model_append_values (GdaDataModel *model, const GList *values, GError **error)
@@ -1046,7 +934,6 @@ gda_data_model_append_values (GdaDataModel *model, const GList *values, GError *
  * #GError from the #GDA_DATA_MODEL_ERROR domain.
  *
  * Returns: the number of the added row, or -1 if an error occurred
- *
  */
 gint
 gda_data_model_append_row (GdaDataModel *model, GError **error)
@@ -1082,7 +969,6 @@ gda_data_model_append_row (GdaDataModel *model, GError **error)
  * #GError from the #GDA_DATA_MODEL_ERROR domain.
  *
  * Returns: %TRUE if successful, %FALSE otherwise.
- *
  */
 gboolean
 gda_data_model_remove_row (GdaDataModel *model, gint row, GError **error)
@@ -1117,7 +1003,6 @@ gda_data_model_remove_row (GdaDataModel *model, gint row, GError **error)
  * NOTE: the @cols_index array MUST contain a column index for each value in @values
  *
  * Returns: the requested row number, of -1 if not found
- *
  */
 gint
 gda_data_model_get_row_from_values (GdaDataModel *model, GSList *values, gint *cols_index)
@@ -1182,11 +1067,10 @@ gda_data_model_get_row_from_values (GdaDataModel *model, GSList *values, gint *c
  * gda_data_model_send_hint: (virtual i_send_hint)
  * @model: a #GdaDataModel
  * @hint: (transfer none): a hint to send to the model
- * @hint_value: (nullable): an optional value to specify the hint, or %NULL
+ * @hint_value: (allow-none): an optional value to specify the hint, or %NULL
  *
  * Sends a hint to the data model. The hint may or may not be handled by the data
  * model, depending on its implementation
- *
  */
 void
 gda_data_model_send_hint (GdaDataModel *model, GdaDataModelHint hint, const GValue *hint_value)
@@ -1209,7 +1093,6 @@ gda_data_model_send_hint (GdaDataModel *model, GdaDataModelHint hint, const GVal
  * Returns: (transfer none) (element-type GLib.Error) (array zero-terminated=1): a pointer to a %NULL terminated array of #GError, or %NULL.
  *
  * Since: 4.2.6
- *
  */
 GError **
 gda_data_model_get_exceptions (GdaDataModel *model)
@@ -1232,9 +1115,9 @@ static gchar *export_to_text_separated (GdaDataModel *model, const gint *cols, g
  * gda_data_model_export_to_string:
  * @model: a #GdaDataModel
  * @format: the format in which to export data
- * @cols: (array length=nb_cols) (nullable): an array containing which columns of @model will be exported, or %NULL for all columns
+ * @cols: (array length=nb_cols) (allow-none): an array containing which columns of @model will be exported, or %NULL for all columns
  * @nb_cols: the number of columns in @cols
- * @rows: (array length=nb_rows) (nullable): an array containing which rows of @model will be exported, or %NULL for all rows
+ * @rows: (array length=nb_rows) (allow-none): an array containing which rows of @model will be exported, or %NULL for all rows
  * @nb_rows: the number of rows in @rows
  * @options: list of options for the export
  *
@@ -1545,9 +1428,9 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
  * @model: a #GdaDataModel
  * @format: the format in which to export data
  * @file: the filename to export to
- * @cols: (array length=nb_cols) (nullable): an array containing which columns of @model will be exported, or %NULL for all columns
+ * @cols: (array length=nb_cols) (allow-none): an array containing which columns of @model will be exported, or %NULL for all columns
  * @nb_cols: the number of columns in @cols
- * @rows: (array length=nb_rows) (nullable): an array containing which rows of @model will be exported, or %NULL for all rows
+ * @rows: (array length=nb_rows) (allow-none): an array containing which rows of @model will be exported, or %NULL for all rows
  * @nb_rows: the number of rows in @rows
  * @options: list of options for the export
  * @error: a place to store errors, or %NULL
@@ -1679,19 +1562,19 @@ export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols,
 
 			value = (GValue*) gda_data_model_iter_get_value_at (iter, cols[c]);
 			if (value && invalid_as_null) {
-				if ((G_VALUE_TYPE (value) == G_TYPE_DATE)) {
+				if (g_type_is_a (G_VALUE_TYPE (value), G_TYPE_DATE)) {
 					GDate *date = (GDate*) g_value_get_boxed (value);
 					if (!g_date_valid (date))
 						value = NULL;
 				}
-				else if ((G_VALUE_TYPE (value) == GDA_TYPE_TIME)) {
+				else if (g_type_is_a (G_VALUE_TYPE (value), GDA_TYPE_TIME)) {
 					const GdaTime *tim = gda_value_get_time (value);
 					if (! gda_time_valid (tim))
 						value = NULL;					
 				}
-				else if ((G_VALUE_TYPE (value) == GDA_TYPE_TIMESTAMP)) {
-					const GdaTimestamp *ts = gda_value_get_timestamp (value);
-					if (! gda_timestamp_valid (ts))
+				else if (g_type_is_a (G_VALUE_TYPE (value), G_TYPE_DATE_TIME)) {
+					GDateTime *ts = g_value_get_boxed (value);
+					if (ts == NULL)
 						value = NULL;
 				}
 			}
@@ -1763,11 +1646,11 @@ xml_set_boolean (xmlNodePtr node, const gchar *name, gboolean value)
 /*
  * gda_data_model_to_xml_node
  * @model: a #GdaDataModel object.
- * @cols: (nullable) (array length=nb_cols): an array containing which columns of @model will be exported, or %NULL for all columns
+ * @cols: (allow-none) (array length=nb_cols): an array containing which columns of @model will be exported, or %NULL for all columns
  * @nb_cols: the number of columns in @cols
- * @rows: (nullable) (array length=nb_rows): an array containing which rows of @model will be exported, or %NULL for all rows
+ * @rows: (allow-none) (array length=nb_rows): an array containing which rows of @model will be exported, or %NULL for all rows
  * @nb_rows: the number of rows in @rows
- * @name: (nullable): name to use for the XML resulting table or %NULL.
+ * @name: (allow-none): name to use for the XML resulting table or %NULL.
  *
  * Converts a #GdaDataModel into a xmlNodePtr (as used in libxml).
  *
@@ -1918,9 +1801,6 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row, GError **error)
 
 		if (strcmp ((gchar*)xml_field->name, "gda_value") && 
 		    strcmp ((gchar*)xml_field->name, "gda_array_value")) {
-			g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_XML_FORMAT_ERROR,
-				     _("Expected tag <gda_value> or <gda_array_value>, "
-				       "got <%s>, ignoring"), (gchar*)xml_field->name);
 			continue;
 		}
 		
@@ -2054,7 +1934,7 @@ gda_data_model_add_data_from_xml_node (GdaDataModel *model, xmlNodePtr node, GEr
  * @to: the destination #GdaDataModel
  * @from: the source #GdaDataModel
  * @overwrite: TRUE if @to is completely overwritten by @from's data, and FALSE if @from's data is appended to @to
- * @cols_trans: (element-type gint gint) (nullable): a #GHashTable for columns translating, or %NULL
+ * @cols_trans: (element-type gint gint) (allow-none): a #GHashTable for columns translating, or %NULL
  * @error: a place to store errors, or %NULL
  *
  * Copy the contents of the @from data model to the @to data model. The copy stops as soon as an error
@@ -2177,7 +2057,7 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from,
 	 * - a GValue of type GDA_VALYE_TYPE_NULL if a null value must be inserted in the dest data model
 	 * - a GValue of a different type if the value must be converted from the src data model
 	 */
-	append_types = g_new0 (GType, to_nb_cols);
+	append_types = g_new0 (GType, (guint)to_nb_cols);
 	for (plist = copy_params, i = to_nb_cols - 1; plist; plist = plist->next, i--) {
 		GdaColumn *column;
 
@@ -2318,7 +2198,7 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from,
  * gda_data_model_import_from_string:
  * @model: a #GdaDataModel
  * @string: the string to import data from
- * @cols_trans: (element-type gint gint) (nullable): a hash table containing which columns of @model will be imported, or %NULL for all columns, see gda_data_model_import_from_model()
+ * @cols_trans: (element-type gint gint) (allow-none): a hash table containing which columns of @model will be imported, or %NULL for all columns, see gda_data_model_import_from_model()
  * @options: list of options for the export
  * @error: a place to store errors, or %NULL
  *
@@ -2353,7 +2233,7 @@ gda_data_model_import_from_string (GdaDataModel *model,
  * gda_data_model_import_from_file:
  * @model: a #GdaDataModel
  * @file: the filename to import from
- * @cols_trans: (element-type gint gint) (nullable): a #GHashTable for columns translating, or %NULL, see gda_data_model_import_from_model()
+ * @cols_trans: (element-type gint gint) (allow-none): a #GHashTable for columns translating, or %NULL, see gda_data_model_import_from_model()
  * @options: list of options for the export
  * @error: a place to store errors, or %NULL
  *
