@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2011 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2011 - 2014 Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -80,8 +80,6 @@ struct _GdaDataPivotPrivate {
 	/* computed data */
 	GArray        *columns; /* Array of GdaColumn objects, for ALL columns! */
 	GdaDataModel  *results;
-	GRecMutex provider_mutex;
-	GdaVirtualProvider *virtual_provider;
 };
 
 /* properties */
@@ -124,6 +122,8 @@ static GdaDataModelAccessFlags gda_data_pivot_get_access_flags(GdaDataModel *mod
 static const GValue        *gda_data_pivot_get_value_at    (GdaDataModel *model, gint col, gint row, GError **error);
 
 static GObjectClass *parent_class = NULL;
+static GMutex provider_mutex;
+static GdaVirtualProvider *virtual_provider = NULL;
 
 /**
  * gda_data_pivot_get_type:
@@ -220,8 +220,6 @@ gda_data_pivot_init (GdaDataPivot *model, G_GNUC_UNUSED GdaDataPivotClass *klass
 	g_return_if_fail (GDA_IS_DATA_PIVOT (model));
 	model->priv = g_new0 (GdaDataPivotPrivate, 1);
 	model->priv->model = NULL;
-	model->priv->virtual_provider = NULL;
-	g_rec_mutex_init (& model->priv->provider_mutex);
 }
 
 static void
@@ -294,20 +292,12 @@ gda_data_pivot_dispose (GObject *object)
 		}
 
 		if (model->priv->vcnc) {
-			if (G_IS_OBJECT (model->priv->vcnc))
-				g_object_unref (model->priv->vcnc);
+			g_object_unref (model->priv->vcnc);
 			model->priv->vcnc = NULL;
 		}
 
 		if (model->priv->model) {
-			if (G_IS_OBJECT (model->priv->model))
-				g_object_unref (model->priv->model);
-			model->priv->model = NULL;
-		}
-
-		if (model->priv->virtual_provider) {
-			if (G_IS_OBJECT (model->priv->virtual_provider))
-				g_object_unref (model->priv->virtual_provider);
+			g_object_unref (model->priv->model);
 			model->priv->model = NULL;
 		}
 	}
@@ -323,8 +313,6 @@ gda_data_pivot_finalize (GObject *object)
 
 	g_return_if_fail (GDA_IS_DATA_PIVOT (model));
 
-	g_rec_mutex_clear (& model->priv->provider_mutex);
-
 	/* free memory */
 	if (model->priv) {
 		g_free (model->priv);
@@ -356,14 +344,14 @@ gda_data_pivot_set_property (GObject *object,
 	if (model->priv) {
 		switch (param_id) {
 		case PROP_MODEL: {
-			GdaDataModel *mod = g_value_dup_object (value);
+			GdaDataModel *mod = g_value_get_object (value);
 
 			clean_previous_population (model);
 
 			if (mod) {
 				g_return_if_fail (GDA_IS_DATA_MODEL (mod));
-
-				if (model->priv->model) {
+  
+                                if (model->priv->model) {
 					if (model->priv->vcnc)
 						gda_vconnection_data_model_remove (GDA_VCONNECTION_DATA_MODEL (model->priv->vcnc),
 										   TABLE_NAME, NULL);
@@ -371,6 +359,7 @@ gda_data_pivot_set_property (GObject *object,
 				}
 
 				model->priv->model = mod;
+				g_object_ref (mod);
 			}
 			break;
 		}
@@ -1672,7 +1661,7 @@ gda_data_pivot_populate (GdaDataPivot *pivot, GError **error)
 		GdaColumn *column;
 		GdaHolder *holder;
 		column = gda_column_new ();
-		holder = GDA_HOLDER (g_slist_nth_data (GDA_SET (iter)->holders, col));
+		holder = GDA_HOLDER (g_slist_nth_data (gda_set_get_holders (GDA_SET (iter)), col));
 		
 		gda_column_set_name (column, gda_holder_get_id (holder));
 		gda_column_set_description (column, gda_holder_get_id (holder));
@@ -2042,21 +2031,23 @@ _gda_value_hash (gconstpointer key)
 			res += g_str_hash (tmp);
 	}
 	else if ((vt == GDA_TYPE_BINARY) || (vt == GDA_TYPE_BLOB)) {
-		const GdaBinary *bin;
+		GdaBinary *bin;
 		if (vt == GDA_TYPE_BLOB) {
 			GdaBlob *blob;
 			blob = (GdaBlob*) gda_value_get_blob ((GValue *) v);
-			bin = (GdaBinary *) blob;
-			if (blob->op &&
-			    (bin->binary_length != gda_blob_op_get_length (blob->op)))
-				gda_blob_op_read_all (blob->op, blob);
+			bin = gda_blob_get_binary (blob);
+			if (gda_blob_get_op (blob) &&
+			    (gda_binary_get_size (bin) != gda_blob_op_get_length (gda_blob_get_op (blob))))
+				gda_blob_op_read_all (gda_blob_get_op (blob), blob);
 		}
 		else
 			bin = gda_value_get_binary ((GValue *) v);
 		if (bin) {
 			glong l;
-			for (l = 0; l < bin->binary_length; l++)
-				res += (guint) bin->data [l];
+			for (l = 0; l < gda_binary_get_size (bin); l++) {
+			  guchar* p = gda_binary_get_data (bin);
+				res += (guint) p[l];
+			}
 		}
 	}
 	else {
@@ -2111,14 +2102,15 @@ create_vcnc (GdaDataPivot *pivot, GError **error)
 {
 	GdaConnection *vcnc;
 	GError *lerror = NULL;
-	if (!pivot->priv->vcnc && GDA_IS_CONNECTION (pivot->priv->vcnc))
+	if (pivot->priv->vcnc)
 		return TRUE;
 
-	g_rec_mutex_lock (&pivot->priv->provider_mutex);
-	if (!pivot->priv->virtual_provider)
-		pivot->priv->virtual_provider = gda_vprovider_data_model_new ();
+	g_mutex_lock (&provider_mutex);
+	if (!virtual_provider)
+		virtual_provider = gda_vprovider_data_model_new ();
+	g_mutex_unlock (&provider_mutex);
 
-	vcnc = gda_virtual_connection_open (pivot->priv->virtual_provider, &lerror);
+	vcnc = gda_virtual_connection_open (virtual_provider, GDA_CONNECTION_OPTIONS_NONE, &lerror);
 	if (! vcnc) {
 		g_print ("Virtual ERROR: %s\n", lerror && lerror->message ? lerror->message : "No detail");
 		if (lerror)
@@ -2129,7 +2121,6 @@ create_vcnc (GdaDataPivot *pivot, GError **error)
 	}
 
 	pivot->priv->vcnc = vcnc;
-	g_rec_mutex_unlock (&pivot->priv->provider_mutex);
 	return TRUE;
 }
 
@@ -2153,12 +2144,14 @@ bind_source_model (GdaDataPivot *pivot, GError **error)
 		return TRUE;
 	}
 
+	GError *lerror = NULL;
 	if (!gda_vconnection_data_model_add_model (GDA_VCONNECTION_DATA_MODEL (pivot->priv->vcnc),
 						   pivot->priv->model,
-						   TABLE_NAME, NULL)) {
+						   TABLE_NAME, &lerror)) {
 		g_set_error (error, GDA_DATA_PIVOT_ERROR, GDA_DATA_PIVOT_SOURCE_MODEL_ERROR,
 			     "%s",
 			     _("Invalid source data model (may have incompatible column names)"));
+		g_clear_error (&lerror);
 		return FALSE;
 	}
 
