@@ -1,23 +1,20 @@
 /*
- * Copyright (C) 2008 - 2011 The GNOME Foundation.
+ * Copyright (C) 2008 - 2012 Vivien Malerba <malerba@gnome-db.org>
  *
- * AUTHORS:
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #ifndef __GDA_DATA_SELECT_H__
@@ -53,6 +50,19 @@ typedef enum {
 	GDA_DATA_SELECT_SAFETY_LOCKED_ERROR
 } GdaDataSelectError;
 
+/**
+ * GdaDataSelectConditionType:
+ * @GDA_DATA_SELECT_COND_PK: only primary key fields are used
+ * @GDA_DATA_SELECT_COND_ALL_COLUMNS: all the columns of the tables are used
+ *
+ * Defines what criteria gda_data_select_compute_modification_statements_ext() uses
+ * to uniquely identify a single row in a table when creating modification statements.
+ */
+typedef enum {
+	GDA_DATA_SELECT_COND_PK,
+	GDA_DATA_SELECT_COND_ALL_COLUMNS
+} GdaDataSelectConditionType;
+
 struct _GdaDataSelect {
 	GObject           object;
 	GdaDataSelectPrivate *priv;
@@ -60,7 +70,7 @@ struct _GdaDataSelect {
 	/* read only information */
 	GdaPStmt         *prep_stmt; /* use the "prepared-stmt" property to set this */
 	gint              nb_stored_rows; /* number of GdaRow objects currently stored */
-	gint              advertized_nrows; /* set when the number of rows becomes known, -1 untill then */
+	gint              advertized_nrows; /* set when the number of rows becomes known, -1 until then */
 
 	/*< private >*/
 	/* Padding for future expansion */
@@ -115,13 +125,26 @@ struct _GdaDataSelectClass {
  *  As the <link linkend="GdaDataModel">GdaDataModel</link> interface is implemented, consult the API
  *  to access and modify the data held in a <link linkend="GdaDataSelect">GdaDataSelect</link> object.
  *
- *  The default behaviour however is to disallow modifications, and this section documents how to characterize
+ * Depending on the requested data model usage (as specified by the "model_usage" parameter of the
+ * <link linkend="gda-connection-statement-execute">gda_connection_statement_execute()</link> and similar
+ * methods, the #GdaDataSelect will allow random access, cursor based access or both.
+ *
+ * Also, when later you'll be reading the data contained in a #GdaDataSelect object, depending on the actual
+ * implementation (which adapts to the API providede by the database server), some calls to the database server
+ * may be necessary to actually obtain the data. If this behaviour is not the one intended and if you need to
+ * access the data without having to contact the database server (for example for performances reasons), then
+ * you can use the <link linkend="gda_data_select_prepare_for_offline">gda_data_select_prepare_for_offline()</link>
+ * method or specify the <link linkend="GDA-STATEMENT-MODEL-OFFLINE:CAPS">GDA_STATEMENT_MODEL_OFFLINE</link>
+ * flag when executing the SELECT statement.
+ *
+ *  The default behaviour however is to disallow modifications, and this section documents how to parametrize
  *  a <link linkend="GdaDataSelect">GdaDataSelect</link> to allow modifications. Once this is done, any modification
  *  done to the data model will be propagated to the modified table in the database using INSERT, UPDATE or DELETE
  *  statements.
  *
  *  After any modification, it is still possible to read values from the data model (even values for rows which have
  *  been modified or inserted). The data model might then execute some SELECT statement to fetch some actualized values.
+ *
  *  Note: there is a corner case where a modification made to a row would make the row not selected at first in the data model
  *  (for example is the original SELECT statement included a clause <![CDATA["WHERE id < 100"]]> and the modification sets the 
  *  <![CDATA["id"]]> value to 110), then the row will still be in the data model even though it would not be if the SELECT statement
@@ -145,11 +168,15 @@ gboolean       gda_data_select_compute_row_selection_condition (GdaDataSelect *m
 gboolean       gda_data_select_set_modification_statement      (GdaDataSelect *model, GdaStatement *mod_stmt, GError **error);
 gboolean       gda_data_select_set_modification_statement_sql  (GdaDataSelect *model, const gchar *sql, GError **error);
 gboolean       gda_data_select_compute_modification_statements (GdaDataSelect *model, GError **error);
+gboolean       gda_data_select_compute_modification_statements_ext (GdaDataSelect *model,
+								    GdaDataSelectConditionType cond_type,
+								    GError **error);
 
 gboolean       gda_data_select_compute_columns_attributes      (GdaDataSelect *model, GError **error);
 GdaConnection *gda_data_select_get_connection                  (GdaDataSelect *model);
 
 gboolean       gda_data_select_rerun                           (GdaDataSelect *model, GError **error);
+gboolean       gda_data_select_prepare_for_offline             (GdaDataSelect *model, GError **error);
 
 G_END_DECLS
 
