@@ -1,11 +1,12 @@
 /*
  * Copyright (C) 2008 Massimo Cora <maxcvs@email.it>
  * Copyright (C) 2008 - 2011 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2008 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2008 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
  * Copyright (C) 2011 Daniel Espinosa <despinosa@src.gnome.org>
+ * Copyright (C) 2015 Corentin Noël <corentin@elementary.io>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -121,7 +122,7 @@ struct _GdaHolderPrivate
 	GdaDataModel    *source_model;
 	gint             source_col;
 
-	GdaMutex        *mutex;
+	GRecMutex        mutex;
 
 	gboolean         validate_changes;
 };
@@ -365,14 +366,14 @@ gda_holder_init (GdaHolder *holder)
 	holder->priv->source_model = NULL;
 	holder->priv->source_col = 0;
 
-	holder->priv->mutex = gda_mutex_new ();
+	g_rec_mutex_init (& (holder->priv->mutex));
 
 	holder->priv->validate_changes = TRUE;
 }
 
 /**
  * gda_holder_new:
- * @type: (transfer none): the #GType requested
+ * @type: the #GType requested
  *
  * Creates a new holder of type @type
  *
@@ -603,8 +604,7 @@ gda_holder_finalize (GObject   * object)
 	holder = GDA_HOLDER (object);
 	if (holder->priv) {
 		g_free (holder->priv->id);
-
-		gda_mutex_free (holder->priv->mutex);
+		g_rec_mutex_clear (& (holder->priv->mutex));
 
 		g_free (holder->priv);
 		holder->priv = NULL;
@@ -751,7 +751,7 @@ gda_holder_get_property (GObject *object,
  * 
  * Get @holder's type
  *
- * Returns: (transfer none): the data type
+ * Returns: the data type
  */
 GType
 gda_holder_get_g_type (GdaHolder *holder)
@@ -966,6 +966,9 @@ gda_holder_set_value_str (GdaHolder *holder, GdaDataHandler *dh, const gchar *va
  * stored internally will be forgiven and replaced by the @value. User should then
  * take care of the 'old' static GValue.
  *
+ * Note4: in any case, the caller should not use @value anymore after this function returns because it may
+ * have been freed. If necessary, use gda_holder_get_value() to get the real value.
+ *
  * Returns: TRUE if value has been set
  */
 gboolean
@@ -1125,10 +1128,10 @@ real_gda_holder_set_value (GdaHolder *holder, GValue *value, gboolean do_copy, G
 				gda_value_free (value);
 		}
 
+		gda_holder_unlock ((GdaLockable*) holder);
 		g_signal_emit (holder, gda_holder_signals[CHANGED], 0);
 	}
 
-	gda_holder_unlock ((GdaLockable*) holder);
 	return newvalid;
 }
 
@@ -1375,11 +1378,14 @@ gda_holder_force_invalid_e (GdaHolder *holder, GError *error)
 	}
 
 	/* if we are an alias, then we forward the value setting to the master */
-	if (holder->priv->full_bind) 
+	if (holder->priv->full_bind) {
 		gda_holder_force_invalid (holder->priv->full_bind);
-	else 
+		gda_holder_unlock ((GdaLockable*) holder);
+	}
+	else {
+		gda_holder_unlock ((GdaLockable*) holder);
 		g_signal_emit (holder, gda_holder_signals[CHANGED], 0);
-	gda_holder_unlock ((GdaLockable*) holder);
+	}
 }
 
 /**
@@ -2024,7 +2030,7 @@ gda_holder_get_attribute (GdaHolder *holder, const gchar *attribute)
  * then the string will not be freed at all).
  *
  * Attributes can have any name, but Libgda proposes some default names, 
- * see <link linkend="libgda-5.0-Attributes-manager.synopsis">this section</link>.
+ * see <link linkend="libgda-6.0-Attributes-manager.synopsis">this section</link>.
  *
  * For example one would use it as:
  *
@@ -2057,19 +2063,19 @@ static void
 gda_holder_lock (GdaLockable *lockable)
 {
 	GdaHolder *holder = (GdaHolder *) lockable;
-	gda_mutex_lock (holder->priv->mutex);
+	g_rec_mutex_lock (& (holder->priv->mutex));
 }
 
 static gboolean
 gda_holder_trylock (GdaLockable *lockable)
 {
 	GdaHolder *holder = (GdaHolder *) lockable;
-	return gda_mutex_trylock (holder->priv->mutex);
+	return g_rec_mutex_trylock (& (holder->priv->mutex));
 }
 
 static void
 gda_holder_unlock (GdaLockable *lockable)
 {
 	GdaHolder *holder = (GdaHolder *) lockable;
-	gda_mutex_unlock (holder->priv->mutex);
+	g_rec_mutex_unlock (& (holder->priv->mutex));
 }
