@@ -1,6 +1,8 @@
 /*
  * Copyright (C) 2008 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2008 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2008 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2012 - 2013 Daniel Espinosa <esodan@gmail.com>
+ * Copyright (C) 2013 Carl-Anton Ingmarsson <ca.ingmarsson@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -62,18 +64,23 @@ typedef enum {
 } GdaMetaStoreChangeType;
 
 /**
- * GdaMetaStoreChange:
- * @c_type:
- * @table_name:
- * @keys: (element-type utf8 GLib.Value):
+ * GdaMetaStoreChange: (ref-func gda_meta_store_change_new) (unref-func gda_meta_store_change_free) (get-value-func gda_value_get_meta_store_change) (set-value-func gda_value_set_meta_store_change)
  */
-typedef struct {
-	/* change general information */
-	GdaMetaStoreChangeType  c_type;
-	gchar                  *table_name;
-	GHashTable             *keys; /* key = ('+' or '-') and a column position in @table (string) starting at 0, 
-	                               * value = a GValue pointer */
-} GdaMetaStoreChange;
+typedef struct _GdaMetaStoreChange GdaMetaStoreChange;
+
+#define GDA_TYPE_META_STORE_CHANGE (gda_meta_store_change_get_type ())
+
+GType                   gda_meta_store_change_get_type (void) G_GNUC_CONST;
+GdaMetaStoreChange*     gda_meta_store_change_new (void);
+void                    gda_meta_store_change_set_change_type (GdaMetaStoreChange *change, GdaMetaStoreChangeType ctype);
+GdaMetaStoreChangeType  gda_meta_store_change_get_change_type (GdaMetaStoreChange *change);
+void                    gda_meta_store_change_set_table_name (GdaMetaStoreChange *change, const gchar *table_name);
+gchar*                  gda_meta_store_change_get_table_name (GdaMetaStoreChange *change);
+GHashTable*              gda_meta_store_change_get_keys (GdaMetaStoreChange *change);
+GdaMetaStoreChange*     gda_meta_store_change_copy (GdaMetaStoreChange *src);
+void                    gda_meta_store_change_free (GdaMetaStoreChange *change);
+void                    gda_value_set_meta_store_change (GValue *value, GdaMetaStoreChange *change);
+GdaMetaStoreChange*     gda_value_get_meta_store_change (GValue *value);
 
 
 /* Pointer type for GdaMetaContext */
@@ -83,9 +90,9 @@ typedef struct {
  * GdaMetaContext:
  * @table_name: the name of the table <emphasis>in the GdaMetaStore's internal database</emphasis>
  * @size: the size of the @column_names and @column_values arrays
- * @column_names: (array length=size): an array of column names (columns of the @table_name table)
- * @column_values: (array length=size): an array of values, one for each column named in @column_names
- * @columns: (element-type utf8 GLib.Value): A #GHashTable storing columns' name as key and #GValue as column's
+ * @column_names: (array length=size) (transfer container): an array of column names (columns of the @table_name table)
+ * @column_values: (array length=size)  (transfer container): an array of values, one for each column named in @column_names
+ * @columns: (element-type utf8 GObject.Value): A #GHashTable storing columns' name as key and #GValue as column's
  * value.
  *
  * The <structname>GdaMetaContext</structname> represents a meta data modification
@@ -207,7 +214,9 @@ gboolean          gda_meta_store_undeclare_foreign_key    (GdaMetaStore *store,
 							   GError **error);
 
 GType             gda_meta_context_get_type               (void) G_GNUC_CONST;
-GdaMetaContext*   gda_meta_context_new                    ();
+GdaMetaContext*   gda_meta_context_new                    (void);
+GdaMetaContext*   gda_meta_context_copy                   (GdaMetaContext *ctx);
+
 void              gda_meta_context_set_table              (GdaMetaContext *ctx, const gchar *table);
 const gchar*      gda_meta_context_get_table              (GdaMetaContext *ctx);
 void              gda_meta_context_set_column             (GdaMetaContext *ctx, const gchar* column, 
