@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2008 - 2010 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2008 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2008 - 2012 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2008 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
@@ -128,9 +128,9 @@ gda_meta_struct_class_init (GdaMetaStructClass *klass) {
         object_class->set_property = gda_meta_struct_set_property;
         object_class->get_property = gda_meta_struct_get_property;
 	g_object_class_install_property (object_class, PROP_STORE,
-					 /* To translators: GdaMetaStore is an object holding meta data information
-					  * which will be used as an information source */
 					 g_param_spec_object ("meta-store", NULL,
+							      /* To translators: GdaMetaStore is an object holding meta data information
+							       * which will be used as an information source */
 							      _ ("GdaMetaStore object to fetch information from"),  
 							      GDA_TYPE_META_STORE, 
 							      (G_PARAM_WRITABLE | G_PARAM_READABLE | 
@@ -194,7 +194,7 @@ gda_meta_struct_dispose (GObject *object) {
 						      G_CALLBACK (meta_store_changed_cb), mstruct);
 		g_signal_handlers_disconnect_by_func (G_OBJECT (mstruct->priv->store),
 						      G_CALLBACK (meta_store_reset_cb), mstruct);
-		g_object_ref (mstruct->priv->store);
+		g_object_unref (mstruct->priv->store);
 		mstruct->priv->store = NULL;
 	}
 	
@@ -932,6 +932,7 @@ _meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 #endif
 			}
 		}
+        	break;
 	}
 	case GDA_META_DB_TABLE: {
 		/* columns */
@@ -1045,9 +1046,6 @@ _meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 					if (!strcmp (tmp, GDA_EXTRA_AUTO_INCREMENT))
 						gda_attributes_manager_set (att_mgr, tcol, GDA_ATTRIBUTE_AUTO_INCREMENT, 
 									    true_value);
-					else
-						g_message ("Unknown EXTRA attribute '%s', please report this bug to "
-							   "http://bugzilla.gnome.org/ for the \"libgda\" product.", tmp);
 				}
 				gda_value_free (true_value);
 				g_strfreev (array);
@@ -1774,7 +1772,11 @@ gda_meta_struct_sort_db_objects (GdaMetaStruct *mstruct, GdaMetaSortType sort_ty
 		ordered_list = mstruct->priv->db_objects;
 		break;
 	case GDA_META_SORT_DEPENDENCIES:
-		g_return_val_if_fail (mstruct, FALSE);
+		if (mstruct == NULL) {
+			g_set_error (error, GDA_META_STRUCT_ERROR, GDA_META_STORE_INTERNAL_ERROR,
+			     _("No meta struct was provided for object sorting"));
+			return FALSE;
+		}
 		for (pass_list = build_pass (mstruct->priv->db_objects, ordered_list); 
 		     pass_list; 
 		     pass_list = build_pass (mstruct->priv->db_objects, ordered_list)) 
@@ -1919,7 +1921,7 @@ gda_meta_struct_get_db_object (GdaMetaStruct *mstruct, const GValue *catalog, co
 }
 
 /**
- * gda_meta_struct_get_table_column: (skip)
+ * gda_meta_struct_get_table_column:
  * @mstruct: a #GdaMetaStruct object
  * @table: the #GdaMetaTable structure to find the column for
  * @col_name: the name of the column to find (as a G_TYPE_STRING GValue)
@@ -2615,7 +2617,7 @@ _gda_meta_struct_add_db_object (GdaMetaStruct *mstruct, GdaMetaDbObject *dbo, GE
  *
  * Get the value associated to a named attribute.
  *
- * Attributes can have any name, but Libgda proposes some default names, see <link linkend="libgda-5.0-Attributes-manager.synopsis">this section</link>.
+ * Attributes can have any name, but Libgda proposes some default names, see <link linkend="libgda-6.0-Attributes-manager.synopsis">this section</link>.
  *
  * Returns: (transfer none): a read-only #GValue, or %NULL if not attribute named @attribute has been set for @column
  */
