@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2006 - 2008 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2006 Rodrigo Moya <rodrigo@gnome-db.org>
- * Copyright (C) 2006 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2006 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2007 Armin Burgmeier <armin@openismus.com>
  * Copyright (C) 2007 Daniel Espinosa <esodan@gmail.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
@@ -30,6 +30,7 @@
 #include <libgda/gda-data-handler.h>
 #include <libgda/gda-util.h>
 #include <glib/gi18n-lib.h>
+#include <gio/gio.h>
 #include <libgda/sql-parser/gda-sql-parser.h>
 
 #include <libgda/handlers/gda-handler-numerical.h>
@@ -103,7 +104,7 @@ gda_server_provider_perform_operation_default (GdaServerProvider *provider, GdaC
 }
 
 /**
- * gda_server_provider_handler_use_default: (skip)
+ * gda_server_provider_handler_use_default:
  * @provider: a server provider
  * @type: a #GType
  *
@@ -165,7 +166,7 @@ gda_server_provider_handler_use_default (GdaServerProvider *provider, GType type
 		}
 	}
 	else if ((type == GDA_TYPE_TIME) ||
-		 (type == GDA_TYPE_TIMESTAMP) ||
+		 (type == G_TYPE_DATE_TIME) ||
 		 (type == G_TYPE_DATE)) {
 		/* no default time related data handler, it's too database specific */
 		dh = NULL;
@@ -190,26 +191,6 @@ gda_server_provider_handler_use_default (GdaServerProvider *provider, GType type
 	return dh;
 }
 
-/**
- * gda_server_provider_get_data_handler_default:
- * @provider: a server provider.
- * @cnc: (allow-none): a #GdaConnection object, or %NULL
- * @type: a #GType
- * @dbms_type: a DBMS type definition
- *
- * Provides the implementation when the default Libgda's data handlers must be used
- * 
- * Returns: (transfer none): a #GdaDataHandler, or %NULL
- *
- * Deprecated: 5.2: use gda_server_provider_handler_use_default() instead
- */
-GdaDataHandler *
-gda_server_provider_get_data_handler_default (GdaServerProvider *provider, G_GNUC_UNUSED GdaConnection *cnc,
-					      GType type, G_GNUC_UNUSED const gchar *dbms_type)
-{
-	return gda_server_provider_handler_use_default (provider, type);
-}
-
 static gboolean
 param_to_null_foreach (GdaSqlAnyPart *part, G_GNUC_UNUSED gpointer data, G_GNUC_UNUSED GError **error)
 {
@@ -431,7 +412,7 @@ gda_server_provider_load_file_contents (const gchar *inst_dir, const gchar *data
 		goto theend;
 	
 	g_free (file);
-	file = g_build_filename (inst_dir, "..", "..", "..", "share", "libgda-5.0", filename, NULL);
+	file = g_build_filename (inst_dir, "..", "..", "..", "share", "libgda-6.0", filename, NULL);
 	if (g_file_get_contents (file, &contents, NULL, NULL))
 		goto theend;
 	contents = NULL;
@@ -440,3 +421,38 @@ gda_server_provider_load_file_contents (const gchar *inst_dir, const gchar *data
 	g_free (file);
 	return contents;
 }
+
+/**
+ * gda_server_provider_load_resource_contents:
+ * @prov_name: the provider's name
+ * @resource: the name of the resource to load
+ *
+ * Loads and returns the contents of the specified resource.
+ * This function should only be used by database provider's implementations
+ *
+ * Returns: (transfer full): a new string containing the resource's contents, or %NULL if not found or if an error occurred
+ *
+ * Since: 6.0
+ */
+gchar *
+gda_server_provider_load_resource_contents (const gchar *prov_name, const gchar *resource)
+{
+	g_return_val_if_fail (prov_name, NULL);
+	g_return_val_if_fail (resource, NULL);
+
+	gchar *rname;
+	rname = g_strdup_printf ("/spec/%s/%s", prov_name, resource);
+
+	GBytes *bytes;
+	bytes = g_resources_lookup_data (rname, G_RESOURCE_LOOKUP_FLAGS_NONE, NULL);
+	g_free (rname);
+	if (!bytes) {
+		g_warning ("Resource: /spec/%s/%s, not found for provider", prov_name, resource);
+		return NULL;
+	}
+
+	gchar *retval;
+	retval = g_strdup ((const gchar*) g_bytes_get_data (bytes, NULL));
+	g_bytes_unref (bytes);
+	return retval;
+}
