@@ -1,9 +1,10 @@
 /*
  * Copyright (C) 2008 - 2011 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2008 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2008 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2013 Daniel Espinosa <esodan@gmail.com>
+ * Copyright (C) 2015 Corentin Noël <corentin@elementary.io>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -44,7 +45,6 @@
 extern xmlDtdPtr gda_paramlist_dtd;
 extern gchar *gda_lang_locale;
 
-#ifdef GSEAL_ENABLE
 /**
  * GdaSetGroup:
  * @nodes: (element-type Gda.SetNode): list of GdaSetNode, at least one entry
@@ -62,8 +62,6 @@ struct _GdaSetGroup {
 	gpointer      _gda_reserved1;
 	gpointer      _gda_reserved2;
 };
-#else
-#endif
 /*
    Register GdaSetGroup type
 */
@@ -241,7 +239,6 @@ gda_set_group_get_n_nodes (GdaSetGroup *sg)
 	return g_slist_length (sg->nodes);
 }
 
-#ifdef GSEAL_ENABLE
 /**
  * GdaSetSource:
  * @data_model: Can't be NULL
@@ -261,8 +258,6 @@ struct _GdaSetSource {
 	gpointer        _gda_reserved3;
 	gpointer        _gda_reserved4;
 };
-#else
-#endif
 
 /*
    Register GdaSetSource type
@@ -421,7 +416,6 @@ gda_set_source_get_n_nodes (GdaSetSource *s)
 	g_return_val_if_fail (s, -1);
 	return g_slist_length (s->nodes);
 }
-#ifdef GSEAL_ENABLE
 /**
  * GdaSetNode:
  * @holder: a #GdaHolder. It can't be NULL
@@ -441,7 +435,6 @@ struct _GdaSetNode {
 	gpointer      _gda_reserved1;
 	gpointer      _gda_reserved2;
 };
-#endif
 /*
    Register GdaSetNode type
 */
@@ -687,6 +680,11 @@ struct _GdaSetPrivate
 	GArray          *holders_array;
 	gboolean         read_only;
 	gboolean         validate_changes;
+
+	GSList         *holders;   /* list of GdaHolder objects */
+	GSList         *nodes_list;   /* list of GdaSetNode */
+	GSList         *sources_list; /* list of GdaSetSource */
+	GSList         *groups_list;  /* list of GdaSetGroup */
 };
 
 static void 
@@ -723,7 +721,7 @@ gda_set_set_property (GObject *object,
 		if (set->priv->validate_changes != g_value_get_boolean (value)) {
 			GSList *list;
 			set->priv->validate_changes = g_value_get_boolean (value);
-			for (list = set->holders; list; list = list->next) {
+			for (list = set->priv->holders; list; list = list->next) {
 				GdaHolder *holder = (GdaHolder*) list->data;
 				g_object_set ((GObject*) holder, "validate-changes",
 					      set->priv->validate_changes, NULL);
@@ -1006,10 +1004,10 @@ static void
 gda_set_init (GdaSet *set)
 {
 	set->priv = g_new0 (GdaSetPrivate, 1);
-	set->holders = NULL;
-	set->nodes_list = NULL;
-	set->sources_list = NULL;
-	set->groups_list = NULL;
+	set->priv->holders = NULL;
+	set->priv->nodes_list = NULL;
+	set->priv->sources_list = NULL;
+	set->priv->groups_list = NULL;
 	set->priv->holders_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	set->priv->holders_array = NULL;
 	set->priv->read_only = FALSE;
@@ -1080,7 +1078,7 @@ gda_set_copy (GdaSet *set)
 	GSList *list, *holders = NULL;
 	g_return_val_if_fail (GDA_IS_SET (set), NULL);
 	
-	for (list = set->holders; list; list = list->next) 
+	for (list = set->priv->holders; list; list = list->next)
 		holders = g_slist_prepend (holders, gda_holder_copy (GDA_HOLDER (list->data)));
 	holders = g_slist_reverse (holders);
 
@@ -1129,47 +1127,47 @@ gda_set_new_inline (gint nb, ...)
 		holder = (GdaHolder *) g_object_new (GDA_TYPE_HOLDER, "g-type", type, "id", id, NULL);
 
 		value = gda_value_new (type);
-		if (type == G_TYPE_BOOLEAN) 
+		if (g_type_is_a (type, G_TYPE_BOOLEAN))
 			g_value_set_boolean (value, va_arg (ap, int));
-                else if (type == G_TYPE_STRING)
+                else if (g_type_is_a (type, G_TYPE_STRING))
 			g_value_set_string (value, va_arg (ap, gchar *));
-                else if (type == G_TYPE_OBJECT)
+                else if (g_type_is_a (type, G_TYPE_OBJECT))
 			g_value_set_object (value, va_arg (ap, gpointer));
-		else if (type == G_TYPE_INT)
+		else if (g_type_is_a (type, G_TYPE_INT))
 			g_value_set_int (value, va_arg (ap, gint));
-		else if (type == G_TYPE_UINT)
+		else if (g_type_is_a (type, G_TYPE_UINT))
 			g_value_set_uint (value, va_arg (ap, guint));
-		else if (type == GDA_TYPE_BINARY)
+		else if (g_type_is_a (type, GDA_TYPE_BINARY))
 			gda_value_set_binary (value, va_arg (ap, GdaBinary *));
-		else if (type == G_TYPE_INT64)
+		else if (g_type_is_a (type, G_TYPE_INT64))
 			g_value_set_int64 (value, va_arg (ap, gint64));
-		else if (type == G_TYPE_UINT64)
+		else if (g_type_is_a (type, G_TYPE_UINT64))
 			g_value_set_uint64 (value, va_arg (ap, guint64));
-		else if (type == GDA_TYPE_SHORT)
+		else if (g_type_is_a (type, GDA_TYPE_SHORT))
 			gda_value_set_short (value, va_arg (ap, int));
-		else if (type == GDA_TYPE_USHORT)
+		else if (g_type_is_a (type, GDA_TYPE_USHORT))
 			gda_value_set_ushort (value, va_arg (ap, guint));
-		else if (type == G_TYPE_CHAR)
+		else if (g_type_is_a (type, G_TYPE_CHAR))
 			g_value_set_schar (value, va_arg (ap, gint));
-		else if (type == G_TYPE_UCHAR)
+		else if (g_type_is_a (type, G_TYPE_UCHAR))
 			g_value_set_uchar (value, va_arg (ap, guint));
-		else if (type == G_TYPE_FLOAT)
+		else if (g_type_is_a (type, G_TYPE_FLOAT))
 			g_value_set_float (value, va_arg (ap, double));
-		else if (type == G_TYPE_DOUBLE)
+		else if (g_type_is_a (type, G_TYPE_DOUBLE))
 			g_value_set_double (value, va_arg (ap, gdouble));
-		else if (type == GDA_TYPE_NUMERIC)
+		else if (g_type_is_a (type, GDA_TYPE_NUMERIC))
 			gda_value_set_numeric (value, va_arg (ap, GdaNumeric *));
-		else if (type == G_TYPE_DATE)
+		else if (g_type_is_a (type, G_TYPE_DATE))
 			g_value_set_boxed (value, va_arg (ap, GDate *));
-		else if (type == G_TYPE_LONG)
+		else if (g_type_is_a (type, G_TYPE_LONG))
 			g_value_set_long (value, va_arg (ap, glong));
-		else if (type == G_TYPE_ULONG)
+		else if (g_type_is_a (type, G_TYPE_ULONG))
 			g_value_set_ulong (value, va_arg (ap, gulong));
-		else if (type == G_TYPE_GTYPE)
+		else if (g_type_is_a (type, G_TYPE_GTYPE))
 			g_value_set_gtype (value, va_arg(ap, GType));
-		else if (type == GDA_TYPE_TIMESTAMP)
-			gda_value_set_timestamp (value, va_arg(ap, GdaTimestamp *));
-		else if (type == GDA_TYPE_TIME)
+		else if (g_type_is_a (type, G_TYPE_DATE_TIME))
+			g_value_set_boxed (value, va_arg(ap, GDateTime *));
+		else if (g_type_is_a (type, GDA_TYPE_TIME))
 			gda_value_set_time (value, va_arg(ap, GdaTime *));
 		else {
 			g_warning (_("%s() does not handle values of type '%s'."),
@@ -1203,7 +1201,7 @@ gda_set_new_inline (gint nb, ...)
 }
 
 /**
- * gda_set_set_holder_value: (skip)
+ * gda_set_set_holder_value:
  * @set: a #GdaSet object
  * @error: (allow-none): a place to store errors, or %NULL
  * @holder_id: the ID of the holder to set the value
@@ -1233,47 +1231,47 @@ gda_set_set_holder_value (GdaSet *set, GError **error, const gchar *holder_id, .
 	type = gda_holder_get_g_type (holder);
 	va_start (ap, holder_id);
 	value = gda_value_new (type);
-	if (type == G_TYPE_BOOLEAN) 
+	if (g_type_is_a (type, G_TYPE_BOOLEAN))
 		g_value_set_boolean (value, va_arg (ap, int));
-	else if (type == G_TYPE_STRING)
+	else if (g_type_is_a (type, G_TYPE_STRING))
 		g_value_set_string (value, va_arg (ap, gchar *));
-	else if (type == G_TYPE_OBJECT)
+	else if (g_type_is_a (type, G_TYPE_OBJECT))
 		g_value_set_object (value, va_arg (ap, gpointer));
-	else if (type == G_TYPE_INT)
+	else if (g_type_is_a (type, G_TYPE_INT))
 		g_value_set_int (value, va_arg (ap, gint));
-	else if (type == G_TYPE_UINT)
+	else if (g_type_is_a (type, G_TYPE_UINT))
 		g_value_set_uint (value, va_arg (ap, guint));
-	else if (type == GDA_TYPE_BINARY)
+	else if (g_type_is_a (type, GDA_TYPE_BINARY))
 		gda_value_set_binary (value, va_arg (ap, GdaBinary *));
-	else if (type == G_TYPE_INT64)
+	else if (g_type_is_a (type, G_TYPE_INT64))
 		g_value_set_int64 (value, va_arg (ap, gint64));
-	else if (type == G_TYPE_UINT64)
+	else if (g_type_is_a (type, G_TYPE_UINT64))
 		g_value_set_uint64 (value, va_arg (ap, guint64));
-	else if (type == GDA_TYPE_SHORT)
+	else if (g_type_is_a (type, GDA_TYPE_SHORT))
 		gda_value_set_short (value, va_arg (ap, int));
-	else if (type == GDA_TYPE_USHORT)
+	else if (g_type_is_a (type, GDA_TYPE_USHORT))
 		gda_value_set_ushort (value, va_arg (ap, guint));
-	else if (type == G_TYPE_CHAR)
+	else if (g_type_is_a (type, G_TYPE_CHAR))
 		g_value_set_schar (value, va_arg (ap, gint));
-	else if (type == G_TYPE_UCHAR)
+	else if (g_type_is_a (type, G_TYPE_UCHAR))
 		g_value_set_uchar (value, va_arg (ap, guint));
-	else if (type == G_TYPE_FLOAT)
+	else if (g_type_is_a (type, G_TYPE_FLOAT))
 		g_value_set_float (value, va_arg (ap, double));
-	else if (type == G_TYPE_DOUBLE)
+	else if (g_type_is_a (type, G_TYPE_DOUBLE))
 		g_value_set_double (value, va_arg (ap, gdouble));
-	else if (type == GDA_TYPE_NUMERIC)
+	else if (g_type_is_a (type, GDA_TYPE_NUMERIC))
 		gda_value_set_numeric (value, va_arg (ap, GdaNumeric *));
-	else if (type == G_TYPE_DATE)
+	else if (g_type_is_a (type, G_TYPE_DATE))
 		g_value_set_boxed (value, va_arg (ap, GDate *));
-	else if (type == GDA_TYPE_TIMESTAMP)
-		gda_value_set_timestamp (value, va_arg (ap, GdaTimestamp*));
-	else if (type == GDA_TYPE_TIME)
+	else if (g_type_is_a (type, G_TYPE_DATE_TIME))
+		g_value_set_boxed (value, va_arg (ap, GDateTime*));
+	else if (g_type_is_a (type, GDA_TYPE_TIME))
 		gda_value_set_time (value, va_arg (ap, GdaTime*));
-	else if (type == G_TYPE_LONG)
+	else if (g_type_is_a (type, G_TYPE_LONG))
 		g_value_set_long (value, va_arg (ap, glong));
-	else if (type == G_TYPE_ULONG)
+	else if (g_type_is_a (type, G_TYPE_ULONG))
 		g_value_set_ulong (value, va_arg (ap, gulong));
-	else if (type == G_TYPE_GTYPE)
+	else if (g_type_is_a (type, G_TYPE_GTYPE))
 		g_value_set_gtype (value, va_arg (ap, GType));	
 	else {
 		g_set_error (error, GDA_SET_ERROR, GDA_SET_IMPLEMENTATION_ERROR,
@@ -1605,7 +1603,7 @@ gda_set_remove_holder (GdaSet *set, GdaHolder *holder)
 
 	g_return_if_fail (GDA_IS_SET (set));
 	g_return_if_fail (set->priv);
-	g_return_if_fail (g_slist_find (set->holders, holder));
+	g_return_if_fail (g_slist_find (set->priv->holders, holder));
 
 	if (set->priv->validate_changes)
 		g_signal_handlers_disconnect_by_func (G_OBJECT (holder),
@@ -1638,7 +1636,7 @@ gda_set_remove_holder (GdaSet *set, GdaHolder *holder)
 	}
 	set_remove_node (set, node);
 
-	set->holders = g_slist_remove (set->holders, holder);
+	set->priv->holders = g_slist_remove (set->priv->holders, holder);
 	g_hash_table_remove (set->priv->holders_hash, gda_holder_get_id (holder));
 	if (set->priv->holders_array) {
 		g_array_free (set->priv->holders_array, TRUE);
@@ -1714,8 +1712,8 @@ gda_set_dispose (GObject *object)
 
 	set = GDA_SET (object);
 	/* free the holders list */
-	if (set->holders) {
-		for (list = set->holders; list; list = list->next) {
+	if (set->priv->holders) {
+		for (list = set->priv->holders; list; list = list->next) {
 			if (set->priv->validate_changes)
 				g_signal_handlers_disconnect_by_func (G_OBJECT (list->data),
 								      G_CALLBACK (validate_change_holder_cb), set);
@@ -1729,7 +1727,7 @@ gda_set_dispose (GObject *object)
 			}
 			g_object_unref (list->data);
 		}
-		g_slist_free (set->holders);
+		g_slist_free (set->priv->holders);
 	}
 	if (set->priv->holders_hash) {
 		g_hash_table_destroy (set->priv->holders_hash);
@@ -1741,14 +1739,14 @@ gda_set_dispose (GObject *object)
 	}
 
 	/* free the nodes if there are some */
-	while (set->nodes_list)
-		set_remove_node (set, GDA_SET_NODE (set->nodes_list->data));
-	while (set->sources_list)
-		set_remove_source (set, GDA_SET_SOURCE (set->sources_list->data));
+	while (set->priv->nodes_list)
+		set_remove_node (set, GDA_SET_NODE (set->priv->nodes_list->data));
+	while (set->priv->sources_list)
+		set_remove_source (set, GDA_SET_SOURCE (set->priv->sources_list->data));
 
-	g_slist_foreach (set->groups_list, (GFunc) group_free, NULL);
-	g_slist_free (set->groups_list);
-	set->groups_list = NULL;
+	g_slist_foreach (set->priv->groups_list, (GFunc) group_free, NULL);
+	g_slist_free (set->priv->groups_list);
+	set->priv->groups_list = NULL;
 
 	/* parent class */
 	parent_class->dispose (object);
@@ -1790,32 +1788,32 @@ compute_public_data (GdaSet *set)
 	/*
 	 * Get rid of all the previous structures
 	 */
-	while (set->nodes_list)
-		set_remove_node (set, GDA_SET_NODE (set->nodes_list->data));
-	while (set->sources_list)
-		set_remove_source (set, GDA_SET_SOURCE (set->sources_list->data));
+	while (set->priv->nodes_list)
+		set_remove_node (set, GDA_SET_NODE (set->priv->nodes_list->data));
+	while (set->priv->sources_list)
+		set_remove_source (set, GDA_SET_SOURCE (set->priv->sources_list->data));
 
-	g_slist_foreach (set->groups_list, (GFunc) group_free, NULL);
-	g_slist_free (set->groups_list);
-	set->groups_list = NULL;
+	g_slist_foreach (set->priv->groups_list, (GFunc) group_free, NULL);
+	g_slist_free (set->priv->groups_list);
+	set->priv->groups_list = NULL;
 
 	/*
 	 * Creation of the GdaSetNode structures
 	 */
-	for (list = set->holders; list; list = list->next) {
+	for (list = set->priv->holders; list; list = list->next) {
 		GdaHolder *holder = GDA_HOLDER (list->data);
 		gint col;
 		node = gda_set_node_new (holder);
 		gda_set_node_set_data_model (node, gda_holder_get_source_model (holder, &col));
 		gda_set_node_set_source_column (node, col);
-		set->nodes_list = g_slist_prepend (set->nodes_list, node);
+		set->priv->nodes_list = g_slist_prepend (set->priv->nodes_list, node);
 	}
-	set->nodes_list = g_slist_reverse (set->nodes_list);
+	set->priv->nodes_list = g_slist_reverse (set->priv->nodes_list);
 
 	/*
 	 * Creation of the GdaSetSource and GdaSetGroup structures 
 	 */
-	for (list = set->nodes_list; list;list = list->next) {
+	for (list = set->priv->nodes_list; list;list = list->next) {
 		node = GDA_SET_NODE (list->data);
 		
 		/* source */
@@ -1827,7 +1825,7 @@ compute_public_data (GdaSet *set)
 			else {
 				source = gda_set_source_new (gda_set_node_get_data_model (node));
 				gda_set_source_add_node (source, node);
-				set->sources_list = g_slist_prepend (set->sources_list, source);
+				set->priv->sources_list = g_slist_prepend (set->priv->sources_list, source);
 			}
 		}
 
@@ -1840,7 +1838,7 @@ compute_public_data (GdaSet *set)
 		else {
 			group = gda_set_group_new (node);
 			gda_set_group_set_source (group, source);
-			set->groups_list = g_slist_prepend (set->groups_list, group);
+			set->priv->groups_list = g_slist_prepend (set->priv->groups_list, group);
 			if (gda_set_node_get_data_model (node)) {
 				if (!groups)
 					groups = g_hash_table_new (NULL, NULL); /* key = source model, 
@@ -1849,7 +1847,7 @@ compute_public_data (GdaSet *set)
 			}
 		}		
 	}
-	set->groups_list = g_slist_reverse (set->groups_list);
+	set->priv->groups_list = g_slist_reverse (set->priv->groups_list);
 	if (groups)
 		g_hash_table_destroy (groups);
 
@@ -1865,7 +1863,7 @@ compute_public_data (GdaSet *set)
 /**
  * gda_set_add_holder:
  * @set: a #GdaSet object
- * @holder: a #GdaHolder object
+ * @holder: (transfer none): a #GdaHolder object
  *
  * Adds @holder to the list of holders managed within @set.
  *
@@ -1873,7 +1871,7 @@ compute_public_data (GdaSet *set)
  * will not be added to the set (even if @holder's type or value is not the same as the
  * one already in @set).
  *
- * Returns: TRUE if @holder has been added to @set (and FALSE if it has not been added because there is another #GdaHolder
+ * Returns: %TRUE if @holder has been added to @set (and FALSE if it has not been added because there is another #GdaHolder
  * with the same ID)
  */
 gboolean
@@ -1927,7 +1925,7 @@ gda_set_real_add_holder (GdaSet *set, GdaHolder *holder)
 	const gchar *hid;
 
 	/* 
-	 * try to find a similar holder in the set->holders:
+	 * try to find a similar holder in the set->priv->holders:
 	 * a holder B is similar to a holder A if it has the same ID
 	 */
 	hid = gda_holder_get_id (holder);
@@ -1939,7 +1937,7 @@ gda_set_real_add_holder (GdaSet *set, GdaHolder *holder)
 	similar = (GdaHolder*) g_hash_table_lookup (set->priv->holders_hash, hid);
 	if (!similar) {
 		/* really add @holder to the set */
-		set->holders = g_slist_append (set->holders, holder);
+		set->priv->holders = g_slist_append (set->priv->holders, holder);
 		g_hash_table_insert (set->priv->holders_hash, (gchar*) hid, holder);
 		if (set->priv->holders_array) {
 			g_array_free (set->priv->holders_array, TRUE);
@@ -1993,7 +1991,7 @@ gda_set_merge_with_set (GdaSet *set, GdaSet *set_to_merge)
 	g_return_if_fail (GDA_IS_SET (set));
 	g_return_if_fail (set_to_merge && GDA_IS_SET (set_to_merge));
 
-	for (holders = set_to_merge->holders; holders; holders = holders->next)
+	for (holders = set_to_merge->priv->holders; holders; holders = holders->next)
 		gda_set_real_add_holder (set, GDA_HOLDER (holders->data));
 	compute_public_data (set);
 }
@@ -2001,17 +1999,17 @@ gda_set_merge_with_set (GdaSet *set, GdaSet *set_to_merge)
 static void
 set_remove_node (GdaSet *set, GdaSetNode *node)
 {
-	g_return_if_fail (g_slist_find (set->nodes_list, node));
+	g_return_if_fail (g_slist_find (set->priv->nodes_list, node));
 	gda_set_node_free (node);
-	set->nodes_list = g_slist_remove (set->nodes_list, node);
+	set->priv->nodes_list = g_slist_remove (set->priv->nodes_list, node);
 }
 
 static void
 set_remove_source (GdaSet *set, GdaSetSource *source)
 {
-	g_return_if_fail (g_slist_find (set->sources_list, source));
+	g_return_if_fail (g_slist_find (set->priv->sources_list, source));
 	gda_set_source_free (source);
-	set->sources_list = g_slist_remove (set->sources_list, source);
+	set->priv->sources_list = g_slist_remove (set->priv->sources_list, source);
 }
 
 /**
@@ -2035,7 +2033,7 @@ gda_set_is_valid (GdaSet *set, GError **error)
 	g_return_val_if_fail (GDA_IS_SET (set), FALSE);
 	g_return_val_if_fail (set->priv, FALSE);
 
-	for (holders = set->holders; holders; holders = holders->next) {
+	for (holders = set->priv->holders; holders; holders = holders->next) {
 		if (!gda_holder_is_valid (GDA_HOLDER (holders->data))) {
 			g_set_error (error, GDA_SET_ERROR, GDA_SET_INVALID_ERROR,
 				     "%s", _("One or more values are invalid"));
@@ -2104,8 +2102,8 @@ gda_set_get_nth_holder (GdaSet *set, gint pos)
 	if (! set->priv->holders_array) {
 		GSList *list;
 		set->priv->holders_array = g_array_sized_new (FALSE, FALSE, sizeof (GdaHolder*),
-							      g_slist_length (set->holders));
-		for (list = set->holders; list; list = list->next)
+							      g_slist_length (set->priv->holders));
+		for (list = set->priv->holders; list; list = list->next)
 			g_array_append_val (set->priv->holders_array, list->data);
 	}
 	if ((guint)pos >= set->priv->holders_array->len)
@@ -2114,6 +2112,49 @@ gda_set_get_nth_holder (GdaSet *set, gint pos)
 		return g_array_index (set->priv->holders_array, GdaHolder*, pos);
 }
 
+/**
+ * gda_set_get_holders:
+ *
+ * Returns: (transfer none) (element-type Gda.Holder): a list of #GdaHolder objects in the set
+ */
+GSList*
+gda_set_get_holders (GdaSet *set) {
+  g_return_val_if_fail (set != NULL, NULL);
+	g_return_val_if_fail (GDA_IS_SET(set), NULL);
+	return set->priv->holders;
+}
+/**
+ * gda_set_get_nodes:
+ *
+ * Returns: (transfer none) (element-type Gda.SetNode): a list of #GdaSetNode objects in the set
+ */
+GSList*
+gda_set_get_nodes (GdaSet *set) {
+	g_return_val_if_fail (GDA_IS_SET(set), NULL);
+	return set->priv->nodes_list;
+}
+
+/**
+ * gda_set_get_sources:
+ *
+ * Returns: (transfer none) (element-type Gda.SetSource): a list of #GdaSetSource objects in the set
+ */
+GSList*
+gda_set_get_sources (GdaSet *set) {
+	g_return_val_if_fail (GDA_IS_SET(set), NULL);
+	return set->priv->sources_list;
+}
+
+/**
+ * gda_set_get_groups:
+ *
+ * Returns: (transfer none) (element-type Gda.SetGroup): a list of #GdaSetGroup objects in the set
+ */
+GSList*
+gda_set_get_groups (GdaSet *set) {
+	g_return_val_if_fail (GDA_IS_SET(set), NULL);
+	return set->priv->groups_list;
+}
 /**
  * gda_set_get_node:
  * @set: a #GdaSet object
@@ -2133,9 +2174,9 @@ gda_set_get_node (GdaSet *set, GdaHolder *holder)
 	g_return_val_if_fail (set->priv, NULL);
 	g_return_val_if_fail (GDA_IS_HOLDER (holder), NULL);
 	/* FIXME: May is better to use holder's hash for better performance */
-	g_return_val_if_fail (g_slist_find (set->holders, holder), NULL);
+	g_return_val_if_fail (g_slist_find (set->priv->holders, holder), NULL);
 
-	for (list = set->nodes_list; list && !retval; list = list->next) {
+	for (list = set->priv->nodes_list; list && !retval; list = list->next) {
 		GdaHolder *node_holder;
 		retval = GDA_SET_NODE (list->data);
 		node_holder = gda_set_node_get_holder (retval);
@@ -2186,32 +2227,29 @@ gda_set_get_group (GdaSet *set, GdaHolder *holder)
 {
 	GdaSetNode *node;
 	GdaSetGroup *retval = NULL;
-	GSList *list, *sublist;
-	GdaHolder *node_holder;
+	GSList *list;
 
 	g_return_val_if_fail (GDA_IS_SET (set), NULL);
 	g_return_val_if_fail (set->priv, NULL);
 	g_return_val_if_fail (GDA_IS_HOLDER (holder), NULL);
-	/* FIXME: May is better to use holder's hash for better performance */
-	g_return_val_if_fail (g_slist_find (set->holders, holder), NULL);
+	g_return_val_if_fail (g_slist_find (set->priv->holders, holder), NULL);
 
-	for (list = set->groups_list; list && !retval; list = list->next) {
+	for (list = set->priv->groups_list; list; list = list->next) {
 		retval = GDA_SET_GROUP (list->data);
-		sublist = gda_set_group_get_nodes (retval);
-		while (sublist && !retval) {
+		GSList *sublist;
+		for (sublist = gda_set_group_get_nodes (retval); sublist; sublist = sublist->next) {
 			node = GDA_SET_NODE (sublist->data);
 			if (node) {
+				GdaHolder *node_holder;
 				node_holder = gda_set_node_get_holder (node);
 				if (node_holder == holder) /* FIXME: May is better to compare holders ID */
 					break;
-				else {
-					sublist = g_slist_next (sublist);
-					retval = NULL;
-				}
 			}
 		}
+		if (sublist)
+			break;
 	}
-	return retval;
+	return list ? retval : NULL;
 }
 
 
@@ -2236,7 +2274,7 @@ gda_set_get_source_for_model (GdaSet *set, GdaDataModel *model)
 	g_return_val_if_fail (set->priv, NULL);
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
 
-	list = set->sources_list;
+	list = set->priv->sources_list;
 	while (list && !retval) {
 		retval = GDA_SET_SOURCE (list->data);
 		source_model = gda_set_source_get_data_model (retval);
@@ -2274,7 +2312,7 @@ gda_set_replace_source_model (GdaSet *set, GdaSetSource *source, GdaDataModel *m
 	
 	g_return_if_fail (GDA_IS_SET (set));
 	g_return_if_fail (source);
-	g_return_if_fail (g_slist_find (set->sources_list, source));
+	g_return_if_fail (g_slist_find (set->priv->sources_list, source));
 	g_return_if_fail (GDA_IS_DATA_MODEL (model));
 	
 	/* compare models */
@@ -2379,10 +2417,10 @@ void
 gda_set_dump (GdaSet *set)
 {
 	g_print ("=== GdaSet %p ===\n", set);
-	g_slist_foreach (set->holders, (GFunc) holder_dump, NULL);
-	g_slist_foreach (set->nodes_list, (GFunc) set_node_dump, NULL);
-	g_slist_foreach (set->sources_list, (GFunc) set_source_dump, NULL);
-	g_slist_foreach (set->groups_list, (GFunc) set_group_dump, NULL);
+	g_slist_foreach (set->priv->holders, (GFunc) holder_dump, NULL);
+	g_slist_foreach (set->priv->nodes_list, (GFunc) set_node_dump, NULL);
+	g_slist_foreach (set->priv->sources_list, (GFunc) set_source_dump, NULL);
+	g_slist_foreach (set->priv->groups_list, (GFunc) set_group_dump, NULL);
 	g_print ("=== GdaSet %p END ===\n", set);
 }
 #endif
