@@ -2,7 +2,7 @@
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2009 Johannes Schmid <jhs@gnome.org>
  * Copyright (C) 2009 - 2011 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2009 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
  * Copyright (C) 2011 Daniel Espinosa <despinosa@src.gnome.org>
@@ -328,7 +328,7 @@ use_part (SqlPart *p, GdaSqlAnyPart *parent)
 /**
  * gda_sql_builder_get_statement:
  * @builder: a #GdaSqlBuilder object
- * @error: (nullable): a place to store errors, or %NULL
+ * @error: (allow-none): a place to store errors, or %NULL
  *
  * Creates a new #GdaStatement statement from @builder's contents.
  *
@@ -352,7 +352,7 @@ gda_sql_builder_get_statement (GdaSqlBuilder *builder, GError **error)
 }
 
 /**
- * gda_sql_builder_get_sql_statement: (skip)
+ * gda_sql_builder_get_sql_statement:
  * @builder: a #GdaSqlBuilder object
  *
  * Creates a new #GdaSqlStatement structure from @builder's contents.
@@ -360,7 +360,7 @@ gda_sql_builder_get_statement (GdaSqlBuilder *builder, GError **error)
  * The returned pointer belongs to @builder's internal representation.
  * Use gda_sql_statement_copy() if you need to keep it.
  *
- * Returns: (transfer none) (nullable): a #GdaSqlStatement pointer
+ * Returns: (transfer none) (allow-none): a #GdaSqlStatement pointer
  *
  * Since: 4.2
  */
@@ -483,8 +483,8 @@ gda_sql_builder_set_where (GdaSqlBuilder *builder, GdaSqlBuilderId cond_id)
  * gda_sql_builder_select_add_field:
  * @builder: a #GdaSqlBuilder object
  * @field_name: a field name
- * @table_name: (nullable): a table name, or %NULL
- * @alias: (nullable): an alias (eg. for the "AS" clause), or %NULL
+ * @table_name: (allow-none): a table name, or %NULL
+ * @alias: (allow-none): an alias (eg. for the "AS" clause), or %NULL
  *
  * Valid only for: SELECT statements.
  *
@@ -574,10 +574,10 @@ create_typed_value (GType type, va_list *ap)
 		timegda = va_arg (*ap, GdaTime *);
 		gda_value_set_time ((v = gda_value_new (GDA_TYPE_TIME)), timegda);
 	}
-	else if (type == GDA_TYPE_TIMESTAMP) {
-		GdaTimestamp *timestamp;
-		timestamp = va_arg (*ap, GdaTimestamp *);
-		gda_value_set_timestamp ((v = gda_value_new (GDA_TYPE_TIMESTAMP)), timestamp);
+	else if (type == G_TYPE_DATE_TIME) {
+		GDateTime *timestamp;
+		timestamp = va_arg (*ap, GDateTime *);
+		g_value_set_boxed ((v = gda_value_new (G_TYPE_DATE_TIME)), timestamp);
 	}
 	else if (type == GDA_TYPE_NULL)
 		v = gda_value_new_null ();
@@ -651,7 +651,7 @@ gda_sql_builder_add_field_value (GdaSqlBuilder *builder, const gchar *field_name
  * gda_sql_builder_add_field_value_as_gvalue:
  * @builder: a #GdaSqlBuilder object
  * @field_name: a field name
- * @value: (nullable): value to set the field to, or %NULL or a GDA_TYPE_NULL value to represent an SQL NULL
+ * @value: (allow-none): value to set the field to, or %NULL or a GDA_TYPE_NULL value to represent an SQL NULL
  *
  * Valid only for: INSERT, UPDATE statements.
  *
@@ -799,8 +799,8 @@ gda_sql_builder_add_field_value_id (GdaSqlBuilder *builder, GdaSqlBuilderId fiel
 /**
  * gda_sql_builder_add_expr_value:
  * @builder: a #GdaSqlBuilder object
- * @dh: (nullable): deprecated useless argument, just pass %NULL
- * @value: (nullable): value to set the expression to, or %NULL or a GDA_TYPE_NULL value to represent an SQL NULL
+ * @dh: (allow-none): deprecated useless argument, just pass %NULL
+ * @value: (allow-none): value to set the expression to, or %NULL or a GDA_TYPE_NULL value to represent an SQL NULL
  *
  * Defines an expression in @builder which may be reused to build other parts of a statement.
  *
@@ -840,9 +840,9 @@ gda_sql_builder_add_expr_value (GdaSqlBuilder *builder, G_GNUC_UNUSED GdaDataHan
 }
 
 /**
- * gda_sql_builder_add_expr: (skip)
+ * gda_sql_builder_add_expr:
  * @builder: a #GdaSqlBuilder object
- * @dh: (nullable): deprecated useless argument, just pass %NULL
+ * @dh: (allow-none): deprecated useless argument, just pass %NULL
  * @type: the GType of the following argument
  * @...: value to set the expression to, of the type specified by @type
  *
@@ -950,7 +950,7 @@ gda_sql_builder_add_id (GdaSqlBuilder *builder, const gchar *str)
  * gda_sql_builder_add_field_id:
  * @builder: a #GdaSqlBuilder object
  * @field_name: a field name
- * @table_name: (nullable): a table name, or %NULL
+ * @table_name: (allow-none): a table name, or %NULL
  *
  * Defines an expression representing a field in @builder,
  * which may be reused to build other parts of a statement,
@@ -1139,7 +1139,7 @@ typedef struct {
  * gda_sql_builder_select_add_target_id:
  * @builder: a #GdaSqlBuilder object
  * @table_id: the ID of the expression holding a table reference (not %0)
- * @alias: (nullable): the alias to give to the target, or %NULL
+ * @alias: (allow-none): the alias to give to the target, or %NULL
  *
  * Adds a new target to a SELECT statement. If there already exists a target representing
  * the same table and the same alias (or with the same absence of alias) then the same target
@@ -1224,7 +1224,7 @@ gda_sql_builder_select_add_target_id (GdaSqlBuilder *builder, GdaSqlBuilderId ta
  * gda_sql_builder_select_add_target:
  * @builder: a #GdaSqlBuilder object
  * @table_name: the name of the target table
- * @alias: (nullable): the alias to give to the target, or %NULL
+ * @alias: (allow-none): the alias to give to the target, or %NULL
  *
  * Adds a new target to a SELECT statement
  *
@@ -1257,7 +1257,7 @@ typedef struct {
 } BuilderJoin;
 
 /**
- * gda_sql_builder_select_join_targets: (skip)
+ * gda_sql_builder_select_join_targets:
  * @builder: a #GdaSqlBuilder object
  * @left_target_id: the ID of the left target to use (not %0)
  * @right_target_id: the ID of the right target to use (not %0)
@@ -1407,7 +1407,7 @@ gda_sql_builder_join_add_field (GdaSqlBuilder *builder, GdaSqlBuilderId join_id,
  * @builder: a #GdaSqlBuiler
  * @expr_id: the ID of the expression to use during sorting (not %0)
  * @asc: %TRUE for an ascending sorting
- * @collation_name: (nullable):  name of the collation to use when sorting, or %NULL
+ * @collation_name: (allow-none):  name of the collation to use when sorting, or %NULL
  *
  * Adds a new ORDER BY expression to a SELECT statement.
  *
@@ -1712,7 +1712,7 @@ gda_sql_builder_add_function_v (GdaSqlBuilder *builder, const gchar *func_name,
 }
 
 /**
- * gda_sql_builder_add_sub_select: (skip)
+ * gda_sql_builder_add_sub_select:
  * @builder: a #GdaSqlBuilder object
  * @sqlst: a pointer to a #GdaSqlStatement, which has to be a SELECT or compound SELECT. This will be copied.
  *
@@ -1775,7 +1775,7 @@ gda_sql_builder_compound_set_type (GdaSqlBuilder *builder, GdaSqlStatementCompou
 }
 
 /**
- * gda_sql_builder_compound_add_sub_select: (skip)
+ * gda_sql_builder_compound_add_sub_select:
  * @builder: a #GdaSqlBuilder object
  * @sqlst: a pointer to a #GdaSqlStatement, which has to be a SELECT or compound SELECT. This will be copied.
  *
@@ -1922,7 +1922,6 @@ gda_sql_builder_add_case (GdaSqlBuilder *builder,
  * Returns: the ID of the new expression, or %0 if there was an error
  *
  * Since: 4.2
- *
  */
 GdaSqlBuilderId
 gda_sql_builder_add_case_v (GdaSqlBuilder *builder,
@@ -1969,7 +1968,7 @@ gda_sql_builder_add_case_v (GdaSqlBuilder *builder,
 }
 
 /**
- * gda_sql_builder_export_expression: (skip)
+ * gda_sql_builder_export_expression:
  * @builder: a #GdaSqlBuilder object
  * @id: the ID of the expression to be exported, (must be a valid ID in @builder, not %0)
  *
@@ -1996,7 +1995,7 @@ gda_sql_builder_export_expression (GdaSqlBuilder *builder, GdaSqlBuilderId id)
 }
 
 /**
- * gda_sql_builder_import_expression: (skip)
+ * gda_sql_builder_import_expression:
  * @builder: a #GdaSqlBuilder object
  * @expr: a #GdaSqlExpr obtained using gda_sql_builder_export_expression()
  *
