@@ -1,9 +1,10 @@
 /*
  * Copyright (C) 2008 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2008 - 2011 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2008 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2008 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
+ * Copyright (C) 2017 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -36,7 +37,7 @@
 #include <libgda/sql-parser/gda-statement-struct-select.h>
 #include <libgda/gda-marshal.h>
 #include <libgda/gda-data-handler.h>
-#include <libgda/gda-server-provider.h>
+#include <libgda/gda-server-provider-private.h>
 #include <libgda/gda-statement-extra.h>
 #include <libgda/gda-holder.h>
 #include <libgda/gda-set.h>
@@ -132,8 +133,9 @@ gda_statement_class_init (GdaStatementClass * klass)
 
 	/**
 	 * GdaStatement::reset:
+	 * @stmt: the #GdaStatement object
 	 *
-	 * Gets emitted whenever the statement has changed
+	 * Gets emitted whenever the @stmt has changed
 	 */
 	gda_statement_signals[RESET] =
 		g_signal_new ("reset",
@@ -145,9 +147,12 @@ gda_statement_class_init (GdaStatementClass * klass)
 			      0);
 	/**
 	 * GdaStatement::checked:
+	 * @stmt: the #GdaStatement object
+	 * @cnc: a #GdaConnection
+	 * @checked: whether @stmt have been verified
 	 *
 	 * Gets emitted whenever the structure and contents
-	 * of the statement have been verified (emitted after gda_statement_check_validity()).
+	 * of @stmt have been verified (emitted after gda_statement_check_validity()).
 	 */
 	gda_statement_signals[CHECKED] =
 		g_signal_new ("checked",
@@ -785,7 +790,7 @@ default_render_value (const GValue *value, GdaSqlRenderingContext *context, GErr
 			dh = gda_data_handler_get_default (G_VALUE_TYPE (value));
 
 		if (!dh) {
-			if (G_VALUE_TYPE (value) == GDA_TYPE_DEFAULT)
+			if (g_type_is_a (G_VALUE_TYPE (value), GDA_TYPE_DEFAULT))
 				return g_strdup ("DEFAULT");
 			else {
 				g_set_error (error, GDA_SQL_ERROR, GDA_SQL_STRUCTURE_CONTENTS_ERROR,
@@ -795,13 +800,13 @@ default_render_value (const GValue *value, GdaSqlRenderingContext *context, GErr
 			}
 		}
 		if (context->flags & GDA_STATEMENT_SQL_TIMEZONE_TO_GMT) {
-			if (G_VALUE_TYPE (value) == GDA_TYPE_TIME) {
+			if (g_type_is_a (G_VALUE_TYPE (value), GDA_TYPE_TIME)) {
 				GdaTime *nts;
 				nts = (GdaTime*) gda_value_get_time (value);
-				if (nts && (nts->timezone != GDA_TIMEZONE_INVALID)) {
+				if (nts && (gda_time_get_timezone (nts) != GDA_TIMEZONE_INVALID)) {
 					nts = gda_time_copy (nts);
 					gda_time_change_timezone (nts, 0);
-					nts->timezone = GDA_TIMEZONE_INVALID;
+					gda_time_set_timezone (nts, GDA_TIMEZONE_INVALID);
 					GValue v = {0};
 					g_value_init (&v, GDA_TYPE_TIME);
 					gda_value_set_time (&v, nts);
@@ -812,21 +817,20 @@ default_render_value (const GValue *value, GdaSqlRenderingContext *context, GErr
 					return tmp;
 				}
 			}
-			else if (G_VALUE_TYPE (value) == GDA_TYPE_TIMESTAMP) {
-				GdaTimestamp *nts;
-				nts = (GdaTimestamp*) gda_value_get_timestamp (value);
-				if (nts && (nts->timezone != GDA_TIMEZONE_INVALID)) {
-					nts = gda_timestamp_copy (nts);
-					gda_timestamp_change_timezone (nts, 0);
-					nts->timezone = GDA_TIMEZONE_INVALID;
-					GValue v = {0};
-					g_value_init (&v, GDA_TYPE_TIMESTAMP);
-					gda_value_set_timestamp (&v, nts);
-					gda_timestamp_free (nts);
-					gchar *tmp;
-					tmp = gda_data_handler_get_sql_from_value (dh, &v);
-					g_value_reset (&v);
-					return tmp;
+			else if (g_type_is_a (G_VALUE_TYPE (value), G_TYPE_DATE_TIME)) {
+				GDateTime *nts;
+				nts = (GDateTime*) g_value_get_boxed (value);
+				if (nts != NULL) {
+					nts = (GDateTime*) g_date_time_to_utc ((GDateTime*) nts);
+          if (nts != NULL) {
+					  GValue v = {0};
+					  g_value_init (&v, G_TYPE_DATE_TIME);
+					  g_value_set_boxed (&v, nts);
+					  gchar *tmp;
+					  tmp = gda_data_handler_get_sql_from_value (dh, &v);
+					  g_value_reset (&v);
+					  return tmp;
+          }
 				}
 			}
 		}
@@ -869,15 +873,12 @@ gda_statement_to_sql_extended (GdaStatement *stmt, GdaConnection *cnc, GdaSet *p
 	context.params = params;
 	context.flags = flags;
 	if (cnc) {
-		GdaServerProvider *prov;
-		prov = gda_connection_get_provider (cnc);
-#define PROV_CLASS(provider) (GDA_SERVER_PROVIDER_CLASS (G_OBJECT_GET_CLASS (provider)))
-		if (prov && PROV_CLASS (prov)->statement_to_sql)
-			return (PROV_CLASS (prov)->statement_to_sql) (prov, 
-								      cnc, stmt, params, flags, 
+		if (gda_connection_is_opened (cnc))
+			return _gda_server_provider_statement_to_sql (gda_connection_get_provider (cnc), cnc,
+								      stmt, params, flags,
 								      params_used, error);
-		context.cnc = cnc;
-		context.provider = gda_connection_get_provider (cnc);
+		else
+			context.provider = gda_connection_get_provider (cnc);
 	}
 
 	str = gda_statement_to_sql_real (stmt, &context, error);
@@ -1949,9 +1950,6 @@ default_render_operation (GdaSqlOperation *op, GdaSqlRenderingContext *context,
 	case GDA_SQL_OPERATOR_TYPE_OR:
 		multi_op = "OR";
 		break;
-	case GDA_SQL_OPERATOR_TYPE_GLOB:
-		str = g_strdup_printf ("%s GLOB %s", SQL_OPERAND (sql_list->data)->sql, SQL_OPERAND (sql_list->next->data)->sql);
-		break;
 	default:
 		g_assert_not_reached ();
 		break;
