@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2006 - 2013 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2007 Armin Burgmeier <armin@openismus.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
@@ -128,6 +128,27 @@ gda_transaction_status_get_type (void)
 	return type;
 }
 
+
+GdaTransactionStatusEvent *
+gda_transaction_status_event_copy (GdaTransactionStatusEvent *src) {
+	GdaTransactionStatusEvent *cp = g_new0(GdaTransactionStatusEvent, 1);
+	cp->trans = src->trans;
+	cp->type = src->type;
+	cp->pl.svp_name = g_strdup (src->pl.svp_name);
+	cp->pl.sql = g_strdup (src->pl.sql);
+	cp->pl.sub_trans = src->pl.sub_trans;
+	cp->conn_event = src->conn_event;
+  return cp;
+}
+void
+gda_transaction_status_event_free (GdaTransactionStatusEvent *te) {
+	g_free (te->pl.svp_name);
+	g_free (te->pl.sql);
+	g_free (te);
+}
+
+G_DEFINE_BOXED_TYPE(GdaTransactionStatusEvent, gda_transaction_status_event, gda_transaction_status_event_copy, gda_transaction_status_event_free)
+
 /**
  * gda_transaction_status_new:
  * @name: name for the transaction
@@ -150,7 +171,7 @@ gda_transaction_status_new (const gchar *name)
 }
 
 /**
- * gda_transaction_status_add_event_svp: (skip)
+ * gda_transaction_status_add_event_svp:
  *
  */
 GdaTransactionStatusEvent *
@@ -171,7 +192,7 @@ gda_transaction_status_add_event_svp (GdaTransactionStatus *tstatus, const gchar
 }
 
 /**
- * gda_transaction_status_add_event_sql: (skip)
+ * gda_transaction_status_add_event_sql:
  *
  */
 GdaTransactionStatusEvent *
@@ -196,7 +217,7 @@ gda_transaction_status_add_event_sql (GdaTransactionStatus *tstatus, const gchar
 }
 
 /**
- * gda_transaction_status_add_event_sub: (skip)
+ * gda_transaction_status_add_event_sub:
  *
  */
 GdaTransactionStatusEvent *
