@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2013 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
  *
@@ -24,12 +24,33 @@
 #include "gda-value.h"
 #include <string.h>
 
+/**
+ * GdaTreeManagerNodesFunc:
+ * @manager: a #GdaTreeManager
+ * @node: (allow-none): a #GdaTreeNode object, or %NULL
+ * @children_nodes: (element-type GdaTreeNode): a list of #GdaTreeNode nodes which have previously been created by a similar call and
+ * need to be updated ore moved
+ * @out_error: (out) (allow-none): a boolean to store if there was an error (can be %NULL)
+ * @error: a place to store errors, or %NULL
+ *
+ * Returns: (transfer container) (element-type GdaTreeNode): a new list of #GdaTreeNode objects.
+ */
+
+/**
+ * GdaTreeManagerNodeFunc:
+ * @manager: a #GdaTreeManager
+ * @parent: (allow-none): the parent the new node may have, or %NULL
+ * @name: (allow-none): name given to the new node, or %NULL
+ *
+ * Returns: (transfer full): a new #GdaTreeNode
+ */
+
 typedef struct {
 	gchar *att_name;
 	GValue *value;
 } AddedAttribute;
 
-struct _GdaTreeManagerPrivate {
+typedef struct {
         GSList  *sub_managers; /* list of GdaTreeManager structures, no ref held */
 	GSList  *ref_managers; /* list of GdaTreeManager structures, ref held */
 	gboolean recursive;
@@ -38,10 +59,10 @@ struct _GdaTreeManagerPrivate {
 	GdaTreeManagerNodesFunc update_func;
 
 	GSList *added_attributes; /* list of #AddedAttribute pointers, managed here */
-};
+} GdaTreeManagerPrivate;
+
+G_DEFINE_TYPE_WITH_PRIVATE (GdaTreeManager, gda_tree_manager, G_TYPE_OBJECT)
 
-static void gda_tree_manager_class_init (GdaTreeManagerClass *klass);
-static void gda_tree_manager_init       (GdaTreeManager *manager, GdaTreeManagerClass *klass);
 static void gda_tree_manager_dispose    (GObject *object);
 static void gda_tree_manager_set_property (GObject *object,
 					   guint param_id,
@@ -56,28 +77,24 @@ static void gda_tree_manager_get_property (GObject *object,
 enum {
 	PROP_0,
 	PROP_RECURS,
-	PROP_FUNC
+	PROP_FUNC,
+	LAST_PROP
 };
 
-static GObjectClass *parent_class = NULL;
+static GParamSpec *properties [LAST_PROP];
 
-/*
- * GdaTreeManager class implementation
- * @klass:
- */
 static void
 gda_tree_manager_class_init (GdaTreeManagerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	/* virtual methods */
 	klass->update_children = NULL;
 
 	/* Properties */
-        object_class->set_property = gda_tree_manager_set_property;
-        object_class->get_property = gda_tree_manager_get_property;
+	object_class->set_property = gda_tree_manager_set_property;
+	object_class->get_property = gda_tree_manager_get_property;
+	object_class->dispose = gda_tree_manager_dispose;
 
 	/**
 	 * GdaTreeManager:recursive:
@@ -89,66 +106,67 @@ gda_tree_manager_class_init (GdaTreeManagerClass *klass)
 	 * This property can typically set to FALSE if the process of creating children nodes is lenghty
 	 * and needs to be postponed while an event occurs.
 	 */
-	g_object_class_install_property (object_class, PROP_RECURS,
-                                         g_param_spec_boolean ("recursive", NULL, "Recursive building/updating of children",
-                                                              TRUE,
-                                                              G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
-/**
-	 * GdaTreeManager:func:
+	properties [PROP_RECURS] =
+                g_param_spec_boolean ("recursive",
+                                      NULL,
+                                      "Recursive building/updating of children",
+                                      TRUE,
+                                      G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT);
+
+	/**
+	 * GdaTreeManager:func: (type GdaTreeManagerNodesFunc)
 	 *
 	 * This property specifies the function which needs to be called when the list of #GdaTreeNode nodes
 	 * managed has to be updated
 	 */
-	g_object_class_install_property (object_class, PROP_FUNC,
-                                         g_param_spec_pointer ("func", NULL, "Function called when building/updating of children",
-                                                              G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
-	object_class->dispose = gda_tree_manager_dispose;
+	properties [PROP_FUNC] =
+                g_param_spec_pointer ("func",
+                                      NULL,
+                                      "Function called when building/updating of children",
+                                      G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT);
+
+	g_object_class_install_properties (object_class, LAST_PROP, properties);
 }
 
 static void
-gda_tree_manager_init (GdaTreeManager *manager, G_GNUC_UNUSED GdaTreeManagerClass *klass)
+gda_tree_manager_init (GdaTreeManager *manager)
 {
+	GdaTreeManagerPrivate *priv;
+
 	g_return_if_fail (GDA_IS_TREE_MANAGER (manager));
 
-	manager->priv = g_new0 (GdaTreeManagerPrivate, 1);
-	manager->priv->sub_managers = NULL;
-	manager->priv->ref_managers = NULL;
-	manager->priv->added_attributes = NULL;
+	priv = gda_tree_manager_get_instance_private (manager);
+	priv->sub_managers = NULL;
+	priv->ref_managers = NULL;
+	priv->added_attributes = NULL;
 }
 
 static void
 gda_tree_manager_dispose (GObject *object)
 {
 	GdaTreeManager *manager = (GdaTreeManager *) object;
+	GdaTreeManagerPrivate *priv;
 
 	g_return_if_fail (GDA_IS_TREE_MANAGER (manager));
 
-	if (manager->priv) {
-		if (manager->priv->sub_managers) {
-			g_slist_free (manager->priv->sub_managers);
-			manager->priv->sub_managers = NULL;
+	priv = gda_tree_manager_get_instance_private (manager);
+	g_clear_pointer (&priv->sub_managers, g_slist_free);
+	g_clear_object (&priv->ref_managers);
+	if (priv->added_attributes) {
+		GSList *list;
+		for (list = priv->added_attributes; list; list = list->next) {
+			AddedAttribute *aa = (AddedAttribute*) list->data;
+			g_free (aa->att_name);
+			if (aa->value)
+				gda_value_free (aa->value);
+			g_free (aa);
 		}
-		if (manager->priv->ref_managers) {
-			g_slist_foreach (manager->priv->ref_managers, (GFunc) g_object_unref, NULL);
-			g_slist_free (manager->priv->ref_managers);
-		}
-		if (manager->priv->added_attributes) {
-			GSList *list;
-			for (list = manager->priv->added_attributes; list; list = list->next) {
-				AddedAttribute *aa = (AddedAttribute*) list->data;
-				g_free (aa->att_name);
-				if (aa->value)
-					gda_value_free (aa->value);
-				g_free (aa);
-			}
-			g_slist_free (manager->priv->added_attributes);
-		}
-		g_free (manager->priv);
-		manager->priv = NULL;
 	}
 
+	g_clear_pointer (&priv->added_attributes, g_slist_free);
+
 	/* chain to parent class */
-	parent_class->dispose (object);
+	G_OBJECT_CLASS (gda_tree_manager_parent_class)->dispose (object);
 }
 
 
@@ -161,64 +179,25 @@ GQuark gda_tree_manager_error_quark (void)
         return quark;
 }
 
-/**
- * gda_tree_manager_get_type:
- * 
- * Registers the #GdaTreeManager class on the GLib type system.
- * 
- * Returns: the GType identifying the class.
- *
- * Since: 4.2
- */
-GType
-gda_tree_manager_get_type (void)
-{
-        static GType type = 0;
-
-        if (G_UNLIKELY (type == 0)) {
-                static GMutex registering;
-                static const GTypeInfo info = {
-                        sizeof (GdaTreeManagerClass),
-                        (GBaseInitFunc) NULL,
-                        (GBaseFinalizeFunc) NULL,
-                        (GClassInitFunc) gda_tree_manager_class_init,
-                        NULL,
-                        NULL,
-                        sizeof (GdaTreeManager),
-                        0,
-                        (GInstanceInitFunc) gda_tree_manager_init,
-			0
-                };
-
-                g_mutex_lock (&registering);
-                if (type == 0)
-                        type = g_type_register_static (G_TYPE_OBJECT, "GdaTreeManager", &info, 0);
-                g_mutex_unlock (&registering);
-        }
-        return type;
-}
-
 static void
 gda_tree_manager_set_property (GObject *object,
 			       guint param_id,
 			       const GValue *value,
 			       GParamSpec *pspec)
 {
-	GdaTreeManager *manager;
+	GdaTreeManager *manager = GDA_TREE_MANAGER (object);
+	GdaTreeManagerPrivate *priv = gda_tree_manager_get_instance_private (manager);
 
-        manager = GDA_TREE_MANAGER (object);
-        if (manager->priv) {
-                switch (param_id) {
-		case PROP_RECURS:
-			manager->priv->recursive = g_value_get_boolean (value);
-			break;
-		case PROP_FUNC:
-			manager->priv->update_func = g_value_get_pointer (value);
-			break;
-		default:
-			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
-			break;
-		}	
+        switch (param_id) {
+	case PROP_RECURS:
+		priv->recursive = g_value_get_boolean (value);
+		break;
+	case PROP_FUNC:
+		priv->update_func = g_value_get_pointer (value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+		break;
 	}
 }
 
@@ -228,30 +207,28 @@ gda_tree_manager_get_property (GObject *object,
 			       GValue *value,
 			       GParamSpec *pspec)
 {
-	GdaTreeManager *manager;
-	
-	manager = GDA_TREE_MANAGER (object);
-	if (manager->priv) {
-		switch (param_id) {
-		case PROP_RECURS:
-			g_value_set_boolean (value, manager->priv->recursive);
-			break;
-		case PROP_FUNC:
-			g_value_set_pointer (value, manager->priv->update_func);
-			break;
-		default:
-			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
-			break;
-		}
-	}	
+	GdaTreeManager *manager = GDA_TREE_MANAGER (object);
+	GdaTreeManagerPrivate *priv = gda_tree_manager_get_instance_private (manager);
+
+	switch (param_id) {
+	case PROP_RECURS:
+		g_value_set_boolean (value, priv->recursive);
+		break;
+	case PROP_FUNC:
+		g_value_set_pointer (value, priv->update_func);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+		break;
+	}
 }
 
 /*
  * @manager: a #GdaTreeManager object
  * @node: (allow-none): a #GdaTreeNode object, or %NULL
- * @children_nodes: a list of #GdaTreeNode nodes which have previously been created by a similar call and
+ * @children_nodes: (element-type GdaTreeNode): a list of #GdaTreeNode nodes which have previously been created by a similar call and
  * need to be updated ore moved
- * @out_error: (allow-none): a boolean to store if there was an error (can be %NULL)
+ * @out_error: (out) (allow-none): a boolean to store if there was an error (can be %NULL)
  * @error: a place to store errors, or %NULL
  *
  * Creates (or updates) the list of #GdaTreeNode objects which are placed as children of @node. The returned
@@ -260,8 +237,6 @@ gda_tree_manager_get_property (GObject *object,
  * If a node is already present in @children_nodes and needs to be kept in the new list, then it should be added
  * to the returned list and its reference count should be increased by one.
  *
- * Returns: a new list of #GdaTreeNode objects.
- *
  * Since: 4.2
  */
 void
@@ -269,17 +244,19 @@ _gda_tree_manager_update_children (GdaTreeManager *manager, GdaTreeNode *node, c
 				   gboolean *out_error, GError **error)
 {
 	GSList *nodes_list = NULL;
+	GdaTreeManagerPrivate *priv;
 
 	g_return_if_fail (GDA_IS_TREE_MANAGER (manager));
 	g_return_if_fail (GDA_IS_TREE_NODE (node));
 
+	priv = gda_tree_manager_get_instance_private (manager);
+
 	if (out_error)
 		*out_error = FALSE;
-	if (manager->priv->update_func)
-		nodes_list = manager->priv->update_func (manager, node, children_nodes, out_error, error);
+	if (priv->update_func)
+		nodes_list = priv->update_func (manager, node, children_nodes, out_error, error);
 	else {
-		GdaTreeManagerClass *klass;
-		klass = (GdaTreeManagerClass*) G_OBJECT_GET_CLASS (manager);
+		GdaTreeManagerClass *klass = GDA_TREE_MANAGER_GET_CLASS (manager);
 		if (klass->update_children)
 			nodes_list = klass->update_children (manager, node, children_nodes, out_error, error);
 	}
@@ -289,11 +266,11 @@ _gda_tree_manager_update_children (GdaTreeManager *manager, GdaTreeNode *node, c
 	GSList *list;
 	for (list = nodes_list; list; list = list->next) {
 		GSList *sl;
-		for (sl = manager->priv->sub_managers; sl; sl = sl->next) {
-			if (manager->priv->recursive) {
+		for (sl = priv->sub_managers; sl; sl = sl->next) {
+			if (priv->recursive) {
 				gboolean lout_error = FALSE;
 				GdaTreeNode *parent = GDA_TREE_NODE (list->data);
-				GdaTreeManager *mgr = (GdaTreeManager *) sl->data;
+				GdaTreeManager *mgr = GDA_TREE_MANAGER (sl->data);
 				_gda_tree_manager_update_children (mgr, parent,
 								   _gda_tree_node_get_children_for_manager (parent, mgr),
 								   &lout_error, error);
@@ -304,7 +281,7 @@ _gda_tree_manager_update_children (GdaTreeManager *manager, GdaTreeNode *node, c
 				}
 			}
 			else 
-				_gda_tree_node_add_children (GDA_TREE_NODE (list->data), (GdaTreeManager *) sl->data, NULL);	
+				_gda_tree_node_add_children (GDA_TREE_NODE (list->data), GDA_TREE_MANAGER (sl->data), NULL);
 		}
 	}
 	if (nodes_list)
@@ -329,7 +306,9 @@ gda_tree_manager_new_with_func (GdaTreeManagerNodesFunc update_func)
 {
 	g_return_val_if_fail (update_func, NULL);
 
-	return (GdaTreeManager*) g_object_new (GDA_TYPE_TREE_MANAGER, "func", update_func, NULL);
+	return (GdaTreeManager*) g_object_new (GDA_TYPE_TREE_MANAGER,
+	                                       "func", update_func,
+	                                       NULL);
 }
 
 /**
@@ -350,10 +329,13 @@ gda_tree_manager_add_new_node_attribute (GdaTreeManager *manager, const gchar *a
 {
 	AddedAttribute *aa = NULL;
 	GSList *list;
+	GdaTreeManagerPrivate *priv;
+
 	g_return_if_fail (GDA_IS_TREE_MANAGER (manager));
 	g_return_if_fail (attribute && *attribute);
-	
-	for (list = manager->priv->added_attributes; list; list = list->next) {
+
+	priv = gda_tree_manager_get_instance_private (manager);
+	for (list = priv->added_attributes; list; list = list->next) {
 		if (!strcmp (((AddedAttribute *) list->data)->att_name, attribute)) {
 			aa = (AddedAttribute *) list->data;
 			break;
@@ -362,7 +344,7 @@ gda_tree_manager_add_new_node_attribute (GdaTreeManager *manager, const gchar *a
 	if (!aa) {
 		aa = g_new0 (AddedAttribute, 1);
 		aa->att_name = g_strdup (attribute);
-		manager->priv->added_attributes = g_slist_append (manager->priv->added_attributes, aa);
+		priv->added_attributes = g_slist_append (priv->added_attributes, aa);
 	}
 	else if (aa->value) {
 		gda_value_free (aa->value);
@@ -394,13 +376,17 @@ gda_tree_manager_create_node (GdaTreeManager *manager, GdaTreeNode *parent, cons
 {
 	GdaTreeNode *node;
 	GSList *list;
+	GdaTreeManagerPrivate *priv;
+
 	g_return_val_if_fail (GDA_IS_TREE_MANAGER (manager), NULL);
-	if (manager->priv->node_create_func)
-		node = manager->priv->node_create_func (manager, parent, name);
+
+	priv = gda_tree_manager_get_instance_private (manager);
+	if (priv->node_create_func)
+		node = priv->node_create_func (manager, parent, name);
 	else
 		node = gda_tree_node_new (name);
 
-	for (list = manager->priv->added_attributes; list; list = list->next) {
+	for (list = priv->added_attributes; list; list = list->next) {
 		AddedAttribute *aa = (AddedAttribute*) list->data;
 		gda_tree_node_set_node_attribute (node, aa->att_name, aa->value, NULL);
 	}
@@ -411,13 +397,18 @@ gda_tree_manager_create_node (GdaTreeManager *manager, GdaTreeNode *parent, cons
 static gboolean
 manager_is_sub_manager_of (GdaTreeManager *mgr, GdaTreeManager *sub)
 {
-	if (g_slist_find (mgr->priv->ref_managers, sub))
+	GdaTreeManagerPrivate *priv;
+
+	priv = gda_tree_manager_get_instance_private (mgr);
+	if (g_slist_find (priv->ref_managers, sub))
 		return TRUE;
+
 	GSList *list;
-	for (list = mgr->priv->sub_managers; list; list = list->next) {
+	for (list = priv->sub_managers; list; list = list->next) {
 		if (manager_is_sub_manager_of (GDA_TREE_MANAGER (list->data), sub))
 			return TRUE;
 	}
+
 	return FALSE;
 }
 
@@ -439,14 +430,16 @@ manager_is_sub_manager_of (GdaTreeManager *mgr, GdaTreeManager *sub)
 void
 gda_tree_manager_add_manager (GdaTreeManager *manager, GdaTreeManager *sub)
 {
+	GdaTreeManagerPrivate *priv;
 	g_return_if_fail (GDA_IS_TREE_MANAGER (manager));
 	g_return_if_fail (GDA_IS_TREE_MANAGER (sub));
 
-	manager->priv->sub_managers = g_slist_append (manager->priv->sub_managers, sub);
+	priv = gda_tree_manager_get_instance_private (manager);
+	priv->sub_managers = g_slist_append (priv->sub_managers, sub);
 
 	/* determine if @sub should be ref'ed or not to avoid circular dependencies */
 	if ((sub != manager) && !manager_is_sub_manager_of (sub, manager)) {
-		manager->priv->ref_managers = g_slist_prepend (manager->priv->ref_managers, sub);
+		priv->ref_managers = g_slist_prepend (priv->ref_managers, sub);
 		g_object_ref (sub);
 	}
 }
@@ -465,8 +458,12 @@ gda_tree_manager_add_manager (GdaTreeManager *manager, GdaTreeManager *sub)
 const GSList *
 gda_tree_manager_get_managers (GdaTreeManager *manager)
 {
+	GdaTreeManagerPrivate *priv;
+
 	g_return_val_if_fail (GDA_IS_TREE_MANAGER (manager), NULL);
-	return manager->priv->sub_managers;
+
+	priv = gda_tree_manager_get_instance_private (manager);
+	return priv->sub_managers;
 }
 
 /**
@@ -485,8 +482,12 @@ gda_tree_manager_get_managers (GdaTreeManager *manager)
 void
 gda_tree_manager_set_node_create_func (GdaTreeManager *manager, GdaTreeManagerNodeFunc func)
 {
+	GdaTreeManagerPrivate *priv;
+
 	g_return_if_fail (GDA_IS_TREE_MANAGER (manager));
-	manager->priv->node_create_func = func;
+
+	priv = gda_tree_manager_get_instance_private (manager);
+	priv->node_create_func = func;
 }
 
 /**
@@ -502,6 +503,10 @@ gda_tree_manager_set_node_create_func (GdaTreeManager *manager, GdaTreeManagerNo
 GdaTreeManagerNodeFunc
 gda_tree_manager_get_node_create_func (GdaTreeManager *manager)
 {
+	GdaTreeManagerPrivate *priv;
+
 	g_return_val_if_fail (GDA_IS_TREE_MANAGER (manager), NULL);
-	return manager->priv->node_create_func;
+
+	priv = gda_tree_manager_get_instance_private (manager);
+	return priv->node_create_func;
 }
