@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2001 - 2003 Rodrigo Moya <rodrigo@gnome-db.org>
  * Copyright (C) 2002 - 2003 Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
- * Copyright (C) 2002 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2002 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2003 Akira TAGOH <tagoh@gnome-db.org>
  * Copyright (C) 2003 Danilo Schoeneberg <dschoene@src.gnome.org>
  * Copyright (C) 2003 Laurent Sansonetti <laurent@datarescue.be>
@@ -9,7 +9,7 @@
  * Copyright (C) 2004 Paisa  Seeluangsawat <paisa@users.sf.net>
  * Copyright (C) 2008 Przemysław Grzegorczyk <pgrzegorczyk@gmail.com>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
- * Copyright (C) 2011 Daniel Espinosa <despinosa@src.gnome.org>
+ * Copyright (C) 2011 - 2017 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -43,103 +43,92 @@ G_BEGIN_DECLS
 /* Definition of the GType's values used in GValue*/
 #define GDA_TYPE_NULL (gda_null_get_type())
 #define GDA_TYPE_DEFAULT (gda_default_get_type())
-#define	GDA_TYPE_BINARY (gda_binary_get_type())
 #define GDA_TYPE_BLOB (gda_blob_get_type())
-#define	GDA_TYPE_GEOMETRIC_POINT (gda_geometricpoint_get_type())
-#define	GDA_TYPE_NUMERIC (gda_numeric_get_type())
+#define	GDA_TYPE_GEOMETRIC_POINT (gda_geometric_point_get_type())
 #define	GDA_TYPE_SHORT (gda_short_get_type()) 
 #define	GDA_TYPE_USHORT (gda_ushort_get_type())
 #define GDA_TYPE_TIME (gda_time_get_type())
-#define GDA_TYPE_TIMESTAMP (gda_timestamp_get_type())
 
 
 /* Definition of the GDA_VALUE_HOLDS macros */
 #define GDA_VALUE_HOLDS_NULL(value)            G_VALUE_HOLDS(value, GDA_TYPE_NULL)
 #define GDA_VALUE_HOLDS_DEFAULT(value)         G_VALUE_HOLDS(value, GDA_TYPE_DEFAULT)
-#define GDA_VALUE_HOLDS_BINARY(value)          G_VALUE_HOLDS(value, GDA_TYPE_BINARY)
 #define GDA_VALUE_HOLDS_BLOB(value)            G_VALUE_HOLDS(value, GDA_TYPE_BLOB)
 #define GDA_VALUE_HOLDS_GEOMETRIC_POINT(value) G_VALUE_HOLDS(value, GDA_TYPE_GEOMETRIC_POINT)
-#define GDA_VALUE_HOLDS_NUMERIC(value)         G_VALUE_HOLDS(value, GDA_TYPE_NUMERIC)
 #define GDA_VALUE_HOLDS_SHORT(value)           G_VALUE_HOLDS(value, GDA_TYPE_SHORT)
 #define GDA_VALUE_HOLDS_USHORT(value)          G_VALUE_HOLDS(value, GDA_TYPE_USHORT)
 #define GDA_VALUE_HOLDS_TIME(value)            G_VALUE_HOLDS(value, GDA_TYPE_TIME)
-#define GDA_VALUE_HOLDS_TIMESTAMP(value)       G_VALUE_HOLDS(value, GDA_TYPE_TIMESTAMP)
-
-/**
- * GdaGeometricPoint:
- * @x:
- * @y:
- */
-typedef struct {
-	gdouble x;
-	gdouble y;
-} GdaGeometricPoint;
 
 
+/* GdaNumeric */
 typedef struct _GdaNumeric GdaNumeric;
-# ifdef GSEAL_ENABLE
-# else
-struct _GdaNumeric {
-	gchar*   GSEAL(number);
-	glong    GSEAL(precision);
-	glong    GSEAL(width);
-	
-	/*< private >*/
-	gpointer reserved; /* reserved for future usage with GMP (http://gmplib.org/) */
-};
-#endif
+
+#define	GDA_TYPE_NUMERIC (gda_numeric_get_type())
+#define GDA_VALUE_HOLDS_NUMERIC(value)         G_VALUE_HOLDS(value, GDA_TYPE_NUMERIC)
+
+GType                             gda_numeric_get_type (void) G_GNUC_CONST;
+GdaNumeric*                       gda_numeric_new (void);
+GdaNumeric*                       gda_numeric_copy (GdaNumeric *src);
+void                              gda_numeric_set_from_string (GdaNumeric *numeric, const gchar* str);
+void                              gda_numeric_set_double (GdaNumeric *numeric, gdouble number);
+gdouble                           gda_numeric_get_double (const GdaNumeric *numeric);
+void                              gda_numeric_set_precision (GdaNumeric *numeric, glong precision);
+glong                             gda_numeric_get_precision (const GdaNumeric *numeric);
+void                              gda_numeric_set_width (GdaNumeric *numeric, glong width);
+glong                             gda_numeric_get_width (const GdaNumeric *numeric);
+gchar*                            gda_numeric_get_string (const GdaNumeric *numeric);
+void                              gda_numeric_free (GdaNumeric *numeric);
 
 /**
- * GdaTime:
- * @hour: hour representation of the time, as a number between 0 and 23
- * @minute: minute representation of the time, as a number between 0 and 59
- * @second: second representation of the time, as a number between 0 and 59
- * @fraction: fractionnal part of the seconds, in millionth' of second
- * @timezone: number of seconds added to the GMT timezone
+ * GdaTime: (ref-func gda_time_new) (unref-func gda_time_free) (get-value-func gda_value_get_time) (set-value-func gda_value_set_time)
  *
  * Represents a time information.
  */
-typedef struct {
-	gushort hour;
-	gushort minute;
-	gushort second;
-	gulong  fraction;
-	glong   timezone;
-} GdaTime;
+typedef struct _GdaTime GdaTime;
+
+GType                             gda_time_get_type (void) G_GNUC_CONST;
+GdaTime*                          gda_time_new (void);
+GdaTime*                          gda_time_new_from_values (gushort hour, gushort minute, gushort second, gulong fraction, glong timezone);
+GdaTime*                          gda_time_copy (const GdaTime* time);
+void                              gda_time_free (GdaTime* time);
+
+gushort                           gda_time_get_hour (const GdaTime* time);
+void                              gda_time_set_hour (GdaTime* time, gushort hour);
+gushort                           gda_time_get_minute (const GdaTime* time);
+void                              gda_time_set_minute (GdaTime* time, gushort minute);
+gushort                           gda_time_get_second (const GdaTime* time);
+void                              gda_time_set_second (GdaTime* time, gushort second);
+gulong                            gda_time_get_fraction (const GdaTime *time);
+void                              gda_time_set_fraction (GdaTime* time, gulong fraction);
+glong                             gda_time_get_timezone (const GdaTime *time);
+void                              gda_time_set_timezone (GdaTime* time, glong timezone);
+
+gboolean                          gda_time_valid (const GdaTime *time);
+void                              gda_time_change_timezone (GdaTime *time, glong ntz);
 
 /**
- * GdaTimestamp:
- * @year: year representation of the time stamp
- * @month: month representation of the time stamp, as a number between 1 and 12
- * @day: day representation of the time stamp, as a number between 1 and 31
- * @hour: hour representation of the time stamp, as a number between 0 and 23
- * @minute: minute representation of the time stamp, as a number between 0 and 59
- * @second: second representation of the time stamp, as a number between 0 and 59
- * @fraction: fractionnal part of the seconds, in millionth' of second
- * @timezone: number of seconds added to the GMT timezone
- *
- * Represents an instant (a time stamp)
+ * GdaBinary: (ref-func gda_binary_new) (unref-func gda_binary_free) (get-value-func gda_value_get_binary) (set-value-func gda_value_set_binary)
  */
-typedef struct {
-	gshort  year;
-	gushort month;
-	gushort day;
-	gushort hour;
-	gushort minute;
-	gushort second;
-	gulong  fraction;
-	glong   timezone;
-} GdaTimestamp;
+typedef struct _GdaBinary GdaBinary;
 
-/**
- * GdaBinary:
- * @data:
- * @binary_length:
- */
-typedef struct {
-	guchar *data;
-	glong   binary_length;
-} GdaBinary;
+#define GDA_TYPE_BINARY (gda_binary_get_type ())
+#define GDA_VALUE_HOLDS_BINARY(value)          G_VALUE_HOLDS(value, GDA_TYPE_BINARY)
+
+GValue*                           gda_value_new_binary (const guchar *val, glong size);
+GdaBinary*                        gda_value_get_binary (const GValue *value);
+void                              gda_value_set_binary (GValue *value, GdaBinary *binary);
+void                              gda_value_take_binary (GValue *value, GdaBinary *binary);
+
+
+GType                             gda_binary_get_type (void) G_GNUC_CONST;
+GdaBinary*                        gda_binary_new (void);
+glong                             gda_binary_get_size (const GdaBinary* binary);
+gpointer                          gda_binary_get_data (GdaBinary* binary);
+void                              gda_binary_reset_data (GdaBinary* binary);
+void                              gda_binary_set_data (GdaBinary *binary, const guchar *val, glong size);
+void                              gda_binary_take_data (GdaBinary *binary, guchar *val, glong size);
+GdaBinary*                        gda_binary_copy (GdaBinary *src);
+void                              gda_binary_free (GdaBinary *binary);
 
 /**
  * GdaBlob:
@@ -150,10 +139,17 @@ typedef struct {
  * @op is generally set up by database providers when giving access to an existing BLOB in
  * a database, but can be modified if needed using gda_blob_set_op().
  */
-typedef struct {
-	GdaBinary  data;
-	GdaBlobOp *op;
-} GdaBlob;
+typedef struct _GdaBlob GdaBlob;
+
+
+GType                             gda_blob_get_type (void) G_GNUC_CONST;
+GdaBlob*                          gda_blob_new (void);
+GdaBinary*                        gda_blob_get_binary (GdaBlob *blob);
+GdaBlobOp*                        gda_blob_get_op (GdaBlob *blob);
+GdaBlob*                          gda_blob_copy (GdaBlob *src);
+void                              gda_blob_free (GdaBlob *blob);
+void                              gda_blob_set_op (GdaBlob *blob, GdaBlobOp *op);
+
 
 #define gda_value_isa(value, type) (G_VALUE_HOLDS(value, type))
 
@@ -198,10 +194,10 @@ GValue                           *gda_value_new (GType type);
 
 GValue                           *gda_value_new_null (void);
 GValue                           *gda_value_new_default (const gchar *default_val);
-GValue                           *gda_value_new_binary (const guchar *val, glong size);
 GValue                           *gda_value_new_blob (const guchar *val, glong size);
 GValue                           *gda_value_new_blob_from_file (const gchar *filename);
-GValue                           *gda_value_new_timestamp_from_timet (time_t val);
+GValue                           *gda_value_new_time_from_timet (time_t val);
+GValue                           *gda_value_new_date_time_from_timet (time_t val);
 
 GValue                           *gda_value_new_from_string (const gchar *as_string, GType type);
 GValue                           *gda_value_new_from_xml (const xmlNodePtr node);
@@ -213,27 +209,33 @@ gboolean                          gda_value_is_null (const GValue *value);
 gboolean                          gda_value_is_number (const GValue *value); 
 GValue                           *gda_value_copy (const GValue *value);
 
-const GdaBinary         *gda_value_get_binary (const GValue *value);
-void                              gda_value_set_binary (GValue *value, const GdaBinary *binary);
-void                              gda_value_take_binary (GValue *value, GdaBinary *binary);
 
-const GdaBlob           *gda_value_get_blob (const GValue *value);
+const GdaBlob                    *gda_value_get_blob (const GValue *value);
 void                              gda_value_set_blob (GValue *value, const GdaBlob *blob);
 void                              gda_value_take_blob (GValue *value, GdaBlob *blob);
 
-const GdaGeometricPoint *gda_value_get_geometric_point (const GValue *value);
+typedef struct _GdaGeometricPoint GdaGeometricPoint;
+GType                             gda_geometric_point_get_type (void) G_GNUC_CONST;
+GdaGeometricPoint                *gda_geometric_point_new (void);
+gdouble                           gda_geometric_point_get_x (GdaGeometricPoint* gp);
+void                              gda_geometric_point_set_x (GdaGeometricPoint* gp, double x);
+gdouble                           gda_geometric_point_get_y (GdaGeometricPoint* gp);
+void                              gda_geometric_point_set_y (GdaGeometricPoint* gp, double y);
+GdaGeometricPoint                *gda_geometric_point_copy (GdaGeometricPoint *gp);
+void                              gda_geometric_point_free (GdaGeometricPoint *gp);
+const GdaGeometricPoint          *gda_value_get_geometric_point (const GValue *value);
 void                              gda_value_set_geometric_point (GValue *value, const GdaGeometricPoint *val);
+
 void                              gda_value_set_null (GValue *value);
-const GdaNumeric        *gda_value_get_numeric (const GValue *value);
+
+const GdaNumeric                 *gda_value_get_numeric (const GValue *value);
 void                              gda_value_set_numeric (GValue *value, const GdaNumeric *val);
 gshort                            gda_value_get_short (const GValue *value);
 void                              gda_value_set_short (GValue *value, const gshort val);
 gushort                           gda_value_get_ushort (const GValue *value);
 void                              gda_value_set_ushort (GValue *value, const gushort val);
-const GdaTime           *gda_value_get_time (const GValue *value);
+const GdaTime                    *gda_value_get_time (const GValue *value);
 void                              gda_value_set_time (GValue *value, const GdaTime *val);
-const GdaTimestamp      *gda_value_get_timestamp (const GValue *value);
-void                              gda_value_set_timestamp (GValue *value, const GdaTimestamp *val);
 
 
 
@@ -247,6 +249,7 @@ gint                              gda_value_differ (const GValue *value1, const
 gint                              gda_value_compare (const GValue *value1, const GValue *value2);
 gchar                            *gda_value_stringify (const GValue *value);
 xmlNodePtr                        gda_value_to_xml (const GValue *value);
+gchar*                            gda_value_to_xml_string (const GValue *value);
 
 gchar                            *gda_binary_to_string (const GdaBinary *bin, guint maxlen);
 GdaBinary                        *gda_string_to_binary (const gchar *str);
@@ -258,44 +261,14 @@ GdaBlob                          *gda_string_to_blob (const gchar *str);
 
 GType                             gda_null_get_type (void) G_GNUC_CONST;
 GType                             gda_default_get_type (void) G_GNUC_CONST;
-GType                             gda_numeric_get_type (void) G_GNUC_CONST;
-GdaNumeric*                       gda_numeric_new (void);
-GdaNumeric*                       gda_numeric_copy (GdaNumeric *src);
-void                              gda_numeric_set_from_string (GdaNumeric *numeric, const gchar* str);
-void                              gda_numeric_set_double (GdaNumeric *numeric, gdouble number);
-gdouble                           gda_numeric_get_double (const GdaNumeric *numeric);
-void                              gda_numeric_set_precision (GdaNumeric *numeric, glong precision);
-glong                             gda_numeric_get_precision (const GdaNumeric *numeric);
-void                              gda_numeric_set_width (GdaNumeric *numeric, glong width);
-glong                             gda_numeric_get_width (const GdaNumeric *numeric);
-gchar*                            gda_numeric_get_string (const GdaNumeric *numeric);
-void                              gda_numeric_free (GdaNumeric *numeric);
-
-GType                             gda_time_get_type (void) G_GNUC_CONST;
-gpointer                          gda_time_copy (gpointer boxed);
-void                              gda_time_free (gpointer boxed);
-gboolean                          gda_time_valid (const GdaTime *time);
-void                              gda_time_change_timezone (GdaTime *time, glong ntz);
-
-GType                             gda_timestamp_get_type (void) G_GNUC_CONST;
-gpointer                          gda_timestamp_copy (gpointer boxed);
-void                              gda_timestamp_free (gpointer boxed);
-gboolean                          gda_timestamp_valid (const GdaTimestamp *timestamp);
-void                              gda_timestamp_change_timezone (GdaTimestamp *ts, glong ntz);
 
 
-GType                             gda_geometricpoint_get_type (void) G_GNUC_CONST;
-gpointer                          gda_geometricpoint_copy (gpointer boxed);
-void                              gda_geometricpoint_free (gpointer boxed);
 
-GType                             gda_binary_get_type (void) G_GNUC_CONST;
-gpointer                          gda_binary_copy (gpointer boxed);
-void                              gda_binary_free (gpointer boxed);
 
-GType                             gda_blob_get_type (void) G_GNUC_CONST;
-gpointer                          gda_blob_copy (gpointer boxed);
-void                              gda_blob_free (gpointer boxed);
-void                              gda_blob_set_op (GdaBlob *blob, GdaBlobOp *op);
+
+/* Timestamp based on GDateTime */
+
+GDateTime                        *gda_date_time_copy (GDateTime *ts);
 
 GType                             gda_short_get_type (void) G_GNUC_CONST;
 GType                             gda_ushort_get_type (void) G_GNUC_CONST;
