@@ -26,145 +26,25 @@
 #include <locale.h>
 #include <glib/gi18n-lib.h>
 
-static void gda_handler_numerical_class_init (GdaHandlerNumericalClass * class);
-static void gda_handler_numerical_init (GdaHandlerNumerical * wid);
-static void gda_handler_numerical_dispose (GObject   * object);
+struct _GdaHandlerNumerical
+{
+	GObject   parent_instance;
 
-
-/* GdaDataHandler interface */
-static void         gda_handler_numerical_data_handler_init      (GdaDataHandlerIface *iface);
-static gchar       *gda_handler_numerical_get_sql_from_value     (GdaDataHandler *dh, const GValue *value);
-static gchar       *gda_handler_numerical_get_str_from_value     (GdaDataHandler *dh, const GValue *value);
-static GValue      *gda_handler_numerical_get_value_from_sql     (GdaDataHandler *dh, const gchar *sql,
-								  GType type);
-static GValue      *gda_handler_numerical_get_value_from_str     (GdaDataHandler *dh, const gchar *str,
-								  GType type);
-static GValue      *gda_handler_numerical_get_sane_init_value    (GdaDataHandler * dh, GType type);
-
-static gboolean     gda_handler_numerical_accepts_g_type       (GdaDataHandler * dh, GType type);
-
-static const gchar *gda_handler_numerical_get_descr              (GdaDataHandler *dh);
-
-struct  _GdaHandlerNumericalPriv {
 	guint           nb_g_types;
 	GType          *valid_g_types;
 };
 
-/* get a pointer to the parents to be able to call their destructor */
-static GObjectClass *parent_class = NULL;
+static void data_handler_iface_init (GdaDataHandlerIface *iface);
 
-GType
-gda_handler_numerical_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0)) {
-		static GMutex registering;
-		static const GTypeInfo info = {
-			sizeof (GdaHandlerNumericalClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) gda_handler_numerical_class_init,
-			NULL,
-			NULL,
-			sizeof (GdaHandlerNumerical),
-			0,
-			(GInstanceInitFunc) gda_handler_numerical_init,
-			NULL
-		};
-
-		static const GInterfaceInfo data_entry_info = {
-			(GInterfaceInitFunc) gda_handler_numerical_data_handler_init,
-			NULL,
-			NULL
-		};
-
-		g_mutex_lock (&registering);
-		if (type == 0) {
-			type = g_type_register_static (G_TYPE_OBJECT, "GdaHandlerNumerical", &info, 0);
-			g_type_add_interface_static (type, GDA_TYPE_DATA_HANDLER, &data_entry_info);
-		}
-		g_mutex_unlock (&registering);
-	}
-	return type;
-}
-
-static void
-gda_handler_numerical_data_handler_init (GdaDataHandlerIface *iface)
-{
-	iface->get_sql_from_value = gda_handler_numerical_get_sql_from_value;
-	iface->get_str_from_value = gda_handler_numerical_get_str_from_value;
-	iface->get_value_from_sql = gda_handler_numerical_get_value_from_sql;
-	iface->get_value_from_str = gda_handler_numerical_get_value_from_str;
-	iface->get_sane_init_value = gda_handler_numerical_get_sane_init_value;
-	iface->accepts_g_type = gda_handler_numerical_accepts_g_type;
-	iface->get_descr = gda_handler_numerical_get_descr;
-}
-
-
-static void
-gda_handler_numerical_class_init (GdaHandlerNumericalClass * class)
-{
-	GObjectClass   *object_class = G_OBJECT_CLASS (class);
-
-	parent_class = g_type_class_peek_parent (class);
-
-	object_class->dispose = gda_handler_numerical_dispose;
-}
-
-static void
-gda_handler_numerical_init (GdaHandlerNumerical * hdl)
-{
-	/* Private structure */
-	hdl->priv = g_new0 (GdaHandlerNumericalPriv, 1);
-	hdl->priv->nb_g_types = 13;
-	hdl->priv->valid_g_types = g_new0 (GType, hdl->priv->nb_g_types);
-	hdl->priv->valid_g_types[0] = G_TYPE_INT64;
-	hdl->priv->valid_g_types[1] = G_TYPE_DOUBLE;
-	hdl->priv->valid_g_types[2] = G_TYPE_INT;
-        hdl->priv->valid_g_types[3] = GDA_TYPE_NUMERIC;
-        hdl->priv->valid_g_types[4] = G_TYPE_FLOAT;
-        hdl->priv->valid_g_types[5] = GDA_TYPE_SHORT;
-        hdl->priv->valid_g_types[6] = G_TYPE_CHAR;
-	hdl->priv->valid_g_types[7] = G_TYPE_UINT64;
-        hdl->priv->valid_g_types[8] = GDA_TYPE_USHORT;
-        hdl->priv->valid_g_types[9] = G_TYPE_UCHAR;
-	hdl->priv->valid_g_types[10] = G_TYPE_UINT;
-	hdl->priv->valid_g_types[11] = G_TYPE_ULONG;
-	hdl->priv->valid_g_types[12] = G_TYPE_LONG;
-
-	g_object_set_data (G_OBJECT (hdl), "name", "InternalNumerical");
-	g_object_set_data (G_OBJECT (hdl), "descr", _("Numerical representation"));
-}
-
-static void
-gda_handler_numerical_dispose (GObject *object)
-{
-	GdaHandlerNumerical *hdl;
-
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (GDA_IS_HANDLER_NUMERICAL (object));
-
-	hdl = GDA_HANDLER_NUMERICAL (object);
-
-	if (hdl->priv) {
-		g_free (hdl->priv->valid_g_types);
-		hdl->priv->valid_g_types = NULL;
-
-		g_free (hdl->priv);
-		hdl->priv = NULL;
-	}
-
-	/* for the parent class */
-	parent_class->dispose (object);
-}
+G_DEFINE_TYPE_EXTENDED (GdaHandlerNumerical, gda_handler_numerical, G_TYPE_OBJECT, 0,
+                        G_IMPLEMENT_INTERFACE (GDA_TYPE_DATA_HANDLER, data_handler_iface_init))
 
 /**
  * gda_handler_numerical_new:
  *
  * Creates a data handler for numerical values
  *
- * Returns: (transfer full): the new object
+ * Returns: (type GdaHandlerNumerical) (transfer full): the new object
  */
 GdaDataHandler *
 gda_handler_numerical_new (void)
@@ -393,8 +273,8 @@ gda_handler_numerical_accepts_g_type (GdaDataHandler *iface, GType type)
 
 	g_assert (iface);
 	hdl = (GdaHandlerNumerical*) (iface);
-	for (i = 0; i < hdl->priv->nb_g_types; i++) {
-		if (hdl->priv->valid_g_types [i] == type)
+	for (i = 0; i < hdl->nb_g_types; i++) {
+		if (hdl->valid_g_types [i] == type)
 			return TRUE;
 	}
 
@@ -407,3 +287,64 @@ gda_handler_numerical_get_descr (GdaDataHandler *iface)
 	g_return_val_if_fail (GDA_IS_HANDLER_NUMERICAL (iface), NULL);
 	return g_object_get_data (G_OBJECT (iface), "descr");
 }
+
+static void
+gda_handler_numerical_init (GdaHandlerNumerical * hdl)
+{
+	/* Private structure */
+	hdl->nb_g_types = 13;
+	hdl->valid_g_types = g_new0 (GType, hdl->nb_g_types);
+	hdl->valid_g_types[0] = G_TYPE_INT64;
+	hdl->valid_g_types[1] = G_TYPE_DOUBLE;
+	hdl->valid_g_types[2] = G_TYPE_INT;
+        hdl->valid_g_types[3] = GDA_TYPE_NUMERIC;
+        hdl->valid_g_types[4] = G_TYPE_FLOAT;
+        hdl->valid_g_types[5] = GDA_TYPE_SHORT;
+        hdl->valid_g_types[6] = G_TYPE_CHAR;
+	hdl->valid_g_types[7] = G_TYPE_UINT64;
+        hdl->valid_g_types[8] = GDA_TYPE_USHORT;
+        hdl->valid_g_types[9] = G_TYPE_UCHAR;
+	hdl->valid_g_types[10] = G_TYPE_UINT;
+	hdl->valid_g_types[11] = G_TYPE_ULONG;
+	hdl->valid_g_types[12] = G_TYPE_LONG;
+
+	g_object_set_data (G_OBJECT (hdl), "name", "InternalNumerical");
+	g_object_set_data (G_OBJECT (hdl), "descr", _("Numerical representation"));
+}
+
+static void
+gda_handler_numerical_dispose (GObject *object)
+{
+	GdaHandlerNumerical *hdl;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (GDA_IS_HANDLER_NUMERICAL (object));
+
+	hdl = GDA_HANDLER_NUMERICAL (object);
+
+	g_clear_pointer (&hdl->valid_g_types, g_free);
+
+	/* for the parent class */
+	G_OBJECT_CLASS (gda_handler_numerical_parent_class)->dispose (object);
+}
+
+static void
+data_handler_iface_init (GdaDataHandlerIface *iface)
+{
+	iface->get_sql_from_value = gda_handler_numerical_get_sql_from_value;
+	iface->get_str_from_value = gda_handler_numerical_get_str_from_value;
+	iface->get_value_from_sql = gda_handler_numerical_get_value_from_sql;
+	iface->get_value_from_str = gda_handler_numerical_get_value_from_str;
+	iface->get_sane_init_value = gda_handler_numerical_get_sane_init_value;
+	iface->accepts_g_type = gda_handler_numerical_accepts_g_type;
+	iface->get_descr = gda_handler_numerical_get_descr;
+}
+
+
+static void
+gda_handler_numerical_class_init (GdaHandlerNumericalClass * class)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (class);
+
+	object_class->dispose = gda_handler_numerical_dispose;
+}
