@@ -28,131 +28,25 @@
 #include <libgda/gda-util.h>
 #include <libgda/gda-server-provider.h>
 
-static void gda_handler_string_class_init (GdaHandlerStringClass * class);
-static void gda_handler_string_init (GdaHandlerString * wid);
-static void gda_handler_string_dispose (GObject   * object);
+struct _GdaHandlerString
+{
+  GObject            parent_instance;
 
-
-/* GdaDataHandler interface */
-static void         gda_handler_string_data_handler_init      (GdaDataHandlerIface *iface);
-static gchar       *gda_handler_string_get_sql_from_value     (GdaDataHandler *dh, const GValue *value);
-static gchar       *gda_handler_string_get_str_from_value     (GdaDataHandler *dh, const GValue *value);
-static GValue      *gda_handler_string_get_value_from_sql     (GdaDataHandler *dh, const gchar *sql, 
-							       GType type);
-static GValue      *gda_handler_string_get_value_from_str     (GdaDataHandler *dh, const gchar *sql, 
-							       GType type);
-static GValue      *gda_handler_string_get_sane_init_value    (GdaDataHandler * dh, GType type);
-
-static gboolean     gda_handler_string_accepts_g_type       (GdaDataHandler * dh, GType type);
-
-static const gchar *gda_handler_string_get_descr              (GdaDataHandler *dh);
-
-struct  _GdaHandlerStringPriv {
-	GdaServerProvider *prov;
-	GdaConnection     *cnc;
+  GdaServerProvider *prov;
+  GdaConnection     *cnc;
 };
 
-/* get a pointer to the parents to be able to call their destructor */
-static GObjectClass *parent_class = NULL;
+static void data_handler_iface_init (GdaDataHandlerIface *iface);
 
-GType
-gda_handler_string_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0)) {
-		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
-		static const GTypeInfo info = {
-			sizeof (GdaHandlerStringClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) gda_handler_string_class_init,
-			NULL,
-			NULL,
-			sizeof (GdaHandlerString),
-			0,
-			(GInstanceInitFunc) gda_handler_string_init,
-			NULL
-		};		
-
-		static const GInterfaceInfo data_entry_info = {
-			(GInterfaceInitFunc) gda_handler_string_data_handler_init,
-			NULL,
-			NULL
-		};
-
-		g_static_mutex_lock (&registering);
-		if (type == 0) {
-			type = g_type_register_static (G_TYPE_OBJECT, "GdaHandlerString", &info, 0);
-			g_type_add_interface_static (type, GDA_TYPE_DATA_HANDLER, &data_entry_info);
-		}
-		g_static_mutex_unlock (&registering);
-	}
-	return type;
-}
-
-static void
-gda_handler_string_data_handler_init (GdaDataHandlerIface *iface)
-{
-	iface->get_sql_from_value = gda_handler_string_get_sql_from_value;
-	iface->get_str_from_value = gda_handler_string_get_str_from_value;
-	iface->get_value_from_sql = gda_handler_string_get_value_from_sql;
-	iface->get_value_from_str = gda_handler_string_get_value_from_str;
-	iface->get_sane_init_value = gda_handler_string_get_sane_init_value;
-	iface->accepts_g_type = gda_handler_string_accepts_g_type;
-	iface->get_descr = gda_handler_string_get_descr;
-}
-
-
-static void
-gda_handler_string_class_init (GdaHandlerStringClass * class)
-{
-	GObjectClass   *object_class = G_OBJECT_CLASS (class);
-	
-	parent_class = g_type_class_peek_parent (class);
-
-	object_class->dispose = gda_handler_string_dispose;
-}
-
-static void
-gda_handler_string_init (GdaHandlerString * hdl)
-{
-	/* Private structure */
-	hdl->priv = g_new0 (GdaHandlerStringPriv, 1);
-	g_object_set_data (G_OBJECT (hdl), "name", _("InternalString"));
-	g_object_set_data (G_OBJECT (hdl), "descr", _("Strings representation"));
-}
-
-static void
-gda_handler_string_dispose (GObject   * object)
-{
-	GdaHandlerString *hdl;
-
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (GDA_IS_HANDLER_STRING (object));
-
-	hdl = GDA_HANDLER_STRING (object);
-
-	if (hdl->priv) {
-		if (hdl->priv->prov)
-			g_object_remove_weak_pointer (G_OBJECT (hdl->priv->prov), (gpointer) &(hdl->priv->prov));
-		if (hdl->priv->cnc)
-			g_object_remove_weak_pointer (G_OBJECT (hdl->priv->cnc), (gpointer) &(hdl->priv->cnc));
-
-		g_free (hdl->priv);
-		hdl->priv = NULL;
-	}
-
-	/* for the parent class */
-	parent_class->dispose (object);
-}
+G_DEFINE_TYPE_EXTENDED (GdaHandlerString, gda_handler_string, G_TYPE_OBJECT, 0,
+                        G_IMPLEMENT_INTERFACE (GDA_TYPE_DATA_HANDLER, data_handler_iface_init))
 
 /**
  * gda_handler_string_new:
  *
  * Creates a data handler for strings
  *
- * Returns: (transfer full): the new object
+ * Returns: (type GdaHandlerString) (transfer full): the new object
  */
 GdaDataHandler *
 gda_handler_string_new (void)
@@ -172,7 +66,7 @@ gda_handler_string_new (void)
  * Creates a data handler for strings, which will use some specific methods implemented
  * by the @prov object (possibly also @cnc).
  *
- * Returns: (transfer full): the new object
+ * Returns: (type GdaHandlerString) (transfer full): the new object
  */
 GdaDataHandler *
 gda_handler_string_new_with_provider (GdaServerProvider *prov, GdaConnection *cnc)
@@ -186,13 +80,13 @@ gda_handler_string_new_with_provider (GdaServerProvider *prov, GdaConnection *cn
 	obj = g_object_new (GDA_TYPE_HANDLER_STRING, NULL);
 	dh = (GdaHandlerString*) obj;
 
-	dh->priv->prov = prov;
+	dh->prov = prov;
 	if (cnc)
-		dh->priv->cnc = cnc;
+		dh->cnc = cnc;
 
-	g_object_add_weak_pointer (G_OBJECT (prov), (gpointer) &(dh->priv->prov));
+	g_object_add_weak_pointer (G_OBJECT (prov), (gpointer) &(dh->prov));
 	if (cnc)
-		g_object_add_weak_pointer (G_OBJECT (cnc), (gpointer) &(dh->priv->cnc));
+		g_object_add_weak_pointer (G_OBJECT (cnc), (gpointer) &(dh->cnc));
 
 	return (GdaDataHandler *) obj;
 }
@@ -211,8 +105,8 @@ gda_handler_string_get_sql_from_value (GdaDataHandler *iface, const GValue *valu
 	str = gda_value_stringify ((GValue *) value);
 	if (str) {
 		gchar *str2;
-		if (hdl->priv->prov) 
-			str2 = gda_server_provider_escape_string (hdl->priv->prov, hdl->priv->cnc, str);
+		if (hdl->prov) 
+			str2 = gda_server_provider_escape_string (hdl->prov, hdl->cnc, str);
 		else 
 			str2 = gda_default_escape_string (str);
 		retval = g_strdup_printf ("'%s'", str2);
@@ -250,8 +144,8 @@ gda_handler_string_get_value_from_sql (GdaDataHandler *iface, const gchar *sql,
 			gchar *unstr;
 
 			str[i-1] = 0;
-			if (hdl->priv->prov)
-				unstr = gda_server_provider_unescape_string (hdl->priv->prov, hdl->priv->cnc, str+1);
+			if (hdl->prov)
+				unstr = gda_server_provider_unescape_string (hdl->prov, hdl->cnc, str+1);
 			else
 				unstr = gda_default_unescape_string (str+1);
 			if (unstr) {
@@ -301,3 +195,50 @@ gda_handler_string_get_descr (GdaDataHandler *iface)
 	g_return_val_if_fail (GDA_IS_HANDLER_STRING (iface), NULL);
 	return g_object_get_data (G_OBJECT (iface), "descr");
 }
+
+static void
+gda_handler_string_init (GdaHandlerString * hdl)
+{
+	/* Private structure */
+	g_object_set_data (G_OBJECT (hdl), "name", _("InternalString"));
+	g_object_set_data (G_OBJECT (hdl), "descr", _("Strings representation"));
+}
+
+static void
+gda_handler_string_dispose (GObject   *object)
+{
+	GdaHandlerString *hdl = (GdaHandlerString *)object;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (GDA_IS_HANDLER_STRING (object));
+
+	hdl = GDA_HANDLER_STRING (object);
+
+	if (hdl->prov)
+		g_object_remove_weak_pointer (G_OBJECT (hdl->prov), (gpointer) &(hdl->prov));
+	if (hdl->cnc)
+		g_object_remove_weak_pointer (G_OBJECT (hdl->cnc), (gpointer) &(hdl->cnc));
+
+	/* for the parent class */
+	G_OBJECT_CLASS (gda_handler_string_parent_class)->dispose (object);
+}
+
+static void
+data_handler_iface_init (GdaDataHandlerIface *iface)
+{
+	iface->get_sql_from_value = gda_handler_string_get_sql_from_value;
+	iface->get_str_from_value = gda_handler_string_get_str_from_value;
+	iface->get_value_from_sql = gda_handler_string_get_value_from_sql;
+	iface->get_value_from_str = gda_handler_string_get_value_from_str;
+	iface->get_sane_init_value = gda_handler_string_get_sane_init_value;
+	iface->accepts_g_type = gda_handler_string_accepts_g_type;
+	iface->get_descr = gda_handler_string_get_descr;
+}
+
+static void
+gda_handler_string_class_init (GdaHandlerStringClass * class)
+{
+	GObjectClass   *object_class = G_OBJECT_CLASS (class);
+
+	object_class->dispose = gda_handler_string_dispose;
+}
