@@ -1,10 +1,10 @@
 /*
- * Copyright (C) 2006 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2006 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2007 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
- * Copyright (C) 2011 Daniel Espinosa <despinosa@src.gnome.org>
+ * Copyright (C) 2011-2017 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,10 +29,24 @@
 #include <glib/gi18n-lib.h>
 #include <libgda/gda-debug-macros.h>
 
-static void gda_handler_time_class_init (GdaHandlerTimeClass *class);
-static void gda_handler_time_init (GdaHandlerTime *hdl);
-static void gda_handler_time_dispose (GObject *object);
-static void handler_compute_locale (GdaHandlerTime *hdl);
+typedef struct _LocaleSetting {
+	GDateDMY        dmy_order[3];
+	gboolean        twodigit_years;
+	gint            current_offset; /* 1900, 2000, etc... */
+	gchar           separator;	
+} LocaleSetting;
+
+struct _GdaHandlerTime
+{
+	GObject   parent_instance;
+
+	guint   nb_g_types;
+	GType  *valid_g_types;
+
+	/* for locale setting */
+	LocaleSetting  *sql_locale;
+	LocaleSetting  *str_locale;
+};
 
 /* General notes:
  * about months representations:
@@ -47,165 +61,19 @@ static void handler_compute_locale (GdaHandlerTime *hdl);
  * in file gdate.c
  */
 
-/* GdaDataHandler interface */
-static void         gda_handler_time_data_handler_init      (GdaDataHandlerIface *iface);
-static gchar       *gda_handler_time_get_sql_from_value     (GdaDataHandler *dh, const GValue *value);
-static gchar       *gda_handler_time_get_str_from_value     (GdaDataHandler *dh, const GValue *value);
-static GValue      *gda_handler_time_get_value_from_sql     (GdaDataHandler *dh, const gchar *sql,
-							     GType type);
-static GValue      *gda_handler_time_get_value_from_str     (GdaDataHandler *dh, const gchar *sql,
-							     GType type);
+static void   data_handler_iface_init (GdaDataHandlerIface *iface);
+static gchar *render_date_locale      (const GDate *date, LocaleSetting *locale);
+static void   handler_compute_locale  (GdaHandlerTime *hdl);
 
-static GValue      *gda_handler_time_get_sane_init_value    (GdaDataHandler * dh, GType type);
-
-static gboolean     gda_handler_time_accepts_g_type       (GdaDataHandler * dh, GType type);
-
-static const gchar *gda_handler_time_get_descr              (GdaDataHandler *dh);
-
-typedef struct _LocaleSetting {
-	GDateDMY        dmy_order[3];
-	gboolean        twodigit_years;
-	gint            current_offset; /* 1900, 2000, etc... */
-	gchar           separator;	
-} LocaleSetting;
-
-static gchar *render_date_locale (const GDate *date, LocaleSetting *locale);
-
-struct  _GdaHandlerTimePriv {
-	guint           nb_g_types;
-	GType          *valid_g_types;
-
-	/* for locale setting */
-	LocaleSetting  *sql_locale;
-	LocaleSetting  *str_locale;
-};
-
-/* get a pointer to the parents to be able to call their destructor */
-static GObjectClass *parent_class = NULL;
-
-GType
-gda_handler_time_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0)) {
-		static GMutex registering;
-		static const GTypeInfo info = {
-			sizeof (GdaHandlerTimeClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) gda_handler_time_class_init,
-			NULL,
-			NULL,
-			sizeof (GdaHandlerTime),
-			0,
-			(GInstanceInitFunc) gda_handler_time_init,
-			NULL
-		};		
-
-		static const GInterfaceInfo data_entry_info = {
-			(GInterfaceInitFunc) gda_handler_time_data_handler_init,
-			NULL,
-			NULL
-		};
-
-		g_mutex_lock (&registering);
-		if (type == 0) {
-			type = g_type_register_static (G_TYPE_OBJECT, "GdaHandlerTime", &info, 0);
-			g_type_add_interface_static (type, GDA_TYPE_DATA_HANDLER, &data_entry_info);
-		}
-		g_mutex_unlock (&registering);
-	}
-	return type;
-}
-
-static void
-gda_handler_time_data_handler_init (GdaDataHandlerIface *iface)
-{
-	iface->get_sql_from_value = gda_handler_time_get_sql_from_value;
-	iface->get_str_from_value = gda_handler_time_get_str_from_value;
-	iface->get_value_from_sql = gda_handler_time_get_value_from_sql;
-	iface->get_value_from_str = gda_handler_time_get_value_from_str;
-	iface->get_sane_init_value = gda_handler_time_get_sane_init_value;
-	iface->accepts_g_type = gda_handler_time_accepts_g_type;
-	iface->get_descr = gda_handler_time_get_descr;
-}
-
-
-static void
-gda_handler_time_class_init (GdaHandlerTimeClass * class)
-{
-	GObjectClass   *object_class = G_OBJECT_CLASS (class);
-	
-	parent_class = g_type_class_peek_parent (class);
-
-	object_class->dispose = gda_handler_time_dispose;
-}
-
-static void
-gda_handler_time_init (GdaHandlerTime *hdl)
-{
-	/* Private structure */
-	hdl->priv = g_new0 (GdaHandlerTimePriv, 1);
-	hdl->priv->nb_g_types = 4;
-	hdl->priv->valid_g_types = g_new0 (GType, 7);
-	hdl->priv->valid_g_types[0] = G_TYPE_DATE;
-	hdl->priv->valid_g_types[1] = GDA_TYPE_TIME;
-	hdl->priv->valid_g_types[2] = GDA_TYPE_TIMESTAMP;
-	hdl->priv->valid_g_types[3] = G_TYPE_DATE_TIME;
-
-	/* taking into accout the locale */
-	hdl->priv->sql_locale = g_new0 (LocaleSetting, 1);
-	hdl->priv->sql_locale->dmy_order[0] = G_DATE_MONTH;
-	hdl->priv->sql_locale->dmy_order[1] = G_DATE_DAY;
-	hdl->priv->sql_locale->dmy_order[2] = G_DATE_YEAR;
-	hdl->priv->sql_locale->twodigit_years = FALSE;
-	hdl->priv->sql_locale->current_offset = 0;
-	hdl->priv->sql_locale->separator = '-';
-
-	hdl->priv->str_locale = g_new0 (LocaleSetting, 1);
-	hdl->priv->str_locale->dmy_order[0] = G_DATE_MONTH;
-	hdl->priv->str_locale->dmy_order[1] = G_DATE_DAY;
-	hdl->priv->str_locale->dmy_order[2] = G_DATE_YEAR;
-	hdl->priv->str_locale->twodigit_years = FALSE;
-	hdl->priv->str_locale->current_offset = 0;
-	hdl->priv->str_locale->separator = '-';
-
-	g_object_set_data (G_OBJECT (hdl), "name", "InternalTime");
-	g_object_set_data (G_OBJECT (hdl), "descr", _("Time, Date and TimeStamp representation"));
-}
-
-static void
-gda_handler_time_dispose (GObject *object)
-{
-	GdaHandlerTime *hdl;
-
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (GDA_IS_HANDLER_TIME (object));
-
-	hdl = GDA_HANDLER_TIME (object);
-
-	if (hdl->priv) {
-		g_free (hdl->priv->valid_g_types);
-		hdl->priv->valid_g_types = NULL;
-
-		g_free (hdl->priv->str_locale);
-		g_free (hdl->priv->sql_locale);
-
-		g_free (hdl->priv);
-		hdl->priv = NULL;
-	}
-
-	/* for the parent class */
-	parent_class->dispose (object);
-}
+G_DEFINE_TYPE_EXTENDED (GdaHandlerTime, gda_handler_time, G_TYPE_OBJECT, 0,
+                        G_IMPLEMENT_INTERFACE (GDA_TYPE_DATA_HANDLER, data_handler_iface_init))
 
 /**
  * gda_handler_time_new:
  *
  * Creates a data handler for time values
  *
- * Returns: (transfer full): the new object
+ * Returns: (type GdaHandlerTime) (transfer full): the new object
  */
 GdaDataHandler *
 gda_handler_time_new (void)
@@ -224,7 +92,7 @@ gda_handler_time_new (void)
  * Creates a data handler for time values, but using the default C locale
  * instead of the current user locale.
  *
- * Returns: (transfer full): the new object
+ * Returns: (type GdaHandlerTime) (transfer full): the new object
  */
 GdaDataHandler *
 gda_handler_time_new_no_locale (void)
@@ -257,16 +125,15 @@ gda_handler_time_set_sql_spec  (GdaHandlerTime *dh, GDateDMY first, GDateDMY sec
 				GDateDMY third, gchar separator, gboolean twodigits_years)
 {
 	g_return_if_fail (GDA_IS_HANDLER_TIME (dh));
-	g_return_if_fail (dh->priv);
 	g_return_if_fail (first != sec);
 	g_return_if_fail (sec != third);
 	g_return_if_fail (first != third);
 
-	dh->priv->sql_locale->dmy_order[0] = first;
-	dh->priv->sql_locale->dmy_order[1] = sec;
-	dh->priv->sql_locale->dmy_order[2] = third;
-	dh->priv->sql_locale->twodigit_years = twodigits_years;
-	dh->priv->sql_locale->separator = separator;
+	dh->sql_locale->dmy_order[0] = first;
+	dh->sql_locale->dmy_order[1] = sec;
+	dh->sql_locale->dmy_order[2] = third;
+	dh->sql_locale->twodigit_years = twodigits_years;
+	dh->sql_locale->separator = separator;
 }
 
 /**
@@ -293,16 +160,15 @@ gda_handler_time_set_str_spec  (GdaHandlerTime *dh, GDateDMY first, GDateDMY sec
 				GDateDMY third, gchar separator, gboolean twodigits_years)
 {
 	g_return_if_fail (GDA_IS_HANDLER_TIME (dh));
-	g_return_if_fail (dh->priv);
 	g_return_if_fail (first != sec);
 	g_return_if_fail (sec != third);
 	g_return_if_fail (first != third);
 
-	dh->priv->str_locale->dmy_order[0] = first;
-	dh->priv->str_locale->dmy_order[1] = sec;
-	dh->priv->str_locale->dmy_order[2] = third;
-	dh->priv->str_locale->twodigit_years = twodigits_years;
-	dh->priv->str_locale->separator = separator;
+	dh->str_locale->dmy_order[0] = first;
+	dh->str_locale->dmy_order[1] = sec;
+	dh->str_locale->dmy_order[2] = third;
+	dh->str_locale->twodigit_years = twodigits_years;
+	dh->str_locale->separator = separator;
 }
 
 static void
@@ -323,7 +189,7 @@ handler_compute_locale (GdaHandlerTime *hdl)
 	while (*ptr && g_ascii_isdigit (*ptr))
 		ptr++;
 	if (*ptr) {
-		hdl->priv->str_locale->separator = *ptr;
+		hdl->str_locale->separator = *ptr;
 		*ptr = 0;
 		nums[0] = atoi (numstart); /* Flawfinder: ignore */
 	}
@@ -366,15 +232,17 @@ handler_compute_locale (GdaHandlerTime *hdl)
 		for (i=0; i < 3; i++) {
 			switch (nums[i]) {
 			case 7:
-				hdl->priv->str_locale->dmy_order[i] = G_DATE_MONTH;
+				hdl->str_locale->dmy_order[i] = G_DATE_MONTH;
 				break;
 			case 4:
-				hdl->priv->str_locale->dmy_order[i] = G_DATE_DAY;
+				hdl->str_locale->dmy_order[i] = G_DATE_DAY;
 				break;
 			case 76:
-				hdl->priv->str_locale->twodigit_years = TRUE;
+				hdl->str_locale->twodigit_years = TRUE;
+				hdl->str_locale->dmy_order[i] = G_DATE_YEAR;
+                                break;
 			case 1976:
-				hdl->priv->str_locale->dmy_order[i] = G_DATE_YEAR;
+				hdl->str_locale->dmy_order[i] = G_DATE_YEAR;
 				break;
 			default:
 				break;
@@ -392,11 +260,11 @@ handler_compute_locale (GdaHandlerTime *hdl)
 #else
                 now_tm = localtime (&now);
 #endif
-		hdl->priv->str_locale->current_offset = ((now_tm->tm_year + 1900) / 100) * 100;
+		hdl->str_locale->current_offset = ((now_tm->tm_year + 1900) / 100) * 100;
 
 #ifdef GDA_DEBUG_NO		
 		for (i=0; i<3; i++) {
-			switch (hdl->priv->str_locale->dmy_order[i]) {
+			switch (hdl->str_locale->dmy_order[i]) {
 			case G_DATE_MONTH:
 				strings[i] = "Month";
 				break;
@@ -413,9 +281,9 @@ handler_compute_locale (GdaHandlerTime *hdl)
 		}
 		g_print ("GdaHandlerTime %p\n", hdl);
 		g_print ("\tlocale order = %s %s %s, separator = %c\n", 
-			 strings[0], strings[1], strings[2], hdl->priv->str_locale->separator);
-		if (hdl->priv->str_locale->twodigit_years)
-			g_print ("\tlocale has 2 digits year, using %d as offset\n", hdl->priv->str_locale->current_offset);
+			 strings[0], strings[1], strings[2], hdl->str_locale->separator);
+		if (hdl->str_locale->twodigit_years)
+			g_print ("\tlocale has 2 digits year, using %d as offset\n", hdl->str_locale->current_offset);
 		else
 			g_print ("\tlocale has 4 digits year\n");
 #endif
@@ -446,7 +314,7 @@ gda_handler_time_get_no_locale_str_from_value (GdaHandlerTime *dh, const GValue
 		const GDate *date;
 
 		date = (GDate *) g_value_get_boxed (value);
-		str = render_date_locale (date, dh->priv->sql_locale);
+		str = render_date_locale (date, dh->sql_locale);
 		if (!str)
 			retval = g_strdup ("NULL");
 		else 
@@ -459,44 +327,23 @@ gda_handler_time_get_no_locale_str_from_value (GdaHandlerTime *dh, const GValue
 		g_string_append_c (string, '\'');
 		tim = gda_value_get_time ((GValue *) value);
 		g_string_append_printf (string, "%02d:%02d:%02d",
-					tim->hour,
-					tim->minute,
-					tim->second);
-		if (tim->timezone != GDA_TIMEZONE_INVALID)
+					gda_time_get_hour (tim),
+					gda_time_get_minute (tim),
+					gda_time_get_second (tim));
+		if (gda_time_get_timezone (tim) != GDA_TIMEZONE_INVALID)
 			g_string_append_printf (string, "%+02d",
-						(int) tim->timezone / 3600);
+						(int) gda_time_get_timezone (tim) / 3600);
 		g_string_append_c (string, '\'');
 		retval = g_string_free (string, FALSE);
 	}
-	else if (type == GDA_TYPE_TIMESTAMP) {
-		const GdaTimestamp *gdats;
-		GDate *vdate;
+	else if (type == G_TYPE_DATE_TIME) {
+		GDateTime *gdats;
 
-		gdats = gda_value_get_timestamp ((GValue *) value);
-		vdate = g_date_new_dmy (gdats->day, gdats->month, gdats->year);
-		str = render_date_locale (vdate, dh->priv->sql_locale);
-		g_date_free (vdate);
-
-		if (str) {
-			GString *string;
-			string = g_string_new ("");
-			g_string_append_printf (string, "%02u:%02u:%02u",
-						gdats->hour,
-						gdats->minute,
-						gdats->second);
-			if (gdats->fraction != 0)
-				g_string_append_printf (string, ".%lu", gdats->fraction);
-			
-			if (gdats->timezone != GDA_TIMEZONE_INVALID)
-				g_string_append_printf (string, "%+02d",
-							(int) gdats->timezone / 3600);
-			
-			retval = g_strdup_printf ("%s %s", str, string->str);
-			g_free (str);
-			g_string_free (string, TRUE);
-		}
+		gdats = (GDateTime*) g_value_get_boxed ((GValue *) value);
+		if (gdats != NULL)
+			retval = g_date_time_format (gdats, "%FT%H:%M:%S%:::z");
 		else
-			retval = g_strdup ("NULL");	
+			retval = g_strdup ("NULL");
 	}
 	else if (type == G_TYPE_DATE_TIME) {
 		GDateTime *ts;
@@ -507,7 +354,7 @@ gda_handler_time_get_no_locale_str_from_value (GdaHandlerTime *dh, const GValue
 			gint y, m, d;
 			g_date_time_get_ymd (ts, &y, &m, &d);
 			vdate = g_date_new_dmy (d, m, y);
-			str = render_date_locale (vdate, dh->priv->sql_locale);
+			str = render_date_locale (vdate, dh->sql_locale);
 			g_date_free (vdate);
 
 			if (str) {
@@ -565,17 +412,17 @@ gda_handler_time_get_format (GdaHandlerTime *dh, GType type)
 	g_return_val_if_fail (GDA_IS_HANDLER_TIME (dh), NULL);
 
 	string = g_string_new ("");
-	if ((type == G_TYPE_DATE) || (type == GDA_TYPE_TIMESTAMP) || (type == G_TYPE_DATE_TIME)) {
+	if ((type == G_TYPE_DATE) || (type == G_TYPE_DATE_TIME) || (type == G_TYPE_DATE_TIME)) {
 		for (i=0; i<3; i++) {
 			if (i > 0)
-				g_string_append_c (string, dh->priv->str_locale->separator);
-			switch (dh->priv->str_locale->dmy_order[i]) {
+				g_string_append_c (string, dh->str_locale->separator);
+			switch (dh->str_locale->dmy_order[i]) {
 			case G_DATE_DAY:
 			case G_DATE_MONTH:
 				g_string_append (string, "00");
 				break;
 			case G_DATE_YEAR:
-				if (dh->priv->str_locale->twodigit_years)
+				if (dh->str_locale->twodigit_years)
 					g_string_append (string, "00");
 				else
 					g_string_append (string, "0000");
@@ -586,10 +433,10 @@ gda_handler_time_get_format (GdaHandlerTime *dh, GType type)
 			}
 		}
 	}
-	if (type == GDA_TYPE_TIMESTAMP)
+	if (type == G_TYPE_DATE_TIME)
 		g_string_append_c (string, ' ');
 
-	if ((type == GDA_TYPE_TIME) || (type == GDA_TYPE_TIMESTAMP) || (type == G_TYPE_DATE_TIME))
+	if ((type == GDA_TYPE_TIME) || (type == G_TYPE_DATE_TIME) || (type == G_TYPE_DATE_TIME))
 		g_string_append (string, "00:00:00");
 
 	str = string->str;
@@ -597,6 +444,66 @@ gda_handler_time_get_format (GdaHandlerTime *dh, GType type)
 	return str;
 }
 
+/**
+ * gda_handler_time_get_hint:
+ * @dh: a #GdaHandlerTime object
+ * @type: the type of data being handled
+ *
+ * Get a string giving the user a hint about the locale-dependent requested format.
+ *
+ * Returns: a new string
+ *
+ * Since: 6.0
+ */
+gchar *
+gda_handler_time_get_hint (GdaHandlerTime *dh, GType type)
+{
+	gchar *str;
+	GString *string;
+	gint i;
+
+	g_return_val_if_fail (GDA_IS_HANDLER_TIME (dh), NULL);
+
+	string = g_string_new ("");
+	if ((type == G_TYPE_DATE) || (type == G_TYPE_DATE_TIME) || (type == G_TYPE_DATE_TIME)) {
+		for (i=0; i<3; i++) {
+			if (i > 0)
+				g_string_append_c (string, dh->str_locale->separator);
+			switch (dh->str_locale->dmy_order[i]) {
+			case G_DATE_DAY:
+				/* To translators: DD represents a place holder in a date string. For example in the "YYYY-MM-DD" format, one knows that she has to replace DD by a day number */
+				g_string_append (string, _("DD"));
+				break;
+			case G_DATE_MONTH:
+				/* To translators: MM represents a place holder in a date string. For example in the "YYYY-MM-DD" format, one knows that she has to replace MM by a month number */
+				g_string_append (string, _("MM"));
+				break;
+			case G_DATE_YEAR:
+				if (dh->str_locale->twodigit_years)
+					/* To translators: YY represents a place holder in a date string. For example in the "YY-MM-DD" format, one knows that she has to replace YY by a year number, on 2 digits */
+					g_string_append (string, _("YY"));
+				else
+					/* To translators: YYYY represents a place holder in a date string. For example in the "YYYY-MM-DD" format, one knows that she has to replace YYYY by a year number, on 4 digits */
+					g_string_append (string, _("YYYY"));
+				break;
+			default:
+				g_assert_not_reached ();
+				break;
+			}
+		}
+	}
+	if (type == G_TYPE_DATE_TIME)
+		g_string_append_c (string, ' ');
+
+	if ((type == GDA_TYPE_TIME) || (type == G_TYPE_DATE_TIME))
+		/* To translators: HH:MM:SS represents a time format. For example in the "HH:MM:SS" format, one knows that she has to replace HH by a number of hours, and so on */
+		g_string_append (string, "HH:MM:SS");
+
+	str = string->str;
+	g_string_free (string, FALSE);
+	return str;
+}
+
 /* Interface implementation */
 
 /* REM: SQL date format is always returned using the MM-DD-YYY format, it's up to the
@@ -619,7 +526,7 @@ gda_handler_time_get_sql_from_value (GdaDataHandler *iface, const GValue *value)
 		const GDate *date;
 
 		date = (GDate *) g_value_get_boxed (value);
-		str = render_date_locale (date, hdl->priv->sql_locale);
+		str = render_date_locale (date, hdl->sql_locale);
 		if (!str)
 			retval = g_strdup ("NULL");
 		else {
@@ -634,46 +541,25 @@ gda_handler_time_get_sql_from_value (GdaDataHandler *iface, const GValue *value)
 		tim = gda_value_get_time ((GValue *) value);
 		g_string_append_c (string, '\'');
 		g_string_append_printf (string, "%02d:%02d:%02d",
-					tim->hour,
-					tim->minute,
-					tim->second);
-		if (tim->timezone != GDA_TIMEZONE_INVALID)
+					gda_time_get_hour (tim),
+					gda_time_get_minute (tim),
+					gda_time_get_second (tim));
+		if (gda_time_get_timezone (tim) != GDA_TIMEZONE_INVALID)
 			g_string_append_printf (string, "%+02d",
-						(int) tim->timezone / 3600);
+						(int) gda_time_get_timezone (tim) / 3600);
 		g_string_append_c (string, '\'');
 		retval = g_string_free (string, FALSE);
 	}
-	else if (type == GDA_TYPE_TIMESTAMP) {
-		const GdaTimestamp *gdats;
-		GDate *vdate;
+	else if (g_type_is_a (type, G_TYPE_DATE_TIME)) {
+		GDateTime *gdats;
 
-		gdats = gda_value_get_timestamp ((GValue *) value);
-		vdate = g_date_new_dmy (gdats->day, gdats->month, gdats->year);
-		str = render_date_locale (vdate, hdl->priv->sql_locale);
-		g_date_free (vdate);
-
-		if (str) {
-			GString *string;
-			string = g_string_new ("");
-			g_string_append_printf (string, "%02u:%02u:%02u",
-						gdats->hour,
-						gdats->minute,
-						gdats->second);
-			if (gdats->fraction != 0)
-				g_string_append_printf (string, ".%lu", gdats->fraction);
-			
-			if (gdats->timezone != GDA_TIMEZONE_INVALID)
-				g_string_append_printf (string, "%+02d",
-							(int) gdats->timezone / 3600);
-			
-			retval = g_strdup_printf ("'%s %s'", str, string->str);
-			g_free (str);
-			g_string_free (string, TRUE);
-		}
+		gdats = (GDateTime*) g_value_get_boxed ((GValue *) value);
+		if (gdats != NULL)
+			retval = g_date_time_format (gdats, "'%FT%H:%M:%S%:::z'");
 		else
 			retval = g_strdup ("NULL");	
 	}
-	else if (type == G_TYPE_DATE_TIME) {
+	else if (type == G_TYPE_DATE_TIME) { // FIXME: Remove
 		GDateTime *ts;
 		GDate *vdate;
 
@@ -682,7 +568,7 @@ gda_handler_time_get_sql_from_value (GdaDataHandler *iface, const GValue *value)
 			gint y, m, d;
 			g_date_time_get_ymd (ts, &y, &m, &d);
 			vdate = g_date_new_dmy (d, m, y);
-			str = render_date_locale (vdate, hdl->priv->sql_locale);
+			str = render_date_locale (vdate, hdl->sql_locale);
 			g_date_free (vdate);
 
 			if (str) {
@@ -704,7 +590,7 @@ gda_handler_time_get_sql_from_value (GdaDataHandler *iface, const GValue *value)
 					g_string_append_printf (string, "-%02d",
 								(int) (-span / G_TIME_SPAN_HOUR));
 
-				retval = g_strdup_printf ("'%s %s'", str, string->str);
+				retval = g_strdup_printf ("'%sT%s'", str, string->str);
 				g_free (str);
 				g_string_free (string, TRUE);
 			}
@@ -738,7 +624,7 @@ gda_handler_time_get_str_from_value (GdaDataHandler *iface, const GValue *value)
 		const GDate *date;
 
 		date = (GDate *) g_value_get_boxed (value);
-		retval = render_date_locale (date, hdl->priv->str_locale);
+		retval = render_date_locale (date, hdl->str_locale);
 		if (!retval)
 			retval = g_strdup ("");
 	}
@@ -747,37 +633,16 @@ gda_handler_time_get_str_from_value (GdaDataHandler *iface, const GValue *value)
 		retval = strip_quotes (str);
 		g_free (str);
 	}
-	else if (type == GDA_TYPE_TIMESTAMP) {
-		const GdaTimestamp *gdats;
-		GDate *vdate;
-
-		gdats = gda_value_get_timestamp ((GValue *) value);
-		vdate = g_date_new_dmy (gdats->day, gdats->month, gdats->year);
-		str = render_date_locale (vdate, hdl->priv->str_locale);
-		g_date_free (vdate);
-
-		if (str) {
-			GString *string;
-			string = g_string_new ("");
-			g_string_append_printf (string, "%02u:%02u:%02u",
-						gdats->hour,
-						gdats->minute,
-						gdats->second);
-			if (gdats->fraction != 0)
-				g_string_append_printf (string, ".%lu", gdats->fraction);
-			
-			if (gdats->timezone != GDA_TIMEZONE_INVALID)
-				g_string_append_printf (string, "%+02d",
-							(int) gdats->timezone / 3600);
-			
-			retval = g_strdup_printf ("%s %s", str, string->str);
-			g_free (str);
-			g_string_free (string, TRUE);
-		}
-		else
-			retval = g_strdup ("");	
-	}
 	else if (type == G_TYPE_DATE_TIME) {
+		GDateTime *gdats;
+
+		gdats = (GDateTime*) g_value_get_boxed ((GValue *) value);
+		if (gdats != NULL)
+			retval = g_date_time_format (gdats, "%xT%H:%M:%S%:::z");
+		else
+			retval = g_strdup ("");
+	}
+	else if (type == G_TYPE_DATE_TIME) { // FIXME: Remove
 		GDateTime *ts;
 		GDate *vdate;
 
@@ -786,7 +651,7 @@ gda_handler_time_get_str_from_value (GdaDataHandler *iface, const GValue *value)
 			gint y, m, d;
 			g_date_time_get_ymd (ts, &y, &m, &d);
 			vdate = g_date_new_dmy (d, m, y);
-			str = render_date_locale (vdate, hdl->priv->str_locale);
+			str = render_date_locale (vdate, hdl->str_locale);
 			g_date_free (vdate);
 
 			if (str) {
@@ -808,7 +673,7 @@ gda_handler_time_get_str_from_value (GdaDataHandler *iface, const GValue *value)
 					g_string_append_printf (string, "-%02d",
 								(int) (-span / G_TIME_SPAN_HOUR));
 
-				retval = g_strdup_printf ("%s %s", str, string->str);
+				retval = g_strdup_printf ("%sT%s", str, string->str);
 				g_free (str);
 				g_string_free (string, TRUE);
 			}
@@ -903,7 +768,7 @@ gda_handler_time_get_value_from_sql (GdaDataHandler *iface, const gchar *sql, GT
 		if ((i>=2) && (*sql=='\'') && (sql[i-1]=='\'')) {
 			gchar *str = g_strdup (sql);
 			str[i-1] = 0;
-			value = gda_handler_time_get_value_from_locale (iface, str+1, type, hdl->priv->sql_locale);
+			value = gda_handler_time_get_value_from_locale (iface, str+1, type, hdl->sql_locale);
 			g_free (str);
 		}
 	}
@@ -925,17 +790,16 @@ gda_handler_time_get_value_from_str (GdaDataHandler *iface, const gchar *str, GT
 	if (*str == '\'')
 		return NULL;
 	else
-		return gda_handler_time_get_value_from_locale (iface, str, type, hdl->priv->str_locale);
+		return gda_handler_time_get_value_from_locale (iface, str, type, hdl->str_locale);
 }
 
 
 
-static gboolean make_timestamp (GdaHandlerTime *hdl, GdaTimestamp *timestamp, 
+static GDateTime * make_timestamp (GdaHandlerTime *hdl,
 				const gchar *value, LocaleSetting *locale);
 static gboolean make_date (GdaHandlerTime *hdl, GDate *date, const gchar *value,
 			   LocaleSetting *locale, const gchar **out_endptr);
 static gboolean make_time (GdaHandlerTime *hdl, GdaTime *timegda, const gchar *value);
-static GDateTime *make_ts (GdaHandlerTime *hdl, const gchar *value, LocaleSetting *locale);
 static GValue *
 gda_handler_time_get_value_from_locale (GdaDataHandler *iface, const gchar *sql, 
 					GType type, LocaleSetting *locale)
@@ -954,25 +818,19 @@ gda_handler_time_get_value_from_locale (GdaDataHandler *iface, const gchar *sql,
 		}
 	}
 	else if (type == GDA_TYPE_TIME) {
-		GdaTime timegda;
-		if (make_time (hdl, &timegda, sql)) {
+		GdaTime* timegda = gda_time_new ();
+		if (make_time (hdl, timegda, sql)) {
 			value = g_value_init (g_new0 (GValue, 1), GDA_TYPE_TIME);
-			gda_value_set_time (value, &timegda);
+			gda_value_set_time (value, timegda);
 		}
+		gda_time_free (timegda);
 	}
-	else if (type == GDA_TYPE_TIMESTAMP) {
-		GdaTimestamp timestamp;
-		if (make_timestamp (hdl, &timestamp, sql, locale)) {
-			value = g_value_init (g_new0 (GValue, 1), GDA_TYPE_TIMESTAMP);
-			gda_value_set_timestamp (value, &timestamp);
-		}
-	}
-	else if (type == G_TYPE_DATE_TIME) {
-		GDateTime *ts;
-		ts = make_ts (hdl, sql, locale);
-		if (ts) {
+	else if (g_type_is_a (type, G_TYPE_DATE_TIME)) {
+		GDateTime* timestamp = make_timestamp (hdl, sql, locale);
+		if (timestamp != NULL) {
 			value = g_value_init (g_new0 (GValue, 1), G_TYPE_DATE_TIME);
-			g_value_take_boxed (value, ts);
+			g_value_set_boxed (value, timestamp);
+			g_date_time_unref (timestamp);
 		}
 	}
 	else
@@ -981,81 +839,60 @@ gda_handler_time_get_value_from_locale (GdaDataHandler *iface, const gchar *sql,
 	return value;
 }
 
-static GDateTime *
-make_ts (GdaHandlerTime *hdl, const gchar *value, LocaleSetting *locale)
-{
-	GDateTime *ts;
-	const gchar *end_ptr;
-	GDate vdate;
-	GdaTime vtime;
-	memset (&vtime, 0, sizeof (GdaTime));
-	vtime.timezone = GDA_TIMEZONE_INVALID;
 
-	gboolean retval;
-	retval = make_date (hdl, &vdate, value, locale, &end_ptr);
-	if (retval) {
-		if (*end_ptr != ' ')
-			retval = FALSE;
-		else
-			retval = make_time (hdl, &vtime, end_ptr + 1);
-	}
-	if (!retval)
-		return NULL;
-
-	GTimeZone *tz;
-	if (vtime.timezone != GDA_TIMEZONE_INVALID) {
-		gchar *tzdef;
-		if (vtime.timezone >= 0)
-			tzdef = g_strdup_printf ("+%02d", (gint) (vtime.timezone / 3600));
-		else
-			tzdef = g_strdup_printf ("-%02d", (gint) (- vtime.timezone / 3600));
-		tz = g_time_zone_new (tzdef);
-		g_free (tzdef);
-	}
-	else
-		tz = g_time_zone_new_local ();
-	ts = g_date_time_new (tz, vdate.year, vdate.month, vdate.day,
-			      vtime.hour, vtime.minute, (gdouble) vtime.second + vtime.fraction);
-	g_time_zone_unref (tz);
-
-	return ts;
-}
-
-/* Makes a GdaTimestamp from a string like "24-12-2003 13:12:01.12+01",
+/* Makes a GDateTime from a string like "24-12-2003T13:12:01.12+01",
  * taken from libgda/gda-value.h
  * with a modification for the date format
  */
-static gboolean
-make_timestamp (GdaHandlerTime *hdl, GdaTimestamp *timestamp, const gchar *value, LocaleSetting *locale)
+static GDateTime*
+make_timestamp (GdaHandlerTime *hdl, const gchar *value, LocaleSetting *locale)
 {
 	gboolean retval;
 	const gchar *end_ptr;
 	GDate vdate;
-	GdaTime vtime;
-	memset (&vtime, 0, sizeof (GdaTime));
-	vtime.timezone = GDA_TIMEZONE_INVALID;
+	GdaTime* vtime;
+	vtime = gda_time_new ();
+	gda_time_set_timezone (vtime, GDA_TIMEZONE_INVALID);
 
 	retval = make_date (hdl, &vdate, value, locale, &end_ptr);
-	timestamp->day = vdate.day;
-	timestamp->month = vdate.month;
-	timestamp->year = vdate.year;
 
 	if (retval) {
-		if (*end_ptr != ' ')
-			retval = FALSE;
+		if (*end_ptr != ' ' && *end_ptr != 'T')
+			return NULL;
 		else
-			retval = make_time (hdl, &vtime, end_ptr + 1);
+			if (!make_time (hdl, vtime, end_ptr + 1))
+				return NULL;
 	}
-
-	timestamp->hour = vtime.hour;
-	timestamp->minute = vtime.minute;
-	timestamp->second = vtime.second;
-	timestamp->fraction = vtime.fraction;
-	timestamp->timezone = vtime.timezone;
-
-	/*g_print ("Value #%s# => %d\n", value, retval);*/
-
-	return retval;
+	gchar *stz;
+	gint ts = gda_time_get_timezone (vtime);
+	if (ts < 0) ts *= -1;
+	gint h = ts/60/60;
+	gint m = ts/60 - h * 60;
+	gint s = ts - h * 60 * 60 - m * 60;
+	stz = g_strdup_printf ("%s%02d:%02d:%02d",
+																gda_time_get_timezone (vtime) >= 0 ? "+" : "-",
+																h, m, s);
+  g_print ("STR for TZ: %s\n" , stz);
+	GTimeZone* tz = g_time_zone_new (stz);
+	g_free (stz);
+	if (tz == NULL)
+		return NULL;
+	gdouble seconds;
+	seconds = (gdouble) (gda_time_get_second (vtime) + gda_time_get_fraction (vtime) / 1000000.0);
+	g_print ("Fraction: %ld, %f\n", gda_time_get_fraction (vtime), gda_time_get_fraction (vtime) / 1000000.0);
+	g_print ("Seconds to set %lf\n", seconds);
+	g_print ("TZ: %d\n", g_time_zone_get_offset (tz, 0));
+	GDateTime *dt = g_date_time_new (tz,
+																	g_date_get_year (&vdate),
+																	g_date_get_month (&vdate),
+																	g_date_get_day (&vdate),
+																	gda_time_get_hour (vtime),
+																	gda_time_get_minute (vtime),
+																	seconds);
+	gchar *tss = g_date_time_format (dt, "%FT%H%:%M:%S:::z");
+	g_print ("String: %s\n", tss);
+	g_free (tss);
+	return (GDateTime*) dt;
 }
 
 static gboolean
@@ -1096,8 +933,7 @@ make_date (G_GNUC_UNUSED GdaHandlerTime *hdl, GDate *date, const gchar *value,
 		return FALSE;
 
 	g_date_clear (date, 1);
-	g_date_set_dmy (date, 1, 1, 1);
-	
+
 	/* 1st number */
 	ptr = g_strdup (value);
 	tofree = ptr;
@@ -1157,35 +993,31 @@ make_date (G_GNUC_UNUSED GdaHandlerTime *hdl, GDate *date, const gchar *value,
 		else if (value [ptr-tofree])
 			error = TRUE;
 	}
+	GDateDay day = 1;
+	GDateMonth month = 1;
+	GDateYear year = 1;
 
 	/* analyse what's parsed */
 	if (!error) {
 		for (i=0; i<3; i++) {
 			switch (locale->dmy_order[i]) {
 			case G_DATE_DAY:
-				if ((nums[i] <= G_MAXUINT8) && g_date_valid_day ((GDateDay) nums[i]))
-					g_date_set_day (date, nums[i]);
-				else
-					retval = FALSE;
+					day = nums[i];
 				break;
 			case G_DATE_MONTH:
-				if ((nums[i] <= 12) && g_date_valid_month ((GDateMonth) nums[i]))
-					g_date_set_month (date, nums[i]);
-				else
-					retval = FALSE;
+				month = nums[i];
 				break;
 			case G_DATE_YEAR:
-				if (g_date_valid_year (nums[i] < 100 ? nums[i] + locale->current_offset : nums[i]))
-					g_date_set_year (date, nums[i] < 100 ? nums[i] + locale->current_offset : nums[i]);
-				else
-					retval = FALSE;
+				year = nums[i] < 100 ? nums[i] + locale->current_offset : nums[i];
 				break;
 			}
 		}
-
-		/* checks */
-		if (retval)
-			retval = g_date_valid (date);
+		if (!g_date_valid_dmy (day, month, year)) {
+			retval = FALSE;
+		} else {
+			g_date_set_dmy (date, day, month, year);
+			retval = TRUE;
+		}
 	}
 	else
 		retval = FALSE;
@@ -1246,30 +1078,36 @@ gda_handler_time_get_sane_init_value (G_GNUC_UNUSED GdaDataHandler *iface, GType
 		g_value_take_boxed (value, gdate);
 	}
 	else if (type == GDA_TYPE_TIME) {
-		GdaTime gtime;
+		GdaTime* gtime;
 
-                gtime.hour = stm->tm_hour;
-		gtime.minute = stm->tm_min;
-		gtime.second = stm->tm_sec;
-		gtime.timezone = GDA_TIMEZONE_INVALID;
+		gtime = gda_time_new ();
+
+                gda_time_set_hour (gtime, stm->tm_hour);
+		gda_time_set_minute (gtime, stm->tm_min);
+		gda_time_set_second (gtime, stm->tm_sec);
+		gda_time_set_timezone (gtime, GDA_TIMEZONE_INVALID);
 		value = g_value_init (g_new0 (GValue, 1), GDA_TYPE_TIME);
-		gda_value_set_time (value, &gtime);
+		gda_value_set_time (value, gtime);
 	}
-	else if (type == GDA_TYPE_TIMESTAMP) {
-		GdaTimestamp gts;
+	else if (g_type_is_a (type, G_TYPE_DATE_TIME)) {
+		GTimeZone *tz = g_time_zone_new ("Z");
+		GDateTime* gts = g_date_time_new (tz,
+																			stm->tm_year + 1900,
+																			stm->tm_mon + 1,
+																			stm->tm_mday,
+																			stm->tm_hour,
+																			stm->tm_min,
+																			stm->tm_sec);
 
-		gts.year = stm->tm_year + 1900;
-		gts.month = stm->tm_mon + 1;
-		gts.day = stm->tm_mday;
-                gts.hour = stm->tm_hour;
-		gts.minute = stm->tm_min;
-		gts.second = stm->tm_sec;
-		gts.fraction = 0;
-		gts.timezone = GDA_TIMEZONE_INVALID;
-		value = g_value_init (g_new0 (GValue, 1), GDA_TYPE_TIMESTAMP);
-		gda_value_set_timestamp (value, &gts);
+		if (gts != NULL) {
+      value = g_value_init (g_new0 (GValue, 1), G_TYPE_DATE_TIME);
+		  g_value_set_boxed (value, gts);
+      g_date_time_unref (gts);
+    } else {
+      g_warning (_("Invalid values from statement to create a timestamp"));
+    }
 	}
-	else if (type == G_TYPE_DATE_TIME) {
+	else if (type == G_TYPE_DATE_TIME) { // FIXME: Remove
 		GDateTime *ts;
 		GTimeZone *tz;
 		tz = g_time_zone_new_local ();
@@ -1293,8 +1131,8 @@ gda_handler_time_accepts_g_type (GdaDataHandler *iface, GType type)
 	g_assert (iface);
 	hdl = (GdaHandlerTime*) (iface);
 
-	for (i = 0; i < hdl->priv->nb_g_types; i++) {
-		if (hdl->priv->valid_g_types [i] == type)
+	for (i = 0; i < hdl->nb_g_types; i++) {
+		if (hdl->valid_g_types [i] == type)
 			return TRUE;
 	}
 
@@ -1307,3 +1145,73 @@ gda_handler_time_get_descr (GdaDataHandler *iface)
 	g_return_val_if_fail (GDA_IS_HANDLER_TIME (iface), NULL);
 	return g_object_get_data (G_OBJECT (iface), "descr");
 }
+
+static void
+data_handler_iface_init (GdaDataHandlerIface *iface)
+{
+	iface->get_sql_from_value = gda_handler_time_get_sql_from_value;
+	iface->get_str_from_value = gda_handler_time_get_str_from_value;
+	iface->get_value_from_sql = gda_handler_time_get_value_from_sql;
+	iface->get_value_from_str = gda_handler_time_get_value_from_str;
+	iface->get_sane_init_value = gda_handler_time_get_sane_init_value;
+	iface->accepts_g_type = gda_handler_time_accepts_g_type;
+	iface->get_descr = gda_handler_time_get_descr;
+}
+
+static void
+gda_handler_time_init (GdaHandlerTime *hdl)
+{
+	/* Private structure */
+	hdl->nb_g_types = 4;
+	hdl->valid_g_types = g_new0 (GType, 7);
+	hdl->valid_g_types[0] = G_TYPE_DATE;
+	hdl->valid_g_types[1] = GDA_TYPE_TIME;
+	hdl->valid_g_types[2] = G_TYPE_DATE_TIME;
+	hdl->valid_g_types[3] = G_TYPE_DATE_TIME;
+
+	/* taking into accout the locale */
+	hdl->sql_locale = g_new0 (LocaleSetting, 1);
+	hdl->sql_locale->dmy_order[0] = G_DATE_MONTH;
+	hdl->sql_locale->dmy_order[1] = G_DATE_DAY;
+	hdl->sql_locale->dmy_order[2] = G_DATE_YEAR;
+	hdl->sql_locale->twodigit_years = FALSE;
+	hdl->sql_locale->current_offset = 0;
+	hdl->sql_locale->separator = '-';
+
+	hdl->str_locale = g_new0 (LocaleSetting, 1);
+	hdl->str_locale->dmy_order[0] = G_DATE_MONTH;
+	hdl->str_locale->dmy_order[1] = G_DATE_DAY;
+	hdl->str_locale->dmy_order[2] = G_DATE_YEAR;
+	hdl->str_locale->twodigit_years = FALSE;
+	hdl->str_locale->current_offset = 0;
+	hdl->str_locale->separator = '-';
+
+	g_object_set_data (G_OBJECT (hdl), "name", "InternalTime");
+	g_object_set_data (G_OBJECT (hdl), "descr", _("Time, Date and TimeStamp representation"));
+}
+
+static void
+gda_handler_time_dispose (GObject *object)
+{
+	GdaHandlerTime *hdl;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (GDA_IS_HANDLER_TIME (object));
+
+	hdl = GDA_HANDLER_TIME (object);
+
+	g_clear_pointer(&hdl->valid_g_types, g_free);
+	g_clear_pointer(&hdl->str_locale, g_free);
+	g_clear_pointer(&hdl->sql_locale, g_free);
+
+	/* for the parent class */
+	G_OBJECT_CLASS (gda_handler_time_parent_class)->dispose (object);
+}
+
+static void
+gda_handler_time_class_init (GdaHandlerTimeClass * class)
+{
+	GObjectClass   *object_class = G_OBJECT_CLASS (class);
+
+	object_class->dispose = gda_handler_time_dispose;
+}
