@@ -2,7 +2,7 @@
  * Copyright (C) 2000 Reinhard Müller <reinhard@src.gnome.org>
  * Copyright (C) 2000 - 2004 Rodrigo Moya <rodrigo@gnome-db.org>
  * Copyright (C) 2001 Carlos Perelló Marín <carlos@gnome-db.org>
- * Copyright (C) 2001 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2001 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2002 Andrew Hill <andru@src.gnome.org>
  * Copyright (C) 2002 - 2003 Gonzalo Paniagua Javier <gonzalo@ximian.com>
  * Copyright (C) 2003 - 2006 Murray Cumming <murrayc@murrayc.com>
@@ -34,7 +34,6 @@
 #include <libgda/gda-config.h>
 #include <libgda/gda-connection-event.h>
 #include <libgda/gda-connection.h>
-#include <libgda/gda-connection-private.h>
 #include <libgda/gda-data-comparator.h>
 #include <libgda/gda-data-model-array.h>
 
@@ -85,13 +84,10 @@
 #include <libgda/gda-tree-mgr-tables.h> 
 #include <libgda/gda-tree-node.h>
 
-#include <libgda/sql-parser/gda-sql-parser.h>
-
 #include <libgda/gda-sql-builder.h>
 
 #include <libgda/gda-meta-store.h>
-
-#include <libgda/gda-mutex.h>
+#include <libgda/gda-ddl-creator.h>
 
 G_BEGIN_DECLS
 
