@@ -2,7 +2,6 @@
  * Copyright (C) 2008 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2010 Jonh Wendell <jwendell@gnome.org>
- * Copyright (C) 2018 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -87,7 +86,7 @@ gda_pstmt_init (GdaPStmt *pstmt, G_GNUC_UNUSED GdaPStmtClass *klass)
 	g_return_if_fail (GDA_IS_PSTMT (pstmt));
 	pstmt->priv = g_new0 (GdaPStmtPrivate, 1);
 	g_rec_mutex_init (& pstmt->priv->mutex);
-	g_weak_ref_init (&pstmt->priv->gda_stmt_ref, NULL);
+	g_weak_ref_init (& pstmt->priv->gda_stmt_ref, NULL);
 	pstmt->sql = NULL;
 	pstmt->param_ids = NULL;
 	pstmt->ncols = -1;
@@ -102,22 +101,20 @@ static void
 gda_stmt_reset_cb (GdaStatement *stmt, GdaPStmt *pstmt)
 {
 	g_rec_mutex_lock (& pstmt->priv->mutex);
-	if (stmt) {
-		g_object_ref (stmt);
+	if (stmt)
 		g_signal_handlers_disconnect_by_func (G_OBJECT (stmt),
 						      G_CALLBACK (gda_stmt_reset_cb), pstmt);
-		g_object_unref (stmt);
-	}
 	else {
-		GdaStatement *stm = g_weak_ref_get (& pstmt->priv->gda_stmt_ref);
-		if (stm) {
-			g_signal_handlers_disconnect_by_func (G_OBJECT (stm),
+		stmt = g_weak_ref_get (& pstmt->priv->gda_stmt_ref);
+		if (stmt) {
+			g_signal_handlers_disconnect_by_func (G_OBJECT (stmt),
 							      G_CALLBACK (gda_stmt_reset_cb), pstmt);
-			g_object_unref (stm);
+			g_object_unref (stmt);
 		}
 	}
 
-	g_weak_ref_set (& pstmt->priv->gda_stmt_ref, NULL);
+	g_weak_ref_clear (& pstmt->priv->gda_stmt_ref);
+	g_weak_ref_init (& pstmt->priv->gda_stmt_ref, NULL);
 	g_rec_mutex_unlock (& pstmt->priv->mutex);
 }
 
@@ -138,8 +135,8 @@ gda_pstmt_finalize (GObject *object)
 	GdaPStmt *pstmt = (GdaPStmt *) object;
 
 	/* free memory */
+	g_weak_ref_clear (& pstmt->priv->gda_stmt_ref);
 	g_rec_mutex_clear (& pstmt->priv->mutex);
-	g_weak_ref_clear (&pstmt->priv->gda_stmt_ref);
 	g_free (pstmt->priv);
 
 	if (pstmt->sql) {
@@ -178,7 +175,6 @@ gda_pstmt_set_gda_statement (GdaPStmt *pstmt, GdaStatement *stmt)
 	g_return_if_fail (!stmt || GDA_IS_STATEMENT (stmt));
 
 	g_rec_mutex_lock (& pstmt->priv->mutex);
-	g_object_ref (stmt);
 
 	GdaStatement *estmt;
 	estmt = g_weak_ref_get (& pstmt->priv->gda_stmt_ref);
@@ -191,9 +187,11 @@ gda_pstmt_set_gda_statement (GdaPStmt *pstmt, GdaStatement *stmt)
 
 	gda_stmt_reset_cb (NULL, pstmt);
 
-	g_weak_ref_set (& pstmt->priv->gda_stmt_ref, stmt);
-	g_signal_connect (G_OBJECT (stmt), "reset", G_CALLBACK (gda_stmt_reset_cb), pstmt);
-	g_object_unref (stmt);
+	if (stmt) {
+		g_object_ref (stmt);
+		g_weak_ref_set (& pstmt->priv->gda_stmt_ref, stmt);
+		g_signal_connect (G_OBJECT (stmt), "reset", G_CALLBACK (gda_stmt_reset_cb), pstmt);
+	}
 	g_rec_mutex_unlock (& pstmt->priv->mutex);
 }
 
