@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2008 Bas Driessen <bas.driessen@xobas.com>
- * Copyright (C) 2008 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2008 - 2012 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2009 - 2011 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2011 Daniel Espinosa <despinosa@src.gnome.org>
  *
@@ -32,57 +32,57 @@
 
 G_BEGIN_DECLS
 /*
- * GdaSqlExpr: (skip)
+ * GdaSqlExpr:
  *
  **/
 typedef struct _GdaSqlExpr      GdaSqlExpr;
 /*
- * GdaSqlField: (skip)
+ * GdaSqlField:
  *
  **/
 typedef struct _GdaSqlField     GdaSqlField;
 /*
- * GdaSqlTable: (skip)
+ * GdaSqlTable:
  *
  **/
 typedef struct _GdaSqlTable     GdaSqlTable;
 /*
- * GdaSqlFunction: (skip)
+ * GdaSqlFunction:
  *
  **/
 typedef struct _GdaSqlFunction  GdaSqlFunction;
 /*
- * GdaSqlOperation: (skip)
+ * GdaSqlOperation:
  *
  **/
 typedef struct _GdaSqlOperation GdaSqlOperation;
 /*
- * GdaSqlCase: (skip)
+ * GdaSqlCase:
  *
  **/
 typedef struct _GdaSqlCase GdaSqlCase;
 /*
- * GdaSqlSelectField: (skip)
+ * GdaSqlSelectField:
  *
  **/
 typedef struct _GdaSqlSelectField GdaSqlSelectField;
 /*
- * GdaSqlSelectTarget: (skip)
+ * GdaSqlSelectTarget:
  *
  **/
 typedef struct _GdaSqlSelectTarget GdaSqlSelectTarget;
 /*
- * GdaSqlSelectJoin: (skip)
+ * GdaSqlSelectJoin:
  *
  **/
 typedef struct _GdaSqlSelectJoin GdaSqlSelectJoin;
 /*
- * GdaSqlSelectFrom: (skip)
+ * GdaSqlSelectFrom:
  *
  **/
 typedef struct _GdaSqlSelectFrom GdaSqlSelectFrom;
 /*
- * GdaSqlSelectOrder: (skip)
+ * GdaSqlSelectOrder:
  *
  **/
 typedef struct _GdaSqlSelectOrder GdaSqlSelectOrder;
@@ -91,7 +91,7 @@ typedef struct _GdaSqlSelectOrder GdaSqlSelectOrder;
  * Any Expression
  */
 /**
- * GdaSqlExpr: (skip)
+ * GdaSqlExpr:
  * @any: inheritance structure
  * @value: (allow-none): a #GValue, or %NULL. Please see specific note about this field. 
  * @param_spec: (allow-none): a #GdaSqlParamSpec, or %NULL if this is not a variable
@@ -150,7 +150,7 @@ void             gda_sql_expr_take_select    (GdaSqlExpr *expr, GdaSqlStatement
  * Any Table's field
  */
 /**
- * GdaSqlField: (skip)
+ * GdaSqlField:
  * any: 
  * @field_name: 
  * @validity_meta_table_column: 
@@ -170,6 +170,9 @@ struct _GdaSqlField {
 	gpointer         _gda_reserved2;
 };
 
+#define GDA_TYPE_SQL_FIELD (gda_sql_field_get_type())
+
+GType            gda_sql_field_get_type  (void) G_GNUC_CONST;
 GdaSqlField     *gda_sql_field_new            (GdaSqlAnyPart *parent);
 void             gda_sql_field_free           (GdaSqlField *field);
 GdaSqlField     *gda_sql_field_copy           (GdaSqlField *field);
@@ -182,7 +185,7 @@ void             gda_sql_field_take_name      (GdaSqlField *field, GValue *value
  * Any table
  */
 /**
- * GdaSqlTable: (skip)
+ * GdaSqlTable:
  * @any: 
  * @table_name: 
  * @validity_meta_object: 
@@ -203,6 +206,9 @@ struct _GdaSqlTable
 	gpointer         _gda_reserved2;
 };
 
+#define GDA_TYPE_SQL_TABLE (gda_sql_table_get_type())
+
+GType            gda_sql_table_get_type  (void) G_GNUC_CONST;
 GdaSqlTable     *gda_sql_table_new            (GdaSqlAnyPart *parent);
 void             gda_sql_table_free           (GdaSqlTable *table);
 GdaSqlTable     *gda_sql_table_copy           (GdaSqlTable *table);
@@ -215,7 +221,7 @@ void             gda_sql_table_take_name      (GdaSqlTable *table, GValue *value
  * A function with any number of arguments
  */
 /**
- * GdaSqlFunction: (skip)
+ * GdaSqlFunction:
  * @any: inheritance structure
  * @function_name: name of the function , in the form [[catalog.]schema.]function_name
  * @args_list: list of #GdaSqlExpr expressions, one for each argument
@@ -233,6 +239,9 @@ struct _GdaSqlFunction {
 	gpointer         _gda_reserved2;
 };
 
+#define GDA_TYPE_SQL_FUNCTION (gda_sql_function_get_type())
+
+GType            gda_sql_function_get_type  (void) G_GNUC_CONST;
 GdaSqlFunction  *gda_sql_function_new            (GdaSqlAnyPart *parent);
 void             gda_sql_function_free           (GdaSqlFunction *function);
 GdaSqlFunction  *gda_sql_function_copy           (GdaSqlFunction *function);
@@ -315,15 +324,15 @@ typedef enum {
 	GDA_SQL_OPERATOR_TYPE_ILIKE,
 
 	GDA_SQL_OPERATOR_TYPE_NOTLIKE,
-	GDA_SQL_OPERATOR_TYPE_NOTILIKE,
-	GDA_SQL_OPERATOR_TYPE_GLOB
+	GDA_SQL_OPERATOR_TYPE_NOTILIKE
 } GdaSqlOperatorType;
 
 /**
- * GdaSqlOperation: (skip)
+ * GdaSqlOperation:
  * @any: inheritance structure
  * @operator_type: 
- * @operands: list of #GdaSqlExpr operands
+ * @operands: (element-type Gda.SqlExpr)
+ * : list of #GdaSqlExpr operands
  *
  * This structure represents an operation between one or more operands.
  */
@@ -338,6 +347,9 @@ struct _GdaSqlOperation {
 	gpointer         _gda_reserved2;
 };
 
+#define GDA_TYPE_SQL_OPERATION (gda_sql_operation_get_type())
+
+GType             gda_sql_operation_get_type  (void) G_GNUC_CONST;
 GdaSqlOperation  *gda_sql_operation_new            (GdaSqlAnyPart *parent);
 void              gda_sql_operation_free           (GdaSqlOperation *operation);
 GdaSqlOperation  *gda_sql_operation_copy           (GdaSqlOperation *operation);
@@ -349,7 +361,7 @@ GdaSqlOperatorType    gda_sql_operation_operator_from_string (const gchar *op);
  * A CASE expression
  */
 /**
- * GdaSqlCase: (skip)
+ * GdaSqlCase:
  * @any: inheritance structure
  * @base_expr: expression to test
  * @when_expr_list: list of #GdaSqlExpr, one for each WHEN clause
@@ -371,6 +383,9 @@ struct _GdaSqlCase
 	gpointer         _gda_reserved1;
 	gpointer         _gda_reserved2;
 };
+#define GDA_TYPE_SQL_CASE (gda_sql_case_get_type())
+
+GType             gda_sql_case_get_type  (void) G_GNUC_CONST;
 
 GdaSqlCase        *gda_sql_case_new            (GdaSqlAnyPart *parent);
 void               gda_sql_case_free           (GdaSqlCase *sc);
@@ -381,7 +396,7 @@ gchar             *gda_sql_case_serialize      (GdaSqlCase *sc);
  * Any expression in a SELECT ... before the FROM clause
  */
 /**
- * GdaSqlSelectField: (skip)
+ * GdaSqlSelectField:
  * @any: inheritance structure
  * @expr: expression
  * @field_name: field name part of @expr if @expr represents a field
@@ -412,6 +427,9 @@ struct _GdaSqlSelectField
 	gpointer         _gda_reserved1;
 	gpointer         _gda_reserved2;
 };
+#define GDA_TYPE_SQL_SELECT_FIELD (gda_sql_select_field_get_type())
+
+GType             gda_sql_select_field_get_type  (void) G_GNUC_CONST;
 
 GdaSqlSelectField *gda_sql_select_field_new            (GdaSqlAnyPart *parent);
 void               gda_sql_select_field_free           (GdaSqlSelectField *field);
@@ -427,7 +445,7 @@ void               gda_sql_select_field_take_alias     (GdaSqlSelectField *field
  * Any TARGET ... in a SELECT statement
  */
 /**
- * GdaSqlSelectTarget: (skip)
+ * GdaSqlSelectTarget:
  * @any: inheritance structure
  * @expr: expression
  * @table_name: table name part of @expr if @expr represents a table
@@ -454,6 +472,9 @@ struct _GdaSqlSelectTarget
 	gpointer         _gda_reserved1;
 	gpointer         _gda_reserved2;
 };
+#define GDA_TYPE_SQL_SELECT_TARGET (gda_sql_select_target_get_type())
+
+GType             gda_sql_select_target_get_type  (void) G_GNUC_CONST;
 
 GdaSqlSelectTarget *gda_sql_select_target_new            (GdaSqlAnyPart *parent);
 void                gda_sql_select_target_free           (GdaSqlSelectTarget *target);
@@ -487,7 +508,7 @@ typedef enum {
 } GdaSqlSelectJoinType;
 
 /**
- * GdaSqlSelectJoin: (skip)
+ * GdaSqlSelectJoin:
  * @any: inheritance structure
  * @type: type of join
  * @position: represents a join between a target at (pos &lt; @position) and the one at @position
@@ -509,7 +530,9 @@ struct _GdaSqlSelectJoin
 	gpointer         _gda_reserved1;
 	gpointer         _gda_reserved2;
 };
+#define GDA_TYPE_SQL_SELECT_JOIN (gda_sql_select_join_get_type())
 
+GType             gda_sql_select_join_get_type  (void) G_GNUC_CONST;
 GdaSqlSelectJoin  *gda_sql_select_join_new            (GdaSqlAnyPart *parent);
 void               gda_sql_select_join_free           (GdaSqlSelectJoin *join);
 GdaSqlSelectJoin  *gda_sql_select_join_copy           (GdaSqlSelectJoin *join);
@@ -522,7 +545,7 @@ const gchar       *gda_sql_select_join_type_to_string (GdaSqlSelectJoinType type
  * Any FROM ... in a SELECT statement
  */
 /**
- * GdaSqlSelectFrom: (skip)
+ * GdaSqlSelectFrom:
  * @any: inheritance structure
  * @targets: (element-type Gda.SqlSelectTarget): list of #GdaSqlSelectTarget
  * @joins: (element-type Gda.SqlSelectJoin): list of #GdaSqlSelectJoin
@@ -540,7 +563,9 @@ struct _GdaSqlSelectFrom
 	gpointer         _gda_reserved1;
 	gpointer         _gda_reserved2;
 };
+#define GDA_TYPE_SQL_SELECT_FROM (gda_sql_select_from_get_type())
 
+GType              gda_sql_select_from_get_type  (void) G_GNUC_CONST;
 GdaSqlSelectFrom  *gda_sql_select_from_new            (GdaSqlAnyPart *parent);
 void               gda_sql_select_from_free           (GdaSqlSelectFrom *from);
 GdaSqlSelectFrom  *gda_sql_select_from_copy           (GdaSqlSelectFrom *from);
@@ -553,7 +578,7 @@ void               gda_sql_select_from_take_new_join  (GdaSqlSelectFrom *from, G
  * Any expression in a SELECT ... after the ORDER BY
  */
 /**
- * GdaSqlSelectOrder: (skip)
+ * GdaSqlSelectOrder:
  * @any: inheritance structure
  * @expr: expression to order on
  * @asc: TRUE is ordering is ascending
@@ -573,7 +598,9 @@ struct _GdaSqlSelectOrder
 	gpointer         _gda_reserved1;
 	gpointer         _gda_reserved2;
 };
+#define GDA_TYPE_SQL_SELECT_ORDER (gda_sql_select_order_get_type())
 
+GType              gda_sql_select_order_get_type  (void) G_GNUC_CONST;
 GdaSqlSelectOrder *gda_sql_select_order_new            (GdaSqlAnyPart *parent);
 void               gda_sql_select_order_free           (GdaSqlSelectOrder *order);
 GdaSqlSelectOrder *gda_sql_select_order_copy           (GdaSqlSelectOrder *order);
