@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006 Bas Driessen <bas.driessen@xobas.com>
- * Copyright (C) 2006 - 2009 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2006 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2008 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  *
@@ -29,8 +29,14 @@ gchar *
 _gda_sqlite_render_CREATE_TABLE (GdaServerProvider *provider, GdaConnection *cnc, 
 				 GdaServerOperation *op, GError **error)
 {
+	g_return_val_if_fail (provider, NULL);
+	g_return_val_if_fail (cnc, NULL);
+	g_return_val_if_fail (op, NULL);
+	g_return_val_if_fail (gda_server_operation_get_op_type (op) == GDA_SERVER_OPERATION_CREATE_TABLE, NULL);
+
 	GString *string;
 	const GValue *value;
+	const GValue *value1;
 	gboolean allok = TRUE;
 	gboolean hasfields = FALSE;
 	gint nrows;
@@ -53,7 +59,12 @@ _gda_sqlite_render_CREATE_TABLE (GdaServerProvider *provider, GdaConnection *cnc
 	if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
 		g_string_append (string, "IF NOT EXISTS ");
 		
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DEF_P/TABLE_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DEF_P/TABLE_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		g_assert (*error != NULL);
+		return NULL;
+	}
 	g_string_append (string, tmp);
 	g_free (tmp);
 	g_string_append (string, " (");
@@ -63,7 +74,11 @@ _gda_sqlite_render_CREATE_TABLE (GdaServerProvider *provider, GdaConnection *cnc
 		GdaServerOperationNode *node;
 
 		node = gda_server_operation_get_node_info (op, "/FIELDS_A");
-		g_assert (node);
+		if (node == NULL) {
+			g_set_error (error, GDA_SERVER_OPERATION_ERROR, GDA_SERVER_OPERATION_INCORRECT_VALUE_ERROR,
+			     "%s", _("No fields are defined for CREATE TABLE operation"));
+			return NULL;
+		}
 
 		/* finding if there is a composed primary key */
 		nrows = gda_data_model_get_n_rows (node->model);
@@ -71,7 +86,13 @@ _gda_sqlite_render_CREATE_TABLE (GdaServerProvider *provider, GdaConnection *cnc
 			value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_PKEY/%d", i);
 			if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value)) {
 				tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider,
-										  "/FIELDS_A/@COLUMN_NAME/%d", i);
+										  "/FIELDS_A/@COLUMN_NAME/%d", error,
+										  i);
+				if (!tmp) {
+					g_string_free (string, TRUE);
+					g_assert (*error != NULL);
+					return NULL;
+				}
 				pkfields = g_slist_append (pkfields, tmp);
 				nbpkfields++;
 			}
@@ -87,7 +108,13 @@ _gda_sqlite_render_CREATE_TABLE (GdaServerProvider *provider, GdaConnection *cnc
 			else
 				g_string_append (string, ", ");
 				
-			tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/FIELDS_A/@COLUMN_NAME/%d", i);
+			tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider,
+									  "/FIELDS_A/@COLUMN_NAME/%d", error, i);
+			if (!tmp) {
+				g_string_free (string, TRUE);
+				g_assert (*error != NULL);
+				return NULL;
+			}
 			g_string_append (string, tmp);
 			g_free (tmp);
 			g_string_append_c (string, ' ');
@@ -107,8 +134,8 @@ _gda_sqlite_render_CREATE_TABLE (GdaServerProvider *provider, GdaConnection *cnc
 			}
 
 			if (!pkautoinc) {
-				value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_TYPE/%d", i);
-				g_string_append (string, g_value_get_string (value));
+				value1 = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_TYPE/%d", i);
+				g_string_append (string, g_value_get_string (value1));
 				
 				value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_SIZE/%d", i);
 				if (value && G_VALUE_HOLDS (value, G_TYPE_UINT)) {
@@ -126,7 +153,16 @@ _gda_sqlite_render_CREATE_TABLE (GdaServerProvider *provider, GdaConnection *cnc
 					const gchar *str = g_value_get_string (value);
 					if (str && *str) {
 						g_string_append (string, " DEFAULT ");
-						g_string_append (string, str);
+            const gchar* valtmp = g_value_get_string (value1);
+            if (!g_ascii_strcasecmp (valtmp,"string") ||
+                !g_ascii_strcasecmp (valtmp,"gchararray")) {
+              g_string_append_c (string,'\'');
+              g_string_append (string, str);
+              g_string_append_c (string,'\'');
+            }
+            else {
+              g_string_append (string, str);
+            }
 					}
 				}
 				
@@ -219,18 +255,22 @@ _gda_sqlite_render_CREATE_TABLE (GdaServerProvider *provider, GdaConnection *cnc
 		allok = FALSE;
 		g_set_error (error, GDA_SERVER_OPERATION_ERROR,
 			     GDA_SERVER_OPERATION_INCORRECT_VALUE_ERROR,
-			     "%s", _("Table to create must have at least one row"));
+			     "%s", _("Table to create must have at least one field"));
+		g_string_free (string, TRUE);
+		return NULL;
 	}
 
 	sql = string->str;
 	g_string_free (string, FALSE);
-
+#ifdef GDA_DEBUG
+	g_print ("Renderer SQL for SQLite: %s\n", sql);
+#endif
 	return sql;
 }
 
 gchar *
 _gda_sqlite_render_DROP_TABLE (GdaServerProvider *provider, GdaConnection *cnc, 
-			       GdaServerOperation *op, G_GNUC_UNUSED GError **error)
+			       GdaServerOperation *op, GError **error)
 {
 	GString *string;
 	const GValue *value;
@@ -244,7 +284,11 @@ _gda_sqlite_render_DROP_TABLE (GdaServerProvider *provider, GdaConnection *cnc,
 	if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
 		g_string_append (string, " IF EXISTS");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DESC_P/TABLE_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DESC_P/TABLE_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
 	g_string_append_c (string, ' ');
 	g_string_append (string, tmp);
 	g_free (tmp);
@@ -257,7 +301,7 @@ _gda_sqlite_render_DROP_TABLE (GdaServerProvider *provider, GdaConnection *cnc,
 
 gchar *
 _gda_sqlite_render_RENAME_TABLE (GdaServerProvider *provider, GdaConnection *cnc, 
-				 GdaServerOperation *op, G_GNUC_UNUSED GError **error)
+				 GdaServerOperation *op, GError **error)
 {
 	GString *string;
 	gchar *sql = NULL;
@@ -266,11 +310,19 @@ _gda_sqlite_render_RENAME_TABLE (GdaServerProvider *provider, GdaConnection *cnc
 	/* DROP TABLE */
 	string = g_string_new ("ALTER TABLE ");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DESC_P/TABLE_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DESC_P/TABLE_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
 	g_string_append (string, tmp);
 	g_free (tmp);
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DESC_P/TABLE_NEW_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DESC_P/TABLE_NEW_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
 	g_string_append (string, " RENAME TO ");
 	g_string_append (string, tmp);
 	g_free (tmp);
@@ -293,13 +345,21 @@ _gda_sqlite_render_ADD_COLUMN (GdaServerProvider *provider, GdaConnection *cnc,
 	/* DROP TABLE */
 	string = g_string_new ("ALTER TABLE ");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/COLUMN_DEF_P/TABLE_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/COLUMN_DEF_P/TABLE_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
 	g_string_append (string, tmp);
 	g_free (tmp);
 
 	g_string_append (string, " ADD COLUMN ");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/COLUMN_DEF_P/COLUMN_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/COLUMN_DEF_P/COLUMN_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
 	g_string_append (string, tmp);
 	g_free (tmp);
 
@@ -377,13 +437,21 @@ _gda_sqlite_render_CREATE_INDEX (GdaServerProvider *provider, GdaConnection *cnc
 		g_string_append (string, " IF NOT EXISTS ");
 	
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/INDEX_DEF_P/INDEX_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/INDEX_DEF_P/INDEX_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
 	g_string_append (string, tmp);
 	g_free (tmp);
 
 	g_string_append (string, " ON ");
 	
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/INDEX_DEF_P/INDEX_ON_TABLE");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/INDEX_DEF_P/INDEX_ON_TABLE", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
 	g_string_append (string, tmp);
 	g_free (tmp);
 
@@ -394,7 +462,7 @@ _gda_sqlite_render_CREATE_INDEX (GdaServerProvider *provider, GdaConnection *cnc
 	nrows = gda_server_operation_get_sequence_size (op, "/INDEX_FIELDS_S");
 	for (i = 0; i < nrows; i++) {
 		tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider,
-								  "/INDEX_FIELDS_S/%d/INDEX_FIELD", i);
+								  "/INDEX_FIELDS_S/%d/INDEX_FIELD", error, i);
 		if (tmp) {
 			if (i != 0)
 				g_string_append (string, ", ");
@@ -420,6 +488,10 @@ _gda_sqlite_render_CREATE_INDEX (GdaServerProvider *provider, GdaConnection *cnc
 				}
 			}
 		}
+		else {
+			g_string_free (string, TRUE);
+			return NULL;
+		}
 	}
 
 	g_string_append (string, ")");
@@ -446,7 +518,11 @@ _gda_sqlite_render_DROP_INDEX (GdaServerProvider *provider, GdaConnection *cnc,
 	if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
 		g_string_append (string, "IF EXISTS ");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/INDEX_DESC_P/INDEX_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/INDEX_DESC_P/INDEX_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
 	g_string_append (string, tmp);
 	g_free (tmp);
 
@@ -477,7 +553,11 @@ _gda_sqlite_render_CREATE_VIEW (GdaServerProvider *provider, GdaConnection *cnc,
 	if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
 		g_string_append (string, "IF NOT EXISTS ");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/VIEW_DEF_P/VIEW_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/VIEW_DEF_P/VIEW_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
 	g_string_append (string, tmp);
 	g_free (tmp);
 	
@@ -515,7 +595,11 @@ _gda_sqlite_render_DROP_VIEW (GdaServerProvider *provider, GdaConnection *cnc,
 	if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
 		g_string_append (string, " IF EXISTS");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/VIEW_DESC_P/VIEW_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/VIEW_DESC_P/VIEW_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
 	g_string_append_c (string, ' ');
 	g_string_append (string, tmp);
 	g_free (tmp);
