@@ -3,13 +3,14 @@
  * Copyright (C) 2002 - 2003 Gonzalo Paniagua Javier <gonzalo@src.gnome.org>
  * Copyright (C) 2002 - 2005 Rodrigo Moya <rodrigo@gnome-db.org>
  * Copyright (C) 2005 Denis Fortin <denis.fortin@free.fr>
- * Copyright (C) 2005 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2005 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2005 Álvaro Peña <alvaropg@telefonica.net>
  * Copyright (C) 2006 - 2008 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2007 Armin Burgmeier <arminb@src.gnome.org>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2011 Marco Ciampa <ciampix@libero.it>
+ * Copyright (C) 2017 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -219,7 +220,7 @@ _gda_sqlite_recordset_new (GdaConnection *cnc, GdaSqlitePStmt *ps, GdaSet *exec_
         g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
         g_return_val_if_fail (ps != NULL, NULL);
 
-	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data_error (cnc, NULL);
 	if (!cdata)
 		return NULL;
 
@@ -287,7 +288,8 @@ _gda_sqlite_recordset_new (GdaConnection *cnc, GdaSqlitePStmt *ps, GdaSet *exec_
 		rflags = GDA_DATA_MODEL_ACCESS_CURSOR_FORWARD;
 
 	/* create data model */
-        model = g_object_new (GDA_TYPE_SQLITE_RECORDSET, "connection", cnc, 
+        model = g_object_new (GDA_TYPE_SQLITE_RECORDSET,
+			      "connection", cnc,
 			      "prepared-stmt", ps, "model-usage", rflags, 
 			      "exec-params", exec_params, 
 			      "auto-reset", force_empty, NULL);
@@ -349,9 +351,11 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 	SqliteConnectionData *cdata;
 	GdaSqlitePStmt *ps;
 	GdaRow *prow = NULL;
+	GdaConnection *cnc;
+	glong length;
 
-	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data_error 
-		(gda_data_select_get_connection ((GdaDataSelect*) model), error);
+	cnc = gda_data_select_get_connection ((GdaDataSelect*) model);
+	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data_error (cnc, error);
 	if (!cdata)
 		return NULL;
 	ps = GDA_SQLITE_PSTMT (GDA_DATA_SELECT (model)->prep_stmt);
@@ -464,16 +468,12 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 				else if (type == GDA_TYPE_BINARY) {
 					GdaBinary *bin;
 					
-					bin = g_new0 (GdaBinary, 1);
-					bin->binary_length = SQLITE3_CALL (sqlite3_column_bytes) (ps->sqlite_stmt, real_col);
-					if (bin->binary_length > 0) {
-						bin->data = g_new (guchar, bin->binary_length);
-						memcpy (bin->data, SQLITE3_CALL (sqlite3_column_blob) (ps->sqlite_stmt, /* Flawfinder: ignore */
-												       real_col),
-							bin->binary_length);
+					bin = gda_binary_new ();
+					length = SQLITE3_CALL (sqlite3_column_bytes) (ps->sqlite_stmt, real_col);
+					if (length > 0) {
+						gda_binary_set_data (bin, SQLITE3_CALL (sqlite3_column_blob) (ps->sqlite_stmt, /* Flawfinder: ignore */
+												       real_col), length);
 					}
-					else
-						bin->binary_length = 0;
 					gda_value_take_binary (value, bin);
 				}
 				else if (type == GDA_TYPE_BLOB) {
@@ -497,7 +497,7 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 						gint64 rowid;
 						rowid = SQLITE3_CALL (sqlite3_column_int64) (ps->sqlite_stmt, oidcol - 1); /* remove 1
 													       because it was added in the first place */
-						bop = _gda_sqlite_blob_op_new (cdata,
+						bop = _gda_sqlite_blob_op_new (cnc,
 									       SQLITE3_CALL (sqlite3_column_database_name) (ps->sqlite_stmt, 
 													    real_col),
 									       SQLITE3_CALL (sqlite3_column_table_name) (ps->sqlite_stmt,
@@ -515,7 +515,7 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 					}
 					else {
 						GdaBlob *blob;
-						blob = g_new0 (GdaBlob, 1);
+						blob = gda_blob_new ();
 						gda_blob_set_op (blob, bop);
 						g_object_unref (bop);
 						gda_value_take_blob (value, blob);
@@ -539,8 +539,8 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 						g_value_set_boxed (value, &date);
 				}
 				else if (type == GDA_TYPE_TIME) {
-					GdaTime timegda;
-					if (!gda_parse_iso8601_time (&timegda, 
+					GdaTime* timegda = gda_time_new ();
+					if (!gda_parse_iso8601_time (timegda,
 								     (gchar *) SQLITE3_CALL (sqlite3_column_text) (ps->sqlite_stmt, 
 												    real_col))) {
 						GError *lerror = NULL;
@@ -551,27 +551,27 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 						gda_row_invalidate_value_e (prow, value, lerror);
 					}
 					else {
-						if (timegda.timezone == GDA_TIMEZONE_INVALID)
-							timegda.timezone = 0; /* set to GMT */
-						gda_value_set_time (value, &timegda);
+						if (gda_time_get_timezone (timegda) == GDA_TIMEZONE_INVALID)
+							gda_time_set_timezone (timegda, 0); /* set to GMT */
+						gda_value_set_time (value, timegda);
 					}
+					gda_time_free (timegda);
 				}
-				else if (type == GDA_TYPE_TIMESTAMP) {
-					GdaTimestamp timestamp;
-					if (!gda_parse_iso8601_timestamp (&timestamp, 
+				else if (g_type_is_a (type, G_TYPE_DATE_TIME)) {
+					GDateTime* timestamp = gda_parse_iso8601_timestamp (
 									  (gchar *) SQLITE3_CALL (sqlite3_column_text) (ps->sqlite_stmt,
-													 real_col))) {
+										real_col));
+					if (timestamp == NULL) {
 						GError *lerror = NULL;
 						g_set_error (&lerror, GDA_SERVER_PROVIDER_ERROR,
 							     GDA_SERVER_PROVIDER_DATA_ERROR,
-							     _("Invalid timestamp '%s' (format should be YYYY-MM-DD HH:MM:SS[.ms])"), 
+							     _("Invalid timestamp '%s' (format should be YYYY-MM-DDTHH:MM:SS[.ms])"),
 							     (gchar *) SQLITE3_CALL (sqlite3_column_text) (ps->sqlite_stmt, real_col));
 						gda_row_invalidate_value_e (prow, value, lerror);
 					}
 					else {
-						if (timestamp.timezone == GDA_TIMEZONE_INVALID)
-							timestamp.timezone = 0; /* set to GMT */
-						gda_value_set_timestamp (value, &timestamp);
+						g_value_set_boxed (value, timestamp);
+					  g_date_time_unref (timestamp);
 					}
 				}
 				else if (type == G_TYPE_CHAR) {
