@@ -32,7 +32,7 @@
 #include "gda-sqlite-recordset.h"
 
 #include <libgda/sqlite/keywords_hash.h>
-#include "keywords_hash.c" /* this one is dynamically generated */
+#include "keywords_hash.code" /* this one is dynamically generated */
 
 static guint
 nocase_str_hash (gconstpointer v)
@@ -91,7 +91,7 @@ _gda_sqlite_compute_types_hash (SqliteConnectionData *cdata)
 		array [i] = type;
 		g_hash_table_insert (hash, "time", array + i);
 
-		type = GDA_TYPE_TIMESTAMP;
+		type = G_TYPE_DATE_TIME;
 		i++;
 		array [i] = type;
 		g_hash_table_insert (hash, "timestamp", array + i);
@@ -165,14 +165,6 @@ _gda_sqlite_compute_g_type (int sqlite_type)
 
 
 
-#ifdef GDA_DEBUG
-void
-_gda_sqlite_test_keywords (void)
-{
-        test_keywords();
-}
-#endif
-
 GdaSqlReservedKeywordsFunc
 _gda_sqlite_get_reserved_keyword_func (void)
 {
