@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2001 - 2002 Carlos Perell Marn <carlos@gnome-db.org>
+ * Copyright (C) 2001 - 2002 Carlos Perelló Marín <carlos@gnome-db.org>
  * Copyright (C) 2001 - 2002 Rodrigo Moya <rodrigo@gnome-db.org>
- * Copyright (C) 2001 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2001 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2002 Gonzalo Paniagua Javier <gonzalo@src.gnome.org>
  * Copyright (C) 2005 Denis Fortin <denis.fortin@free.fr>
  * Copyright (C) 2008 Przemysław Grzegorczyk <pgrzegorczyk@gmail.com>
@@ -27,6 +27,7 @@
 
 #include <glib.h>
 #include <libgda/libgda.h>
+#include <libgda/gda-connection-private.h>
 
 #ifdef WITH_BDBSQLITE
   #include <dbsql.h>
@@ -55,6 +56,7 @@
  * Provider's specific connection data
  */
 typedef struct {
+	GdaServerProviderConnectionData parent;
 	GdaConnection *gdacnc;
 	sqlite3      *connection;
 	gchar        *file;
