@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2012 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -107,7 +107,7 @@ main (int argc,char** argv)
 		
 		/* copy the file into the buffer: */
 		n_read = fread (buffer + buffer_index, 1, fsize, file);
-		if (n_read != fsize) {
+		if (n_read != (uint) fsize) {
 			fprintf (stderr, "Reading error");
 			fclose (file);
 			return 1;
@@ -116,8 +116,8 @@ main (int argc,char** argv)
 		/* alter buffer */
 		int added = 0;
 		int real_added = 0;
-		for (i = buffer_index; i < buffer_index + n_read + added; i++) {
-			if (buffer_index + n_read + added + 2 >= maxlen) {
+		for (i = buffer_index; (uint) i < buffer_index + n_read + added; i++) {
+			if (buffer_index + n_read + added + 2 >= (uint) maxlen) {
 				fprintf (stderr, "Max buffer size reached\nIncrease MAXSIZE constant and re-run\n");
 				fclose (file);
 				return 1;
