@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2011 - 2015 Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -42,8 +42,7 @@ typedef struct {
 static void
 _ldap_table_map_free (LdapTableMap *map)
 {
-	if (map->ldap_cnc)
-		g_object_unref (map->ldap_cnc);
+	map->ldap_cnc = NULL;
 	g_free (map->table_name);
 	g_free (map->base_dn);
 	g_free (map->filter);
@@ -165,23 +164,30 @@ vtable_dropped (GdaVconnectionDataModel *cnc, const gchar *table_name)
 	LdapTableMap *map = NULL;
 	GSList *list;
 
-	lcnc = GDA_LDAP_CONNECTION (cnc);
-	for (list = lcnc->priv->maps; list; list = list->next) {
-		if (!strcmp (((LdapTableMap*)list->data)->table_name, table_name)) {
-			map = (LdapTableMap*)list->data;
-			break;
+	lcnc = (GdaLdapConnection*) cnc;
+
+	/* if @lcnc->priv is NULL, then @cnc is currently being destroyed */
+	if (lcnc->priv) {
+		for (list = lcnc->priv->maps; list; list = list->next) {
+			if (!strcmp (((LdapTableMap*)list->data)->table_name, table_name)) {
+				map = (LdapTableMap*)list->data;
+				break;
+			}
+		}
+		if (map) {
+			lcnc->priv->maps = g_slist_remove (lcnc->priv->maps, map);
+#ifdef GDA_DEBUG_NO
+			g_print ("VTable dropped: %s\n", table_name);
+			dump_vtables (lcnc);
+#endif
 		}
 	}
-	if (map) {
-		lcnc->priv->maps = g_slist_remove (lcnc->priv->maps, map);
-#ifdef GDA_DEBUG_NO
-		g_print ("VTable dropped: %s\n", table_name);
-		dump_vtables (lcnc);
-#endif
-	}
+
 	if (GDA_VCONNECTION_DATA_MODEL_CLASS (parent_class)->vtable_dropped)
 		GDA_VCONNECTION_DATA_MODEL_CLASS (parent_class)->vtable_dropped (cnc, table_name);
-	update_connection_startup_file (GDA_LDAP_CONNECTION (cnc));
+
+	if (lcnc->priv)
+		update_connection_startup_file (GDA_LDAP_CONNECTION (cnc));
 }
 
 /*
@@ -268,7 +274,7 @@ gda_ldap_connection_init (GdaLdapConnection *cnc, G_GNUC_UNUSED GdaLdapConnectio
 
 	g_signal_connect (cnc, "notify::dsn",
 			  G_CALLBACK (dsn_set_cb), NULL);
-	g_signal_connect (cnc, "conn-opened",
+	g_signal_connect (cnc, "opened",
 			  G_CALLBACK (conn_opened_cb), NULL);
 }
 
@@ -646,17 +652,17 @@ _ldap_table_create_model_func (GdaVconnectionDataModelSpec *spec, G_GNUC_UNUSED
 		/*g_print ("FILTER to use: LDAPFilter=> [%s] LDAPDn => [%s] SCOPE => [%d]\n",
 			 real_filter ? real_filter->str : NULL,
 			 real_dn ? real_dn : map->base_dn, real_scope);*/
-		model = gda_data_model_ldap_new (map->ldap_cnc,
+		model = GDA_DATA_MODEL (gda_data_model_ldap_new_with_config (map->ldap_cnc,
 						 real_dn ? real_dn : map->base_dn,
 						 real_filter ? real_filter->str : NULL,
-						 map->attributes, real_scope);
+						 map->attributes, real_scope));
 		if (real_filter)
 			g_string_free (real_filter, TRUE);
 		g_free (real_dn);
 	}
 	else
-		model = gda_data_model_ldap_new (map->ldap_cnc, map->base_dn, map->filter,
-						 map->attributes, map->scope);
+		model = GDA_DATA_MODEL (gda_data_model_ldap_new_with_config (map->ldap_cnc, map->base_dn, map->filter,
+						 map->attributes, map->scope));
 
 	return model;
 }
@@ -726,7 +732,8 @@ gda_ldap_connection_declare_table (GdaLdapConnection *cnc, const gchar *table_na
         GDA_VCONNECTION_DATA_MODEL_SPEC (map)->create_model_func = NULL;
         GDA_VCONNECTION_DATA_MODEL_SPEC (map)->create_filter_func = _ldap_table_create_filter;
         GDA_VCONNECTION_DATA_MODEL_SPEC (map)->create_filtered_model_func = _ldap_table_create_model_func;
-	map->ldap_cnc = g_object_ref (cnc);
+	map->ldap_cnc = GDA_CONNECTION (cnc); /* we can't hold a reference here because it would be self referencing,
+					       * and we don't need to because the @map is destroyed when @cnc is destroyed */
 	map->table_name = gda_sql_identifier_quote (table_name, GDA_CONNECTION (cnc), NULL, TRUE, FALSE);
 	map->filters_hash = NULL;
 	if (base_dn)
@@ -990,7 +997,7 @@ GdaLdapEntry **_gda_ldap_get_entry_children (GdaLdapConnection *cnc, const gchar
  * Describes the LDAP entry which DN is @dn. If @dn is %NULL, then the top entry (as specified when 
  * the LDAP connection was opened) is described.
  *
- * Returns: (transfer full) (Free-function: gda_ldap_entry_free): a new #GdaLdapEntry, or %NULL if an error occurred or if the @dn entry does not exist
+ * Returns: (transfer full): a new #GdaLdapEntry, or %NULL if an error occurred or if the @dn entry does not exist
  *
  * Since: 4.2.8
  */
@@ -1014,7 +1021,7 @@ gda_ldap_describe_entry (GdaLdapConnection *cnc, const gchar *dn, GError **error
  *
  * If @dn is %NULL, then the top entry (as specified when the LDAP connection was opened) is used.
  *
- * Returns: (transfer full) (element_type GdaLdapEntry) (array zero-terminated=1): a %NULL terminated array of #GdaLdapEntry for each child entry, or %NULL if an error occurred or if the @dn entry does not exist
+ * Returns: (transfer full) (array zero-terminated=1): a %NULL terminated array of #GdaLdapEntry for each child entry, or %NULL if an error occurred or if the @dn entry does not exist
  *
  * Since: 4.2.8
  */
@@ -1035,7 +1042,7 @@ gchar **_gda_ldap_dn_split (const gchar *dn, gboolean all);
  *
  * Splits @dn into its components.
  *
- * Returns: (transfer full) (Free-function: g_strfreev): a %NULL terminated array containing the DN parts (free using g_strfreev()), or %NULL if an error occurred because @dn is not a valid DN expression
+ * Returns: (transfer full): a %NULL terminated array containing the DN parts (free using g_strfreev()), or %NULL if an error occurred because @dn is not a valid DN expression
  *
  * Since: 4.2.8
  */
@@ -1199,7 +1206,7 @@ gda_ldap_add_entry (GdaLdapConnection *cnc, GdaLdapEntry *entry, GError **error)
 /**
  * gda_ldap_remove_entry:
  * @cnc: a #GdaLdapConnection
- * @entry: a #GdaLDapEntry describing the LDAP entry to remove
+ * @dn: the DN of the LDAP entry to remove
  * @error: (allow-none): a place to store an error, or %NULL
  *
  * Delete an LDAP entry.
