@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2011 - 2014 Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -72,7 +72,7 @@ struct _GdaLdapConnectionClass {
  *   <listitem>
  *      <para>The CREATE LDAP TABLE: <synopsis>CREATE LDAP TABLE &lt;table name&gt; [BASE=&lt;base DN&gt;] [FILTER=&lt;LDAP filter&gt;] [ATTRIBUTES=&lt;LDAP attributes&gt;] [SCOPE=&lt;search scope&gt;]</synopsis>
  *      </para>
- *      <para>Each of the BASE, FILTER, ATTRIBUTES and SCOPE specifications is optional. Use this command to declare a table, for example: <programlisting>CREATE LDAP TABLE users FILTER='(cn=*doe*)' SCOPE= 'SUBTREE';</programlisting>.
+ *      <para>Each of the BASE, FILTER, ATTRIBUTES and SCOPE specifications is optional. Use this command to declare a table, for example: <programlisting>CREATE LDAP TABLE users FILTER='(cn=*doe*)' SCOPE= 'SUBTREE';</programlisting>
  *        The allowed SCOPE values are: 'BASE', 'ONELEVEL' and 'SUBTREE'.
  *      </para>
  *      <para>See the <link linkend="gda-ldap-connection-declare-table">gda_ldap_connection_declare_table()</link>
@@ -94,6 +94,9 @@ struct _GdaLdapConnectionClass {
  *        specified parameters are altered (ie. you don't need to repeat the parameters you don't want to
  *        be modified)
  *      </para>
+ *      <para>Again, refer to the <link linkend="gda-ldap-connection-declare-table">gda_ldap_connection_declare_table()</link>
+ *        section for more information about the ATTRIBUTES syntax.
+ *      </para>
  *   </listitem>
  *   <listitem>
  *      <para>The DESCRIBE LDAP TABLE: <synopsis>DESCRIBE LDAP TABLE &lt;table name&gt;</synopsis> 
@@ -162,7 +165,11 @@ typedef struct {
 	GHashTable        *attributes_hash;
 } GdaLdapEntry;
 
+GdaLdapEntry  *gda_ldap_entry_new                  (const gchar *dn);
+void           gda_ldap_entry_add_attribute        (GdaLdapEntry *entry, gboolean merge, const gchar *attr_name,
+						    guint nb_values, GValue **values);
 void           gda_ldap_entry_free                 (GdaLdapEntry *entry);
+
 GdaLdapEntry  *gda_ldap_describe_entry             (GdaLdapConnection *cnc, const gchar *dn, GError **error);
 GdaLdapEntry **gda_ldap_get_entry_children         (GdaLdapConnection *cnc, const gchar *dn,
 						    gchar **attributes, GError **error);
@@ -170,6 +177,14 @@ GdaLdapEntry **gda_ldap_get_entry_children         (GdaLdapConnection *cnc, cons
 gchar        **gda_ldap_dn_split                   (const gchar *dn, gboolean all);
 gboolean       gda_ldap_is_dn                      (const gchar *dn);
 
+typedef struct {
+	gchar   *name;
+	GType    g_type;
+	gboolean required;
+} GdaLdapAttributeDefinition;
+void           gda_ldap_attributes_list_free      (GSList *list);
+GSList        *gda_ldap_entry_get_attributes_list (GdaLdapConnection *cnc, GdaLdapEntry *entry);
+
 
 /**
  * GdaLdapClassKind:
@@ -217,13 +232,40 @@ typedef struct {
 	guint             nb_opt_attributes;
 	gchar           **opt_attributes;
 
-	GSList           *parents; /* list of #LdapClass */
-	GSList           *children; /* list of #LdapClass */
+	GSList           *parents; /* list of #GdaLdapClass */
+	GSList           *children; /* list of #GdaLdapClass */
 } GdaLdapClass;
 
 GdaLdapClass   *gda_ldap_get_class_info (GdaLdapConnection *cnc, const gchar *classname);
 const GSList   *gda_ldap_get_top_classes (GdaLdapConnection *cnc);
 
+
+/**
+ * GdaLdapModificationType:
+ * @GDA_LDAP_MODIFICATION_INSERT: modification corresponds to a new LDAP entry
+ * @GDA_LDAP_MODIFICATION_DELETE: modification corresponds to removing an LDAP entry
+ * @GDA_LDAP_MODIFICATION_ATTR_ADD: modification correspond to adding attributes to an existing LDAP entry
+ * @GDA_LDAP_MODIFICATION_ATTR_DEL: modification correspond to removing attributes from an existing LDAP entry
+ * @GDA_LDAP_MODIFICATION_ATTR_REPL: modification correspond to replacing attributes of an existing LDAP entry
+ * @GDA_LDAP_MODIFICATION_ATTR_DIFF: modification correspond to modifying attributes to an existing LDAP entry
+ *
+ * Speficies the type of operation requested when writing to an LDAP directory.
+ */
+typedef enum {
+	GDA_LDAP_MODIFICATION_INSERT,
+	GDA_LDAP_MODIFICATION_DELETE,
+	GDA_LDAP_MODIFICATION_ATTR_ADD,
+	GDA_LDAP_MODIFICATION_ATTR_DEL,
+	GDA_LDAP_MODIFICATION_ATTR_REPL,
+	GDA_LDAP_MODIFICATION_ATTR_DIFF
+} GdaLdapModificationType;
+
+gboolean        gda_ldap_add_entry    (GdaLdapConnection *cnc, GdaLdapEntry *entry, GError **error);
+gboolean        gda_ldap_remove_entry (GdaLdapConnection *cnc, const gchar *dn, GError **error);
+gboolean        gda_ldap_rename_entry (GdaLdapConnection *cnc, const gchar *current_dn, const gchar *new_dn, GError **error);
+gboolean        gda_ldap_modify_entry (GdaLdapConnection *cnc, GdaLdapModificationType modtype,
+				       GdaLdapEntry *entry, GdaLdapEntry *ref_entry, GError **error);
+
 G_END_DECLS
 
 #endif
