@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2007 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2008 - 2011 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
@@ -21,6 +21,7 @@
  */
 
 #include <glib/gi18n-lib.h>
+#include <glib.h>
 #include <string.h>
 #include "gda-vconnection-data-model.h"
 #include "gda-vconnection-data-model-private.h"
@@ -30,9 +31,10 @@
 #include "../gda-sqlite.h"
 
 struct _GdaVconnectionDataModelPrivate {
-	GSList       *table_data_list; /* list of GdaVConnectionTableData structures */
+	GSList *table_data_list; /* list of GdaVConnectionTableData structures */
+	gboolean being_disposed;
 
-	GRecMutex     lock_context;
+	GMutex        lock_context;
 	GdaStatement *executed_stmt;
 };
 
@@ -81,7 +83,8 @@ vtable_dropped (GdaVconnectionDataModel *cnc, const gchar *table_name)
 	td = _gda_vconnection_get_table_data_by_name (cnc, table_name);
 	if (td)
 		cnc->priv->table_data_list = g_slist_remove (cnc->priv->table_data_list, td);
-	_gda_connection_signal_meta_table_update ((GdaConnection *)cnc, table_name);
+	if (! cnc->priv->being_disposed)
+		_gda_connection_signal_meta_table_update ((GdaConnection *)cnc, table_name);
 #ifdef GDA_DEBUG_NO
 	dump_all_tables (cnc);
 #endif
@@ -139,7 +142,8 @@ gda_vconnection_data_model_init (GdaVconnectionDataModel *cnc, G_GNUC_UNUSED Gda
 {
 	cnc->priv = g_new (GdaVconnectionDataModelPrivate, 1);
 	cnc->priv->table_data_list = NULL;
-	g_rec_mutex_init (& (cnc->priv->lock_context));
+	cnc->priv->being_disposed = FALSE;
+	g_mutex_init (& (cnc->priv->lock_context));
 
 	g_object_set (G_OBJECT (cnc), "cnc-string", "_IS_VIRTUAL=TRUE", NULL);
 }
@@ -153,14 +157,15 @@ gda_vconnection_data_model_dispose (GObject *object)
 
 	/* free memory */
 	if (cnc->priv) {
+		cnc->priv->being_disposed = TRUE;
 		while (cnc->priv->table_data_list) {
 			GdaVConnectionTableData *td;
 			td = (GdaVConnectionTableData *) cnc->priv->table_data_list->data;
 			get_rid_of_vtable (cnc, td, TRUE, NULL);
 		}
-		gda_connection_close_no_warning ((GdaConnection *) cnc);
+		gda_connection_close ((GdaConnection *) cnc, NULL);
 
-		g_rec_mutex_clear (& (cnc->priv->lock_context));
+		g_mutex_clear (& (cnc->priv->lock_context));
 		g_free (cnc->priv);
 		cnc->priv = NULL;
 	}
@@ -246,13 +251,13 @@ gboolean
 gda_vconnection_data_model_add_model (GdaVconnectionDataModel *cnc, 
 				      GdaDataModel *model, const gchar *table_name, GError **error)
 {
-	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
 	GdaVconnectionDataModelSpec *spec;
 	gboolean retval;
 
 	spec = g_new0 (GdaVconnectionDataModelSpec, 1);
-	spec->data_model = g_object_ref (model);
+	spec->data_model = model;
 	spec->create_columns_func = create_columns;
+	g_object_ref (model);
 	retval = gda_vconnection_data_model_add (cnc, spec, (GDestroyNotify) spec_destroy_func, table_name, error);
 
 	return retval;
@@ -597,6 +602,7 @@ _gda_vconnection_data_model_table_data_free (GdaVConnectionTableData *td)
 	g_free (td);
 }
 
+
 static void
 vcontext_object_weak_notify_cb (VContext *context, GObject *old_context_object)
 {
@@ -612,8 +618,13 @@ vcontext_free (VContext *context)
 		g_object_weak_unref (context->context_object,
 				     (GWeakNotify) vcontext_object_weak_notify_cb, context);
 	if (context->context_data) {
+		guint i;
+		for (i = 0; i < context->context_data->len; i++) {
+			VirtualFilteredData *data;
+			data = g_array_index (context->context_data, VirtualFilteredData*, i);
+			_gda_vconnection_virtual_filtered_data_unref (data);
+		}
 		g_array_free (context->context_data, TRUE);
-		context->context_data = NULL;
 	}
 	g_free (context);
 #ifdef DEBUG_VCONTEXT
@@ -626,7 +637,7 @@ _gda_vconnection_set_working_obj (GdaVconnectionDataModel *cnc, GObject *obj)
 {
 	GSList *list;
 	if (obj) {
-		g_rec_mutex_lock (& (cnc->priv->lock_context));
+		g_mutex_lock (& (cnc->priv->lock_context));
 		for (list = cnc->priv->table_data_list; list; list = list->next) {
 			GdaVConnectionTableData *td = (GdaVConnectionTableData*) list->data;
 			VContext *vc = NULL;
@@ -644,7 +655,6 @@ _gda_vconnection_set_working_obj (GdaVconnectionDataModel *cnc, GObject *obj)
 				vc->context_object = obj;
 				vc->context_data = g_array_new (FALSE, FALSE,
 								sizeof (VirtualFilteredData*));
-				g_array_set_clear_func (vc->context_data, (GDestroyNotify) _gda_vconnection_virtual_filtered_data_unref);
 				vc->vtable = td;
 				g_object_weak_ref (obj, (GWeakNotify) vcontext_object_weak_notify_cb, vc);
 				g_hash_table_insert (td->context.hash, obj, vc);
@@ -662,7 +672,7 @@ _gda_vconnection_set_working_obj (GdaVconnectionDataModel *cnc, GObject *obj)
 			 * an exception already occurred */
 			td->context.current_vcontext = NULL;
 		}
-		g_rec_mutex_unlock (& (cnc->priv->lock_context));
+		g_mutex_unlock (& (cnc->priv->lock_context));
 	}
 }
 
