@@ -1,8 +1,9 @@
 /*
- * Copyright (C) 2007 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2007 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2008 - 2011 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2017 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,6 +30,7 @@
 #include <libgda/gda-data-select.h>
 #include <gda-sql-builder.h>
 #include "../gda-sqlite.h"
+#include <libgda/gda-connection-internal.h>
 
 typedef struct {
 	GdaVconnectionHub *hub;
@@ -86,7 +88,7 @@ gda_vconnection_hub_dispose (GObject *object)
 
 	/* free memory */
 	if (cnc->priv) {
-		gda_connection_close_no_warning ((GdaConnection *) cnc);
+		gda_connection_close ((GdaConnection *) cnc, NULL);
 		g_assert (!cnc->priv->hub_connections);
 
 		g_free (cnc->priv);
@@ -681,8 +683,8 @@ create_value_from_sqlite3_gvalue (GType type, GValue *svalue, GError **error)
 		if (G_VALUE_TYPE (svalue) != G_TYPE_STRING)
 			allok = FALSE;
 		else {
-			GdaTime timegda;
-			if (!gda_parse_iso8601_time (&timegda, g_value_get_string (svalue))) {
+			GdaTime* timegda = gda_time_new ();
+			if (!gda_parse_iso8601_time (timegda, g_value_get_string (svalue))) {
 				g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 					     GDA_SERVER_PROVIDER_DATA_ERROR,
 					     _("Invalid time '%s' (time format should be HH:MM:SS[.ms])"),
@@ -690,23 +692,26 @@ create_value_from_sqlite3_gvalue (GType type, GValue *svalue, GError **error)
 				allok = FALSE;
 			}
 			else
-				gda_value_set_time (value, &timegda);
+				gda_value_set_time (value, timegda);
+			gda_time_free (timegda);
 		}
 	}
-	else if (type == GDA_TYPE_TIMESTAMP) {
-		if (G_VALUE_TYPE (svalue) != G_TYPE_STRING)
+	else if (g_type_is_a (type, G_TYPE_DATE_TIME)) {
+		if (!g_type_is_a (G_VALUE_TYPE (svalue), G_TYPE_STRING))
 			allok = FALSE;
 		else {
-			GdaTimestamp timestamp;
-			if (!gda_parse_iso8601_timestamp (&timestamp, g_value_get_string (svalue))) {
+			GDateTime* timestamp = gda_parse_iso8601_timestamp (g_value_get_string (svalue));
+			if (timestamp == NULL) {
 				g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 					     GDA_SERVER_PROVIDER_DATA_ERROR,
-					     _("Invalid timestamp '%s' (format should be YYYY-MM-DD HH:MM:SS[.ms])"),
+					     _("Invalid timestamp '%s' (format should be YYYY-MM-DDTHH:MM:SS[.ms])"),
 					     g_value_get_string (svalue));
 				allok = FALSE;
 			}
-			else
-				gda_value_set_timestamp (value, &timestamp);
+			else {
+        g_value_set_boxed (value, timestamp);
+			  g_date_time_unref (timestamp);
+      }
 		}
 	}
 	else
@@ -737,8 +742,8 @@ dict_table_create_model_func (GdaVconnectionDataModelSpec *spec, G_GNUC_UNUSED i
 		if (! gda_statement_get_parameters (stmt, &params, NULL))
 			return NULL;
 		if (argc > 0) {
-			g_assert (params && ((guint)argc == g_slist_length (params->holders)));
-			for (i = 0, list = params->holders; i < argc; i++, list = list->next) {
+			g_assert (params && ((guint)argc == g_slist_length (gda_set_get_holders (params))));
+			for (i = 0, list = gda_set_get_holders (params); i < argc; i++, list = list->next) {
 				GdaHolder *holder = GDA_HOLDER (list->data);
 				GValue *value;
 				value = create_value_from_sqlite3_gvalue (gda_holder_get_g_type (holder),
@@ -883,39 +888,39 @@ meta_changed_cb (G_GNUC_UNUSED GdaMetaStore *store, GSList *changes, HubConnecti
 		GValue *tsn, *tn;
 
 		/* we are only interested in changes occurring in the "_tables" table */
-		if (!strcmp (ch->table_name, "_tables")) {
-			switch (ch->c_type) {
+		if (!strcmp (gda_meta_store_change_get_table_name (ch), "_tables")) {
+			switch (gda_meta_store_change_get_change_type (ch)) {
 			case GDA_META_STORE_ADD: {
 				/* we only want tables where table_short_name = table_name */
-				tsn = g_hash_table_lookup (ch->keys, "+6");
-				tn = g_hash_table_lookup (ch->keys, "+2");
+				tsn = g_hash_table_lookup (gda_meta_store_change_get_keys (ch), "+6");
+				tn = g_hash_table_lookup (gda_meta_store_change_get_keys (ch), "+2");
 				if (tn && tsn && !gda_value_compare (tsn, tn))
 					table_add (hc, tn, NULL);
 				break;
 			}
 			case GDA_META_STORE_REMOVE: {
 				/* we only want tables where table_short_name = table_name */
-				tsn = g_hash_table_lookup (ch->keys, "-6");
-				tn = g_hash_table_lookup (ch->keys, "-2");
+				tsn = g_hash_table_lookup (gda_meta_store_change_get_keys (ch), "-6");
+				tn = g_hash_table_lookup (gda_meta_store_change_get_keys (ch), "-2");
 				if (tn && tsn && !gda_value_compare (tsn, tn))
 					table_remove (hc, tn);
 				break;
 			}
 			case GDA_META_STORE_MODIFY: {
 				/* we only want tables where table_short_name = table_name */
-				tsn = g_hash_table_lookup (ch->keys, "-6");
-				tn = g_hash_table_lookup (ch->keys, "-2");
+				tsn = g_hash_table_lookup (gda_meta_store_change_get_keys (ch), "-6");
+				tn = g_hash_table_lookup (gda_meta_store_change_get_keys (ch), "-2");
 				if (tn && tsn && !gda_value_compare (tsn, tn))
 					table_remove (hc, tn);
-				tsn = g_hash_table_lookup (ch->keys, "+6");
-				tn = g_hash_table_lookup (ch->keys, "+2");
+				tsn = g_hash_table_lookup (gda_meta_store_change_get_keys (ch), "+6");
+				tn = g_hash_table_lookup (gda_meta_store_change_get_keys (ch), "+2");
 				if (tn && tsn && !gda_value_compare (tsn, tn))
 					table_add (hc, tn, NULL);
 				break;
 			}
 			}
 		}
-		else if (!strcmp (ch->table_name, "_columns")) {
+		else if (!strcmp (gda_meta_store_change_get_table_name (ch), "_columns")) {
 			/* TODO */
 		}
 	}
