@@ -1,7 +1,8 @@
 /*
- * Copyright (C) 2007 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2007 - 2016 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2017 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,9 +30,10 @@
 #include <libgda/gda-connection-internal.h>
 #include <libgda/gda-data-model-iter.h>
 #include <libgda/gda-blob-op.h>
-#include "../gda-sqlite.h"
+#include <gda-sqlite.h>
 #include <sql-parser/gda-statement-struct-util.h>
 #include <libgda/gda-debug-macros.h>
+#include <libgda/gda-server-provider-impl.h>
 
 #define GDA_DEBUG_VIRTUAL
 #undef GDA_DEBUG_VIRTUAL
@@ -47,17 +49,16 @@ static GObjectClass  *parent_class = NULL;
 
 static GdaConnection *gda_vprovider_data_model_create_connection (GdaServerProvider *provider);
 static gboolean       gda_vprovider_data_model_open_connection (GdaServerProvider *provider, GdaConnection *cnc,
-								GdaQuarkList *params, GdaQuarkList *auth,
-								guint *task_id, GdaServerProviderAsyncCallback async_cb, 
-								gpointer cb_data);
+								GdaQuarkList *params, GdaQuarkList *auth);
+static gboolean       gda_vprovider_data_model_prepare_connection (GdaServerProvider *provider, GdaConnection *cnc,
+								   GdaQuarkList *params, GdaQuarkList *auth);
 static gboolean       gda_vprovider_data_model_close_connection (GdaServerProvider *provider,
 								 GdaConnection *cnc);
 static GObject        *gda_vprovider_data_model_statement_execute (GdaServerProvider *provider, GdaConnection *cnc,
 								   GdaStatement *stmt, GdaSet *params,
 								   GdaStatementModelUsage model_usage,
 								   GType *col_types, GdaSet **last_inserted_row,
-								   guint *task_id, GdaServerProviderExecCallback async_cb,
-								   gpointer cb_data, GError **error);
+								   GError **error);
 static const gchar   *gda_vprovider_data_model_get_name (GdaServerProvider *provider);
 
 static GValue **create_gvalues_array_from_sqlite3_array (int argc, sqlite3_value **argv);
@@ -66,29 +67,56 @@ static GValue **create_gvalues_array_from_sqlite3_array (int argc, sqlite3_value
 /*
  * GdaVproviderDataModel class implementation
  */
+GdaServerProviderBase data_model_base_functions = {
+	gda_vprovider_data_model_get_name,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	gda_vprovider_data_model_create_connection,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	gda_vprovider_data_model_open_connection,
+	gda_vprovider_data_model_prepare_connection,
+	gda_vprovider_data_model_close_connection,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	gda_vprovider_data_model_statement_execute,
+
+	NULL, NULL, NULL, NULL, /* padding */
+};
+
 static void
 gda_vprovider_data_model_class_init (GdaVproviderDataModelClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GdaServerProviderClass *server_class = GDA_SERVER_PROVIDER_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
+	/* set virtual functions */
+	gda_server_provider_set_impl_functions (GDA_SERVER_PROVIDER_CLASS (klass),
+						GDA_SERVER_PROVIDER_FUNCTIONS_BASE,
+						(gpointer) &data_model_base_functions);
+
 	object_class->finalize = gda_vprovider_data_model_finalize;
-	server_class->create_connection = gda_vprovider_data_model_create_connection;
-	server_class->open_connection = gda_vprovider_data_model_open_connection;
-	server_class->close_connection = gda_vprovider_data_model_close_connection;
-	server_class->statement_execute = gda_vprovider_data_model_statement_execute;
-
-	server_class->get_name = gda_vprovider_data_model_get_name;
-
-	/* explicitly unimplement the DDL queries */
-	server_class->supports_operation = NULL;
-        server_class->create_operation = NULL;
-        server_class->render_operation = NULL;
-        server_class->perform_operation = NULL;
 }
 
+
 static void
 gda_vprovider_data_model_init (GdaVproviderDataModel *prov, G_GNUC_UNUSED GdaVproviderDataModelClass *klass)
 {
@@ -249,7 +277,6 @@ virtual_filtered_data_new (VirtualTable *vtable, GdaDataModel *model,
 static void
 virtual_filtered_data_free (VirtualFilteredData *data)
 {
-	g_return_if_fail (data != NULL);
 	if (data->argv) {
 		int i;
 		for (i = 0; i < data->argc; i++)
@@ -280,7 +307,6 @@ virtual_filtered_data_free (VirtualFilteredData *data)
 static VirtualFilteredData *
 virtual_filtered_data_ref (VirtualFilteredData *data)
 {
-	g_return_val_if_fail (data != NULL, NULL);
 	data->refcount ++;
 	return data;
 }
@@ -288,7 +314,6 @@ virtual_filtered_data_ref (VirtualFilteredData *data)
 void
 _gda_vconnection_virtual_filtered_data_unref (VirtualFilteredData *data)
 {
-	g_return_if_fail (data != NULL);
 	data->refcount --;
 	if (data->refcount == 0)
 		virtual_filtered_data_free (data);
@@ -301,7 +326,6 @@ virtual_cursor_free (VirtualCursor *cursor)
 		return;
 
 	_gda_vconnection_virtual_filtered_data_unref (cursor->data);
-	cursor->data = NULL;
 	g_free (cursor);
 }
 
@@ -366,19 +390,13 @@ gda_vprovider_data_model_create_connection (GdaServerProvider *provider)
 
 static gboolean
 gda_vprovider_data_model_open_connection (GdaServerProvider *provider, GdaConnection *cnc,
-					  GdaQuarkList *params, GdaQuarkList *auth,
-					  G_GNUC_UNUSED guint *task_id, GdaServerProviderAsyncCallback async_cb, G_GNUC_UNUSED gpointer cb_data)
+					  GdaQuarkList *params, GdaQuarkList *auth)
 {
 	GdaQuarkList *m_params;
 
 	g_return_val_if_fail (GDA_IS_VPROVIDER_DATA_MODEL (provider), FALSE);
 	g_return_val_if_fail (GDA_IS_VCONNECTION_DATA_MODEL (cnc), FALSE);
 
-	if (async_cb) {
-		gda_connection_add_event_string (cnc, _("Provider does not support asynchronous connection open"));
-                return FALSE;
-	}
-
 	if (params) {
 		m_params = gda_quark_list_copy (params);
 		gda_quark_list_add_from_string (m_params, "_IS_VIRTUAL=TRUE;EXTRA_FUNCTIONS=TRUE", TRUE);
@@ -386,23 +404,21 @@ gda_vprovider_data_model_open_connection (GdaServerProvider *provider, GdaConnec
 	else
 		m_params = gda_quark_list_new_from_string ("_IS_VIRTUAL=TRUE;EXTRA_FUNCTIONS=TRUE");
 
-	if (! GDA_SERVER_PROVIDER_CLASS (parent_class)->open_connection (GDA_SERVER_PROVIDER (provider), cnc, m_params,
-									 auth, NULL, NULL, NULL)) {
-		if (auth)
-			gda_quark_list_protect_values (auth);
+	GdaServerProviderBase *parent_functions;
+	parent_functions = gda_server_provider_get_impl_functions_for_class (parent_class, GDA_SERVER_PROVIDER_FUNCTIONS_BASE);
+	
+	if (! parent_functions->open_connection (GDA_SERVER_PROVIDER (provider), cnc, m_params, auth)) {
 		gda_quark_list_free (m_params);
 		return FALSE;
 	}
-	if (auth)
-		gda_quark_list_protect_values (auth);
+
 	gda_quark_list_free (m_params);
 
 	SqliteConnectionData *scnc;
-	scnc = (SqliteConnectionData*) gda_connection_internal_get_provider_data ((GdaConnection *) cnc);
+	scnc = (SqliteConnectionData*) gda_connection_internal_get_provider_data_error ((GdaConnection *) cnc, NULL);
 	if (!scnc) {
-		gda_connection_close_no_warning (cnc);
-		gda_connection_add_event_string (cnc,
-						 _("Connection is closed"));
+		gda_connection_close (cnc, NULL);
+		gda_connection_add_event_string (cnc, _("Connection is closed"));
 		return FALSE;
 	}
 
@@ -414,6 +430,25 @@ gda_vprovider_data_model_open_connection (GdaServerProvider *provider, GdaConnec
 	return TRUE;
 }
 
+static gboolean
+gda_vprovider_data_model_prepare_connection (GdaServerProvider *provider, GdaConnection *cnc,
+					     GdaQuarkList *params, GdaQuarkList *auth)
+{
+	g_return_val_if_fail (GDA_IS_VPROVIDER_DATA_MODEL (provider), FALSE);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+
+	/* calling the parent's function first */
+	GdaServerProviderBase *parent_functions;
+        parent_functions = gda_server_provider_get_impl_functions_for_class (parent_class, GDA_SERVER_PROVIDER_FUNCTIONS_BASE);
+	if (parent_functions->prepare_connection) {
+		if (! parent_functions->prepare_connection (GDA_SERVER_PROVIDER (provider), cnc, params, auth))
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
+
 static void
 cnc_close_foreach_func (G_GNUC_UNUSED GdaDataModel *model, const gchar *table_name, GdaVconnectionDataModel *cnc)
 {
@@ -435,29 +470,25 @@ gda_vprovider_data_model_close_connection (GdaServerProvider *provider, GdaConne
 	gda_vconnection_data_model_foreach (GDA_VCONNECTION_DATA_MODEL (cnc),
 					    (GdaVconnectionDataModelFunc) cnc_close_foreach_func, cnc);
 
-	return GDA_SERVER_PROVIDER_CLASS (parent_class)->close_connection (GDA_SERVER_PROVIDER (provider), cnc);
+	GdaServerProviderBase *parent_functions;
+	parent_functions = gda_server_provider_get_impl_functions_for_class (parent_class, GDA_SERVER_PROVIDER_FUNCTIONS_BASE);
+	return parent_functions->close_connection (provider, cnc);
 }
 
 static GObject *
 gda_vprovider_data_model_statement_execute (GdaServerProvider *provider, GdaConnection *cnc,
 					    GdaStatement *stmt, GdaSet *params,
 					    GdaStatementModelUsage model_usage,
-					    GType *col_types, GdaSet **last_inserted_row,
-					    guint *task_id, GdaServerProviderExecCallback async_cb,
-					    gpointer cb_data, GError **error)
+					    GType *col_types, GdaSet **last_inserted_row,GError **error)
 {
-	GObject *retval;
-	if (async_cb) {
-                g_set_error (error, GDA_SERVER_PROVIDER_ERROR, GDA_SERVER_PROVIDER_METHOD_NON_IMPLEMENTED_ERROR,
-                             "%s", _("Provider does not support asynchronous statement execution"));
-                return NULL;
-        }
-
+	GObject *retval = NULL;
 	_gda_vconnection_set_working_obj ((GdaVconnectionDataModel*) cnc, (GObject*) stmt);
-	retval = GDA_SERVER_PROVIDER_CLASS (parent_class)->statement_execute (provider, cnc, stmt, params,
-									      model_usage, col_types,
-									      last_inserted_row, task_id,
-									      async_cb, cb_data, error);
+
+	GdaServerProviderBase *parent_functions;
+	parent_functions = gda_server_provider_get_impl_functions_for_class (parent_class, GDA_SERVER_PROVIDER_FUNCTIONS_BASE);
+	retval = parent_functions->statement_execute (provider, cnc, stmt, params,
+						      model_usage, col_types,
+						      last_inserted_row, error);
 
 	if (retval) {
 		if (! GDA_IS_DATA_MODEL (retval))
@@ -515,7 +546,7 @@ gda_vprovider_data_model_statement_execute (GdaServerProvider *provider, GdaConn
 static const gchar *
 gda_vprovider_data_model_get_name (G_GNUC_UNUSED GdaServerProvider *provider)
 {
-	return "Virtual";
+	return "Virtual data model";
 }
 
 static int
@@ -667,7 +698,7 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 			type = "date";
 		else if (gtype == GDA_TYPE_TIME)
 			type = "time";
-		else if (gtype == GDA_TYPE_TIMESTAMP)
+		else if (gtype == G_TYPE_DATE_TIME)
 			type = "timestamp";
 		else if (gtype == GDA_TYPE_SHORT)
 			type = "short";
@@ -704,7 +735,11 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 	*ppVtab = &(vtable->base);
 
 	if (SQLITE3_CALL (sqlite3_declare_vtab) (db, sql->str) != SQLITE_OK) {
-		*pzErr = SQLITE3_CALL (sqlite3_mprintf) (_("Can't declare virtual table (%s)"), sql->str);
+		sqlite3_mutex_enter (sqlite3_db_mutex (db));
+		*pzErr = SQLITE3_CALL (sqlite3_mprintf) (_("Can't declare virtual table (\"%s\"): %s"), sql->str,
+							 sqlite3_errmsg (db));
+		sqlite3_mutex_leave (sqlite3_db_mutex (db));
+
 		g_string_free (sql, TRUE);
 		g_free (vtable);
 		*ppVtab = NULL;
@@ -857,7 +892,7 @@ virtualNext (sqlite3_vtab_cursor *cur)
 			/* load data for row */
 			GSList *list;
 			gint count;
-			for (count = 0, list = ((GdaSet*) data->iter)->holders; list;
+			for (count = 0, list = gda_set_get_holders ((GdaSet*) data->iter); list;
 			     count++, list = list->next) {
 				GdaHolder *h = (GdaHolder*) list->data;
 				GError *lerror = NULL;
@@ -984,9 +1019,14 @@ virtualColumn (sqlite3_vtab_cursor *cur, sqlite3_context *ctx, int i)
 	else if (G_VALUE_TYPE (value) == G_TYPE_ERROR) {
 		GError *lerror;
 		lerror = g_value_get_boxed (value);
-		//SQLITE3_CALL (sqlite3_result_error) (ctx, lerror && lerror->message ? lerror->message : _("No detail"), -1);
+		/* Can't call:
+		 * SQLITE3_CALL (sqlite3_result_error) (ctx, lerror && lerror->message ? lerror->message : _("No detail"), -1);
+		 * because then the whole processing stops and no further rows are read */
 		if (lerror)
 			lerror = g_error_copy (lerror);
+		else
+			g_set_error (&lerror, GDA_SERVER_PROVIDER_ERROR, GDA_SERVER_PROVIDER_DATA_ERROR,
+				     _("No detail"));
 		g_hash_table_insert (error_blobs_hash, lerror, GINT_TO_POINTER (1));
 		SQLITE3_CALL (sqlite3_result_blob) (ctx, lerror, sizeof (GError), NULL);
 	}
@@ -1003,15 +1043,17 @@ virtualColumn (sqlite3_vtab_cursor *cur, sqlite3_context *ctx, int i)
 		GdaBinary *bin;
 		blob = (GdaBlob *) gda_value_get_blob (value);
 		bin = (GdaBinary *) blob;
-		if (blob->op &&
-		    (bin->binary_length != gda_blob_op_get_length (blob->op)))
-			gda_blob_op_read_all (blob->op, blob);
-		SQLITE3_CALL (sqlite3_result_blob) (ctx, blob->data.data, blob->data.binary_length, SQLITE_TRANSIENT);
+		if (gda_blob_get_op (blob) &&
+		    (gda_binary_get_size (bin) != gda_blob_op_get_length (gda_blob_get_op(blob))))
+			gda_blob_op_read_all (gda_blob_get_op (blob), blob);
+		SQLITE3_CALL (sqlite3_result_blob) (ctx, gda_binary_get_data (gda_blob_get_binary (blob)),
+		                                    gda_binary_get_size (gda_blob_get_binary (blob)),
+		                                    SQLITE_TRANSIENT);
 	}
 	else if (G_VALUE_TYPE (value) == GDA_TYPE_BINARY) {
-		const GdaBinary *bin;
+		GdaBinary *bin;
 		bin = gda_value_get_binary (value);
-		SQLITE3_CALL (sqlite3_result_blob) (ctx, bin->data, bin->binary_length, SQLITE_TRANSIENT);
+		SQLITE3_CALL (sqlite3_result_blob) (ctx, gda_binary_get_data (bin), gda_binary_get_size (bin), SQLITE_TRANSIENT);
 	}
 	else {
 		gchar *str = gda_value_stringify (value);
@@ -1052,15 +1094,12 @@ create_value_from_sqlite3_value_notype (sqlite3_value *svalue)
 	case SQLITE_BLOB: {
 		GdaBinary *bin;
 		g_value_init (value, GDA_TYPE_BINARY);
-		bin = g_new0 (GdaBinary, 1);
-		bin->binary_length = SQLITE3_CALL (sqlite3_value_bytes) (svalue);
-		if (bin->binary_length > 0) {
-			bin->data = g_new (guchar, bin->binary_length);
-			memcpy (bin->data, SQLITE3_CALL (sqlite3_value_blob) (svalue),
-				bin->binary_length);
+		bin = gda_binary_new ();
+		glong length = SQLITE3_CALL (sqlite3_value_bytes) (svalue);
+		if (length > 0) {
+			//gpointer data = g_new (guchar, length);
+			gda_binary_set_data (bin, SQLITE3_CALL (sqlite3_value_blob) (svalue), length);
 		}
-		else
-			bin->binary_length = 0;
 		gda_value_take_binary (value, bin);
 		break;
 	}
@@ -1296,6 +1335,7 @@ map_sqlite3_info_to_gda_filter (sqlite3_index_info *info, GdaVconnectionDataMode
 			if (! info->aConstraint[i].usable)
 				continue;
 
+			filter->aConstraint[j].iColumn = info->aConstraint[i].iColumn;
 			switch (info->aConstraint[i].op) {
 			case SQLITE_INDEX_CONSTRAINT_EQ:
 				filter->aConstraint[j].op = GDA_SQL_OPERATOR_TYPE_EQ;
@@ -1315,18 +1355,9 @@ map_sqlite3_info_to_gda_filter (sqlite3_index_info *info, GdaVconnectionDataMode
 			case SQLITE_INDEX_CONSTRAINT_MATCH:
 				filter->aConstraint[j].op = GDA_SQL_OPERATOR_TYPE_REGEXP;
 				break;
-#if HAVE_SQLITE
-			case SQLITE_INDEX_CONSTRAINT_LIKE:
-				filter->aConstraint[j].op = GDA_SQL_OPERATOR_TYPE_LIKE;
-				break;
-			case SQLITE_INDEX_CONSTRAINT_GLOB:
-				filter->aConstraint[j].op = GDA_SQL_OPERATOR_TYPE_GLOB;
-				break;
-#endif
 			default:
 				g_assert_not_reached ();
 			}
-			filter->aConstraint[j].iColumn = info->aConstraint[i].iColumn;
 
 			filter->aConstraintUsage[j].argvIndex = 0;
 			filter->aConstraintUsage[j].omit = FALSE;
@@ -1470,7 +1501,7 @@ update_data_select_model (sqlite3_vtab *tab, gint optype, int nData, sqlite3_val
 	/* bind parameters */
 	GSList *list;
 
-	for (list = vtable->td->modif_params [ptype]->holders; list; list = list->next) {
+	for (list = gda_set_get_holders (vtable->td->modif_params [ptype]); list; list = list->next) {
 		const gchar *id;
 		GdaHolder *holder = GDA_HOLDER (list->data);
 		gboolean holder_value_set = FALSE;
