@@ -6,7 +6,7 @@
  * Copyright (C) 2004 Julio M. Merino Vidal <jmmv@menta.net>
  * Copyright (C) 2004 Jürg Billeter <j@bitron.ch>
  * Copyright (C) 2004 Szalai Ferenc <szferi@einstein.ki.iif.hu>
- * Copyright (C) 2005 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2005 - 2014 Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,7 +28,7 @@
 #include <string.h>
 #include <glib/gi18n-lib.h>
 #include <virtual/gda-ldap-connection.h>
-#include <libgda/gda-connection-private.h>
+#include <libgda/gda-server-provider-impl.h>
 #include <libgda/gda-data-model-iter.h>
 #include <libgda/gda-util.h>
 #include <libgda/gda-data-model-array.h>
@@ -37,6 +37,8 @@
 #include "gda-ldap-provider.h"
 #include "gdaprov-data-model-ldap.h"
 #include "gda-ldap-util.h"
+#include <libgda/gda-server-provider-private.h> /* for gda_server_provider_get_real_main_context () */
+#include <libgda/gda-connection-internal.h> /* for gda_connection_increase/decrease_usage() */
 
 static void gda_ldap_provider_class_init (GdaLdapProviderClass *klass);
 static void gda_ldap_provider_init       (GdaLdapProvider *provider,
@@ -46,18 +48,15 @@ static void gda_ldap_provider_finalize   (GObject *object);
 static const gchar *gda_ldap_provider_get_name (GdaServerProvider *provider);
 static const gchar *gda_ldap_provider_get_version (GdaServerProvider *provider);
 static GdaConnection *gda_ldap_provider_create_connection (GdaServerProvider *provider);
-static gboolean gda_ldap_provider_open_connection (GdaServerProvider *provider, GdaConnection *cnc, 
-						   GdaQuarkList *params, GdaQuarkList *auth,
-						   guint *task_id, GdaServerProviderAsyncCallback async_cb, gpointer cb_data);
+static gboolean gda_ldap_provider_prepare_connection (GdaServerProvider *provider, GdaConnection *cnc, 
+						      GdaQuarkList *params, GdaQuarkList *auth);
+static gboolean gda_ldap_provider_close_connection (GdaServerProvider *provider, GdaConnection *cnc);
 static GObject *gda_ldap_provider_statement_execute (GdaServerProvider *provider, GdaConnection *cnc,
 						     GdaStatement *stmt, GdaSet *params,
 						     GdaStatementModelUsage model_usage,
-						     GType *col_types, GdaSet **last_inserted_row,
-						     guint *task_id, GdaServerProviderExecCallback async_cb,
-						     gpointer cb_data, GError **error);
+						     GType *col_types, GdaSet **last_inserted_row, GError **error);
 static const gchar *gda_ldap_provider_get_server_version (GdaServerProvider *provider,
 							  GdaConnection *cnc);
-static const gchar *gda_ldap_provider_get_database (GdaServerProvider *provider, GdaConnection *cnc);
 
 static GObjectClass *parent_class = NULL;
 
@@ -69,23 +68,61 @@ static void gda_ldap_free_cnc_data (LdapConnectionData *cdata);
 /*
  * GdaLdapProvider class implementation
  */
+GdaServerProviderBase ldap_base_functions = {
+	gda_ldap_provider_get_name,
+	gda_ldap_provider_get_version,
+	gda_ldap_provider_get_server_version,
+	NULL,
+	NULL,
+	gda_ldap_provider_create_connection,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	gda_ldap_provider_prepare_connection,
+	gda_ldap_provider_close_connection,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	gda_ldap_provider_statement_execute,
+
+	NULL, NULL, NULL, NULL, /* padding */
+};
+
 static void
 gda_ldap_provider_class_init (GdaLdapProviderClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GdaServerProviderClass *provider_class = GDA_SERVER_PROVIDER_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
-	object_class->finalize = gda_ldap_provider_finalize;
-	provider_class->create_connection = gda_ldap_provider_create_connection;
+	/* set virtual functions */
+	gda_server_provider_set_impl_functions (GDA_SERVER_PROVIDER_CLASS (klass),
+						GDA_SERVER_PROVIDER_FUNCTIONS_BASE,
+						(gpointer) &ldap_base_functions);
 
-	provider_class->get_name = gda_ldap_provider_get_name;
-	provider_class->get_version = gda_ldap_provider_get_version;
-	provider_class->open_connection = gda_ldap_provider_open_connection;
-	provider_class->get_server_version = gda_ldap_provider_get_server_version;
-	provider_class->get_database = gda_ldap_provider_get_database;
-	provider_class->statement_execute = gda_ldap_provider_statement_execute;
+	gda_server_provider_set_impl_functions (GDA_SERVER_PROVIDER_CLASS (klass),
+						GDA_SERVER_PROVIDER_FUNCTIONS_META,
+						(gpointer) NULL);
+
+	gda_server_provider_set_impl_functions (GDA_SERVER_PROVIDER_CLASS (klass),
+						GDA_SERVER_PROVIDER_FUNCTIONS_XA,
+						(gpointer) NULL);
+
+	object_class->finalize = gda_ldap_provider_finalize;
 }
 
 static void
@@ -234,6 +271,8 @@ LdapAuthMapping mappings[] = {
 };
 
 /*
+ * Function called during initialization phase => no need to use the GdaWorker object
+ *
  * Using @url and @username, performs the following tasks:
  * - bind to the LDAP server anonymously
  * - search the directory to identify the entry for the provided user name,
@@ -327,33 +366,24 @@ fetch_user_dn (const gchar *url, const gchar *base, const gchar *username, LdapA
 }
 
 /* 
- * Open connection request
+ * Prepare connection request
  *
  * In this function, the following _must_ be done:
  *   - check for the presence and validify of the parameters required to actually open a connection,
  *     using @params
  *   - open the real connection to the database using the parameters previously checked, create one or
  *     more GdaDataModel objects and declare them to the virtual connection with table names
- *   - open virtual (SQLite) connection
  *   - create a LdapConnectionData structure and associate it to @cnc
  *
  * Returns: TRUE if no error occurred, or FALSE otherwise (and an ERROR connection event must be added to @cnc)
  */
 static gboolean
-gda_ldap_provider_open_connection (GdaServerProvider *provider, GdaConnection *cnc,
-				   GdaQuarkList *params, GdaQuarkList *auth,
-				   G_GNUC_UNUSED guint *task_id, GdaServerProviderAsyncCallback async_cb,
-				   G_GNUC_UNUSED gpointer cb_data)
+gda_ldap_provider_prepare_connection (GdaServerProvider *provider, GdaConnection *cnc,
+				      GdaQuarkList *params, GdaQuarkList *auth)
 {
 	g_return_val_if_fail (GDA_IS_LDAP_PROVIDER (provider), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 
-	/* Don't allow asynchronous connection opening for virtual providers */
-	if (async_cb) {
-		gda_connection_add_event_string (cnc, _("Provider does not support asynchronous connection open"));
-                return FALSE;
-	}
-
 	/* Check for connection parameters */
 	const gchar *base_dn;
 	const gchar *host;
@@ -370,6 +400,14 @@ gda_ldap_provider_open_connection (GdaServerProvider *provider, GdaConnection *c
 	gint rport;
 	gboolean use_ssl, use_cache;
 
+	/* calling the parent's function first */
+	GdaServerProviderBase *parent_functions;
+        parent_functions = gda_server_provider_get_impl_functions_for_class (parent_class, GDA_SERVER_PROVIDER_FUNCTIONS_BASE);
+	if (parent_functions->prepare_connection) {
+		if (! parent_functions->prepare_connection (GDA_SERVER_PROVIDER (provider), cnc, params, auth))
+			return FALSE;
+	}
+
         base_dn = gda_quark_list_find (params, "DB_NAME");
         if (!base_dn) {
                 gda_connection_add_event_string (cnc, "%s",
@@ -595,60 +633,114 @@ gda_ldap_provider_open_connection (GdaServerProvider *provider, GdaConnection *c
 	g_free (fname);
 
 	/* open virtual connection */
-	g_object_set_data ((GObject*) cnc, "__gda_connection_LDAP", (gpointer) 0x01);
 	gda_virtual_connection_internal_set_provider_data (GDA_VIRTUAL_CONNECTION (cnc), 
 							   cdata, (GDestroyNotify) gda_ldap_free_cnc_data);
-	if (! GDA_SERVER_PROVIDER_CLASS (parent_class)->open_connection (GDA_SERVER_PROVIDER (provider), cnc, params,
-                                                                         NULL, NULL, NULL, NULL)) {
-		gda_virtual_connection_internal_set_provider_data (GDA_VIRTUAL_CONNECTION (cnc), NULL, NULL);
-                gda_connection_add_event_string (cnc, _("Can't open virtual connection"));
-		gda_ldap_free_cnc_data (cdata);
-                return FALSE;
-        }
-
-	gda_ldap_may_unbind (cdata);
+	gda_ldap_may_unbind (GDA_LDAP_CONNECTION (cnc));
 	return TRUE;
 }
 
+/*
+ * Close connection request
+ *
+ * In this function, the following _must_ be done:
+ *   - Actually close the connection to the database using @cnc's associated LdapConnectionData structure
+ *   - Free the LdapConnectionData structure and its contents
+ *
+ * Returns: TRUE if no error occurred, or FALSE otherwise (and an ERROR connection event must be added to @cnc)
+ */
+static gboolean
+gda_ldap_provider_close_connection (GdaServerProvider *provider, GdaConnection *cnc)
+{
+	LdapConnectionData *cdata;
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, FALSE);
+
+	/* Close the connection using the C API */
+	cdata = (LdapConnectionData*) gda_virtual_connection_internal_get_provider_data (GDA_VIRTUAL_CONNECTION (cnc));
+	if (!cdata)
+		return FALSE;
+
+	if (cdata->handle) {
+		ldap_unbind_ext (cdata->handle, NULL, NULL);
+		cdata->handle = NULL;
+	}
+
+	GdaServerProviderBase *fset;
+	fset = gda_server_provider_get_impl_functions_for_class (parent_class, GDA_SERVER_PROVIDER_FUNCTIONS_BASE);
+	return fset->close_connection (provider, cnc);
+}
+
+
+gpointer
+worker_gda_ldap_may_unbind (LdapConnectionData *cdata, GError **error)
+{
+	if (cdata->handle) {
+                ldap_unbind_ext (cdata->handle, NULL, NULL);
+		cdata->handle = NULL;
+	}
+
+	return NULL;
+}
+
 /*
  * Unbinds the connection if possible (i.e. if cdata->keep_bound_count is 0)
  * This allows to avoid keeping the connection to the LDAP server if unused
  */
 void
-gda_ldap_may_unbind (LdapConnectionData *cdata)
+gda_ldap_may_unbind (GdaLdapConnection *cnc)
 {
-	if (!cdata || (cdata->keep_bound_count > 0))
+	gda_lockable_lock ((GdaLockable*) cnc); /* CNC LOCK */
+
+	LdapConnectionData *cdata;
+	cdata = (LdapConnectionData*) gda_virtual_connection_internal_get_provider_data (GDA_VIRTUAL_CONNECTION (cnc));
+	if (!cdata || (cdata->keep_bound_count > 0)) {
+		gda_lockable_unlock ((GdaLockable*) cnc); /* CNC UNLOCK */
 		return;
-	if (cdata->handle) {
-                ldap_unbind_ext (cdata->handle, NULL, NULL);
-		cdata->handle = NULL;
 	}
+
+	GdaServerProviderConnectionData *pcdata;
+	pcdata = gda_connection_internal_get_provider_data_error ((GdaConnection*) cnc, NULL);
+
+	GdaWorker *worker;
+	worker = gda_worker_ref (gda_connection_internal_get_worker (pcdata));
+
+	GMainContext *context;
+	context = gda_server_provider_get_real_main_context ((GdaConnection *) cnc);
+
+	gpointer retval;
+	gda_worker_do_job (worker, context, 0, &retval, NULL,
+			   (GdaWorkerFunc) worker_gda_ldap_may_unbind, (gpointer) cdata, NULL, NULL, NULL);
+	if (context)
+		g_main_context_unref (context);
+
+	gda_lockable_unlock ((GdaLockable*) cnc); /* CNC UNLOCK */
+
+	gda_worker_unref (worker);
 }
 
 /*
  * Makes sure the connection is opened
  */
 gboolean
-gda_ldap_ensure_bound (LdapConnectionData *cdata, GError **error)
+gda_ldap_ensure_bound (GdaLdapConnection *cnc, GError **error)
 {
+	LdapConnectionData *cdata;
+	cdata = (LdapConnectionData*) gda_virtual_connection_internal_get_provider_data (GDA_VIRTUAL_CONNECTION (cnc));
+
 	if (!cdata)
 		return FALSE;
 	else if (cdata->handle)
 		return TRUE;
 
-	return gda_ldap_rebind (cdata, error);
+	return gda_ldap_rebind (cnc, error);
 }
 
-/*
- * Reopens a connection after the server has closed it (possibly because of a timeout)
- *
- * If it fails, then @cdata is left unchanged, otherwise it is modified to be useable again.
- */
-gboolean
-gda_ldap_rebind (LdapConnectionData *cdata, GError **error)
+
+gpointer
+worker_gda_ldap_rebind (LdapConnectionData *cdata, GError **error)
 {
 	if (!cdata)
-		return FALSE;
+		return NULL;
 
 	/*g_print ("Trying to reconnect...\n");*/
 	LDAP *ld;
@@ -657,7 +749,7 @@ gda_ldap_rebind (LdapConnectionData *cdata, GError **error)
 	if (res != LDAP_SUCCESS) {
 		g_set_error (error, GDA_CONNECTION_ERROR, GDA_CONNECTION_OPEN_ERROR,
 			     "%s", ldap_err2string (res));
-		return FALSE;
+		return NULL;
 	}
 
 	/* set protocol version to 3 by default */
@@ -672,7 +764,7 @@ gda_ldap_rebind (LdapConnectionData *cdata, GError **error)
 			g_set_error (error, GDA_CONNECTION_ERROR, GDA_CONNECTION_OPEN_ERROR,
 				     "%s", ldap_err2string (res));
 			ldap_unbind_ext (ld, NULL, NULL);
-			return FALSE;
+			return NULL;
 		}
         }
 
@@ -696,7 +788,7 @@ gda_ldap_rebind (LdapConnectionData *cdata, GError **error)
 		g_set_error (error, GDA_CONNECTION_ERROR, GDA_CONNECTION_OPEN_ERROR,
 			     "%s", ldap_err2string (res));
 		ldap_unbind_ext (ld, NULL, NULL);
-                return FALSE;
+                return NULL;
 	}
 
 	/* time limit */
@@ -706,7 +798,7 @@ gda_ldap_rebind (LdapConnectionData *cdata, GError **error)
 		g_set_error (error, GDA_CONNECTION_ERROR, GDA_CONNECTION_OPEN_ERROR,
 			     "%s", ldap_err2string (res));
 		ldap_unbind_ext (ld, NULL, NULL);
-                return FALSE;
+                return NULL;
 	}
 
 	/* size limit */
@@ -716,7 +808,7 @@ gda_ldap_rebind (LdapConnectionData *cdata, GError **error)
 		g_set_error (error, GDA_CONNECTION_ERROR, GDA_CONNECTION_OPEN_ERROR,
 			     "%s", ldap_err2string (res));
 		ldap_unbind_ext (ld, NULL, NULL);
-                return FALSE;
+                return NULL;
 	}
 
 	/* all ok */
@@ -727,7 +819,46 @@ gda_ldap_rebind (LdapConnectionData *cdata, GError **error)
 	cdata->handle = ld;
 
 	/*g_print ("Reconnected!\n");*/
-	return TRUE;
+	return (gpointer) 0x01;
+}
+
+/*
+ * Reopens a connection after the server has closed it (possibly because of a timeout)
+ */
+gboolean
+gda_ldap_rebind (GdaLdapConnection *cnc, GError **error)
+{
+	g_return_val_if_fail (GDA_IS_LDAP_CONNECTION (cnc), FALSE);
+	gda_lockable_lock ((GdaLockable*) cnc); /* CNC LOCK */
+
+	LdapConnectionData *cdata;
+	cdata = (LdapConnectionData*) gda_virtual_connection_internal_get_provider_data (GDA_VIRTUAL_CONNECTION (cnc));
+        if (!cdata) {
+		gda_lockable_unlock ((GdaLockable*) cnc); /* CNC UNLOCK */
+		g_warning ("cdata != NULL failed");
+		return FALSE;
+	}
+
+	GdaServerProviderConnectionData *pcdata;
+	pcdata = gda_connection_internal_get_provider_data_error ((GdaConnection*) cnc, NULL);
+
+	GdaWorker *worker;
+	worker = gda_worker_ref (gda_connection_internal_get_worker (pcdata));
+
+	GMainContext *context;
+	context = gda_server_provider_get_real_main_context ((GdaConnection *) cnc);
+
+	gpointer retval;
+	gda_worker_do_job (worker, context, 0, &retval, NULL,
+			   (GdaWorkerFunc) worker_gda_ldap_rebind, (gpointer) cdata, NULL, NULL, error);
+	if (context)
+		g_main_context_unref (context);
+
+	gda_lockable_unlock ((GdaLockable*) cnc); /* CNC UNLOCK */
+
+	gda_worker_unref (worker);
+
+	return retval ? TRUE : FALSE;
 }
 
 /*
@@ -751,23 +882,6 @@ gda_ldap_provider_get_server_version (GdaServerProvider *provider, GdaConnection
         return cdata->server_version;
 }
 
-/*
- * Get database request
- */
-static const gchar *
-gda_ldap_provider_get_database (GdaServerProvider *provider, GdaConnection *cnc)
-{
-	LdapConnectionData *cdata;
-
-        g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
-        g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
-
-        cdata = (LdapConnectionData*) gda_virtual_connection_internal_get_provider_data (GDA_VIRTUAL_CONNECTION (cnc));
-        if (!cdata)
-                return NULL;
-        return cdata->base_dn;
-}
-
 /*
  * Extra SQL
  */
@@ -792,15 +906,8 @@ static GObject *
 gda_ldap_provider_statement_execute (GdaServerProvider *provider, GdaConnection *cnc,
 				     GdaStatement *stmt, GdaSet *params,
 				     GdaStatementModelUsage model_usage,
-				     GType *col_types, GdaSet **last_inserted_row,
-				     guint *task_id, GdaServerProviderExecCallback async_cb,
-				     gpointer cb_data, GError **error)
+				     GType *col_types, GdaSet **last_inserted_row, GError **error)
 {
-	if (async_cb) {
-                g_set_error (error, GDA_SERVER_PROVIDER_ERROR, GDA_SERVER_PROVIDER_METHOD_NON_IMPLEMENTED_ERROR,
-			     "%s", _("Provider does not support asynchronous statement execution"));
-                return NULL;
-        }
 	gchar *sql;
 	sql = gda_statement_to_sql (stmt, params, NULL);
 	if (sql) {
@@ -822,7 +929,7 @@ gda_ldap_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 									       cmde->table_name, cmde->base_dn,
 									       cmde->filter, cmde->attributes,
 									       cmde->scope, &lerror))
-					retval = (GObject*) gda_set_new (NULL);
+						retval = (GObject*) gda_set_new (NULL);
 					else {
 						event = gda_connection_point_available_event (cnc,
 											      GDA_CONNECTION_EVENT_ERROR);
@@ -964,10 +1071,16 @@ gda_ldap_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 		}
 		g_free (sql);
 	}
-	return GDA_SERVER_PROVIDER_CLASS (parent_class)->statement_execute (provider, cnc, stmt, params,
-									    model_usage, col_types,
-									    last_inserted_row, task_id, 
-									    async_cb, cb_data, error);
+
+	/* check connection to LDAP is Ok */
+	if (! gda_ldap_ensure_bound (GDA_LDAP_CONNECTION (cnc), error))
+		return NULL;
+
+	GdaServerProviderBase *fset;
+	fset = gda_server_provider_get_impl_functions_for_class (parent_class, GDA_SERVER_PROVIDER_FUNCTIONS_BASE);
+	return fset->statement_execute (provider, cnc, stmt, params,
+					model_usage, col_types,
+					last_inserted_row, error);
 }
 
 /*
