@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006 - 2009 Bas Driessen <bas.driessen@xobas.com>
- * Copyright (C) 2006 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2006 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2008 - 2011 Murray Cumming <murrayc@murrayc.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -40,19 +40,29 @@ gda_mysql_render_CREATE_DB (GdaServerProvider *provider, GdaConnection *cnc,
 	if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
 		g_string_append (string, "IF NOT EXISTS ");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/DB_DEF_P/DB_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/DB_DEF_P/DB_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	g_string_append (string, tmp);
 	g_free (tmp);
 
-	value = gda_server_operation_get_value_at (op, "/DB_DEF_P/DB_CSET");
+	value = gda_server_operation_get_value_at_path (op, "/DB_DEF_P/DB_CSET");
 	if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && g_value_get_string (value)) {
 		g_string_append (string, " CHARACTER SET ");
 		g_string_append (string, g_value_get_string (value));
 		first = FALSE;
 	}
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/DB_DEF_P/DB_COLLATION");
-	if (tmp) {
+	if (gda_server_operation_get_value_at_path (op, "/DB_DEF_P/DB_COLLATION")) {
+		tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/DB_DEF_P/DB_COLLATION", error);
+		if (!tmp) {
+			g_string_free (string, TRUE);
+			return NULL;
+		}
+
 		if (first)
 			first = FALSE;
 		else
@@ -83,7 +93,11 @@ gda_mysql_render_DROP_DB (GdaServerProvider *provider, GdaConnection *cnc,
 	if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
 		g_string_append (string, "IF EXISTS ");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/DB_DESC_P/DB_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/DB_DESC_P/DB_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
 	g_string_append (string, tmp);
 	g_free (tmp);
 
@@ -100,7 +114,7 @@ gda_mysql_render_CREATE_TABLE (GdaServerProvider *provider, GdaConnection *cnc,
 {
 	GString *string;
 	const GValue *value;
-	gboolean allok = TRUE;
+	const GValue *value1;
 	gboolean hasfields = FALSE;
 	gint nrows;
 	gint i;
@@ -120,110 +134,132 @@ gda_mysql_render_CREATE_TABLE (GdaServerProvider *provider, GdaConnection *cnc,
 	if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
 		g_string_append (string, "IF NOT EXISTS ");
 		
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DEF_P/TABLE_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DEF_P/TABLE_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	g_string_append (string, tmp);
 	g_free (tmp);
 	g_string_append (string, " (");
 		
 	/* FIELDS */
-	if (allok) {
-		GdaServerOperationNode *node;
+	GdaServerOperationNode *node;
 
-		node = gda_server_operation_get_node_info (op, "/FIELDS_A");
-		g_assert (node);
+	node = gda_server_operation_get_node_info (op, "/FIELDS_A");
+	g_assert (node);
 
-		/* finding if there is a composed primary key */
-		nrows = gda_data_model_get_n_rows (node->model);
-		for (i = 0; i < nrows; i++) {
-			value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_PKEY/%d", i);
-			if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value)) {
-				tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, 
-										  "/FIELDS_A/@COLUMN_NAME/%d", i);
-				pkfields = g_slist_append (pkfields, tmp);
-				nbpkfields ++;
+	/* finding if there is a composed primary key */
+	nrows = gda_data_model_get_n_rows (node->model);
+	for (i = 0; i < nrows; i++) {
+		value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_PKEY/%d", i);
+		if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value)) {
+			tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, 
+									  "/FIELDS_A/@COLUMN_NAME/%d",
+									  error, i);
+			if (!tmp) {
+				g_string_free (string, TRUE);
+				return NULL;
+			}
+
+			pkfields = g_slist_append (pkfields, tmp);
+			nbpkfields ++;
+		}
+	}
+
+	/* manually defined fields */
+	first = TRUE;
+	for (i = 0; i < nrows; i++) {
+		hasfields = TRUE;
+		if (first) 
+			first = FALSE;
+		else
+			g_string_append (string, ", ");
+
+		tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider,
+								  "/FIELDS_A/@COLUMN_NAME/%d", error, i);
+		if (!tmp) {
+			g_string_free (string, TRUE);
+			return NULL;
+		}
+
+		g_string_append (string, tmp);
+		g_free (tmp);
+		g_string_append_c (string, ' ');
+
+		value1 = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_TYPE/%d", i);
+		g_string_append (string, g_value_get_string (value1));
+				
+		value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_SIZE/%d", i);
+		if (value && G_VALUE_HOLDS (value, G_TYPE_UINT)) {
+			g_string_append_printf (string, "(%d", g_value_get_uint (value));
+
+			value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_SCALE/%d", i);
+			if (value && G_VALUE_HOLDS (value, G_TYPE_UINT))
+				g_string_append_printf (string, ",%d)", g_value_get_uint (value));
+			else
+				g_string_append (string, ")");
+		}
+
+		value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_DEFAULT/%d", i);
+		if (value && G_VALUE_HOLDS (value, G_TYPE_STRING)) {
+			const gchar *str = g_value_get_string (value);
+			if (str && *str) {
+				g_string_append (string, " DEFAULT ");
+        const gchar* valtmp = g_value_get_string (value1);
+        if (!g_ascii_strcasecmp (valtmp,"string") ||
+            !g_ascii_strcasecmp (valtmp,"gchararray")) {
+          g_string_append_c (string,'\'');
+          g_string_append (string, str);
+          g_string_append_c (string,'\'');
+        }
+        else
+          g_string_append (string, str);
 			}
 		}
 
-		/* manually defined fields */
-		first = TRUE;
-		for (i = 0; i < nrows; i++) {
-			hasfields = TRUE;
-			if (first) 
-				first = FALSE;
-			else
-				g_string_append (string, ", ");
-				
-			tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider,
-									  "/FIELDS_A/@COLUMN_NAME/%d", i);
-			g_string_append (string, tmp);
-			g_free (tmp);
-			g_string_append_c (string, ' ');
-				
-			value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_TYPE/%d", i);
-			g_string_append (string, g_value_get_string (value));
-				
-			value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_SIZE/%d", i);
-			if (value && G_VALUE_HOLDS (value, G_TYPE_UINT)) {
-				g_string_append_printf (string, "(%d", g_value_get_uint (value));
-
-				value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_SCALE/%d", i);
-				if (value && G_VALUE_HOLDS (value, G_TYPE_UINT))
-					g_string_append_printf (string, ",%d)", g_value_get_uint (value));
-				else
-					g_string_append (string, ")");
-			}
-
-			value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_DEFAULT/%d", i);
-			if (value && G_VALUE_HOLDS (value, G_TYPE_STRING)) {
-				const gchar *str = g_value_get_string (value);
-				if (str && *str) {
-					g_string_append (string, " DEFAULT ");
-					g_string_append (string, str);
-				}
-			}
-				
-			value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_NNUL/%d", i);
-			if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
-				g_string_append (string, " NOT NULL");
+		value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_NNUL/%d", i);
+		if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
+			g_string_append (string, " NOT NULL");
 			
-			value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_AUTOINC/%d", i);
-			if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
-				g_string_append (string, " AUTO_INCREMENT");
+		value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_AUTOINC/%d", i);
+		if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
+			g_string_append (string, " AUTO_INCREMENT");
 
-			value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_UNIQUE/%d", i);
+		value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_UNIQUE/%d", i);
+		if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
+			g_string_append (string, " UNIQUE");
+				
+		if (nbpkfields == 1) {
+			value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_PKEY/%d", i);
 			if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
-				g_string_append (string, " UNIQUE");
-				
-			if (nbpkfields == 1) {
-				value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_PKEY/%d", i);
-				if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
-					g_string_append (string, " PRIMARY KEY");
-			}
+				g_string_append (string, " PRIMARY KEY");
+		}
 
-			value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_COMMENT/%d", i);
-			if (value && G_VALUE_HOLDS (value, G_TYPE_STRING)) {
-				GdaDataHandler *dh;
-				gchar *str;
-				
-				dh = gda_server_provider_get_data_handler_g_type (provider, cnc, G_TYPE_STRING);
-				str = gda_data_handler_get_sql_from_value (dh, value);
-				if (str) {
-					if (*str) {
-						g_string_append (string, " COMMENT ");
-						g_string_append (string, str);
-					}
-					g_free (str);
-				}
-			}
-				
-			value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_CHECK/%d", i);
-			if (value && G_VALUE_HOLDS (value, G_TYPE_STRING)) {
-				const gchar *str = g_value_get_string (value);
-				if (str && *str) {
-					g_string_append (string, " CHECK (");
+		value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_COMMENT/%d", i);
+		if (value && G_VALUE_HOLDS (value, G_TYPE_STRING)) {
+			GdaDataHandler *dh;
+			gchar *str;
+
+			dh = gda_server_provider_get_data_handler_g_type (provider, cnc, G_TYPE_STRING);
+			str = gda_data_handler_get_sql_from_value (dh, value);
+			if (str) {
+				if (*str) {
+					g_string_append (string, " COMMENT ");
 					g_string_append (string, str);
-					g_string_append_c (string, ')');
 				}
+				g_free (str);
+			}
+		}
+
+		value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_CHECK/%d", i);
+		if (value && G_VALUE_HOLDS (value, G_TYPE_STRING)) {
+			const gchar *str = g_value_get_string (value);
+			if (str && *str) {
+				g_string_append (string, " CHECK (");
+				g_string_append (string, str);
+				g_string_append_c (string, ')');
 			}
 		}
 	}
@@ -244,232 +280,225 @@ gda_mysql_render_CREATE_TABLE (GdaServerProvider *provider, GdaConnection *cnc,
 	g_slist_free (pkfields);
 
 	/* foreign keys */
-	if (allok) {
-		GdaServerOperationNode *node;
+	first = TRUE;
+	node = gda_server_operation_get_node_info (op, "/FKEY_S");
+	if (node) {
+		nrows = gda_server_operation_get_sequence_size (op, "/FKEY_S");
+		for (i = 0; i < nrows; i++) {
+			gint nbfields = 0, j;
 
-		first = TRUE;
-		node = gda_server_operation_get_node_info (op, "/FKEY_S");
-		if (node) {
-			nrows = gda_server_operation_get_sequence_size (op, "/FKEY_S");
-			for (i = 0; i < nrows; i++) {
-				gint nbfields = 0, j;
-
-				g_string_append (string, ", FOREIGN KEY (");
-				node = gda_server_operation_get_node_info (op, "/FKEY_S/%d/FKEY_FIELDS_A", i);
-				if (!node || ((nbfields = gda_data_model_get_n_rows (node->model)) == 0)) {
-					allok = FALSE;
-					g_set_error (error, GDA_SERVER_OPERATION_ERROR,
-						     GDA_SERVER_OPERATION_INCORRECT_VALUE_ERROR,
-						     "%s", _("No field specified in foreign key constraint"));
-				}
-				else {
-					for (j = 0; j < nbfields; j++) {
-						if (j != 0)
-							g_string_append (string, ", ");
-						tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider,
-												  "/FKEY_S/%d/FKEY_FIELDS_A/@FK_FIELD/%d", i, j);
-						if (tmp) {
-							g_string_append (string, tmp);
-							g_free (tmp);
-						}
-						else {
-							allok = FALSE;
-							g_set_error (error, GDA_SERVER_OPERATION_ERROR,
-								     GDA_SERVER_OPERATION_INCORRECT_VALUE_ERROR,
-								     "%s",  
-								     _("Empty field specified in foreign key constraint"));
-						}
-					}
-				}
-				g_string_append (string, ") REFERENCES ");
-				tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider,
-										  "/FKEY_S/%d/FKEY_REF_TABLE", i);
-				if (tmp) {
-					g_string_append (string, tmp);
-					g_free (tmp);
-				}
-				else {
-					allok = FALSE;
-					g_set_error (error, GDA_SERVER_OPERATION_ERROR,
-						     GDA_SERVER_OPERATION_INCORRECT_VALUE_ERROR,
-						     "%s", _("No referenced table specified in foreign key constraint"));
-				}
-
-				g_string_append (string, " (");
+			g_string_append (string, ", FOREIGN KEY (");
+			node = gda_server_operation_get_node_info (op, "/FKEY_S/%d/FKEY_FIELDS_A", i);
+			if (!node || ((nbfields = gda_data_model_get_n_rows (node->model)) == 0)) {
+				g_string_free (string, TRUE);
+				g_set_error (error, GDA_SERVER_OPERATION_ERROR,
+					     GDA_SERVER_OPERATION_INCORRECT_VALUE_ERROR,
+					     "%s", _("No field specified in foreign key constraint"));
+				return NULL;
+			}
+			else {
 				for (j = 0; j < nbfields; j++) {
 					if (j != 0)
 						g_string_append (string, ", ");
 					tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider,
-											  "/FKEY_S/%d/FKEY_FIELDS_A/@FK_REF_PK_FIELD/%d", i, j);
+											  "/FKEY_S/%d/FKEY_FIELDS_A/@FK_FIELD/%d",
+											  error, i, j);
 					if (tmp) {
 						g_string_append (string, tmp);
 						g_free (tmp);
 					}
 					else {
-						allok = FALSE;
-						g_set_error (error, GDA_SERVER_OPERATION_ERROR,
-							     GDA_SERVER_OPERATION_INCORRECT_VALUE_ERROR,
-							     "%s",  
-							     _("Empty referenced field specified in foreign key constraint"));
+						g_string_free (string, TRUE);
+						return NULL;
 					}
 				}
-				g_string_append_c (string, ')');
-				value = gda_server_operation_get_value_at (op, "/FKEY_S/%d/FKEY_MATCH_TYPE", i);
-				if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && g_value_get_string (value))
-					g_string_append_printf (string, " %s", g_value_get_string (value));
-				value = gda_server_operation_get_value_at (op, "/FKEY_S/%d/FKEY_ONUPDATE", i);
-				if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && g_value_get_string (value))
-					g_string_append_printf (string, " ON UPDATE %s", g_value_get_string (value));
-				value = gda_server_operation_get_value_at (op, "/FKEY_S/%d/FKEY_ONDELETE", i);
-				if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && g_value_get_string (value))
-					g_string_append_printf (string, " ON DELETE %s", g_value_get_string (value));
 			}
+			g_string_append (string, ") REFERENCES ");
+			tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider,
+									  "/FKEY_S/%d/FKEY_REF_TABLE", error, i);
+			if (tmp) {
+				g_string_append (string, tmp);
+				g_free (tmp);
+			}
+			else {
+				g_string_free (string, TRUE);
+				return NULL;
+			}
+
+			g_string_append (string, " (");
+			for (j = 0; j < nbfields; j++) {
+				if (j != 0)
+					g_string_append (string, ", ");
+				tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider,
+										  "/FKEY_S/%d/FKEY_FIELDS_A/@FK_REF_PK_FIELD/%d",
+										  error, i, j);
+				if (tmp) {
+					g_string_append (string, tmp);
+					g_free (tmp);
+				}
+				else {
+					g_string_free (string, TRUE);
+					return NULL;
+				}
+			}
+			g_string_append_c (string, ')');
+			value = gda_server_operation_get_value_at (op, "/FKEY_S/%d/FKEY_MATCH_TYPE", i);
+			if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && g_value_get_string (value))
+				g_string_append_printf (string, " %s", g_value_get_string (value));
+			value = gda_server_operation_get_value_at (op, "/FKEY_S/%d/FKEY_ONUPDATE", i);
+			if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && g_value_get_string (value))
+				g_string_append_printf (string, " ON UPDATE %s", g_value_get_string (value));
+			value = gda_server_operation_get_value_at (op, "/FKEY_S/%d/FKEY_ONDELETE", i);
+			if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && g_value_get_string (value))
+				g_string_append_printf (string, " ON DELETE %s", g_value_get_string (value));
 		}
 	}
 
 	g_string_append (string, ")");
 
 	if (!hasfields) {
-		allok = FALSE;
 		g_set_error (error, GDA_SERVER_OPERATION_ERROR,
                              GDA_SERVER_OPERATION_INCORRECT_VALUE_ERROR,
 			     "%s", _("Table to create must have at least one row"));
+		g_string_free (string, TRUE);
+		return NULL;
 	}
 
 	/* other options */
-	if (allok) {
-		value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_ENGINE");
-		if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
-		    g_value_get_string (value) && *g_value_get_string (value))
-			g_string_append (string, " ENGINE = ");
+	value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_ENGINE");
+	if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
+	    g_value_get_string (value) && *g_value_get_string (value))
+		g_string_append (string, " ENGINE = ");
+	g_string_append (string, g_value_get_string (value));
+		
+	value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_AUTOINC_VALUE");
+	if (value && G_VALUE_HOLDS (value, G_TYPE_INT)) 
+		g_string_append_printf (string, " AUTO_INCREMENT = %d", g_value_get_int (value));
+
+	value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_AVG_ROW_LENTGH");
+	if (value && G_VALUE_HOLDS (value, G_TYPE_INT)) 
+		g_string_append_printf (string, " AVG_ROW_LENGTH = %d", g_value_get_int (value));
+
+	value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_CSET");
+	if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
+	    g_value_get_string (value) && *g_value_get_string (value)) {
+		g_string_append (string, " CHARACTER SET ");
 		g_string_append (string, g_value_get_string (value));
-		
-		value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_AUTOINC_VALUE");
-		if (value && G_VALUE_HOLDS (value, G_TYPE_INT)) 
-			g_string_append_printf (string, " AUTO_INCREMENT = %d", g_value_get_int (value));
-
-		value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_AVG_ROW_LENTGH");
-		if (value && G_VALUE_HOLDS (value, G_TYPE_INT)) 
-			g_string_append_printf (string, " AVG_ROW_LENGTH = %d", g_value_get_int (value));
-
-		
-		value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_CSET");
-		if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
-		    g_value_get_string (value) && *g_value_get_string (value)) {
-			g_string_append (string, " CHARACTER SET ");
-			g_string_append (string, g_value_get_string (value));
 
+		if (gda_server_operation_get_value_at_path (op, "/TABLE_OPTIONS_P/TABLE_COLLATION")) {
 			tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider,
-									  "/TABLE_OPTIONS_P/TABLE_COLLATION");
-			if (tmp) {
-				g_string_append (string, " COLLATE ");
-				g_string_append (string, tmp);
-				g_free (tmp);
+									  "/TABLE_OPTIONS_P/TABLE_COLLATION", error);
+			if (!tmp) {
+				g_string_free (string, TRUE);
+				return NULL;
 			}
+			g_string_append (string, " COLLATE ");
+			g_string_append (string, tmp);
+			g_free (tmp);
 		}
-		
-		value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_CHECKSUM");
-		if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value)) 
-			g_string_append (string, " CHECKSUM = 1");
-		
-		value = gda_server_operation_get_value_at (op, "/TABLE_DEF_P/TABLE_COMMENT");
-		if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
-		    g_value_get_string (value) && *g_value_get_string (value)) {
-			GdaDataHandler *dh;
-			gchar *str;
-			
-			dh = gda_server_provider_get_data_handler_g_type (provider, cnc, G_TYPE_STRING);
-			str = gda_data_handler_get_sql_from_value (dh, value);
-			if (str) {
-				g_string_append (string, " COMMENT = ");
-				g_string_append (string, str);
-				g_free (str);
-			}
+	}
+
+	value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_CHECKSUM");
+	if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value)) 
+		g_string_append (string, " CHECKSUM = 1");
+
+	value = gda_server_operation_get_value_at (op, "/TABLE_DEF_P/TABLE_COMMENT");
+	if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
+	    g_value_get_string (value) && *g_value_get_string (value)) {
+		GdaDataHandler *dh;
+		gchar *str;
+
+		dh = gda_server_provider_get_data_handler_g_type (provider, cnc, G_TYPE_STRING);
+		str = gda_data_handler_get_sql_from_value (dh, value);
+		if (str) {
+			g_string_append (string, " COMMENT = ");
+			g_string_append (string, str);
+			g_free (str);
 		}
-		
-		value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_MAX_ROWS");
-		if (value && G_VALUE_HOLDS (value, G_TYPE_INT)) 
-			g_string_append_printf (string, " MAX_ROWS = %d", g_value_get_int (value));
-		
-		value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_MIN_ROWS");
-		if (value && G_VALUE_HOLDS (value, G_TYPE_INT)) 
-			g_string_append_printf (string, " MIN_ROWS = %d", g_value_get_int (value));
-		
-		value = gda_server_operation_get_value_at (op, "/TABLE_DEF_P/TABLE_PACK_KEYS");
-		if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
-		    g_value_get_string (value) && *g_value_get_string (value)) {
-			g_string_append (string, " PACK_KEYS = ");
-			g_string_append (string, g_value_get_string (value));
+	}
+
+	value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_MAX_ROWS");
+	if (value && G_VALUE_HOLDS (value, G_TYPE_INT)) 
+		g_string_append_printf (string, " MAX_ROWS = %d", g_value_get_int (value));
+
+	value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_MIN_ROWS");
+	if (value && G_VALUE_HOLDS (value, G_TYPE_INT)) 
+		g_string_append_printf (string, " MIN_ROWS = %d", g_value_get_int (value));
+
+	value = gda_server_operation_get_value_at (op, "/TABLE_DEF_P/TABLE_PACK_KEYS");
+	if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
+	    g_value_get_string (value) && *g_value_get_string (value)) {
+		g_string_append (string, " PACK_KEYS = ");
+		g_string_append (string, g_value_get_string (value));
+	}
+
+	value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_PASSWORD");
+	if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
+	    g_value_get_string (value) && *g_value_get_string (value)) {
+		GdaDataHandler *dh;
+		gchar *str;
+
+		dh = gda_server_provider_get_data_handler_g_type (provider, cnc, G_TYPE_STRING);
+		str = gda_data_handler_get_sql_from_value (dh, value);
+		if (str) {
+			g_string_append (string, " PASSWORD = ");
+			g_string_append (string, str);
+			g_free (str);
 		}
+	}
 
-		value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_PASSWORD");
-		if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
-		    g_value_get_string (value) && *g_value_get_string (value)) {
-			GdaDataHandler *dh;
-			gchar *str;
+	value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_DELAY_KEY_WRITE");
+	if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value)) 
+		g_string_append (string, " DELAY_KEY_WRITE = 1");
 
-			dh = gda_server_provider_get_data_handler_g_type (provider, cnc, G_TYPE_STRING);
-			str = gda_data_handler_get_sql_from_value (dh, value);
-			if (str) {
-				g_string_append (string, " PASSWORD = ");
-				g_string_append (string, str);
-				g_free (str);
-			}
+	value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_ROW_FORMAT");
+	if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
+	    g_value_get_string (value) && *g_value_get_string (value)) {
+		g_string_append (string, " ROW_FORMAT = ");
+		g_string_append (string, g_value_get_string (value));
+	}
+
+	value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_UNION");
+	if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
+	    g_value_get_string (value) && *g_value_get_string (value)) {
+		g_string_append (string, " UNION = ");
+		g_string_append (string, g_value_get_string (value));
+	}
+
+	value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_INSERT_METHOD");
+	if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
+	    g_value_get_string (value) && *g_value_get_string (value)) {
+		g_string_append (string, " INSERT_METHOD = ");
+		g_string_append (string, g_value_get_string (value));
+	}
+
+	value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_DATA_DIR");
+	if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
+	    g_value_get_string (value) && *g_value_get_string (value)) {
+		GdaDataHandler *dh;
+		gchar *str;
+
+		dh = gda_server_provider_get_data_handler_g_type (provider, cnc, G_TYPE_STRING);
+		str = gda_data_handler_get_sql_from_value (dh, value);
+		if (str) {
+			g_string_append (string, " DATA_DIRECTORY = ");
+			g_string_append (string, str);
+			g_free (str);
 		}
+	}
 
-		value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_DELAY_KEY_WRITE");
-		if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value)) 
-			g_string_append (string, " DELAY_KEY_WRITE = 1");
+	value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_INDEX_DIR");
+	if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
+	    g_value_get_string (value) && *g_value_get_string (value)) {
+		GdaDataHandler *dh;
+		gchar *str;
 
-		value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_ROW_FORMAT");
-		if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
-		    g_value_get_string (value) && *g_value_get_string (value)) {
-			g_string_append (string, " ROW_FORMAT = ");
-			g_string_append (string, g_value_get_string (value));
-		}
-
-		value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_UNION");
-		if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
-		    g_value_get_string (value) && *g_value_get_string (value)) {
-			g_string_append (string, " UNION = ");
-			g_string_append (string, g_value_get_string (value));
-		}
-
-		value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_INSERT_METHOD");
-		if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
-		    g_value_get_string (value) && *g_value_get_string (value)) {
-			g_string_append (string, " INSERT_METHOD = ");
-			g_string_append (string, g_value_get_string (value));
-		}
-
-		value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_DATA_DIR");
-		if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
-		    g_value_get_string (value) && *g_value_get_string (value)) {
-			GdaDataHandler *dh;
-			gchar *str;
-
-			dh = gda_server_provider_get_data_handler_g_type (provider, cnc, G_TYPE_STRING);
-			str = gda_data_handler_get_sql_from_value (dh, value);
-			if (str) {
-				g_string_append (string, " DATA_DIRECTORY = ");
-				g_string_append (string, str);
-				g_free (str);
-			}
-		}
-
-		value = gda_server_operation_get_value_at (op, "/TABLE_OPTIONS_P/TABLE_INDEX_DIR");
-		if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && 
-		    g_value_get_string (value) && *g_value_get_string (value)) {
-			GdaDataHandler *dh;
-			gchar *str;
-
-			dh = gda_server_provider_get_data_handler_g_type (provider, cnc, G_TYPE_STRING);
-			str = gda_data_handler_get_sql_from_value (dh, value);
-			if (str) {
-				g_string_append (string, " INDEX_DIRECTORY = ");
-				g_string_append (string, str);
-				g_free (str);
-			}
+		dh = gda_server_provider_get_data_handler_g_type (provider, cnc, G_TYPE_STRING);
+		str = gda_data_handler_get_sql_from_value (dh, value);
+		if (str) {
+			g_string_append (string, " INDEX_DIRECTORY = ");
+			g_string_append (string, str);
+			g_free (str);
 		}
 	}
 
@@ -500,7 +529,12 @@ gda_mysql_render_DROP_TABLE (GdaServerProvider *provider, GdaConnection *cnc,
 	if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
 		g_string_append (string, " IF EXISTS");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DESC_P/TABLE_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DESC_P/TABLE_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	g_string_append_c (string, ' ');
 	g_string_append (string, tmp);
 	g_free (tmp);
@@ -527,11 +561,21 @@ gda_mysql_render_RENAME_TABLE (GdaServerProvider *provider, GdaConnection *cnc,
 
 	string = g_string_new ("ALTER TABLE ");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DESC_P/TABLE_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DESC_P/TABLE_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	g_string_append (string, tmp);
 	g_free (tmp);
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DESC_P/TABLE_NEW_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DESC_P/TABLE_NEW_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	g_string_append (string, " RENAME TO ");
 	g_string_append (string, tmp);
 	g_free (tmp);
@@ -554,7 +598,12 @@ gda_mysql_render_COMMENT_TABLE (GdaServerProvider *provider, GdaConnection *cnc,
 
 	string = g_string_new ("ALTER TABLE ");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DESC_P/TABLE_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DESC_P/TABLE_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	g_string_append (string, tmp);
 	g_free (tmp);
 
@@ -582,13 +631,23 @@ gda_mysql_render_ADD_COLUMN (GdaServerProvider *provider, GdaConnection *cnc,
 
 	string = g_string_new ("ALTER TABLE ");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/COLUMN_DEF_P/TABLE_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/COLUMN_DEF_P/TABLE_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	g_string_append (string, tmp);
 	g_free (tmp);
 
 	g_string_append (string, " ADD COLUMN ");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/COLUMN_DEF_P/COLUMN_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/COLUMN_DEF_P/COLUMN_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	g_string_append (string, tmp);
 	g_free (tmp);
 
@@ -689,11 +748,21 @@ gda_mysql_render_DROP_COLUMN (GdaServerProvider *provider, GdaConnection *cnc,
 
 	string = g_string_new ("ALTER TABLE ");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/COLUMN_DESC_P/TABLE_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/COLUMN_DESC_P/TABLE_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	g_string_append (string, tmp);
 	g_free (tmp);
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/COLUMN_DESC_P/COLUMN_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/COLUMN_DESC_P/COLUMN_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	g_string_append (string, " DROP COLUMN ");
 	g_string_append (string, tmp);
 	g_free (tmp);
@@ -718,12 +787,22 @@ gda_mysql_render_COMMENT_COLUMN (GdaServerProvider *provider, GdaConnection *cnc
 
 	string = g_string_new ("ALTER TABLE ");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/COLUMN_DESC_P/TABLE_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/COLUMN_DESC_P/TABLE_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	table_name = tmp;
 	g_string_append (string, tmp);
 	g_free (tmp);
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/COLUMN_DESC_P/COLUMN_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/COLUMN_DESC_P/COLUMN_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	column_name = tmp;
 	g_string_append (string, " CHANGE COLUMN ");
 	g_string_append (string, tmp);
@@ -805,7 +884,12 @@ gda_mysql_render_CREATE_INDEX (GdaServerProvider *provider, GdaConnection *cnc,
 
 	g_string_append (string, "INDEX ");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/INDEX_DEF_P/INDEX_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/INDEX_DEF_P/INDEX_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	g_string_append (string, tmp);
 	g_free (tmp);
 
@@ -817,7 +901,12 @@ gda_mysql_render_CREATE_INDEX (GdaServerProvider *provider, GdaConnection *cnc,
 
 	g_string_append (string, " ON ");
 	
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/INDEX_DEF_P/INDEX_ON_TABLE");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/INDEX_DEF_P/INDEX_ON_TABLE", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	g_string_append (string, tmp);
 	g_free (tmp);
 
@@ -828,23 +917,25 @@ gda_mysql_render_CREATE_INDEX (GdaServerProvider *provider, GdaConnection *cnc,
 	nrows = gda_server_operation_get_sequence_size (op, "/INDEX_FIELDS_S");
 	for (i = 0; i < nrows; i++) {
 		tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider,
-								  "/INDEX_FIELDS_S/%d/INDEX_FIELD", i);
-                if (tmp) {
+								  "/INDEX_FIELDS_S/%d/INDEX_FIELD", error, i);
+		if (!tmp) {
+			g_string_free (string, TRUE);
+			return NULL;
+		}
 
-			if (i != 0)
-				g_string_append (string, ", ");
-			g_string_append (string, tmp);
-			g_free (tmp);
+		if (i != 0)
+			g_string_append (string, ", ");
+		g_string_append (string, tmp);
+		g_free (tmp);
 			
-			value = gda_server_operation_get_value_at (op, "/INDEX_FIELDS_S/%d/INDEX_LENGTH", i);
-			if (value && G_VALUE_HOLDS (value, G_TYPE_INT) && (g_value_get_int (value) > 0))
-				g_string_append_printf (string, " (%d)", g_value_get_int (value));
+		value = gda_server_operation_get_value_at (op, "/INDEX_FIELDS_S/%d/INDEX_LENGTH", i);
+		if (value && G_VALUE_HOLDS (value, G_TYPE_INT) && (g_value_get_int (value) > 0))
+			g_string_append_printf (string, " (%d)", g_value_get_int (value));
 
-			value = gda_server_operation_get_value_at (op, "/INDEX_FIELDS_S/%d/INDEX_SORT_ORDER", i);
-			if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && g_value_get_string (value)) {
-				g_string_append_c (string, ' ');
-				g_string_append (string, g_value_get_string (value));
-			}
+		value = gda_server_operation_get_value_at (op, "/INDEX_FIELDS_S/%d/INDEX_SORT_ORDER", i);
+		if (value && G_VALUE_HOLDS (value, G_TYPE_STRING) && g_value_get_string (value)) {
+			g_string_append_c (string, ' ');
+			g_string_append (string, g_value_get_string (value));
 		}
 	}
 
@@ -866,11 +957,21 @@ gda_mysql_render_DROP_INDEX (GdaServerProvider *provider, GdaConnection *cnc,
 
 	string = g_string_new ("DROP INDEX ");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/INDEX_DESC_P/INDEX_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/INDEX_DESC_P/INDEX_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	g_string_append (string, tmp);
 	g_free (tmp);
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/INDEX_DESC_P/INDEX_ON_TABLE");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/INDEX_DESC_P/INDEX_ON_TABLE", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	g_string_append (string, " ON ");
 	g_string_append (string, tmp);
 	g_free (tmp);
@@ -900,7 +1001,12 @@ gda_mysql_render_CREATE_VIEW (GdaServerProvider *provider, GdaConnection *cnc,
 
 	g_string_append (string, "VIEW ");
 	
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/VIEW_DEF_P/VIEW_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/VIEW_DEF_P/VIEW_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	g_string_append (string, tmp);
 	g_free (tmp);
 
@@ -915,21 +1021,18 @@ gda_mysql_render_CREATE_VIEW (GdaServerProvider *provider, GdaConnection *cnc,
 				g_string_append (string, " (");
 
 			tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider,
-									  "/FIELDS_A/@COLUMN_NAME/%d", i);
-			if (tmp) {
-				if (i != 0) 
-					g_string_append (string, ", ");
+									  "/FIELDS_A/@COLUMN_NAME/%d", error, i);
+			if (!tmp) {
+				g_string_free (string, TRUE);
+				return NULL;
+			}
+
+			if (i != 0) 
+				g_string_append (string, ", ");
 				
-				g_string_append (string, tmp);
-				g_string_append_c (string, ' ');
-				g_free (tmp);
-			}
-			else {
-				g_set_error (error, GDA_SERVER_OPERATION_ERROR,
-					     GDA_SERVER_OPERATION_INCORRECT_VALUE_ERROR,
-					     "%s", _("Incorrect specified column name"));
-				allok = FALSE;
-			}
+			g_string_append (string, tmp);
+			g_string_append_c (string, ' ');
+			g_free (tmp);
 		}
 		if (i > 0)
 			g_string_append (string, ")");
@@ -969,7 +1072,12 @@ gda_mysql_render_DROP_VIEW (GdaServerProvider *provider, GdaConnection *cnc,
 	if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value))
 		g_string_append (string, " IF EXISTS");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/VIEW_DESC_P/VIEW_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/VIEW_DESC_P/VIEW_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
+
 	g_string_append_c (string, ' ');
 	g_string_append (string, tmp);
 	g_free (tmp);
