@@ -7,7 +7,7 @@
  * Copyright (C) 2003 Chris Silles <csilles@src.gnome.org>
  * Copyright (C) 2003 Laurent Sansonetti <lrz@gnome.org>
  * Copyright (C) 2003 Paisa Seeluangsawat <paisa@users.sf.net>
- * Copyright (C) 2004 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2004 - 2016 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2005 Alan Knowles <alan@akbkhome.com>
  * Copyright (C) 2005 - 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2005 Mike Fisk <mfisk@woozle.org>
@@ -339,7 +339,7 @@ _gda_mysql_type_to_gda (G_GNUC_UNUSED MysqlConnectionData *cdata,
 		break;
 	case MYSQL_TYPE_TIMESTAMP:
 	case MYSQL_TYPE_DATETIME:
-		gtype = GDA_TYPE_TIMESTAMP;
+		gtype = G_TYPE_DATE_TIME;
 		break;
 	case MYSQL_TYPE_DATE:
 		gtype = G_TYPE_DATE;
@@ -408,7 +408,7 @@ gda_mysql_recordset_new_direct (GdaConnection *cnc, GdaDataModelAccessFlags flag
 
         g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
-	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data_error (cnc, NULL);
 	if (!cdata)
 		return NULL;
 
@@ -546,7 +546,7 @@ gda_mysql_recordset_new (GdaConnection            *cnc,
         g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
         g_return_val_if_fail (ps != NULL, NULL);
 
-	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	cdata = (MysqlConnectionData*) gda_connection_internal_get_provider_data_error (cnc, NULL);
 	if (!cdata)
 		return NULL;
 
@@ -859,14 +859,13 @@ new_row_from_mysql_stmt (GdaMysqlRecordset *imodel, G_GNUC_UNUSED gint rownum, G
 			memmove (&bvalue, mysql_bind_result[i].buffer, sizeof (bvalue));
 
 			if (type == GDA_TYPE_TIME) {
-				GdaTime time = {
-					.hour = bvalue.hour,
-					.minute = bvalue.minute,
-					.second = bvalue.second,
-					.fraction = bvalue.second_part,
-					.timezone = 0 /* GMT */
-				};
-				gda_value_set_time (value, &time);
+				GdaTime *time = gda_time_new_from_values (bvalue.hour,
+					                                      bvalue.minute,
+					                                      bvalue.second,
+					                                      bvalue.second_part,
+					                                      0); /* GMT */
+				gda_value_set_time (value, time);
+                gda_time_free (time);
 			}
 			else if (type == G_TYPE_DATE) {
 				GDate *date = g_date_new_dmy
@@ -875,18 +874,17 @@ new_row_from_mysql_stmt (GdaMysqlRecordset *imodel, G_GNUC_UNUSED gint rownum, G
 					 (bvalue.year != 0) ? bvalue.year : 1970);
 				g_value_take_boxed (value, date);
 			}
-			else if (type == GDA_TYPE_TIMESTAMP) {
-				GdaTimestamp timestamp = {
-					.year = bvalue.year,
-					.month = bvalue.month,
-					.day = bvalue.day,
-					.hour = bvalue.hour,
-					.minute = bvalue.minute,
-					.second = bvalue.second,
-					.fraction = bvalue.second_part,
-					.timezone = 0 /* GMT */
-				};
-				gda_value_set_timestamp (value, &timestamp);
+			else if (type == G_TYPE_DATE_TIME) {
+				GTimeZone *tz = g_time_zone_new ("Z"); /* UTC */
+				GDateTime *timestamp = g_date_time_new (tz,
+																								bvalue.year,
+																								bvalue.month,
+																								bvalue.day,
+																								bvalue.hour,
+																								bvalue.minute,
+																								(gdouble) bvalue.second + bvalue.second_part / 1000000.0);
+				g_value_set_boxed (value, timestamp);
+				g_date_time_unref (timestamp);
 			}
 			else {
 				gda_row_invalidate_value (row, value);
@@ -949,18 +947,19 @@ new_row_from_mysql_stmt (GdaMysqlRecordset *imodel, G_GNUC_UNUSED gint rownum, G
 			if (type == G_TYPE_STRING)
 				g_value_set_string (value, bvalue);
 			else if (type == GDA_TYPE_BINARY) {
-				GdaBinary binary = {
-					.data = (guchar*) bvalue,
-					.binary_length = length
-				};
-				gda_value_set_binary (value, &binary);
+				GdaBinary *bin;
+				bin = gda_binary_new ();
+				gda_binary_set_data (bin, (guchar*) bvalue, length);
+				gda_value_take_binary (value, bin);
 			}
 			else if (type == GDA_TYPE_BLOB) {
 				/* we don't use GdaMysqlBlobOp because it looks like the MySQL
 				 * API does not support BLOBs accessed in a random way,
 				 * so we return the whole BLOB at once */
-				GdaBlob blob = { {(guchar*) bvalue, length}, NULL };
-				gda_value_set_blob (value, &blob);
+				GdaBlob *blob = gda_blob_new ();
+				GdaBinary *bin = gda_blob_get_binary (blob);
+				gda_binary_set_data (bin, (guchar*) bvalue, length);
+				gda_value_take_blob (value, blob);
 			}
 			else if (type == GDA_TYPE_NUMERIC) {
 				if (length > 0) {
