@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2007 - 2016 Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,7 @@
 #include <libgda/libgda.h>
 #include "gda-oracle.h"
 #include "gda-oracle-blob-op.h"
+#include <libgda/gda-blob-op-impl.h>
 #include <libgda/gda-debug-macros.h>
 
 struct _GdaOracleBlobOpPrivate {
@@ -91,9 +92,9 @@ gda_oracle_blob_op_class_init (GdaOracleBlobOpClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_oracle_blob_op_finalize;
-	blob_class->get_length = gda_oracle_blob_op_get_length;
-	blob_class->read = gda_oracle_blob_op_read;
-	blob_class->write = gda_oracle_blob_op_write;
+	GDA_BLOB_OP_FUNCTIONS (blob_class->functions)->get_length = gda_oracle_blob_op_get_length;
+	GDA_BLOB_OP_FUNCTIONS (blob_class->functions)->read = gda_oracle_blob_op_read;
+	GDA_BLOB_OP_FUNCTIONS (blob_class->functions)->write = gda_oracle_blob_op_write;
 }
 
 static void
@@ -119,7 +120,7 @@ gda_oracle_blob_op_new (GdaConnection *cnc, OCILobLocator *lobloc)
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
-	bop = g_object_new (GDA_TYPE_ORACLE_BLOB_OP, NULL);
+	bop = g_object_new (GDA_TYPE_ORACLE_BLOB_OP, "connection", cnc, NULL);
 	bop->priv->cnc = cnc;
 	
 	return GDA_BLOB_OP (bop);
@@ -149,7 +150,6 @@ static glong
 gda_oracle_blob_op_read (GdaBlobOp *op, GdaBlob *blob, glong offset, glong size)
 {
 	GdaOracleBlobOp *bop;
-	GdaBinary *bin;
 
 	g_return_val_if_fail (GDA_IS_ORACLE_BLOB_OP (op), -1);
 	bop = GDA_ORACLE_BLOB_OP (op);
@@ -159,16 +159,10 @@ gda_oracle_blob_op_read (GdaBlobOp *op, GdaBlob *blob, glong offset, glong size)
 		return -1;
 	g_return_val_if_fail (blob, -1);
 
-	bin = (GdaBinary *) blob;
-	if (bin->data) 
-		g_free (bin->data);
-	bin->data = g_new0 (guchar, size);
-	bin->binary_length = 0;
-
 	/* fetch blob data using C API into bin->data, and set bin->binary_length */
 	TO_IMPLEMENT;
 
-	return bin->binary_length;
+	return -1;
 }
 
 /*
@@ -186,25 +180,27 @@ gda_oracle_blob_op_write (GdaBlobOp *op, GdaBlob *blob, glong offset)
 	g_return_val_if_fail (GDA_IS_CONNECTION (bop->priv->cnc), -1);
 	g_return_val_if_fail (blob, -1);
 
-	if (blob->op && (blob->op != op)) {
+	GdaBlobOp *blob_op;
+	blob_op = gda_blob_get_op (blob);
+	if (blob_op && (blob_op != op)) {
 		/* use data through blob->op */
 		#define buf_size 16384
 		gint nread = 0;
-		GdaBlob *tmpblob = g_new0 (GdaBlob, 1);
-		gda_blob_set_op (tmpblob, blob->op);
+		GdaBlob *tmpblob = gda_blob_new ();
+		gda_blob_set_op (tmpblob, blob_op);
 
 		nbwritten = 0;
 
-		for (nread = gda_blob_op_read (tmpblob->op, tmpblob, nbwritten, buf_size);
+		for (nread = gda_blob_op_read (blob_op, tmpblob, nbwritten, buf_size);
 		     nread > 0;
-		     nread = gda_blob_op_read (tmpblob->op, tmpblob, nbwritten, buf_size)) {
+		     nread = gda_blob_op_read (blob_op, tmpblob, nbwritten, buf_size)) {
 			glong tmp_written;
 
 			tmp_written = -1; TO_IMPLEMENT;
 			
 			if (tmp_written < 0) {
 				/* treat error */
-				gda_blob_free ((gpointer) tmpblob);
+				gda_blob_free (tmpblob);
 				return -1;
 			}
 			nbwritten += tmp_written;
@@ -212,7 +208,7 @@ gda_oracle_blob_op_write (GdaBlobOp *op, GdaBlob *blob, glong offset)
 				/* nothing more to read */
 				break;
 		}
-		gda_blob_free ((gpointer) tmpblob);
+		gda_blob_free (tmpblob);
 	}
 	else {
 		/* write blob using bin->data and bin->binary_length */
