@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2007 Armin Burgmeier <armin@openismus.com>
- * Copyright (C) 2007 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2007 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2007 - 2012 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2007 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  *
@@ -182,7 +182,7 @@ gda_postgres_handler_bin_get_sql_from_value (GdaDataHandler *iface, const GValue
 
 	if (hdl->priv->cnc) {
 		g_return_val_if_fail (GDA_IS_CONNECTION (hdl->priv->cnc), NULL);
-		cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data (hdl->priv->cnc);
+		cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data_error (hdl->priv->cnc, NULL);
 	}
 
 	GdaBinary *data = (GdaBinary *) gda_value_get_binary ((GValue *) value);
@@ -194,8 +194,8 @@ gda_postgres_handler_bin_get_sql_from_value (GdaDataHandler *iface, const GValue
 			/*tmp = PQescapeByteaConn (cdata->pconn, data, length, &retlength);*/
 		}
 		else
-			tmp = (gchar *) PQescapeBytea (data->data, 
-						       data->binary_length, &retlength);
+			tmp = (gchar *) PQescapeBytea (gda_binary_get_data (data), 
+						       gda_binary_get_size (data), &retlength);
 		if (tmp) {
 			retval = g_strdup_printf ("'%s'", tmp);
 			PQfreemem (tmp);
