@@ -7,13 +7,13 @@
  * Copyright (C) 2004 Andrew Hill <andru@src.gnome.org>
  * Copyright (C) 2004 - 2005 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2004 Szalai Ferenc <szferi@einstein.ki.iif.hu>
- * Copyright (C) 2004 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2004 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2005 Alex <alex@igalia.com>
  * Copyright (C) 2005 Álvaro Peña <alvaropg@telefonica.net>
  * Copyright (C) 2006 - 2011 Murray Cumming <murrayc@murrayc.com>
  * Copyright (C) 2007 Armin Burgmeier <armin@openismus.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
- * Copyright (C) 2011 Daniel Espinosa <despinosa@src.gnome.org>
+ * Copyright (C) 2011-2017 Daniel Espinosa <esodan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -334,7 +334,7 @@ gda_postgres_recordset_new_random (GdaConnection *cnc, GdaPostgresPStmt *ps, Gda
         g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
         g_return_val_if_fail (ps, NULL);
 
-	cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data_error (cnc, NULL);
 	if (!cdata)
 		return NULL;
 
@@ -365,7 +365,7 @@ gda_postgres_recordset_new_cursor (GdaConnection *cnc, GdaPostgresPStmt *ps, Gda
         g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
         g_return_val_if_fail (ps, NULL);
 
-	cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data_error (cnc, NULL);
 	if (!cdata)
 		return NULL;
 
@@ -589,10 +589,10 @@ static void
 make_point (GdaGeometricPoint *point, const gchar *value)
 {
 	value++;
-	point->x = g_ascii_strtod (value, NULL);
+	gda_geometric_point_set_x (point, g_ascii_strtod (value, NULL));
 	value = strchr (value, ',');
 	value++;
-	point->y = g_ascii_strtod (value, NULL);
+	gda_geometric_point_set_y (point, g_ascii_strtod (value, NULL));
 }
 
 static void
@@ -630,18 +630,19 @@ set_value (GdaConnection *cnc, GdaRow *row, GValue *value, GType type, const gch
 				     "%s", _("Internal error"));
 	}
 	else if (type == GDA_TYPE_TIME) {
-		GdaTime timegda;
-		if (!gda_parse_iso8601_time (&timegda, thevalue)) {
+		GdaTime* timegda = gda_time_new ();
+		if (!gda_parse_iso8601_time (timegda, thevalue)) {
 			gda_row_invalidate_value (row, value); 
 			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 				     GDA_SERVER_PROVIDER_DATA_ERROR,
 				     _("Invalid time '%s' (time format should be HH:MM:SS[.ms])"), thevalue);
 		}
 		else {
-			if (timegda.timezone == GDA_TIMEZONE_INVALID)
-				timegda.timezone = 0; /* set to GMT */
-			gda_value_set_time (value, &timegda);
+			if (gda_time_get_timezone (timegda) == GDA_TIMEZONE_INVALID)
+				gda_time_set_timezone (timegda, 0); /* set to GMT */
+			gda_value_set_time (value, timegda);
 		}
+		gda_time_free (timegda);
 	}
 	else if (type == G_TYPE_INT64)
 		g_value_set_int64 (value, atoll (thevalue));
@@ -666,20 +667,20 @@ set_value (GdaConnection *cnc, GdaRow *row, GValue *value, GType type, const gch
 		gda_numeric_free (numeric);
 	}
 	else if (type == GDA_TYPE_GEOMETRIC_POINT) {
-		GdaGeometricPoint point;
-		make_point (&point, thevalue);
-		gda_value_set_geometric_point (value, &point);
+		GdaGeometricPoint* point = gda_geometric_point_new ();
+		make_point (point, thevalue);
+		gda_value_set_geometric_point (value, point);
+		gda_geometric_point_free (point);
 	}
-	else if (type == GDA_TYPE_TIMESTAMP) {
+	else if (type == G_TYPE_DATE_TIME) {
 		PostgresConnectionData *cdata;
 		cdata = (PostgresConnectionData*) gda_connection_internal_get_provider_data_error (cnc, error);
 		if (cdata) {
-			GdaTimestamp timestamp;
-			if (gda_parse_formatted_timestamp (&timestamp, thevalue, cdata->date_first, cdata->date_second,
-							   cdata->date_third, cdata->date_sep)) {
-				if (timestamp.timezone == GDA_TIMEZONE_INVALID)
-					timestamp.timezone = 0; /* set to GMT */
-				gda_value_set_timestamp (value, &timestamp);
+			GDateTime* timestamp = gda_parse_formatted_timestamp (thevalue, cdata->date_first, cdata->date_second,
+							   cdata->date_third, cdata->date_sep);
+			if (timestamp != NULL) {
+				g_value_set_boxed (value, timestamp);
+			  g_date_time_unref (timestamp);
 			}
 			else {
 				gda_row_invalidate_value (row, value);
@@ -737,9 +738,8 @@ set_value (GdaConnection *cnc, GdaRow *row, GValue *value, GType type, const gch
 					}
 					if (! *ptr) {
 						GdaBinary *bin;
-						bin = g_new (GdaBinary, 1);
-						bin->data = unescaped;
-						bin->binary_length = pqlength;
+						bin = gda_binary_new ();
+						gda_binary_set_data (bin, unescaped, pqlength);
 						gda_value_take_binary (value, bin);
 						valueset = TRUE;
 					}
@@ -748,10 +748,9 @@ set_value (GdaConnection *cnc, GdaRow *row, GValue *value, GType type, const gch
 			else {
 				unescaped = PQunescapeBytea ((guchar*) thevalue, &pqlength);
 				if (unescaped) {
-					GdaBinary bin;
-					bin.data = unescaped;
-					bin.binary_length = pqlength;
-					gda_value_set_binary (value, &bin);
+					GdaBinary* bin = gda_binary_new ();
+					gda_binary_set_data (bin, unescaped, pqlength);
+					gda_value_take_binary (value, bin);
 					PQfreemem (unescaped);
 					valueset = TRUE;
 				}
@@ -771,7 +770,7 @@ set_value (GdaConnection *cnc, GdaRow *row, GValue *value, GType type, const gch
 	else if (type == GDA_TYPE_BLOB) {
 		GdaBlob *blob;
 		GdaBlobOp *op;
-		blob = g_new0 (GdaBlob, 1);
+		blob = gda_blob_new ();
 		op = gda_postgres_blob_op_new_with_id (cnc, thevalue);
 		gda_blob_set_op (blob, op);
 		g_object_unref (op);
