@@ -43,7 +43,11 @@ gda_capi_render_CREATE_TABLE (GdaServerProvider *provider, GdaConnection *cnc,
 	/* CREATE TABLE */
 	string = g_string_new ("CREATE TABLE ");
 
-	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DEF_P/TABLE_NAME");
+	tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider, "/TABLE_DEF_P/TABLE_NAME", error);
+	if (!tmp) {
+		g_string_free (string, TRUE);
+		return NULL;
+	}
 	g_string_append (string, tmp);
 	g_free (tmp);
 	g_string_append (string, " (");
@@ -61,7 +65,11 @@ gda_capi_render_CREATE_TABLE (GdaServerProvider *provider, GdaConnection *cnc,
 			value = gda_server_operation_get_value_at (op, "/FIELDS_A/@COLUMN_PKEY/%d", i);
 			if (value && G_VALUE_HOLDS (value, G_TYPE_BOOLEAN) && g_value_get_boolean (value)) {
 				tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider,
-										  "/FIELDS_A/@COLUMN_NAME/%d", i);
+										  "/FIELDS_A/@COLUMN_NAME/%d", error, i);
+				if (!tmp) {
+					g_string_free (string, TRUE);
+					return NULL;
+				}
 				pkfields = g_slist_append (pkfields, tmp);
 				nbpkfields++;
 			}
@@ -77,7 +85,12 @@ gda_capi_render_CREATE_TABLE (GdaServerProvider *provider, GdaConnection *cnc,
 				g_string_append (string, ", ");
 				
 			tmp = gda_server_operation_get_sql_identifier_at (op, cnc, provider,
-									  "/FIELDS_A/@COLUMN_NAME/%d", i);
+									  "/FIELDS_A/@COLUMN_NAME/%d", error, i);
+			if (!tmp) {
+				g_string_free (string, TRUE);
+				return NULL;
+			}
+
 			g_string_append (string, tmp);
 			g_free (tmp);
 			g_string_append_c (string, ' ');
