@@ -35,7 +35,7 @@
 #include <libgda/gda-holder.h>
 #include <libgda/gda-debug-macros.h>
 #include <libgda/sqlite/keywords_hash.h>
-#include "keywords_hash.c" /* this one is dynamically generated */
+#include "keywords_hash.code" /* this one is dynamically generated */
 
 /*
  * predefined statements' IDs
@@ -90,10 +90,6 @@ _gda_capi_provider_meta_init (GdaServerProvider *provider)
 
 	g_mutex_unlock (&init_mutex);
 
-#ifdef GDA_DEBUG
-	/* make sure the reserved keywords hash works correctly */
-        test_keywords ();
-#endif
 }
 
 gboolean
