@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
@@ -90,7 +90,7 @@ gda_web_pstmt_finalize (GObject *object)
 
 	if (pstmt->pstmt_hash) {
 		WebConnectionData *cdata;
-		cdata = (WebConnectionData*) gda_connection_internal_get_provider_data (pstmt->cnc);
+		cdata = (WebConnectionData*) gda_connection_internal_get_provider_data_error (pstmt->cnc, NULL);
 		if (!cdata) 
 			goto next;
 
