@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2009 - 2013 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2014 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2014 Anders Jonsson <anders.jonsson@norsjovallen.se>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,98 +25,23 @@
 #include <string.h>
 #include "../reuseable/reuse-all.h"
 
-/* Use the RSA reference implementation included in the RFC-1321, http://www.freesoft.org/CIE/RFC/1321/ */
-#include "global.h"
-#include "md5.h"
-
 #ifdef HAVE_INTTYPES_H
 #include <inttypes.h>
 #endif
 
-#define PAD_LEN 64  /* PAD length */
-#define SIG_LEN 16  /* MD5 digest length */
-/*
- * From RFC 2104
- */
-static void
-hmac_md5 (uint8_t*  text,            /* pointer to data stream */
-	  int   text_len,            /* length of data stream */
-	  uint8_t*  key,             /* pointer to authentication key */
-	  int   key_len,             /* length of authentication key */
-	  uint8_t  *hmac)            /* returned hmac-md5 */
-{
-	MD5_CTX md5c;
-	uint8_t k_ipad[PAD_LEN];    /* inner padding - key XORd with ipad */
-	uint8_t k_opad[PAD_LEN];    /* outer padding - key XORd with opad */
-	uint8_t keysig[SIG_LEN];
-	int i;
-
-	/* if key is longer than PAD length, reset it to key=MD5(key) */
-	if (key_len > PAD_LEN) {
-		MD5_CTX md5key;
-
-		MD5Init (&md5key);
-		MD5Update (&md5key, key, key_len);
-		MD5Final (keysig, &md5key);
-
-		key = keysig;
-		key_len = SIG_LEN;
-	}
-
-	/*
-	 * the HMAC_MD5 transform looks like:
-	 *
-	 * MD5(Key XOR opad, MD5(Key XOR ipad, text))
-	 *
-	 * where Key is an n byte key
-	 * ipad is the byte 0x36 repeated 64 times
-
-	 * opad is the byte 0x5c repeated 64 times
-	 * and text is the data being protected
-	 */
-
-	/* Zero pads and store key */
-	memset (k_ipad, 0, PAD_LEN);
-	memcpy (k_ipad, key, key_len);
-	memcpy (k_opad, k_ipad, PAD_LEN);
-
-	/* XOR key with ipad and opad values */
-	for (i=0; i<PAD_LEN; i++) {
-		k_ipad[i] ^= 0x36;
-		k_opad[i] ^= 0x5c;
-	}
-
-	/* perform inner MD5 */
-	MD5Init (&md5c);                    /* start inner hash */
-	MD5Update (&md5c, k_ipad, PAD_LEN); /* hash inner pad */
-	MD5Update (&md5c, text, text_len);  /* hash text */
-	MD5Final (hmac, &md5c);             /* store inner hash */
-
-	/* perform outer MD5 */
-	MD5Init (&md5c);                    /* start outer hash */
-	MD5Update (&md5c, k_opad, PAD_LEN); /* hash outer pad */
-	MD5Update (&md5c, hmac, SIG_LEN);   /* hash inner hash */
-	MD5Final (hmac, &md5c);             /* store results */
-}
-
 static gboolean
 check_hash (const gchar *key, const gchar *data, const gchar *expected_hash)
 {
-	uint8_t hmac[16];
-	GString *md5str;
-	gint i;
 	gboolean retval = TRUE;
+	gchar *md5str;
+
+	md5str = g_compute_hmac_for_string (G_CHECKSUM_MD5, key, strlen (key),
+					    data, -1);
 	
-	hmac_md5 ((uint8_t *) data, strlen (data),
-		  (uint8_t *) key, strlen (key), hmac);
-	md5str = g_string_new ("");
-	for (i = 0; i < 16; i++)
-		g_string_append_printf (md5str, "%02x", hmac[i]);
-	
-	if (strcmp (md5str->str, expected_hash))
+	if (strcmp (md5str, expected_hash))
 		retval = FALSE;	
 	
-	g_string_free (md5str, TRUE);
+	g_free (md5str);
 	return retval;
 }
 
@@ -256,9 +182,9 @@ worker_got_chunk_cb (SoupMessage *msg, SoupBuffer *chunk, ThreadData *thdata)
 					xmlFreeDoc (doc);
 			}
 			else {
-				gda_mutex_lock (thdata->cdata->mutex);
+				g_rec_mutex_lock (& (thdata->cdata->mutex));
 				g_assert (thdata->cdata->worker_counter == counter_id);
-				gda_mutex_unlock (thdata->cdata->mutex);
+				g_rec_mutex_unlock (& (thdata->cdata->mutex));
 				xmlFreeDoc (doc);
 			}
 		}
@@ -276,7 +202,7 @@ start_worker_in_sub_thread (ThreadData *thdata)
 	
 	while (runagain) {
 		GString *real_url;
-		gda_mutex_lock (thdata->cdata->mutex);
+		g_rec_mutex_lock (& (thdata->cdata->mutex));
 		real_url = g_string_new (thdata->cdata->worker_url);
 		if (thdata->cdata->session_id)
 			g_string_append_printf (real_url, "?%s", thdata->cdata->session_id);
@@ -285,16 +211,16 @@ start_worker_in_sub_thread (ThreadData *thdata)
 			thdata->cdata->worker_counter = 1;
 		else
 			thdata->cdata->worker_counter ++;
-		gda_mutex_unlock (thdata->cdata->mutex);
+		g_rec_mutex_unlock (& (thdata->cdata->mutex));
 
 		msg = soup_message_new ("GET", real_url->str);
 		/*g_print ("=== WORKER Request URL: [%s]\n", real_url->str);*/
 		if (!msg) {
 			g_warning (_("Invalid HOST/SCRIPT '%s'"), real_url->str);
 			g_string_free (real_url, TRUE);
-			gda_mutex_lock (thdata->cdata->mutex);
+			g_rec_mutex_lock (& (thdata->cdata->mutex));
 			thdata->cdata->worker_running = FALSE;
-			gda_mutex_unlock (thdata->cdata->mutex);
+			g_rec_mutex_unlock (& (thdata->cdata->mutex));
 			g_free (thdata);
 			return NULL;
 		}
@@ -305,11 +231,11 @@ start_worker_in_sub_thread (ThreadData *thdata)
 		guint res;
 		res = soup_session_send_message (thdata->cdata->worker_session, msg);
 		
-		gda_mutex_lock (thdata->cdata->mutex);
+		g_rec_mutex_lock (& (thdata->cdata->mutex));
 		thdata->cdata->worker_running = FALSE;
 		runagain = thdata->cdata->worker_needed;
 		runagain = runagain && SOUP_STATUS_IS_SUCCESSFUL (res);
-		gda_mutex_unlock (thdata->cdata->mutex);
+		g_rec_mutex_unlock (& (thdata->cdata->mutex));
 
 		g_signal_handler_disconnect (msg, sigid);
 		g_object_unref (msg);
@@ -334,9 +260,9 @@ start_worker (GdaConnection *cnc, WebConnectionData *cdata)
 	thdata->cdata = cdata;
 
 	/* set cdata->worker_running to TRUE to avoid having to add a delay */
-	gda_mutex_lock (cdata->mutex);
+	g_rec_mutex_lock (& (cdata->mutex));
 	cdata->worker_running = TRUE;
-	gda_mutex_unlock (cdata->mutex);
+	g_rec_mutex_unlock (& (cdata->mutex));
 
 	if (! g_thread_new ("web-worker", (GThreadFunc) start_worker_in_sub_thread,
 			    thdata)) {
@@ -348,21 +274,21 @@ start_worker (GdaConnection *cnc, WebConnectionData *cdata)
 	gint nb_retries;
 	for (nb_retries = 0; nb_retries < 10; nb_retries++) {
 		gboolean wait_over;
-		gda_mutex_lock (cdata->mutex);
+		g_rec_mutex_lock (& (cdata->mutex));
 		wait_over = !cdata->worker_running || cdata->session_id;
-		gda_mutex_unlock (cdata->mutex);
+		g_rec_mutex_unlock (& (cdata->mutex));
 		if (wait_over)
 			break;
 		else
 			g_usleep (200000);
 	}
 
-	gda_mutex_lock (cdata->mutex);
+	g_rec_mutex_lock (& (cdata->mutex));
 	if (!cdata->session_id) {
 		/* there was an error */
 		cdata->worker_running = FALSE;
 	}
-	gda_mutex_unlock (cdata->mutex);
+	g_rec_mutex_unlock (& (cdata->mutex));
 }
 
 /*
@@ -387,17 +313,17 @@ _gda_web_send_message_to_frontend (GdaConnection *cnc, WebConnectionData *cdata,
 		*out_status_chr = 0;
 
 	/* handle the need to run the worker to get an initial sessionID */
-	gda_mutex_lock (cdata->mutex);
+	g_rec_mutex_lock (& (cdata->mutex));
 	cdata->worker_needed = TRUE;
 	if (!cdata->worker_running && !cdata->session_id) {
-		gda_mutex_unlock (cdata->mutex);
+		g_rec_mutex_unlock (& (cdata->mutex));
 		start_worker (cnc, cdata);
 
-		gda_mutex_lock (cdata->mutex);
+		g_rec_mutex_lock (& (cdata->mutex));
 		if (! cdata->worker_running) {
 			gda_connection_add_event_string (cnc, _("Could not run PHP script on the server"));
 			cdata->worker_needed = FALSE;
-			gda_mutex_unlock (cdata->mutex);
+			g_rec_mutex_unlock (& (cdata->mutex));
 			return NULL;
 		}
 	}
@@ -405,7 +331,7 @@ _gda_web_send_message_to_frontend (GdaConnection *cnc, WebConnectionData *cdata,
 	/* prepare new message */
 	g_assert (cdata->session_id);
 	real_url = g_strdup_printf ("%s?%s&c=%d", cdata->front_url, cdata->session_id, counter++);
-	gda_mutex_unlock (cdata->mutex);
+	g_rec_mutex_unlock (& (cdata->mutex));
 	msg = soup_message_new ("POST", real_url);
 	if (!msg) {
 		gda_connection_add_event_string (cnc, _("Invalid HOST/SCRIPT '%s'"), real_url);
@@ -415,7 +341,7 @@ _gda_web_send_message_to_frontend (GdaConnection *cnc, WebConnectionData *cdata,
 	g_free (real_url);
 
 	/* check context */
-	gda_mutex_lock (cdata->mutex);
+	g_rec_mutex_lock (& (cdata->mutex));
 	if (gda_connection_get_transaction_status (cnc) &&
 	    (!cdata->worker_running ||
 	     ((msgtype == MESSAGE_EXEC) && (cdata->last_exec_counter != cdata->worker_counter)))) {
@@ -426,37 +352,35 @@ _gda_web_send_message_to_frontend (GdaConnection *cnc, WebConnectionData *cdata,
 		g_object_unref (msg);
 
 		gda_connection_internal_reset_transaction_status (cnc);
-		gda_mutex_unlock (cdata->mutex);
+		g_rec_mutex_unlock (& (cdata->mutex));
 		return NULL;
 	}
 	if (! cdata->worker_running) {
-		gda_mutex_unlock (cdata->mutex);
+		g_rec_mutex_unlock (& (cdata->mutex));
 		start_worker (cnc, cdata);
 
-		gda_mutex_lock (cdata->mutex);
+		g_rec_mutex_lock (& (cdata->mutex));
 		if (! cdata->worker_running) {
 			gda_connection_add_event_string (cnc, _("Could not run PHP script on the server"));
 			g_object_unref (msg);
-			gda_mutex_unlock (cdata->mutex);
+			g_rec_mutex_unlock (& (cdata->mutex));
 			return NULL;
 		}
 	}
-	gda_mutex_unlock (cdata->mutex);
+	g_rec_mutex_unlock (& (cdata->mutex));
 
 	/* finalize and send message */
  	if (hash_key) {
-		uint8_t hmac[16];
-		GString *md5str;
-		gint i;
-		
-		hmac_md5 ((uint8_t *) message, strlen (message),
-			  (uint8_t *) hash_key, strlen (hash_key), hmac);
-		md5str = g_string_new ("");
-		for (i = 0; i < 16; i++)
-			g_string_append_printf (md5str, "%02x", hmac[i]);
-		g_string_append_c (md5str, '\n');
-		g_string_append (md5str, message);
-		h_message = g_string_free (md5str, FALSE);
+		gchar *md5str;
+		md5str = g_compute_hmac_for_string (G_CHECKSUM_MD5, hash_key, strlen (hash_key),
+						    message, -1);
+
+		GString *string;
+		string = g_string_new (md5str);
+		g_free (md5str);
+		g_string_append_c (string, '\n');
+		g_string_append (string, message);
+		h_message = g_string_free (string, FALSE);
 	}
 	else
 		h_message = g_strdup_printf ("NOHASH\n%s", message);
@@ -470,9 +394,9 @@ _gda_web_send_message_to_frontend (GdaConnection *cnc, WebConnectionData *cdata,
 	g_object_set (G_OBJECT (cdata->front_session), SOUP_SESSION_TIMEOUT, 20, NULL);
 	status = soup_session_send_message (cdata->front_session, msg);
 
-	gda_mutex_lock (cdata->mutex);
+	g_rec_mutex_lock (& (cdata->mutex));
 	cdata->worker_needed = FALSE;
-	gda_mutex_unlock (cdata->mutex);
+	g_rec_mutex_unlock (& (cdata->mutex));
 
 	if (!SOUP_STATUS_IS_SUCCESSFUL (status)) {
 		gda_connection_add_event_string (cnc, msg->reason_phrase);
@@ -485,10 +409,10 @@ _gda_web_send_message_to_frontend (GdaConnection *cnc, WebConnectionData *cdata,
 	doc = _gda_web_decode_response (cnc, cdata, msg->response_body, out_status_chr, &counter_id);
 	g_object_unref (msg);
 	
-	gda_mutex_lock (cdata->mutex);
+	g_rec_mutex_lock (& (cdata->mutex));
 	if (msgtype == MESSAGE_EXEC)
 		cdata->last_exec_counter = counter_id;
-	gda_mutex_unlock (cdata->mutex);
+	g_rec_mutex_unlock (& (cdata->mutex));
 
 	return doc;
 }
@@ -523,19 +447,12 @@ _gda_web_decode_response (GdaConnection *cnc, WebConnectionData *cdata, SoupMess
 gchar *
 _gda_web_compute_token (WebConnectionData *cdata)
 {
-	uint8_t hmac[16];
-	GString *md5str;
-	gint i;
-	
 	g_return_val_if_fail (cdata->next_challenge && cdata->key, NULL);
 
-	hmac_md5 ((uint8_t *) cdata->next_challenge, strlen (cdata->next_challenge),
-		  (uint8_t *) cdata->key, strlen (cdata->key), hmac);
-	md5str = g_string_new ("");
-	for (i = 0; i < 16; i++)
-		g_string_append_printf (md5str, "%02x", hmac[i]);
-	
-	return g_string_free (md5str, FALSE);
+	gchar *md5str;
+	md5str = g_compute_hmac_for_string (G_CHECKSUM_MD5, cdata->key, strlen (cdata->key),
+					    cdata->next_challenge, -1);	
+	return md5str;
 }
 
 /*
@@ -550,13 +467,13 @@ _gda_web_do_server_cleanup (GdaConnection *cnc, WebConnectionData *cdata)
 	gint nb_retries;
 
 	/* wait for worker to finish */
-	gda_mutex_lock (cdata->mutex);
+	g_rec_mutex_lock (& (cdata->mutex));
 	for (nb_retries = 0; (nb_retries < 10) && cdata->worker_running; nb_retries ++) {
-		gda_mutex_unlock (cdata->mutex);
+		g_rec_mutex_unlock (& (cdata->mutex));
 		g_usleep (50000);
-		gda_mutex_lock (cdata->mutex);
+		g_rec_mutex_lock (& (cdata->mutex));
 	}
-	gda_mutex_unlock (cdata->mutex);
+	g_rec_mutex_unlock (& (cdata->mutex));
  
 	real_url = g_strdup_printf ("%s/gda-clean.php?%s", cdata->server_base_url, cdata->session_id);
 	msg = soup_message_new ("GET", real_url);
@@ -604,7 +521,7 @@ _gda_web_set_connection_error_from_xmldoc (GdaConnection *cnc, xmlDocPtr doc, GE
 				xmlFree (prop);
 			}
 			else
-				ev = gda_connection_add_event_string (cnc, _("Non detailled error"));
+				ev = gda_connection_add_event_string (cnc, _("Non detailed error"));
 			break;
 		}
 	}
@@ -625,7 +542,7 @@ void
 _gda_web_change_connection_to_closed (GdaConnection *cnc, WebConnectionData *cdata)
 {
 	cdata->forced_closing = TRUE;
-	gda_connection_close_no_warning (cnc);
+	gda_connection_close (cnc, NULL);
 }
 
 /*
@@ -648,8 +565,7 @@ _gda_web_free_cnc_data (WebConnectionData *cdata)
 	g_free (cdata->server_base_url);
 	g_free (cdata->front_url);
 	g_free (cdata->worker_url);
-	if (cdata->mutex)
-		gda_mutex_free (cdata->mutex);
+	g_rec_mutex_clear (& (cdata->mutex));
 	if (cdata->worker_session)
 		g_object_unref (cdata->worker_session);
 	if (cdata->front_session)
