@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2007 Armin Burgmeier <armin@openismus.com>
  * Copyright (C) 2007 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2007 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2007 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -22,6 +22,7 @@
 #include <sql-parser/gda-sql-parser.h>
 #include <glib/gi18n-lib.h>
 #include <string.h>
+#include <libgda/gda-connection-private.h>
 
 /* options */
 gchar *pass = NULL;
@@ -184,7 +185,8 @@ main (int argc, char **argv)
 	*/
 
 	gda_connection_commit_transaction (cnc, NULL, NULL);
-	gda_connection_close (cnc);
+	if (! gda_connection_close (cnc, &error))
+		g_error ("Can't close connection: %s", error && error->message ? error->message : "No detail");
 
 	return 0;
 }
@@ -360,7 +362,7 @@ exec_statement (GdaConnection *cnc, GdaStatement *stmt, GdaSet *plist, GError **
 			GSList *list;
 
 			g_print ("Query returned a GdaSet:\n");
-			for (list = GDA_SET (exec_res)->holders; list; list = list->next) {
+			for (list = gda_set_get_holders (GDA_SET (exec_res)); list; list = list->next) {
 				gchar *str;
 				str = gda_holder_get_value_str (GDA_HOLDER (list->data), NULL);
 				g_print (" %s => %s\n", gda_holder_get_id (GDA_HOLDER (list->data)), str);
