@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -101,7 +102,7 @@ get_handler (GType for_type)
 		g_hash_table_insert (hash, (gpointer) G_TYPE_DOUBLE, gda_handler_numerical_new ());
 		g_hash_table_insert (hash, (gpointer) GDA_TYPE_NUMERIC, gda_handler_numerical_new ());
 		g_hash_table_insert (hash, (gpointer) GDA_TYPE_TIME, gda_handler_time_new ());
-		g_hash_table_insert (hash, (gpointer) GDA_TYPE_TIMESTAMP, gda_handler_time_new ());
+		g_hash_table_insert (hash, (gpointer) G_TYPE_DATE_TIME, gda_handler_time_new ());
 		g_hash_table_insert (hash, (gpointer) G_TYPE_ULONG, gda_handler_type_new ());
 	}
 
@@ -139,15 +140,16 @@ main (int argc, char **argv)
         g_option_context_free (context);
 	
 	/* Initialize the widget set */
-	gdaui_init ();
 	gtk_init (&argc, &argv);
+	gdaui_init ();
 
 	/* init main conf */
-	GType tested_gtypes [] = {G_TYPE_INT64, G_TYPE_UINT64, GDA_TYPE_BINARY, G_TYPE_BOOLEAN, GDA_TYPE_BLOB,
+	GType tested_gtypes [] = {G_TYPE_STRING,
+				  G_TYPE_INT64, G_TYPE_UINT64, GDA_TYPE_BINARY, G_TYPE_BOOLEAN, GDA_TYPE_BLOB,
 				  G_TYPE_DATE, G_TYPE_DOUBLE,
 				  GDA_TYPE_GEOMETRIC_POINT, G_TYPE_OBJECT, G_TYPE_INT, 
-				  GDA_TYPE_NUMERIC, G_TYPE_FLOAT, GDA_TYPE_SHORT, GDA_TYPE_USHORT, G_TYPE_STRING, 
-				  GDA_TYPE_TIME, GDA_TYPE_TIMESTAMP, G_TYPE_CHAR, G_TYPE_UCHAR, G_TYPE_UINT};
+				  GDA_TYPE_NUMERIC, G_TYPE_FLOAT, GDA_TYPE_SHORT, GDA_TYPE_USHORT,
+				  GDA_TYPE_TIME, G_TYPE_DATE_TIME, G_TYPE_CHAR, G_TYPE_UCHAR, G_TYPE_UINT};
 	mainconf.test_type = TESTED_BASIC;
 	if (test_type) {
 		if (*test_type == 'f')
@@ -195,7 +197,9 @@ main (int argc, char **argv)
 			      "This test displays GdauiDataEntry widgets and helpers to test "
 			      "them in pages of a notebook. Each page presents the default "
 			      "data entry for the corresponding data type.");
-	gtk_misc_set_alignment (GTK_MISC (label), 0., 0.5);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
+	gtk_widget_set_margin_start (label, 10);
+	gtk_widget_set_margin_end (label, 10);
 	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 5);
 	gtk_widget_show (label);
 
@@ -219,7 +223,9 @@ main (int argc, char **argv)
 			      "This test displays GdauiDataEntry widgets and helpers to test "
 			      "them in pages of a notebook. Each page tests a plugin for a given "
 			      "data type");
-	gtk_misc_set_alignment (GTK_MISC (label), 0., 0.5);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
+	gtk_widget_set_margin_start (label, 10);
+	gtk_widget_set_margin_end (label, 10);
 	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 5);
 	gtk_widget_show (label);
 
@@ -261,7 +267,7 @@ build_menu (GtkWidget *mainwin, GtkWidget *top_nb)
 	menuitem1_menu = gtk_menu_new ();
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem1), menuitem1_menu);
 	
-	mitem = gtk_image_menu_item_new_from_stock ("gtk-quit", accel_group);
+	mitem = gtk_menu_item_new_with_label ("_Quit");
 	gtk_widget_show (mitem);
 	gtk_container_add (GTK_CONTAINER (menuitem1_menu), mitem);
 	g_signal_connect (mitem, "activate",
@@ -298,21 +304,21 @@ build_menu (GtkWidget *mainwin, GtkWidget *top_nb)
 	mitem = gtk_separator_menu_item_new ();
 	gtk_container_add (GTK_CONTAINER (menuitem2_menu), mitem);
 
-	mitem = gtk_menu_item_new_with_mnemonic ("Plugins individual data entry widgets");
+	mitem = gtk_menu_item_new_with_mnemonic ("Plugins: data entry widgets alone");
 	gtk_widget_show (mitem);
 	gtk_container_add (GTK_CONTAINER (menuitem2_menu), mitem);
 	g_object_set_data (G_OBJECT (mitem), "test_type", GINT_TO_POINTER (TESTED_BASIC));
 	g_signal_connect (mitem, "activate",
 			  G_CALLBACK (on_plugins_activate), top_nb);
 
-	mitem = gtk_menu_item_new_with_mnemonic ("Plugins data entry widgets in a form");
+	mitem = gtk_menu_item_new_with_mnemonic ("Plugins: data entry widgets in a form");
 	gtk_widget_show (mitem);
 	gtk_container_add (GTK_CONTAINER (menuitem2_menu), mitem);
 	g_object_set_data (G_OBJECT (mitem), "test_type", GINT_TO_POINTER (TESTED_FORM));
 	g_signal_connect (mitem, "activate",
 			  G_CALLBACK (on_plugins_activate), top_nb);
 
-	mitem = gtk_menu_item_new_with_mnemonic ("Plugins data cell renderers in a grid");
+	mitem = gtk_menu_item_new_with_mnemonic ("Plugins: data cell renderers");
 	gtk_widget_show (mitem);
 	gtk_container_add (GTK_CONTAINER (menuitem2_menu), mitem);
 	g_object_set_data (G_OBJECT (mitem), "test_type", GINT_TO_POINTER (TESTED_GRID));
@@ -392,55 +398,57 @@ plugin_hash_foreach_func (const gchar *plugin_name, GdauiPlugin *plugin, GtkWidg
 
 static void create_plugin_nb (GtkWidget *vbox, GdauiPlugin *plugin);
 static void options_form_holder_changed_cb (GdauiBasicForm *form, GdaHolder *param, gboolean user_modif,
-					   GtkWidget *table);
+					   GtkWidget *grid);
 static GtkWidget *
 build_test_for_plugin_struct (GdauiPlugin *plugin)
 {
-	GtkWidget *label, *table;
+	GtkWidget *label, *grid;
 	GString *string;
 	gchar *str, *txt = "";
 
 	g_assert (plugin);
 
-	table = gtk_table_new (4, 2, FALSE);	
+	grid = gtk_grid_new ();
+	gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+	gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
 
 	/* explain top label */
 	if (plugin->plugin_file)
 		txt = "Plugin ";
 	str = g_strdup_printf ("<b>%sName:</b>", txt);
 	label = gtk_label_new ("");
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
 	gtk_label_set_markup (GTK_LABEL (label), str);
 	g_free (str);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
+	gtk_grid_attach (GTK_GRID (grid), label, 0, 0, 1, 1);
 	gtk_widget_show (label);
 
 	label = gtk_label_new (plugin->plugin_name);
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), label, 1, 2, 0, 1, GTK_FILL | GTK_EXPAND, 0, 0, 0);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
+	gtk_grid_attach (GTK_GRID (grid), label, 1, 0, 1, 1);
 	gtk_widget_show (label);
 
 	str = g_strdup_printf ("<b>%sDescription:</b>", txt);
 	label = gtk_label_new ("");
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
 	gtk_label_set_markup (GTK_LABEL (label), str);
 	g_free (str);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_SHRINK, 0, 0);
+	gtk_grid_attach (GTK_GRID (grid), label, 0, 1, 1, 1);
 	gtk_widget_show (label);
 
 	label = gtk_label_new (plugin->plugin_descr);
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), label, 1, 2, 1, 2, GTK_FILL | GTK_EXPAND, 0, 0, 0);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
+	gtk_grid_attach (GTK_GRID (grid), label, 1, 1, 1, 1);
 	gtk_widget_show (label);
 
 	/* options */
 	string = g_string_new ("");
 	g_string_append_printf (string, "<b>%sOptions:</b>", txt);
 	label = gtk_label_new ("");
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
 	gtk_label_set_markup (GTK_LABEL (label), string->str);
 	g_string_free (string, TRUE);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_grid_attach (GTK_GRID (grid), label, 0, 2, 1, 1);
 	gtk_widget_show (label);
 
 	if (plugin->options_xml_spec) {
@@ -453,55 +461,65 @@ build_test_for_plugin_struct (GdauiPlugin *plugin)
 			str = g_strdup_printf ("Cannot parse XML spec for %soptions: %s", txt,
 					       error && error->message ? error->message : "No detail");
 			label = gtk_label_new (str);
-			gtk_table_attach (GTK_TABLE (table), label, 1, 2, 2, 3, GTK_FILL, 0, 0, 0);
+			gtk_grid_attach (GTK_GRID (grid), label, 1, 2, 1, 1);
 			g_warning ("%s", str);
 			g_error_free (error);
 		}
 		else {
-			GtkWidget *form;
-
+			GtkWidget *form, *frame;
+			frame = gtk_frame_new ("");
 			form = gdaui_basic_form_new (plist);
 			g_object_unref (plist);
-			gtk_table_attach (GTK_TABLE (table), form, 1, 2, 2, 3, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
-			gtk_widget_show (form);
-			g_object_set_data (G_OBJECT (table), "options", form);
+			gtk_widget_set_margin_top (form, 10);
+			gtk_widget_set_margin_bottom (form, 10);
+			gtk_widget_set_margin_start (form, 10);
+			gtk_widget_set_margin_end (form, 10);
+			gtk_grid_attach (GTK_GRID (grid), frame, 1, 2, 1, 1);
+			gtk_container_add (GTK_CONTAINER (frame), form);
+			gtk_widget_show_all (frame);
+			g_object_set_data (G_OBJECT (grid), "options", form);
 			g_signal_connect (G_OBJECT (form), "holder-changed",
-					  G_CALLBACK (options_form_holder_changed_cb), table);
+					  G_CALLBACK (options_form_holder_changed_cb), grid);
 		}
 	}
 	else {
 		label = gtk_label_new ("None");
-		gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-		gtk_table_attach (GTK_TABLE (table), label, 1, 2, 2, 3, GTK_FILL | GTK_EXPAND, 0, 0, 0);
+		gtk_widget_set_halign (label, GTK_ALIGN_START);
+		gtk_grid_attach (GTK_GRID (grid), label, 1, 2, 1, 1);
 		gtk_widget_show (label);
 	}
 
 	/* notebook */
-	create_plugin_nb (table, plugin);
+	create_plugin_nb (grid, plugin);
 
-	gtk_widget_show (table);
+	gtk_widget_show (grid);
 
-	return table;
+	return grid;
 }
-static void plugin_nb_page_changed_cb (GtkNotebook *nb, GtkWidget *page, gint pageno, GtkWidget *table);
+static void plugin_nb_page_changed_cb (GtkNotebook *nb, GtkWidget *page, gint pageno, GtkWidget *grid);
 static void vbox_destroyed_cb (GtkWidget *widget, gpointer data);
 static void
-create_plugin_nb (GtkWidget *table, GdauiPlugin *plugin)
+create_plugin_nb (GtkWidget *grid, GdauiPlugin *plugin)
 {
 	GtkWidget *wid, *nb, *label;
 	gsize i;
 	GType type;
 	GdaDataHandler *dh;
 
+	label = gtk_label_new ("");
+	gtk_label_set_markup (GTK_LABEL (label), "<b>Handled GTypes:</b>");
+	gtk_grid_attach (GTK_GRID (grid), label, 0, 3, 1, 1);
+	gtk_widget_show (label);
+
 	nb = gtk_notebook_new ();
 	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (nb), GTK_POS_LEFT);
-	gtk_table_attach_defaults (GTK_TABLE (table), nb, 0, 2, 3, 4);
+	gtk_grid_attach (GTK_GRID (grid), nb, 1, 3, 1, 1);
 	gtk_widget_show (nb);
 	g_signal_connect (G_OBJECT (nb), "switch-page",
-			  G_CALLBACK (plugin_nb_page_changed_cb), table);
+			  G_CALLBACK (plugin_nb_page_changed_cb), grid);
 
-	g_object_set_data (G_OBJECT (table), "nb", nb);
-	g_object_set_data (G_OBJECT (table), "plugin", plugin);
+	g_object_set_data (G_OBJECT (grid), "nb", nb);
+	g_object_set_data (G_OBJECT (grid), "plugin", plugin);
 
 	if (plugin->nb_g_types > 0) {
 		for (i = 0; i < plugin->nb_g_types; i++) {
@@ -516,7 +534,7 @@ create_plugin_nb (GtkWidget *table, GdauiPlugin *plugin)
 				tdata->type = type;
 				tdata->dh = dh;
 				tdata->plugin = plugin;
-				tdata->options = g_object_get_data (G_OBJECT (table), "options");
+				tdata->options = g_object_get_data (G_OBJECT (grid), "options");
 				g_hash_table_insert (mainconf.tests_hash, wid, tdata);
 
 				gtk_widget_show (wid);
@@ -541,7 +559,7 @@ create_plugin_nb (GtkWidget *table, GdauiPlugin *plugin)
 				tdata->type = type;
 				tdata->dh = dh;
 				tdata->plugin = plugin;
-				tdata->options = g_object_get_data (G_OBJECT (table), "options");
+				tdata->options = g_object_get_data (G_OBJECT (grid), "options");
 				g_hash_table_insert (mainconf.tests_hash, wid, tdata);
 
 				gtk_widget_show (wid);
@@ -566,7 +584,7 @@ static GtkWidget *build_grid_test_for_gtype (GdaDataHandler *dh, GType type, con
 
 static void
 plugin_nb_page_changed_cb (GtkNotebook *nb, G_GNUC_UNUSED GtkWidget *page, gint pageno,
-			   G_GNUC_UNUSED GtkWidget *table)
+			   G_GNUC_UNUSED GtkWidget *grid)
 {
 	GtkWidget *vbox;
 	TestWidgetData *tdata;
@@ -591,7 +609,7 @@ build_test_widget (TestWidgetData *tdata)
 			
 			plist = gdaui_basic_form_get_data_set (GDAUI_BASIC_FORM (tdata->options));
 			g_assert (plist);
-			for (list = plist->holders; list; list = list->next) {
+			for (list = gda_set_get_holders (plist); list; list = list->next) {
 				GdaHolder *param = GDA_HOLDER (list->data);
 				const GValue *value;
 				value = gda_holder_get_value (param);
@@ -647,26 +665,26 @@ build_test_widget (TestWidgetData *tdata)
 
 static void
 options_form_holder_changed_cb (G_GNUC_UNUSED GdauiBasicForm *form, G_GNUC_UNUSED GdaHolder *param,
-				G_GNUC_UNUSED gboolean user_modif, GtkWidget *table)
+				G_GNUC_UNUSED gboolean user_modif, GtkWidget *grid)
 {
 	GtkWidget *nb;
 	GdauiPlugin *plugin;
 	
-	nb = g_object_get_data (G_OBJECT (table), "nb");
-	plugin = g_object_get_data (G_OBJECT (table), "plugin");
+	nb = g_object_get_data (G_OBJECT (grid), "nb");
+	plugin = g_object_get_data (G_OBJECT (grid), "plugin");
 	g_assert (plugin);
 	
 	if (nb)
 		gtk_widget_destroy (nb);
-	g_object_set_data (G_OBJECT (table), "nb", NULL);
-	create_plugin_nb (table, plugin);
+	g_object_set_data (G_OBJECT (grid), "nb", NULL);
+	create_plugin_nb (grid, plugin);
 }
 
 
 /*
  * Basic test (individual GdauiDataEntry widget being tested)
  */
-void entry_contents_modified (GtkWidget *entry, gpointer data);
+void entry_contents_or_attrs_changed (GtkWidget *entry, gpointer data);
 void null_toggled_cb (GtkToggleButton *button, GtkWidget *entry);
 void default_toggled_cb (GtkToggleButton *button, GtkWidget *entry);
 void actions_toggled_cb (GtkToggleButton *button, GtkWidget *entry);
@@ -676,16 +694,15 @@ void default_clicked_cb (GtkButton *button, GtkWidget *entry);
 static GtkWidget *
 build_basic_test_for_gtype (GdaDataHandler *dh, GType type, const gchar *plugin_name)
 {
-	GtkWidget *wid, *label, *table, *button, *bbox, *entry, *vbox;
+	GtkWidget *wid, *label, *grid, *button, *bbox, *entry, *vbox;
 	GString *string;
-	gboolean expand;
 	
 	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 
-	table = gtk_table_new (7, 3, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (table), 5);
-	gtk_table_set_row_spacings (GTK_TABLE (table), 5);
-	gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);
+	grid = gtk_grid_new ();
+	gtk_container_set_border_width (GTK_CONTAINER (grid), 5);
+	gtk_grid_set_row_spacing (GTK_GRID (grid), 5);
+	gtk_box_pack_start (GTK_BOX (vbox), grid, TRUE, TRUE, 0);
 
 	/* explain top label */
 	string = g_string_new ("");
@@ -696,52 +713,61 @@ build_basic_test_for_gtype (GdaDataHandler *dh, GType type, const gchar *plugin_
 		g_string_append_printf (string, "<b>Data handler description:</b> %s\n",
 					"No GdaDataHandler available for this type");
 	label = gtk_label_new ("");
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
 	gtk_label_set_markup (GTK_LABEL (label), string->str);
 	g_string_free (string, TRUE);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 2, 0, 1, GTK_FILL | GTK_EXPAND, 0, 0, 0);
+	gtk_grid_attach (GTK_GRID (grid), label, 0, 0, 2, 1);
 	gtk_widget_show (label);
 
 	/* widget being tested */
-	wid = GTK_WIDGET (gdaui_new_data_entry (type, plugin_name));
-	g_object_set (G_OBJECT (wid), "handler", dh, NULL);
+	label = gtk_label_new ("Data entry widget: ");
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
+	gtk_grid_attach (GTK_GRID (grid), label, 0, 1, 1, 1);
+	gtk_widget_show (label);
 
-	expand = gdaui_data_entry_can_expand (GDAUI_DATA_ENTRY (wid), FALSE);
-	if (expand)
-		gtk_table_attach_defaults (GTK_TABLE (table), wid, 0, 3, 1, 2);
-	else
-		gtk_table_attach (GTK_TABLE (table), wid, 0, 3, 1, 2, GTK_FILL | GTK_EXPAND, 0, 0, 0);
-	gtk_widget_show (wid);
+	GtkWidget *frame;
+	frame = gtk_frame_new ("");
+	gtk_grid_attach (GTK_GRID (grid), frame, 1, 1, 2, 1);
+	wid = GTK_WIDGET (gdaui_new_data_entry (type, plugin_name));
+	gtk_widget_set_margin_top (wid, 10);
+	gtk_widget_set_margin_bottom (wid, 10);
+	gtk_widget_set_margin_start (wid, 10);
+	gtk_widget_set_margin_end (wid, 10);
+	g_object_set (G_OBJECT (wid), "handler", dh, NULL);
+	gtk_container_add (GTK_CONTAINER (frame), wid);
+	gtk_widget_show_all (frame);
 
 	/* Other widgets */
 	label = gtk_label_new ("Current flags: ");
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, 0, 0, 0);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
+	gtk_grid_attach (GTK_GRID (grid), label, 0, 2, 1, 1);
 	gtk_widget_show (label);
 
 	label = gtk_label_new ("--");
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), label, 1, 3, 2, 3, 0, 0, 0, 0);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
+	gtk_grid_attach (GTK_GRID (grid), label, 1, 2, 2, 1);
 	g_object_set_data (G_OBJECT (wid), "flags", label);
 	gtk_widget_show (label);
 
 	label = gtk_label_new ("Current value: ");
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL, 0, 0, 0);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
+	gtk_grid_attach (GTK_GRID (grid), label, 0, 3, 1, 1);
 	gtk_widget_show (label);
 	
 	label = gtk_label_new ("");
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), label, 1, 3, 3, 4, 0, 0, 0, 0);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
+	gtk_grid_attach (GTK_GRID (grid), label, 1, 3, 1, 1);
 	g_object_set_data (G_OBJECT (wid), "value", label);
 	g_signal_connect (G_OBJECT (wid), "contents-modified",
-			  G_CALLBACK (entry_contents_modified), NULL);
+			  G_CALLBACK (entry_contents_or_attrs_changed), NULL);
+	g_signal_connect (G_OBJECT (wid), "status-changed",
+			  G_CALLBACK (entry_contents_or_attrs_changed), NULL);
+	entry_contents_or_attrs_changed (wid, GTK_LABEL (label));
 	gtk_widget_show (label);
 
-	entry_contents_modified (wid, GTK_LABEL (label));
-
 	bbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
-	gtk_table_attach (GTK_TABLE (table), bbox, 0, 3, 4, 5, 0, 0, 0, 0);
+	gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_START);
+	gtk_grid_attach (GTK_GRID (grid), bbox, 0, 4, 2, 1);
 		
 	button = gtk_toggle_button_new_with_label ("NULL ok");
 	g_signal_connect (G_OBJECT (button), "toggled",
@@ -759,34 +785,28 @@ build_basic_test_for_gtype (GdaDataHandler *dh, GType type, const gchar *plugin_
 				      gdaui_data_entry_get_attributes (GDAUI_DATA_ENTRY (wid)) & 
 				      GDA_VALUE_ATTR_CAN_BE_DEFAULT);
 
-	button = gtk_toggle_button_new_with_label ("Actions?");
-	g_signal_connect (G_OBJECT (button), "toggled",
-			  G_CALLBACK (actions_toggled_cb), wid);
-	gtk_container_add (GTK_CONTAINER (bbox), button);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
-				      gdaui_data_entry_get_attributes (GDAUI_DATA_ENTRY (wid)) & 
-				      GDA_VALUE_ATTR_ACTIONS_SHOWN);
-
 	button = gtk_toggle_button_new_with_label ("Editable?");
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (editable_toggled_cb), wid);
 	gtk_container_add (GTK_CONTAINER (bbox), button);
-			      
+
+	entry = gtk_entry_new ();
+	gtk_entry_set_placeholder_text (GTK_ENTRY (entry), "Grab focus here!");
+	gtk_widget_set_can_focus (entry, TRUE);
+	gtk_container_add (GTK_CONTAINER (bbox), entry);
+
 	gtk_widget_show_all (bbox);
 
 	/* to set the original value */
 	entry = GTK_WIDGET (gdaui_new_data_entry (type, plugin_name));
 	g_object_set (G_OBJECT (entry), "handler", dh, NULL);
 
-	if (expand)
-		gtk_table_attach_defaults (GTK_TABLE (table), entry, 0, 2, 5, 6);
-	else
-		gtk_table_attach (GTK_TABLE (table), entry, 0, 2, 5, 6, GTK_FILL | GTK_EXPAND, 0, 0, 0);
+	gtk_grid_attach (GTK_GRID (grid), entry, 1, 5, 1, 1);
 	gtk_widget_show (entry);
 	button = gtk_button_new_with_label ("Set as original");
 	g_object_set_data (G_OBJECT (button), "entry", entry);
-	gtk_table_attach (GTK_TABLE (table), button, 2, 3, 5, 6, 0, 0, 0, 0);
+	gtk_grid_attach (GTK_GRID (grid), button, 0, 5, 1, 1);
 	gtk_widget_show (button);
 	g_signal_connect (G_OBJECT (button), "clicked",
 			  G_CALLBACK (orig_clicked_cb), wid);
@@ -795,26 +815,16 @@ build_basic_test_for_gtype (GdaDataHandler *dh, GType type, const gchar *plugin_
 	entry = GTK_WIDGET (gdaui_new_data_entry (type, plugin_name));
 	g_object_set (G_OBJECT (entry), "handler", dh, NULL);
 
-	if (expand)
-		gtk_table_attach_defaults (GTK_TABLE (table), entry, 0, 2, 6, 7);
-	else
-		gtk_table_attach (GTK_TABLE (table), entry, 0, 2, 6, 7, GTK_FILL | GTK_EXPAND, 0, 0, 0);
+	gtk_grid_attach (GTK_GRID (grid), entry, 1, 6, 1, 1);
 	gtk_widget_show (entry);
 	button = gtk_button_new_with_label ("Set as default");
 	g_object_set_data (G_OBJECT (button), "entry", entry);
-	gtk_table_attach (GTK_TABLE (table), button, 2, 3, 6, 7, 0, 0, 0, 0);
+	gtk_grid_attach (GTK_GRID (grid), button, 0, 6, 1, 1);
 	gtk_widget_show (button);
 	g_signal_connect (G_OBJECT (button), "clicked",
 			  G_CALLBACK (default_clicked_cb), wid);
 
-	gtk_widget_show (table);
-
-	if (!expand) {
-		label = gtk_label_new ("");
-		gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
-		gtk_widget_show (label);
-	}
-
+	gtk_widget_show (grid);
 	gtk_widget_show (vbox);
 
 	return vbox;
@@ -848,20 +858,6 @@ default_toggled_cb (GtkToggleButton *button, GtkWidget *entry)
 				      GDA_VALUE_ATTR_CAN_BE_DEFAULT);
 }
 
-void
-actions_toggled_cb (GtkToggleButton *button, GtkWidget *entry)
-{
-	guint action = 0;
-	if (gtk_toggle_button_get_active (button))
-		action = GDA_VALUE_ATTR_ACTIONS_SHOWN;
-
-	gdaui_data_entry_set_attributes (GDAUI_DATA_ENTRY (entry),
-				      action, GDA_VALUE_ATTR_ACTIONS_SHOWN);
-	gtk_toggle_button_set_active (button,
-				      gdaui_data_entry_get_attributes (GDAUI_DATA_ENTRY (entry)) & 
-				      GDA_VALUE_ATTR_ACTIONS_SHOWN);
-}
-
 void
 editable_toggled_cb (GtkToggleButton *button, GtkWidget *entry)
 {
@@ -872,27 +868,51 @@ editable_toggled_cb (GtkToggleButton *button, GtkWidget *entry)
 }
 
 void 
-entry_contents_modified (GtkWidget *entry, G_GNUC_UNUSED gpointer data)
+entry_contents_or_attrs_changed (GtkWidget *entry, G_GNUC_UNUSED gpointer data)
 {
 	guint attrs;
 	GtkLabel *label;
-	GString *str = g_string_new ("");
+	GString *str = NULL;
 	GValue *value;
 	GdaDataHandler *dh;
 
 	/* flags */
 	label = g_object_get_data (G_OBJECT (entry), "flags");
 	attrs = gdaui_data_entry_get_attributes (GDAUI_DATA_ENTRY (entry));
-	if (attrs & GDA_VALUE_ATTR_IS_NULL)
-		g_string_append (str, "N ");
-	if (attrs & GDA_VALUE_ATTR_IS_DEFAULT)
-		g_string_append (str, "D ");
-	if (!(attrs & GDA_VALUE_ATTR_IS_UNCHANGED))
-		g_string_append (str, "M ");
-	if (attrs & GDA_VALUE_ATTR_DATA_NON_VALID)
-		g_string_append (str, "U ");
+	if (attrs & GDA_VALUE_ATTR_IS_NULL) {
+		if (!str) str = g_string_new (""); else	g_string_append (str, ", ");
+		g_string_append (str, "IS_NULL");
+	}
+	if (attrs & GDA_VALUE_ATTR_CAN_BE_NULL) {
+		if (!str) str = g_string_new (""); else	g_string_append (str, ", ");
+		g_string_append (str, "CAN_BE_NULL");
+	}
+	if (attrs & GDA_VALUE_ATTR_IS_DEFAULT) {
+		if (!str) str = g_string_new (""); else	g_string_append (str, ", ");
+		g_string_append (str, "IS_DEFAULT");
+	}
+	if (attrs & GDA_VALUE_ATTR_CAN_BE_DEFAULT) {
+		if (!str) str = g_string_new (""); else	g_string_append (str, ", ");
+		g_string_append (str, "CAN_BE_DEFAULT");
+	}
+	if (attrs & GDA_VALUE_ATTR_IS_UNCHANGED) {
+		if (!str) str = g_string_new (""); else	g_string_append (str, ", ");
+		g_string_append (str, "IS_UNCHANGED");
+	}
+	if (attrs & GDA_VALUE_ATTR_DATA_NON_VALID) {
+		if (!str) str = g_string_new (""); else	g_string_append (str, ", ");
+		g_string_append (str, "NON_VALID");
+	}
+	if (attrs & GDA_VALUE_ATTR_READ_ONLY) {
+		if (!str) str = g_string_new (""); else	g_string_append (str, ", ");
+		g_string_append (str, "READ_ONLY");
+	}
+	if (attrs & GDA_VALUE_ATTR_HAS_VALUE_ORIG) {
+		if (!str) str = g_string_new (""); else	g_string_append (str, ", ");
+		g_string_append (str, "HAS_VALUE_ORIG");
+	}
 
-	gtk_label_set_text (label, str->str);
+	gtk_label_set_text (label, str ? str->str : "");
 	g_string_free (str, TRUE);
 
 	/* value */
@@ -904,14 +924,13 @@ entry_contents_modified (GtkWidget *entry, G_GNUC_UNUSED gpointer data)
 		if (G_VALUE_TYPE (value) == GDA_TYPE_BINARY) {
 			const GdaBinary *bin;
 			bin = gda_value_get_binary (value);
-			strval = g_strdup_printf ("Binary data, size=%ld", bin->binary_length);
+			strval = g_strdup_printf ("Binary data, size=%ld", gda_binary_get_size (bin));
 		}
 		else if (G_VALUE_TYPE (value) == GDA_TYPE_BLOB) {
 			const GdaBlob *blob;
 			blob = gda_value_get_blob (value);
-			strval = g_strdup_printf ("Blob data, size=%ld", ((GdaBinary*) blob)->binary_length);
+			strval = g_strdup_printf ("Blob data, size=%ld", gda_binary_get_size (gda_blob_get_binary (blob)));
 		}
-
 		else
 			strval = gda_data_handler_get_sql_from_value (dh, value);
 		gda_value_free (value);
@@ -941,7 +960,6 @@ orig_clicked_cb (GtkButton *button, GtkWidget *entry)
 	
 	gdaui_data_entry_set_reference_value (GDAUI_DATA_ENTRY (entry), value);
 	gda_value_free (value);
-	entry_contents_modified (entry, NULL);
 }
 
 void
@@ -970,8 +988,7 @@ build_form_test_for_gtype (G_GNUC_UNUSED GdaDataHandler *dh, GType type, const g
 
 		wid = gdaui_form_new (model);
 		plist = GDA_SET (gdaui_data_selector_get_data_set (GDAUI_DATA_SELECTOR (wid)));
-		gdaui_data_proxy_column_show_actions (GDAUI_DATA_PROXY (wid), -1, TRUE);
-		param = plist->holders->data;
+		param = gda_set_get_holders (plist)->data;
 
 		value = gda_value_new_from_string (plugin_name, G_TYPE_STRING);
 		gda_holder_set_attribute_static (param, GDAUI_ATTRIBUTE_PLUGIN, value);
@@ -1003,8 +1020,7 @@ build_grid_test_for_gtype (G_GNUC_UNUSED GdaDataHandler *dh, GType type, const g
 		
 		wid = gdaui_grid_new (model);
 		plist = GDA_SET (gdaui_data_selector_get_data_set (GDAUI_DATA_SELECTOR (wid)));
-		gdaui_data_proxy_column_show_actions (GDAUI_DATA_PROXY (wid), -1, TRUE);
-		param = plist->holders->data;
+		param = gda_set_get_holders (plist)->data;
 
 		value = gda_value_new_from_string (plugin_name, G_TYPE_STRING);
 		gda_holder_set_attribute_static (param, GDAUI_ATTRIBUTE_PLUGIN, value);
@@ -1197,13 +1213,13 @@ fill_tested_models (void)
 	gda_data_model_set_column_title (model, 0, "CHAR value");
 
 	row = gda_data_model_append_row (model, NULL);
-        g_value_set_char ((value = gda_value_new (G_TYPE_CHAR)), -128);
+        g_value_set_schar ((value = gda_value_new (G_TYPE_CHAR)), (gint8) -128);
         gda_data_model_set_value_at (model, 0, row, value, NULL); gda_value_free (value);
 	row = gda_data_model_append_row (model, NULL);
-        g_value_set_char ((value = gda_value_new (G_TYPE_CHAR)), 0);
+        g_value_set_schar ((value = gda_value_new (G_TYPE_CHAR)), (gint8) 0);
         gda_data_model_set_value_at (model, 0, row, value, NULL); gda_value_free (value);
 	row = gda_data_model_append_row (model, NULL);
-        g_value_set_char ((value = gda_value_new (G_TYPE_CHAR)), 127);
+        g_value_set_schar ((value = gda_value_new (G_TYPE_CHAR)), (gint8) 127);
         gda_data_model_set_value_at (model, 0, row, value, NULL); gda_value_free (value);
 
 	/* G_TYPE_UCHAR */
