@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -22,7 +22,7 @@
 #include <libgda/libgda.h>
 #include <sql-parser/gda-sql-parser.h>
 #include <sql-parser/gda-sql-statement.h>
-#include <tests/gda-ddl-creator.h>
+#include <tests/raw-ddl-creator.h>
 #include <glib/gstdio.h>
 #include "../test-cnc-utils.h"
 #include "../data-model-errors.h"
@@ -336,7 +336,7 @@ test2 (GdaConnection *cnc)
 	GdaDataModelIter *iter = NULL;
 	GdaStatement *stmt;
 	virtual_provider = gda_vprovider_data_model_new ();
-	vcnc = gda_virtual_connection_open (virtual_provider, &lerror);
+	vcnc = gda_virtual_connection_open (virtual_provider, GDA_CONNECTION_OPTIONS_NONE, &lerror);
 	if (!vcnc) {
 #ifdef CHECK_EXTRA_INFO
 		g_print ("Virtual ERROR: %s\n", lerror && lerror->message ? lerror->message : "No detail");
