@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2008 - 2011 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2008 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2008 - 2013 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -23,7 +23,7 @@
 #include <libgda/libgda.h>
 #include <sql-parser/gda-sql-parser.h>
 #include <sql-parser/gda-sql-statement.h>
-#include <tests/gda-ddl-creator.h>
+#include <tests/raw-ddl-creator.h>
 #include <glib/gstdio.h>
 #include "../test-cnc-utils.h"
 #include "../test-errors.h"
@@ -133,7 +133,7 @@ main (int argc, char **argv)
 static GdaConnection *
 setup_connection (void)
 {
-	GdaDDLCreator *ddl;
+	RawDDLCreator *ddl;
         GError *error = NULL;
         GdaConnection *cnc;
         gchar *str;
@@ -147,18 +147,18 @@ setup_connection (void)
         }
 
 	/* Setup structure */
-        ddl = gda_ddl_creator_new ();
-        gda_ddl_creator_set_connection (ddl, cnc);
+        ddl = raw_ddl_creator_new ();
+        raw_ddl_creator_set_connection (ddl, cnc);
 	str = g_build_filename (CHECK_FILES, "tests", "data-models", "pmodel_dbstruct.xml", NULL);
-        if (!gda_ddl_creator_set_dest_from_file (ddl, str, &error)) {
-                g_print ("Error creating GdaDDLCreator: %s\n", error && error->message ? error->message : "No detail");
+        if (!raw_ddl_creator_set_dest_from_file (ddl, str, &error)) {
+                g_print ("Error creating RawDDLCreator: %s\n", error && error->message ? error->message : "No detail");
                 g_error_free (error);
                 exit (EXIT_FAILURE);
         }
 	g_free (str);
 
 #ifdef SHOW_SQL
-        str = gda_ddl_creator_get_sql (ddl, &error);
+        str = raw_ddl_creator_get_sql (ddl, &error);
         if (!str) {
                 g_print ("Error getting SQL: %s\n", error && error->message ? error->message : "No detail");
                 g_error_free (error);
@@ -168,7 +168,7 @@ setup_connection (void)
         g_free (str);
 #endif
 
-        if (!gda_ddl_creator_execute (ddl, &error)) {
+        if (!raw_ddl_creator_execute (ddl, &error)) {
                 g_print ("Error creating database objects: %s\n", error && error->message ? error->message : "No detail");
                 g_error_free (error);
                 exit (EXIT_FAILURE);
@@ -450,8 +450,8 @@ test3 (GdaConnection *cnc)
 	clear_signals ();
 
 	/****/
-	gda_value_set_from_string ((value = gda_value_new (GDA_TYPE_TIMESTAMP)), 
-				   "2009-11-30 11:22:33+0", GDA_TYPE_TIMESTAMP);
+	gda_value_set_from_string ((value = gda_value_new (G_TYPE_DATE_TIME)),
+				   "2009-11-30T11:22:33+00", G_TYPE_DATE_TIME);
 	if (! check_set_value_at (model, 2, 1, value, cnc, stmt, NULL)) {
 		nfailed ++;
 		goto out;
@@ -878,8 +878,8 @@ test7 (GdaConnection *cnc)
 		goto out;
 	}
 
-	gda_value_set_from_string ((value = gda_value_new (GDA_TYPE_TIMESTAMP)), 
-				   "2004-03-22 11:22:44+0", GDA_TYPE_TIMESTAMP);
+	gda_value_set_from_string ((value = gda_value_new (G_TYPE_DATE_TIME)),
+				   "2004-03-22T11:22:44+00", G_TYPE_DATE_TIME);
 	if (! check_set_value_at (model, 2, 1, value, cnc, stmt, params)) {
 		nfailed ++;
 		goto out;
@@ -1452,8 +1452,8 @@ test13 (GdaConnection *cnc)
 	clear_signals ();
 
 	/****/
-	gda_value_set_from_string ((value = gda_value_new (GDA_TYPE_TIMESTAMP)), 
-				   "2009-11-30 11:22:33+0", GDA_TYPE_TIMESTAMP);
+	gda_value_set_from_string ((value = gda_value_new (G_TYPE_DATE_TIME)),
+				   "2009-11-30T11:22:33+00", G_TYPE_DATE_TIME);
 	if (! check_set_value_at (model, 1, 1, value, cnc, stmt, NULL)) {
 		nfailed ++;
 		goto out;
@@ -1547,8 +1547,8 @@ test14 (GdaConnection *cnc)
 	clear_signals ();
 
 	/****/
-	gda_value_set_from_string ((value = gda_value_new (GDA_TYPE_TIMESTAMP)), 
-				   "2019-11-30 11:22:33+0", GDA_TYPE_TIMESTAMP);
+	gda_value_set_from_string ((value = gda_value_new (G_TYPE_DATE_TIME)),
+				   "2019-11-30T11:22:33+00", G_TYPE_DATE_TIME);
 	if (! check_set_value_at (model, 1, 1, value, cnc, stmt, NULL)) {
 		nfailed ++;
 		goto out;
@@ -1768,7 +1768,7 @@ test16 (GdaConnection *cnc)
         g_object_unref (rerun);
 
 	/**/
-	gda_holder_force_invalid (GDA_HOLDER (params->holders->data));
+	gda_holder_force_invalid (GDA_HOLDER (gda_set_get_holders (params)->data));
 	check_expected_signal (model, 'R', -1);
 	dump_data_model (model);
 
@@ -1809,7 +1809,7 @@ test17 (GdaConnection *cnc)
 	gint i, nullcol = -1;
 	GdaColumn *column;
 	GSList *list;
-	for (i = 0, list = ((GdaSet*) iter)->holders;
+	for (i = 0, list = gda_set_get_holders ((GdaSet*) iter);
 	     list;
 	     i++, list = list->next) {
 		if (gda_holder_get_g_type ((GdaHolder *) list->data) == GDA_TYPE_NULL) 
@@ -1845,13 +1845,13 @@ test17 (GdaConnection *cnc)
         }
 
 	column = gda_data_model_describe_column (model, nullcol);
-	if (gda_holder_get_g_type ((GdaHolder *) g_slist_nth_data (((GdaSet*) iter)->holders, nullcol)) !=
+	if (gda_holder_get_g_type ((GdaHolder *) g_slist_nth_data (gda_set_get_holders ((GdaSet*) iter), nullcol)) !=
 	    gda_column_get_g_type (column)) {
 		nfailed++;
 #ifdef CHECK_EXTRA_INFO
                 g_print ("Iter's GdaHolder for column %d reports the '%s' type when it should be '%s'",
 			 nullcol, 
-			 g_type_name (gda_holder_get_g_type ((GdaHolder *) g_slist_nth_data (((GdaSet*) iter)->holders, 
+			 g_type_name (gda_holder_get_g_type ((GdaHolder *) g_slist_nth_data (gda_set_get_holders ((GdaSet*) iter),
 											     nullcol))),
 			 g_type_name (gda_column_get_g_type (column)));
 #endif
