@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2011 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2011 - 2014 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2017 Daniel Espinosa <esodan@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -55,7 +56,7 @@ main (int argc, char *argv[])
 	gda_init ();
 
 	provider = gda_vprovider_hub_new ();
-	virtual = gda_virtual_connection_open (provider, NULL);
+	virtual = gda_virtual_connection_open (provider, GDA_CONNECTION_OPTIONS_NONE, NULL);
 	g_assert (virtual);
 
 	/* load CSV data models */
@@ -90,15 +91,22 @@ main (int argc, char *argv[])
 
 	g_print ("*** Copying data into 'countries' virtual table...\n");
 	assert_run_sql_non_select (virtual, "INSERT INTO out.countries SELECT * FROM country", NULL);
-#if HAVE_SQLITE
+
 	check_simultanous_select_random (virtual);
-#endif
 	check_simultanous_select_forward (virtual);
 	check_threads_select_random (virtual);
 	check_date (virtual);
 
-        gda_connection_close (virtual);
-        gda_connection_close (out_cnc);
+        if (! gda_connection_close (virtual, &error)) {
+		g_print ("gda_connection_close(virtual) error: %s\n",
+                         error && error->message ? error->message : "No detail");
+                exit (1);
+	}
+        if (! gda_connection_close (out_cnc, &error)) {
+		g_print ("gda_connection_close(out_cnc) error: %s\n",
+                         error && error->message ? error->message : "No detail");
+                exit (1);
+	}
 
 	g_print ("All Ok\n");
         return 0;
@@ -110,15 +118,20 @@ open_destination_connection (void)
         /* create connection */
         GdaConnection *cnc;
         GError *error = NULL;
-        cnc = gda_connection_open_from_string ("SQLite", "DB_DIR=.;DB_NAME=vcnc",
-                                               NULL,
-                                               GDA_CONNECTION_OPTIONS_NONE,
-                                               &error);
+        cnc = gda_connection_new_from_string ("SQLite", "DB_DIR=.;DB_NAME=vcnc",
+					      NULL,
+					      GDA_CONNECTION_OPTIONS_NONE,
+					      &error);
         if (!cnc) {
-                g_print ("Could not open connection to local SQLite database: %s\n",
+                g_print ("Could not create connection to local SQLite database: %s\n",
                          error && error->message ? error->message : "No detail");
                 exit (1);
         }
+	if (! gda_connection_open (cnc, &error)) {
+		 g_print ("gda_connection_open() error: %s\n",
+                         error && error->message ? error->message : "No detail");
+                exit (1);
+	}
 
         /* table "cities" */
         assert_run_sql_non_select (cnc, "DROP table IF EXISTS cities", NULL);
@@ -507,7 +520,7 @@ test_multiple_threads (GThreadFunc func, GdaConnection *virtual)
 #ifdef DEBUG_PRINT
 		g_print ("Running thread %d\n", d->th_id);
 #endif
-		d->thread = g_thread_create (func, d, TRUE, NULL);
+		d->thread = g_thread_new ("For_test_multiple_threads", func, d);
 	}
 
 	for (i = 0; i < NTHREADS; i++) {
@@ -551,8 +564,8 @@ threads_select_random_start_thread (ThData *data)
 	g_free (str);
 
 	g_object_unref (iter);
-	g_object_unref (model);
 	g_object_unref (ref);
+	g_object_unref (model);
 	g_print ("thread %p finished\n", self);
 
 	return NULL;
@@ -569,17 +582,20 @@ check_date (GdaConnection *virtual)
 {
 	g_print ("*** insert dates into 'misc' table...\n");
 	GdaSet *set;
-	GdaTimestamp ts = {2011, 01, 31, 12, 34, 56, 0, 0};
-	GdaTime atime = {13, 45, 59, 0, 0};
+	GDateTime *ts = g_date_time_new_utc (2011, 01, 31, 12, 34, 56.0);
+  gchar* tss = g_date_time_format ((GDateTime*) ts, "%FT%T");
+  g_print ("Created Timestamp: %s\n", tss);
+  g_free (tss);
+	GdaTime* atime = gda_time_new_from_values (13, 45, 59, 0, 0);
 	GDate *adate;
 	GdaDataModel *model;
 	GError *error = NULL;
 
 	adate = g_date_new_dmy (23, G_DATE_FEBRUARY, 2010);
 	set = gda_set_new_inline (3,
-				  "ts", GDA_TYPE_TIMESTAMP, &ts,
+				  "ts", G_TYPE_DATE_TIME, ts,
 				  "adate", G_TYPE_DATE, adate,
-				  "atime", GDA_TYPE_TIME, &atime);
+				  "atime", GDA_TYPE_TIME, atime);
 	g_date_free (adate);
 
 	assert_run_sql_non_select (virtual, "INSERT INTO out.misc VALUES (##ts::timestamp, "
@@ -629,4 +645,5 @@ check_date (GdaConnection *virtual)
 	}
 
 	g_object_unref (set);
+	g_date_time_unref (ts);
 }
