@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2007 - 2013 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2012 Daniel Espinosa <despinosa@src.gnome.org>
+ * Copyright (C) 2012-2017 Daniel Espinosa <esodan@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -717,21 +717,12 @@ prov_test_common_check_data_select (void)
 static gboolean
 timestamp_equal (const GValue *cv1, const GValue *cv2)
 {
-	g_assert (G_VALUE_TYPE (cv1) == GDA_TYPE_TIMESTAMP);
-	g_assert (G_VALUE_TYPE (cv2) == GDA_TYPE_TIMESTAMP);
-	const GdaTimestamp *ts1, *ts2;
-	ts1 = gda_value_get_timestamp (cv1);
-	ts2 = gda_value_get_timestamp (cv2);
-	if (ts1->timezone == ts2->timezone)
-		return gda_value_differ (cv1, cv2) ? FALSE : TRUE;
-
-	GdaTimestamp *ts;
-	ts = gda_timestamp_copy ((GdaTimestamp*) ts1);
-	gda_timestamp_change_timezone (ts, ts2->timezone);
-	gboolean res;
-	res = memcmp (ts2, ts, sizeof (GdaTimestamp)) ? FALSE : TRUE;
-	gda_timestamp_free (ts);
-	return res;
+	g_assert (G_VALUE_TYPE (cv1) == G_TYPE_DATE_TIME);
+	g_assert (G_VALUE_TYPE (cv2) == G_TYPE_DATE_TIME);
+	GDateTime *ts1, *ts2;
+	ts1 = g_value_get_boxed (cv1);
+	ts2 = g_value_get_boxed (cv2);
+	return g_date_time_compare (ts1, ts2) == 0 ? TRUE : FALSE;
 }
 
 int
@@ -753,13 +744,13 @@ prov_test_common_check_timestamp (void)
 		parser = gda_sql_parser_new ();
 
 	GValue *tso;
-	tso = gda_value_new_timestamp_from_timet (time (NULL));
+	tso = gda_value_new_date_time_from_timet (time (NULL));
 
 	/* insert timestamp */
 	stmt = gda_sql_parser_parse_string (parser, "INSERT INTO tstest (ts) VALUES (##ts::timestamp)", NULL, &error);
 	if (!stmt ||
 	    ! gda_statement_get_parameters (stmt, &params, &error) ||
-	    ! gda_set_set_holder_value (params, &error, "ts", gda_value_get_timestamp (tso)) ||
+	    ! gda_set_set_holder_value (params, &error, "ts", g_value_get_boxed (tso)) ||
 	    (gda_connection_statement_execute_non_select (cnc, stmt, params, NULL, &error) == -1)) {
 		number_failed ++;
 		goto out;
@@ -788,7 +779,7 @@ prov_test_common_check_timestamp (void)
 	}
 
 	const GValue *cvalue;
-	cvalue = gda_data_model_get_typed_value_at (model, 0, 0, GDA_TYPE_TIMESTAMP, FALSE, &error);
+	cvalue = gda_data_model_get_typed_value_at (model, 0, 0, G_TYPE_DATE_TIME, FALSE, &error);
 	if (!cvalue) {
 		number_failed ++;
 		goto out;
@@ -808,7 +799,7 @@ prov_test_common_check_timestamp (void)
 	/* check that data handler is correctly configured: compare the same timestamp rendered by a data handler for
 	 * timestamps with the date rendered as a string by the server (by appending a string to the timestamp field) */
 	GdaDataHandler *dh;
-	dh = gda_server_provider_get_data_handler_g_type (gda_connection_get_provider (cnc), cnc, GDA_TYPE_TIMESTAMP);
+	dh = gda_server_provider_get_data_handler_g_type (gda_connection_get_provider (cnc), cnc, G_TYPE_DATE_TIME);
 	gchar *str;
 	str = gda_data_handler_get_str_from_value (dh, cvalue);
 	g_object_unref (model);
@@ -1027,3 +1018,96 @@ out:
 
 	return number_failed;
 }
+
+int
+prov_test_common_check_bigint (void)
+{
+	GdaSqlParser *parser = NULL;
+	GdaStatement *stmt = NULL;
+	GdaSet *params = NULL;
+	GError *error = NULL;
+	int number_failed = 0;
+	GdaDataModel *model = NULL;
+
+#ifdef CHECK_EXTRA_INFO
+	g_print ("\n============= %s() =============\n", __FUNCTION__);
+#endif
+
+	parser = gda_connection_create_parser (cnc);
+	if (!parser)
+		parser = gda_sql_parser_new ();
+
+	GValue *tso;
+	tso = gda_value_new (G_TYPE_INT64);
+	g_value_set_int64 (tso, 4294967296);
+
+	/* insert date */
+	stmt = gda_sql_parser_parse_string (parser, "INSERT INTO bigint (thebigint) VALUES (##thebigint::gint64)", NULL, &error);
+	if (!stmt ||
+	    ! gda_statement_get_parameters (stmt, &params, &error) ||
+	    ! gda_set_set_holder_value (params, &error, "thebigint", g_value_get_int64 (tso)) ||
+	    (gda_connection_statement_execute_non_select (cnc, stmt, params, NULL, &error) == -1)) {
+		number_failed ++;
+		goto out;
+	}
+
+	g_print ("Inserted int %s\n", gda_value_stringify (tso));
+
+	/* retreive date */
+	stmt = gda_sql_parser_parse_string (parser, "SELECT thebigint FROM bigint", NULL, &error);
+	if (!stmt) {
+		number_failed ++;
+		goto out;
+	}
+
+	model = gda_connection_statement_execute_select (cnc, stmt, NULL, &error);
+	if (!model) {
+		number_failed ++;
+		goto out;
+	}
+	gda_data_model_dump (model, NULL);
+	if (gda_data_model_get_n_rows (model) != 1) {
+		g_set_error (&error, TEST_ERROR, TEST_ERROR_GENERIC,
+			     "Data model should have exactly 1 row");
+		number_failed ++;
+		goto out;
+	}
+
+	const GValue *cvalue;
+	cvalue = gda_data_model_get_typed_value_at (model, 0, 0, G_TYPE_INT64, FALSE, &error);
+	if (!cvalue) {
+		number_failed ++;
+		goto out;
+	}
+	if (g_value_get_int64 (tso) != g_value_get_int64 (cvalue)) {
+		gchar *tmpg, *tmpe;
+		tmpg = gda_value_stringify (cvalue);
+		tmpe = gda_value_stringify (tso);
+		g_set_error (&error, TEST_ERROR, TEST_ERROR_GENERIC,
+			     "Retreived date differs from expected: got '%s' and expected '%s'", tmpg, tmpe);
+		g_free (tmpg);
+		g_free (tmpe);
+		number_failed ++;
+		goto out;
+	}
+
+
+out:
+	if (stmt)
+		g_object_unref (stmt);
+	if (params)
+		g_object_unref (params);
+	if (model)
+		g_object_unref (model);
+	g_object_unref (parser);
+
+#ifdef CHECK_EXTRA_INFO
+	g_print ("Date test resulted in %d error(s)\n", number_failed);
+	if (number_failed != 0)
+		g_print ("error: %s\n", error && error->message ? error->message : "No detail");
+	if (error)
+		g_error_free (error);
+#endif
+
+	return number_failed;
+}
