@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2008 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2008 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2008 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  *
@@ -22,7 +22,7 @@
 #include <string.h>
 #include "test-cnc-utils.h"
 #include "test-errors.h"
-#include "gda-ddl-creator.h"
+#include "raw-ddl-creator.h"
 #include <libgda/gda-server-provider-extra.h>
 #include <libgda/sql-parser/gda-sql-parser.h>
 #include <libgda/sql-parser/gda-sql-statement.h>
@@ -95,7 +95,6 @@ test_cnc_open_connection (const gchar *provider, const gchar *dbname, GError **e
 	const gchar *cnc_params;
 	GdaProviderInfo *prov_info;
 	GdaQuarkList *db_quark_list = NULL, *cnc_quark_list = NULL;
-	gboolean db_created = FALSE;
 
 	g_return_val_if_fail (dbname && *dbname, NULL);
 
@@ -134,7 +133,7 @@ test_cnc_open_connection (const gchar *provider, const gchar *dbname, GError **e
 		g_print ("Open connection string: %s\n", data.string->str);
 
 		gchar *auth_string = NULL;
-		GSList *current = prov_info->auth_params->holders;
+		GSList *current = gda_set_get_holders (prov_info->auth_params);
 		while (current) {
 			GdaHolder *holder = (GdaHolder *) current->data;
 
@@ -207,9 +206,9 @@ test_cnc_setup_connection (const gchar *provider, const gchar *dbname, GError **
 {
 	GdaConnection *cnc = NULL;
 	gchar *str, *upname;
-	const gchar *db_params, *cnc_params;
+	const gchar *db_params;
 	GdaProviderInfo *prov_info;
-	GdaQuarkList *db_quark_list = NULL, *cnc_quark_list = NULL;
+	GdaQuarkList *db_quark_list = NULL;
 	gboolean db_created = FALSE;
 
 	g_return_val_if_fail (dbname && *dbname, NULL);
@@ -262,17 +261,17 @@ test_cnc_setup_connection (const gchar *provider, const gchar *dbname, GError **
 gboolean
 test_cnc_setup_db_structure (GdaConnection *cnc, const gchar *schema_file, GError **error)
 {
-	GdaDDLCreator *ddl;
+	RawDDLCreator *ddl;
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	
-	ddl = gda_ddl_creator_new ();
-        if (!gda_ddl_creator_set_dest_from_file (ddl, schema_file, error)) {
+	ddl = raw_ddl_creator_new ();
+        if (!raw_ddl_creator_set_dest_from_file (ddl, schema_file, error)) {
 		g_object_unref (ddl);
 		return FALSE;
         }
 
-	gda_ddl_creator_set_connection (ddl, cnc);
-	if (!gda_ddl_creator_execute (ddl, error)) {
+	raw_ddl_creator_set_connection (ddl, cnc);
+	if (!raw_ddl_creator_execute (ddl, error)) {
 		g_object_unref (ddl);
 		return FALSE;
         }
@@ -330,7 +329,7 @@ test_cnc_clean_connection (GdaConnection *cnc, GError **error)
 		g_assert (dbname);
 		dbname = g_strdup (dbname);
 		
-		gda_connection_close (cnc);
+		g_assert (gda_connection_close (cnc, NULL));
 		g_object_unref (cnc);
 
 #ifdef CHECK_EXTRA_INFO
@@ -354,7 +353,7 @@ test_cnc_clean_connection (GdaConnection *cnc, GError **error)
 	}
 	else {
 		TO_IMPLEMENT;
-		gda_connection_close (cnc);
+		g_assert (gda_connection_close (cnc, NULL));
 		g_object_unref (cnc);
 	}
 	g_free (upname);
@@ -458,7 +457,7 @@ test_cnc_load_data_from_file (GdaConnection *cnc, const gchar *table, const gcha
 	for (i = 0; i < nrows; i++) {
 		gint j;
 		GSList *list;
-		for (list = params->holders, j = 0; list && (j < ncols); list = list->next, j++) {
+		for (list = gda_set_get_holders (params), j = 0; list && (j < ncols); list = list->next, j++) {
 			const GValue *cvalue = gda_data_model_get_value_at (import, j, i, error);
 			if (!cvalue) {
 				gda_connection_rollback_transaction (cnc, NULL, NULL);
