@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2008 - 2013 Vivien Malerba <malerba@gnome-db.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,6 +20,8 @@
 #include <json-glib/json-glib.h>
 #endif
 
+static gchar *_json_quote_string (const gchar *str);
+
 void
 tests_common_display_value (const gchar *prefix, const GValue *value)
 {
@@ -106,10 +108,10 @@ tests_common_set_serialize (GdaSet *set)
 	string = g_string_new ("{");
 
 	/* holders */
-	if (set->holders) {
+	if (gda_set_get_holders (set)) {
 		g_string_append (string, "\"holders\":[");
-		for (list = set->holders; list; list = list->next) {
-			if (list != set->holders)
+		for (list = gda_set_get_holders (set); list; list = list->next) {
+			if (list != gda_set_get_holders (set))
 				g_string_append_c (string, ',');
 			str = tests_common_holder_serialize (GDA_HOLDER (list->data));
 			g_string_append (string, str);
@@ -147,15 +149,15 @@ tests_common_set_serialize (GdaSet *set)
 	}
 
 	/* public data */
-	if (set->nodes_list) {
+	if (gda_set_get_nodes (set)) {
 		g_string_append (string, ",\"nodes\":[");
-		for (list = set->nodes_list; list; list = list->next) {
+		for (list = gda_set_get_nodes (set); list; list = list->next) {
 			GdaSetNode *node = (GdaSetNode*) list->data;
-			if (list != set->nodes_list)
+			if (list != gda_set_get_nodes (set))
 				g_string_append_c (string, ',');
 
 			g_string_append_c (string, '{');
-			g_string_append_printf (string, "\"holder\":%d", g_slist_index (set->holders, gda_set_node_get_holder (node)));
+			g_string_append_printf (string, "\"holder\":%d", g_slist_index (gda_set_get_holders (set), gda_set_node_get_holder (node)));
 
 			GdaDataModel *source_model;
 			source_model = gda_set_node_get_data_model (node);
@@ -182,11 +184,11 @@ tests_common_set_serialize (GdaSet *set)
 		g_string_append_c (string, ']');
 	}
 
-	if (set->sources_list) {
+	if (gda_set_get_sources (set)) {
 		g_string_append (string, ",\"sources\":[");
-		for (list = set->sources_list; list; list = list->next) {
+		for (list = gda_set_get_sources (set); list; list = list->next) {
 			GdaSetSource *source = (GdaSetSource*) list->data;
-			if (list != set->sources_list)
+			if (list != gda_set_get_sources (set))
 				g_string_append_c (string, ',');
 			g_string_append_c (string, '{');
 			g_string_append (string, "\"model\":");
@@ -209,7 +211,7 @@ tests_common_set_serialize (GdaSet *set)
 			for (nodes = gda_set_source_get_nodes (source); nodes; nodes = nodes->next) {
 				if (nodes != gda_set_source_get_nodes (source))
 					g_string_append_c (string, ',');
-				g_string_append_printf (string, "%d", g_slist_index (set->nodes_list, nodes->data));
+				g_string_append_printf (string, "%d", g_slist_index (gda_set_get_nodes (set), nodes->data));
 			}
 			g_string_append_c (string, ']');
 
@@ -224,6 +226,72 @@ tests_common_set_serialize (GdaSet *set)
 	return str;
 }
 
+gchar *
+_json_quote_string (const gchar *str)
+{
+	gchar *retval, *rptr;
+	const gchar *sptr;
+	gint len;
+
+	if (!str)
+		return g_strdup ("null");
+
+	len = strlen (str);
+	retval = g_new (gchar, 2*len + 3);
+	*retval = '"';
+	for (rptr = retval+1, sptr = str; *sptr; sptr++, rptr++) {
+		switch (*sptr) {
+		case '"':
+			*rptr = '\\';
+			rptr++;
+			*rptr = *sptr;
+			break;
+		case '\\':
+			*rptr = '\\';
+			rptr++;
+			*rptr = *sptr;
+			break;
+		case '/':
+			*rptr = '\\';
+			rptr++;
+			*rptr = *sptr;
+			break;
+		case '\b':
+			*rptr = '\\';
+			rptr++;
+			*rptr = 'b';
+			break;
+		case '\f':
+			*rptr = '\\';
+			rptr++;
+			*rptr = 'f';
+			break;
+		case '\n':
+			*rptr = '\\';
+			rptr++;
+			*rptr = 'n';
+			break;
+		case '\r':
+			*rptr = '\\';
+			rptr++;
+			*rptr = 'r';
+			break;
+		case '\t':
+			*rptr = '\\';
+			rptr++;
+			*rptr = 't';
+			break;
+		default:
+			*rptr = *sptr;
+			break;
+		}
+	}
+	*rptr = '"';
+	rptr++;
+	*rptr = 0;
+	return retval;
+}
+
 /*
  * Loads @filename where each line (terminated by a '\n') is in the form <id>|<text>, and lines starting
  * with a '#' will be ignored. The new hash table is indexed on the <id> part.
