@@ -1,6 +1,7 @@
 /*
- * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,11 +26,11 @@
 #include "browser-canvas-table.h"
 #include "browser-canvas-column.h"
 #include "browser-canvas-fkey.h"
-#include "../common/objects-cloud.h"
-#include "../common/fk-declare.h"
+#include "../objects-cloud.h"
+#include "../fk-declare.h"
 #include <libgda-ui/internal/popup-container.h>
 #include "../browser-window.h"
-#include "../support.h"
+#include "../ui-support.h"
 
 static void browser_canvas_db_relations_class_init (BrowserCanvasDbRelationsClass *class);
 static void browser_canvas_db_relations_init       (BrowserCanvasDbRelations *canvas);
@@ -418,8 +419,8 @@ popup_func_declare_fk_cb (G_GNUC_UNUSED GtkMenuItem *mitem, BrowserCanvasTable *
 		if (! fk_declare_write (FK_DECLARE (dlg),
 					BROWSER_IS_WINDOW (parent) ? BROWSER_WINDOW (parent) : NULL,
 					&error)) {
-			browser_show_error ((GtkWindow *) parent, _("Failed to declare foreign key: %s"),
-					    error && error->message ? error->message : _("No detail"));
+			ui_show_error ((GtkWindow *) parent, _("Failed to declare foreign key: %s"),
+				       error && error->message ? error->message : _("No detail"));
 			g_clear_error (&error);
 		}
 		else if (BROWSER_IS_WINDOW (parent))
@@ -427,8 +428,8 @@ popup_func_declare_fk_cb (G_GNUC_UNUSED GtkMenuItem *mitem, BrowserCanvasTable *
 						    GTK_MESSAGE_INFO, "fkdeclare",
 						    _("Successfully declared foreign key"));
 		else
-			browser_show_message ((GtkWindow *) parent, "%s",
-					      _("Successfully declared foreign key"));
+			ui_show_message ((GtkWindow *) parent, "%s",
+					 _("Successfully declared foreign key"));
 	}
 	
 	gtk_widget_destroy (dlg);
@@ -649,7 +650,6 @@ build_context_menu (BrowserCanvas *canvas)
 
 	for (list = all_dbo; list; list = list->next) {
 		GdaMetaDbObject *dbo = GDA_META_DB_OBJECT (list->data);
-		GtkWidget *img;
 		if (dbo->obj_type != GDA_META_DB_TABLE)
 			continue;
 		if (g_hash_table_lookup (dbrel->priv->hash_tables, dbo))
@@ -657,9 +657,7 @@ build_context_menu (BrowserCanvas *canvas)
 			continue;
 		
 		if (strcmp (dbo->obj_short_name, dbo->obj_full_name)) {
-			mitem = gtk_image_menu_item_new_with_label (dbo->obj_short_name);
-			img = gtk_image_new_from_pixbuf (browser_get_pixbuf_icon (BROWSER_ICON_TABLE));
-			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mitem), img);
+			mitem = gtk_menu_item_new_with_label (dbo->obj_short_name);
 			g_object_set_data (G_OBJECT (mitem), "dbtable", GDA_META_TABLE (dbo));
 			gtk_menu_shell_prepend (GTK_MENU_SHELL (submenu), mitem);
 			g_signal_connect (mitem, "activate",
@@ -669,9 +667,7 @@ build_context_menu (BrowserCanvas *canvas)
 		GtkWidget *schema_menu;
 		schema_menu = g_hash_table_lookup (schemas, dbo->obj_schema);
 		if (!schema_menu) {
-			mitem = gtk_image_menu_item_new_with_label (dbo->obj_schema);
-			img = gtk_image_new_from_pixbuf (browser_get_pixbuf_icon (BROWSER_ICON_SCHEMA));
-			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mitem), img);
+			mitem = gtk_menu_item_new_with_label (dbo->obj_schema);
 			gtk_menu_shell_append (GTK_MENU_SHELL (submenu), mitem);
 
 			schema_menu = gtk_menu_new ();
@@ -681,9 +677,7 @@ build_context_menu (BrowserCanvas *canvas)
 			added_schemas = g_slist_prepend (added_schemas, schema_menu);
 		}
 		
-		mitem = gtk_image_menu_item_new_with_label (dbo->obj_short_name);
-		img = gtk_image_new_from_pixbuf (browser_get_pixbuf_icon (BROWSER_ICON_TABLE));
-		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mitem), img);
+		mitem = gtk_menu_item_new_with_label (dbo->obj_short_name);
 		g_object_set_data (G_OBJECT (mitem), "dbtable", GDA_META_TABLE (dbo));
 		gtk_menu_shell_prepend (GTK_MENU_SHELL (schema_menu), mitem);
 		g_signal_connect (mitem, "activate",
@@ -823,13 +817,12 @@ static void
 popup_add_table_cb (G_GNUC_UNUSED GtkMenuItem *mitem, BrowserCanvasDbRelations *dbrels)
 {
 	if (! dbrels->priv->add_dialog) {
-		GtkWidget *vbox, *cloud, *find, *dcontents;
-		dbrels->priv->add_dialog = gtk_dialog_new_with_buttons (_("Select tables to add to diagram"),
-									(GtkWindow*) gtk_widget_get_toplevel ((GtkWidget*) dbrels),
-									0,
-									NULL);
-		g_signal_connect (dbrels->priv->add_dialog, "close",
-				  G_CALLBACK (gtk_widget_hide), NULL);
+		GtkWidget *vbox, *cloud, *find;
+		dbrels->priv->add_dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+		gtk_window_set_title (GTK_WINDOW (dbrels->priv->add_dialog),
+				      _("Select tables to add to diagram"));
+		gtk_window_set_transient_for (GTK_WINDOW (dbrels->priv->add_dialog),
+					      (GtkWindow*) gtk_widget_get_toplevel ((GtkWidget*) dbrels));
 		g_signal_connect (dbrels->priv->add_dialog, "delete-event",
 				  G_CALLBACK (add_dialog_delete_event), NULL);
 		gtk_window_set_default_size (GTK_WINDOW (dbrels->priv->add_dialog), 430, 400);
@@ -837,8 +830,7 @@ popup_add_table_cb (G_GNUC_UNUSED GtkMenuItem *mitem, BrowserCanvasDbRelations *
 		g_object_set_data (G_OBJECT (dbrels->priv->add_dialog), "__canvas", dbrels);
 
 		vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-		dcontents = gtk_dialog_get_content_area (GTK_DIALOG (dbrels->priv->add_dialog));
-		gtk_container_add (GTK_CONTAINER (dcontents), vbox);
+		gtk_container_add (GTK_CONTAINER (dbrels->priv->add_dialog), vbox);
 		
 		cloud = objects_cloud_new (dbrels->priv->mstruct, OBJECTS_CLOUD_TYPE_TABLE);
 		dbrels->priv->cloud = OBJECTS_CLOUD (cloud);
