@@ -1,7 +1,8 @@
 /*
- * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2010 - 2011 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2013 Haïkel Guémar <hguemar@fedoraproject.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -332,15 +333,15 @@ canvas_event_cb (BrowserCanvas *canvas, GdkEvent *event, GooCanvas *gcanvas)
 					gtk_widget_show (mitem);
 					gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 				}
-				mitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_ZOOM_IN, NULL);
+				mitem = gtk_menu_item_new_with_label (_("Zoom in"));
 				gtk_widget_show (mitem);
 				gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 				g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (popup_zoom_in_cb), canvas);
-				mitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_ZOOM_OUT, NULL);
+				mitem = gtk_menu_item_new_with_label (_("Zoom out"));
 				gtk_widget_show (mitem);
 				gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 				g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (popup_zoom_out_cb), canvas);
-				mitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_ZOOM_FIT, NULL);
+				mitem = gtk_menu_item_new_with_label (_("Adjusted zoom"));
 				gtk_widget_show (mitem);
 				gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 				g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (popup_zoom_fit_cb), canvas);
@@ -365,19 +366,17 @@ canvas_event_cb (BrowserCanvas *canvas, GdkEvent *event, GooCanvas *gcanvas)
 				gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 #endif
 				
-				mitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_SAVE_AS, NULL);
+				mitem = gtk_menu_item_new_with_label (_("Save as"));
 				gtk_widget_show (mitem);
 				gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 				g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (popup_export_cb), canvas);
 
-				mitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_PRINT, NULL);
+				mitem = gtk_menu_item_new_with_label (_("Print"));
 				gtk_widget_show (mitem);
 				g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (popup_print_cb), canvas);
 				gtk_menu_shell_append (GTK_MENU_SHELL (menu), mitem);
 
-				gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
-						NULL, NULL, ((GdkEventButton *)event)->button,
-						((GdkEventButton *)event)->time);
+				gtk_menu_popup_at_pointer (GTK_MENU (menu), event);
 			}
 		}
 		done = TRUE;
@@ -464,8 +463,8 @@ popup_export_cb (G_GNUC_UNUSED GtkMenuItem *mitem, BrowserCanvas *canvas)
 
 	dlg = gtk_file_chooser_dialog_new (_("Save diagram as"), (GtkWindow*) toplevel,
 					   GTK_FILE_CHOOSER_ACTION_SAVE, 
-					   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					   GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+					   _("_Cancel"), GTK_RESPONSE_CANCEL,
+					   _("_Save"), GTK_RESPONSE_ACCEPT,
 					   NULL);
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dlg),
 					     gdaui_get_default_path ());
@@ -851,13 +850,23 @@ browser_canvas_perform_auto_layout (BrowserCanvas *canvas, gboolean animate, Bro
 	if (!gvc)
 		gvc = gvContext ();
 
+#ifdef GRAPHVIZ_NEW_API
 	graph = agopen ("BrowserCanvasLayout", Agdirected, NULL);
-        agnode (graph, "shape", "box");
+        agset (graph, "shape", "box");
         agset (graph, "height", ".1");
         agset (graph, "width", ".1");
         agset (graph, "fixedsize", "true");
         agset (graph, "pack", "true");
 	agset (graph, "packmode", "node");
+#else
+	graph = agopen ("BrowserCanvasLayout", AGRAPH);
+        agnodeattr (graph, "shape", "box");
+        agnodeattr (graph, "height", ".1");
+        agnodeattr (graph, "width", ".1");
+        agnodeattr (graph, "fixedsize", "true");
+        agnodeattr (graph, "pack", "true");
+	agnodeattr (graph, "packmode", "node");
+#endif
 
 
 	if (class->get_layout_items)
@@ -885,26 +894,34 @@ browser_canvas_perform_auto_layout (BrowserCanvas *canvas, gboolean animate, Bro
 		nodes_list = g_slist_prepend (nodes_list, nl);
 		
 		tmp = g_strdup_printf ("%p", item);
-		node = agnode (graph, tmp, 0);
+#ifdef GRAPHVIZ_NEW_API
+		node = agnode (graph, tmp, 1);
+#else
+		node = agnode (graph, tmp);
+#endif
+		g_free (tmp);
 		nl->node = node;
 		g_hash_table_insert (nodes_hash, item, node);
-		
-		tmp = g_strdup_printf ("%p", node);
-		agset (node, "label", tmp);
-		g_free (tmp);
-		
 		goo_canvas_item_get_bounds (GOO_CANVAS_ITEM (item), &bounds);
 		nl->width = bounds.x2 - bounds.x1;
 		nl->height = bounds.y2 - bounds.y1;
 		val = (bounds.y2 - bounds.y1) / GV_SCALE;
-		tmp = g_strdup_printf ("%.3f", val);
-		agset (node, "height", tmp);
-		g_free (tmp);
-		val = (bounds.x2 - bounds.x1) / GV_SCALE;
-		tmp = g_strdup_printf ("%.3f", val);
-		agset (node, "width", tmp);
-		g_free (tmp);
-		
+
+		if (node) {
+			tmp = g_strdup_printf ("%p", node);
+			agset (node, "label", tmp);
+			g_free (tmp);
+
+			tmp = g_strdup_printf ("%.3f", val);
+			agset (node, "height", tmp);
+			g_free (tmp);
+
+			val = (bounds.x2 - bounds.x1) / GV_SCALE;
+			tmp = g_strdup_printf ("%.3f", val);
+			agset (node, "width", tmp);
+			g_free (tmp);
+		}
+
 		nl->start_x = bounds.x1;
 		nl->start_y = bounds.y1;
 		nl->cur_x = nl->start_x;
@@ -928,8 +945,13 @@ browser_canvas_perform_auto_layout (BrowserCanvas *canvas, gboolean animate, Bro
 			Agnode_t *from_node, *to_node;
 			from_node = (Agnode_t*) g_hash_table_lookup (nodes_hash, from);
 			to_node = (Agnode_t*) g_hash_table_lookup (nodes_hash, to);
-			if (from_node && to_node)
-			  agedge (graph, from_node, to_node, "", 0);
+			if (from_node && to_node) {
+#ifdef GRAPHVIZ_NEW_API
+				agedge (graph, from_node, to_node, "", 0);
+#else
+				agedge (graph, from_node, to_node);
+#endif
+			}
 		}
 	}
 
