@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2010 David King <davidk@openismus.com>
- * Copyright (C) 2010 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -21,7 +21,7 @@
 #include <string.h>
 #include <glib/gi18n-lib.h>
 #include "data-source-editor.h"
-#include "../common/widget-overlay.h"
+#include "../widget-overlay.h"
 
 /* signals */
 enum {
@@ -242,9 +242,9 @@ update_dependencies_display (DataSourceEditor *editor)
 		GdaSet *import;
 		import = data_source_get_import ( editor->priv->source);
 		g_string_append_printf (string, "**%s**\n", _("Requires:"));
-		if (import && import->holders) {
+		if (import && gda_set_get_holders (import)) {
 			GSList *list;
-			for (list = import->holders; list; list = list->next)
+			for (list = gda_set_get_holders (import); list; list = list->next)
 				g_string_append_printf (string, "%s\n",
 							gda_holder_get_id (GDA_HOLDER (list->data)));
 		}
