@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2010 David King <davidk@openismus.com>
- * Copyright (C) 2010 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 - 2014 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -20,10 +20,9 @@
 
 #include <string.h>
 #include <glib/gi18n-lib.h>
-#include "browser-connection.h"
-#include <libgda/thread-wrapper/gda-thread-wrapper.h>
-#include "support.h"
-#include "marshal.h"
+#include "common/t-connection.h"
+#include "../ui-support.h"
+#include "data-manager-marshal.h"
 #include <sql-parser/gda-sql-parser.h>
 #include <libgda/gda-data-model-extra.h>
 #include <libgda/gda-sql-builder.h>
@@ -54,7 +53,7 @@ static void ensure_source_unique_id (DataSourceManager *mgr, DataSource *source)
 static GObjectClass  *parent_class = NULL;
 
 struct _DataSourceManagerPrivate {
-	BrowserConnection *bcnc;
+	TConnection *tcnc;
 	GSList *sources_list;
         GdaSet *params; /* execution params */
 	gboolean emit_changes;
@@ -206,8 +205,8 @@ data_source_manager_dispose (GObject *object)
 			mgr->priv->sources_list = NULL;
 		}
 
-		if (mgr->priv->bcnc)
-			g_object_unref (mgr->priv->bcnc);
+		if (mgr->priv->tcnc)
+			g_object_unref (mgr->priv->tcnc);
 
 		g_free (mgr->priv);
 		mgr->priv = NULL;
@@ -219,20 +218,20 @@ data_source_manager_dispose (GObject *object)
 
 /**
  * data_source_manager_new
- * @bcnc:
+ * @tcnc:
  *
  * Creates a new #DataSourceManager object
  *
  * Returns: a new object
  */
 DataSourceManager*
-data_source_manager_new (BrowserConnection *bcnc)
+data_source_manager_new (TConnection *tcnc)
 {
 	DataSourceManager *mgr;
-	g_return_val_if_fail (BROWSER_IS_CONNECTION (bcnc), NULL);
+	g_return_val_if_fail (T_IS_CONNECTION (tcnc), NULL);
 
 	mgr = DATA_SOURCE_MANAGER (g_object_new (DATA_SOURCE_MANAGER_TYPE, NULL));
-	mgr->priv->bcnc = g_object_ref (bcnc);
+	mgr->priv->tcnc = g_object_ref (tcnc);
 
 	return mgr;
 }
@@ -260,7 +259,7 @@ source_depends_on (DataSource *source1, DataSource *source2)
 	if (!export_columns)
 		return FALSE;
 
-	for (holders = import->holders; holders; holders = holders->next) {
+	for (holders = gda_set_get_holders (import); holders; holders = holders->next) {
 		GdaHolder *holder = (GdaHolder*) holders->data;
 		if (GPOINTER_TO_INT (g_hash_table_lookup (export_columns, gda_holder_get_id (holder))) >= 1) {
 #ifdef DEBUG_SOURCES_SORT
@@ -422,7 +421,7 @@ compute_params (DataSourceManager *mgr)
 {
 	/* cleanning process */
 	if (mgr->priv->params) {
-		browser_connection_keep_variables (mgr->priv->bcnc, mgr->priv->params);
+		t_connection_keep_variables (mgr->priv->tcnc, mgr->priv->params);
 		g_object_unref (mgr->priv->params);
         }
         mgr->priv->params = NULL;
@@ -441,7 +440,7 @@ compute_params (DataSourceManager *mgr)
 
 			GSList *holders;
 			gboolean found;
-			for (found = FALSE, holders = set->holders; holders; holders = holders->next) {
+			for (found = FALSE, holders = gda_set_get_holders (set); holders; holders = holders->next) {
 				GSList *list2;
 				for (list2 = mgr->priv->sources_list; list2; list2 = list2->next) {
 					if (list2 == list)
@@ -465,7 +464,7 @@ compute_params (DataSourceManager *mgr)
 		}
 	}
 
-	browser_connection_load_variables (mgr->priv->bcnc, mgr->priv->params);
+	t_connection_load_variables (mgr->priv->tcnc, mgr->priv->params);
 }
 
 
@@ -627,11 +626,11 @@ data_source_manager_destroy_sources_array (GArray *array)
  * data_source_manager_get_browser_cnc
  * @mgr:
  */
-BrowserConnection *
+TConnection *
 data_source_manager_get_browser_cnc (DataSourceManager *mgr)
 {
 	g_return_val_if_fail (IS_DATA_SOURCE_MANAGER (mgr), NULL);
-	return mgr->priv->bcnc;
+	return mgr->priv->tcnc;
 }
 
 /**
