@@ -2,7 +2,7 @@
  * Copyright (C) 2010 Claude Paroz <claude@2xlibre.net>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2010 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2010 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 - 2015 Vivien Malerba <malerba@gnome-db.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,15 +21,14 @@
 
 #include <string.h>
 #include <glib/gi18n-lib.h>
-#include "browser-connection.h"
-#include <libgda/thread-wrapper/gda-thread-wrapper.h>
-#include "support.h"
-#include "marshal.h"
+#include "../common/t-connection.h"
+#include "../ui-support.h"
+#include "data-manager-marshal.h"
 #include <sql-parser/gda-sql-parser.h>
 #include <libgda/gda-data-model-extra.h>
 #include <libgda/gda-sql-builder.h>
-#include "../common/ui-formgrid.h"
-#include "../../tool-utils.h"
+#include "../ui-formgrid.h"
+#include "common/t-errors.h"
 
 #include "data-source.h"
 #define DEFAULT_DATA_SOURCE_NAME "DataSource"
@@ -131,7 +130,7 @@ dependency_find (GSList *dep_list, const gchar *id, const gchar *table, gint siz
 }
 
 struct _DataSourcePrivate {
-	BrowserConnection *bcnc;
+	TConnection *tcnc;
 	gchar             *title;
 	gchar             *impl_title;
 	gchar             *id;
@@ -228,7 +227,7 @@ data_source_init (DataSource *source)
 {
 	source->priv = g_new0 (DataSourcePrivate, 1);
 	source->priv->id = g_strdup (DEFAULT_DATA_SOURCE_NAME);
-	source->priv->bcnc = NULL;
+	source->priv->tcnc = NULL;
 	source->priv->source_type = DATA_SOURCE_UNKNOWN;
 	source->priv->need_rerun = FALSE;
 	source->priv->exec_id = 0;
@@ -317,8 +316,8 @@ data_source_dispose (GObject *object)
 
 	source = DATA_SOURCE (object);
 	if (source->priv) {
-		if (source->priv->bcnc)
-			g_object_unref (source->priv->bcnc);
+		if (source->priv->tcnc)
+			g_object_unref (source->priv->tcnc);
 		data_source_reset (source);
 		g_free (source->priv->id);
 		g_free (source->priv->title);
@@ -337,20 +336,20 @@ static gboolean init_from_table_node (DataSource *source, xmlNodePtr node, GErro
 
 /**
  * data_source_new
- * @bcnc: a #BrowserConnection
+ * @tcnc: a #TConnection
  * @type: the new data source's requested type
  *
  * Returns: a new #DataSource object
  */
 DataSource *
-data_source_new (BrowserConnection *bcnc, DataSourceType type)
+data_source_new (TConnection *tcnc, DataSourceType type)
 {
 	DataSource *source;
 
-	g_return_val_if_fail (BROWSER_IS_CONNECTION (bcnc), NULL);
+	g_return_val_if_fail (T_IS_CONNECTION (tcnc), NULL);
 
 	source = DATA_SOURCE (g_object_new (DATA_SOURCE_TYPE, NULL));
-	source->priv->bcnc = g_object_ref (bcnc);
+	source->priv->tcnc = g_object_ref (tcnc);
 	source->priv->source_type = type;
 
 	return source;
@@ -358,7 +357,7 @@ data_source_new (BrowserConnection *bcnc, DataSourceType type)
 
 /**
  * data_source_new_from_xml_node
- * @bcnc: a #BrowserConnection
+ * @tcnc: a #TConnection
  * @node:
  * @error:
  *
@@ -367,15 +366,15 @@ data_source_new (BrowserConnection *bcnc, DataSourceType type)
  * Returns: a new object
  */
 DataSource*
-data_source_new_from_xml_node (BrowserConnection *bcnc, xmlNodePtr node, GError **error)
+data_source_new_from_xml_node (TConnection *tcnc, xmlNodePtr node, GError **error)
 {
 	DataSource *source;
 
-	g_return_val_if_fail (BROWSER_IS_CONNECTION (bcnc), NULL);
+	g_return_val_if_fail (T_IS_CONNECTION (tcnc), NULL);
 	g_return_val_if_fail (node, NULL);
 
 	source = DATA_SOURCE (g_object_new (DATA_SOURCE_TYPE, NULL));
-	source->priv->bcnc = g_object_ref (bcnc);
+	source->priv->tcnc = g_object_ref (tcnc);
 	xmlChar *prop;
 	prop = xmlGetProp (node, BAD_CAST "title");
 	if (prop) {
@@ -400,7 +399,7 @@ data_source_new_from_xml_node (BrowserConnection *bcnc, xmlNodePtr node, GError
 		init_from_query (source, node);
 	}
 	else {
-		g_set_error (error, GDA_TOOLS_ERROR, GDA_TOOLS_COMMAND_ARGUMENTS_ERROR,
+		g_set_error (error, T_ERROR, T_COMMAND_ARGUMENTS_ERROR,
 			     /* Translators: Do not translate "table" nor "query" */
 			     _("Node must be \"table\" or \"query\", and is \"%s\""), (gchar*)node->name);
 		g_object_unref (source);
@@ -432,16 +431,16 @@ get_meta_table (DataSource *source, const gchar *table_name, GError **error)
 	gchar **split;
 	gint len;
 
-	mstruct = browser_connection_get_meta_struct (source->priv->bcnc);
+	mstruct = t_connection_get_meta_struct (source->priv->tcnc);
 	if (! mstruct) {
-		g_set_error (error, GDA_TOOLS_ERROR, GDA_TOOLS_STORED_DATA_ERROR,
+		g_set_error (error, T_ERROR, T_STORED_DATA_ERROR,
 			     "%s", _("Not ready"));
 		return NULL;
 	}
 
 	split = gda_sql_identifier_split (table_name);
 	if (! split) {
-		g_set_error (error, GDA_TOOLS_ERROR, GDA_TOOLS_COMMAND_ARGUMENTS_ERROR,
+		g_set_error (error, T_ERROR, T_COMMAND_ARGUMENTS_ERROR,
 			     _("Malformed table name \"%s\""), table_name);
 		return NULL;
 	}
@@ -458,12 +457,12 @@ get_meta_table (DataSource *source, const gchar *table_name, GError **error)
 	if (vname[2]) gda_value_free (vname[2]);
 
 	if (! dbo) {
-		g_set_error (error, GDA_TOOLS_ERROR, GDA_TOOLS_INTERNAL_COMMAND_ERROR,
+		g_set_error (error, T_ERROR, T_INTERNAL_COMMAND_ERROR,
 			     _("Could not find the \"%s\" table"), table_name);
 		return NULL;
 	}
 	if ((dbo->obj_type != GDA_META_DB_TABLE) && (dbo->obj_type != GDA_META_DB_VIEW)) {
-		g_set_error (error, GDA_TOOLS_ERROR, GDA_TOOLS_INTERNAL_COMMAND_ERROR,
+		g_set_error (error, T_ERROR, T_INTERNAL_COMMAND_ERROR,
 			     _("The \"%s\" object is not a table"), table_name);
 		return NULL;
 	}
@@ -481,7 +480,7 @@ init_from_table_node (DataSource *source, xmlNodePtr node, GError **error)
 #endif
 	tname = xmlGetProp (node, BAD_CAST "name");
 	if (!tname) {
-		g_set_error (error, GDA_TOOLS_ERROR, GDA_TOOLS_COMMAND_ARGUMENTS_ERROR,
+		g_set_error (error, T_ERROR, T_COMMAND_ARGUMENTS_ERROR,
 			     /* Translators: Do not translate "name" */
 			     "%s", _("Missing attribute \"name\" for table"));
 		return FALSE;
@@ -622,12 +621,12 @@ data_source_add_dependency (DataSource *source, const gchar *table,
 		}
 	}
 	if (!fk) {
-		g_set_error (error, GDA_TOOLS_ERROR, GDA_TOOLS_INTERNAL_COMMAND_ERROR,
+		g_set_error (error, T_ERROR, T_INTERNAL_COMMAND_ERROR,
 			     _("Could not find any foreign key to \"%s\""), table);
 		return FALSE;
 	}
 	else if (fk->cols_nb <= 0) {
-		g_set_error (error, GDA_TOOLS_ERROR, GDA_TOOLS_INTERNAL_COMMAND_ERROR,
+		g_set_error (error, T_ERROR, T_INTERNAL_COMMAND_ERROR,
 			     _("The fields involved in the foreign key to \"%s\" are not known"),
 			     table);
 		return FALSE;
@@ -780,59 +779,6 @@ data_source_to_xml_node (DataSource *source)
 	return node;
 }
 
-static gboolean
-exec_end_timeout_cb (DataSource *source)
-{
-	GObject *obj;
-
-	g_return_val_if_fail (source->priv->exec_id > 0, FALSE);
-
-	g_clear_error (&source->priv->exec_error);
-	obj = browser_connection_execution_get_result (source->priv->bcnc,
-						       source->priv->exec_id,
-						       NULL,
-						       &source->priv->exec_error);
-	if (obj) {
-		if (GDA_IS_DATA_MODEL (obj)) {
-			if (source->priv->model != GDA_DATA_MODEL (obj)) {
-				if (source->priv->model)
-					g_object_unref (source->priv->model);
-				source->priv->model = GDA_DATA_MODEL (obj);
-				g_object_set (source->priv->model, "auto-reset", FALSE, NULL);
-			}
-			else {
-				gda_data_model_thaw (source->priv->model);
-				gda_data_model_reset (source->priv->model);
-			}
-			/*gda_data_model_dump (source->priv->model, NULL);*/
-		}
-		else {
-			g_object_unref (obj);
-			g_set_error (&source->priv->exec_error, GDA_TOOLS_ERROR, GDA_TOOLS_COMMAND_ARGUMENTS_ERROR,
-				     "%s", _("Statement to execute is not a selection statement"));
-		}
-
-		source->priv->exec_id = 0;
-		g_signal_emit (source, data_source_signals [EXEC_FINISHED], 0, source->priv->exec_error);
-		if (source->priv->exec_again) {
-			source->priv->exec_again = FALSE;
-			data_source_execute (source, NULL);
-		}
-		return FALSE;
-	}
-	else if (source->priv->exec_error) {
-		source->priv->exec_id = 0;
-		g_signal_emit (source, data_source_signals [EXEC_FINISHED], 0, source->priv->exec_error);
-		if (source->priv->exec_again) {
-			source->priv->exec_again = FALSE;
-			data_source_execute (source, NULL);
-		}
-		return FALSE;
-	}
-	else
-		return TRUE; /* keep timer */
-}
-
 /**
  *data_source_get_statement
  */
@@ -884,7 +830,7 @@ data_source_set_params (DataSource *source, GdaSet *params)
 
 	if (source->priv->params) {
 		GSList *list;
-		for (list = source->priv->params->holders; list; list = list->next) {
+		for (list = gda_set_get_holders (source->priv->params); list; list = list->next) {
 			GdaHolder *holder = GDA_HOLDER (list->data);
 			GdaHolder *bind = NULL;
 			if (params)
@@ -932,13 +878,9 @@ void
 data_source_execute (DataSource *source, GError **error)
 {
 	GError *lerror = NULL;
-	gboolean has_exec = TRUE;
-	guint exec_id = 0;
 	g_return_if_fail (IS_DATA_SOURCE (source));
 
-	if (source->priv->exec_again)
-		return;
-	if (source->priv->executing || (source->priv->exec_id > 0)) {
+	if (source->priv->exec_again || source->priv->executing) {
 		source->priv->exec_again = TRUE;
 		return;
 	}
@@ -948,49 +890,59 @@ data_source_execute (DataSource *source, GError **error)
 		if (source->priv->init_error)
 			g_propagate_error (error, source->priv->init_error);
 		else
-			g_set_error (error, GDA_TOOLS_ERROR, GDA_TOOLS_COMMAND_ARGUMENTS_ERROR,
+			g_set_error (error, T_ERROR, T_COMMAND_ARGUMENTS_ERROR,
 				     "%s", _("No SELECT statement to execute"));
 	}
 
 	if (source->priv->model) {
 		if (source->priv->need_rerun) {
-			/* freeze source->priv->model to avoid that it emits signals while being in the
-			 * wrong thread */
 			source->priv->need_rerun = FALSE;
-			gda_data_model_freeze (source->priv->model);
-			exec_id = browser_connection_rerun_select (source->priv->bcnc,
-								   source->priv->model, &lerror);
-		}
-		else
-			has_exec = FALSE;
-	}
-	else
-		exec_id = browser_connection_execute_statement (source->priv->bcnc,
-								source->priv->stmt,
-								source->priv->params,
-								GDA_STATEMENT_MODEL_RANDOM_ACCESS |
-								GDA_STATEMENT_MODEL_ALLOW_NOPARAM,
-								FALSE, &lerror);
-
-	if (has_exec) {
-		g_signal_emit (source, data_source_signals [EXEC_STARTED], 0);
-		if (! exec_id) {
-			gda_data_model_thaw (source->priv->model);
-			gda_data_model_reset (source->priv->model);
+			g_signal_emit (source, data_source_signals [EXEC_STARTED], 0);
+			t_connection_rerun_select (source->priv->tcnc, source->priv->model, &lerror);
+			gda_data_model_dump (source->priv->model, NULL);
 			g_signal_emit (source, data_source_signals [EXEC_FINISHED], 0, lerror);
-			g_propagate_error (error, lerror);
-		}
-		else {
-			/* monitor the end of execution */
-			source->priv->exec_id = exec_id;
-			g_timeout_add (50, (GSourceFunc) exec_end_timeout_cb, source);
 		}
 	}
+	else {
+		GObject *result;
+		g_signal_emit (source, data_source_signals [EXEC_STARTED], 0);
+		result = t_connection_execute_statement (source->priv->tcnc,
+							       source->priv->stmt,
+							       source->priv->params,
+							       GDA_STATEMENT_MODEL_RANDOM_ACCESS |
+							       GDA_STATEMENT_MODEL_ALLOW_NOPARAM,
+							       NULL, &lerror);
+		if (result) {
+			if (GDA_IS_DATA_MODEL (result)) {
+				if (source->priv->model != GDA_DATA_MODEL (result)) {
+					if (source->priv->model)
+						g_object_unref (source->priv->model);
+					source->priv->model = GDA_DATA_MODEL (result);
+					g_object_set (source->priv->model, "auto-reset", FALSE, NULL);
+				}
+				gda_data_model_dump (source->priv->model, NULL);
+			}
+			else {
+				g_object_unref (result);
+				g_set_error (&lerror, T_ERROR, T_COMMAND_ARGUMENTS_ERROR,
+					     "%s", _("Statement to execute is not a selection statement"));
+			}
+			g_signal_emit (source, data_source_signals [EXEC_FINISHED], 0, lerror);
+		}
+
+		g_signal_emit (source, data_source_signals [EXEC_FINISHED], 0, lerror);
+	}
+
+	if (source->priv->exec_again) {
+		source->priv->exec_again = FALSE;
+		data_source_execute (source, NULL);
+	}
+
 	source->priv->executing = FALSE;
 }
 
 static void
-action_refresh_cb (GtkAction *action, DataSource *source)
+action_refresh_cb (GtkButton *button, DataSource *source)
 {
 	source->priv->need_rerun = TRUE;
 	data_source_execute (source, NULL);
@@ -1012,28 +964,7 @@ data_source_create_grid (DataSource *source)
 	GtkWidget *fg;
 	fg = ui_formgrid_new (source->priv->model, FALSE, GDAUI_DATA_PROXY_INFO_ROW_MODIFY_BUTTONS);
 
-	/* add a refresh action */
-	GtkUIManager *uimanager;
-	GtkActionGroup *agroup;
-	GtkAction *action;
-	guint mid;
-
-	agroup = gtk_action_group_new ("DSGroup");
-	gtk_action_group_set_translation_domain (agroup, GETTEXT_PACKAGE);
-	action = gtk_action_new ("Refresh", "Refresh",
-				 _("Refresh data"), GTK_STOCK_EXECUTE);
-	gtk_action_group_add_action (agroup, action);
-	g_signal_connect (G_OBJECT (action), "activate",
-			  G_CALLBACK (action_refresh_cb), source);
-	g_object_unref (action);
-	uimanager = ui_formgrid_get_ui_manager (UI_FORMGRID (fg));
-	gtk_ui_manager_insert_action_group (uimanager, agroup, 0);
-	g_object_unref (agroup);
-
-	mid = gtk_ui_manager_new_merge_id (uimanager);
-	gtk_ui_manager_add_ui (uimanager, mid, "/ToolBar/RowModifExtension", "Refresh", "Refresh",
-			       GTK_UI_MANAGER_AUTO, TRUE);
-	gtk_ui_manager_ensure_update (uimanager);
+	ui_formgrid_set_refresh_func (UI_FORMGRID (fg), G_CALLBACK (action_refresh_cb), source);
 
 	return fg;
 }
@@ -1132,7 +1063,7 @@ update_export_information (DataSource *source)
 	/* Get GdaSqlStatement */
 	GdaSqlStatement *sqlst;
 	g_object_get ((GObject*) source->priv->stmt, "structure", &sqlst, NULL);
-	if (browser_connection_check_sql_statement_validify (source->priv->bcnc, sqlst, NULL))
+	if (t_connection_check_sql_statement_validify (source->priv->tcnc, sqlst, NULL))
 		g_object_set ((GObject*) source->priv->stmt, "structure", sqlst, NULL);
 	if (! sqlst)
 		return;
@@ -1218,7 +1149,7 @@ data_source_set_table (DataSource *source, const gchar *table, GError **error)
 	b = gda_sql_builder_new (GDA_SQL_STATEMENT_SELECT);
 	source->priv->builder = b;
 	if (! gda_sql_builder_select_add_target (b, table, NULL)) {
-		g_set_error (error, GDA_TOOLS_ERROR, GDA_TOOLS_INTERNAL_COMMAND_ERROR,
+		g_set_error (error, T_ERROR, T_INTERNAL_COMMAND_ERROR,
 			     "%s", _("Could not build SELECT statement"));
 		return FALSE;
 	}
@@ -1275,7 +1206,7 @@ data_source_set_query (DataSource *source, const gchar *sql, GError **warning)
 
 	GdaSqlParser *parser;
 	const gchar *remain;
-	parser = browser_connection_create_parser (source->priv->bcnc);
+	parser = t_connection_create_parser (source->priv->tcnc);
 	source->priv->stmt = gda_sql_parser_parse_string (parser, sql,
 							  &remain, warning);
 	g_object_unref (parser);
@@ -1285,13 +1216,13 @@ data_source_set_query (DataSource *source, const gchar *sql, GError **warning)
 	}
 
 	if (remain)
-		g_set_error (warning, GDA_TOOLS_ERROR, GDA_TOOLS_COMMAND_ARGUMENTS_ERROR,
+		g_set_error (warning, T_ERROR, T_COMMAND_ARGUMENTS_ERROR,
 			     "%s", _("Multiple statements detected, only the first will be used"));
 
 	/* try to normalize the statement */
 	GdaSqlStatement *sqlst;
 	g_object_get ((GObject*) source->priv->stmt, "structure", &sqlst, NULL);
-	if (browser_connection_normalize_sql_statement (source->priv->bcnc, sqlst, NULL))
+	if (t_connection_normalize_sql_statement (source->priv->tcnc, sqlst, NULL))
 		g_object_set ((GObject*) source->priv->stmt, "structure", sqlst, NULL);
 	gda_sql_statement_free (sqlst);
 	
@@ -1340,14 +1271,14 @@ compute_import_params (DataSource *source)
 				      &source->priv->init_error);
 	if (source->priv->params) {
 		GSList *list;
-		for (list = source->priv->params->holders; list; list = list->next) {
+		for (list = gda_set_get_holders (source->priv->params); list; list = list->next) {
 			gda_holder_set_not_null (GDA_HOLDER (list->data), FALSE);
 #ifdef DEBUG_SOURCE
 			g_print ("\tIMPORT [%s]\n", gda_holder_get_id (GDA_HOLDER (list->data)));
 #endif
 		}
 
-		browser_connection_define_ui_plugins_for_stmt (source->priv->bcnc, source->priv->stmt,
+		t_connection_define_ui_plugins_for_stmt (source->priv->tcnc, source->priv->stmt,
 							       source->priv->params);
 
 		g_signal_connect (source->priv->params, "holder-changed",
