@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2010 David King <davidk@openismus.com>
- * Copyright (C) 2010 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -21,11 +21,10 @@
 #include <glib/gi18n-lib.h>
 #include <string.h>
 #include "data-widget.h"
-#include "../browser-connection.h"
-#include "../browser-spinner.h"
-#include "../common/ui-formgrid.h"
+#include "common/t-connection.h"
+#include "../ui-formgrid.h"
 #include "../browser-window.h"
-#include "../support.h"
+#include "../ui-support.h"
 #include "data-source-editor.h"
 #include "analyser.h"
 
@@ -44,7 +43,7 @@ typedef struct {
 	gint edit_widget_page;
 	gint edit_widget_previous_page;
 
-	BrowserSpinner *spinner;
+	GtkWidget *spinner;
 	guint spinner_show_timer_id;
 	GtkWidget *data_widget;
 	GtkWidget *error_widget;
@@ -113,7 +112,7 @@ data_widget_init (DataWidget *dwid, G_GNUC_UNUSED DataWidgetClass *klass)
 	info = gtk_info_bar_new ();
 	gtk_notebook_append_page (GTK_NOTEBOOK (dwid->priv->top_nb), info, NULL);
 	dwid->priv->info_label = gtk_label_new ("");
-	gtk_misc_set_alignment (GTK_MISC (dwid->priv->info_label), 0., -1);
+	gtk_widget_set_halign (dwid->priv->info_label, GTK_ALIGN_START);
 	gtk_label_set_ellipsize (GTK_LABEL (dwid->priv->info_label), PANGO_ELLIPSIZE_END);
 	gtk_container_add (GTK_CONTAINER (gtk_info_bar_get_content_area (GTK_INFO_BAR (info))),
 			   dwid->priv->info_label);
@@ -231,12 +230,12 @@ create_or_reuse_part (DataWidget *dwid, DataSource *source, gboolean *out_reused
 	}
 	else
 		gtk_label_set_markup (GTK_LABEL (label), "<b><small> </small></b>");
-	gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
 	gtk_widget_set_size_request (label, 150, -1);
 	gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
 	gtk_box_pack_start (GTK_BOX (header), label, TRUE, TRUE, 0);
 
-	image = gtk_image_new_from_pixbuf (browser_get_pixbuf_icon (BROWSER_ICON_MENU_INDICATOR));
+	image = gtk_image_new_from_pixbuf (ui_get_pixbuf_icon (UI_ICON_MENU_INDICATOR));
 	button = gtk_button_new ();
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 	gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
@@ -255,11 +254,10 @@ create_or_reuse_part (DataWidget *dwid, DataSource *source, gboolean *out_reused
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (nb), FALSE);
 	part->nb = GTK_NOTEBOOK (nb);
 
-	part->spinner = BROWSER_SPINNER (browser_spinner_new ());
-	browser_spinner_set_size ((BrowserSpinner*) part->spinner, GTK_ICON_SIZE_LARGE_TOOLBAR);
-	page = gtk_alignment_new (0.5, 0.5, 0., 0.);
-	gtk_container_add (GTK_CONTAINER (page), (GtkWidget*) part->spinner);
-	gtk_notebook_append_page (GTK_NOTEBOOK (nb), page, NULL);
+	part->spinner = gtk_spinner_new ();
+	gtk_widget_set_halign (part->spinner, GTK_ALIGN_CENTER);
+	gtk_widget_set_valign (part->spinner, GTK_ALIGN_CENTER);
+	gtk_notebook_append_page (GTK_NOTEBOOK (nb), part->spinner, NULL);
 	part->data_widget = NULL;
 
 	gtk_box_pack_start (GTK_BOX (vbox), nb, TRUE, TRUE, 0);
@@ -361,9 +359,7 @@ data_source_menu_clicked_cb (G_GNUC_UNUSED GtkButton *button, DataPart *part)
 			g_slist_free (added_list);
 	}
 
-	gtk_menu_popup (GTK_MENU (part->menu), NULL, NULL,
-                        NULL, NULL, 0,
-                        gtk_get_current_event_time ());
+	gtk_menu_popup_at_pointer (GTK_MENU (part->menu), NULL);
 }
 
 static void
@@ -611,7 +607,7 @@ data_part_show_error (DataPart *part, GError *error)
 				       error && error->message ? error->message : _("no detail"));
 	if (! part->error_widget) {
 		part->error_widget = gtk_label_new ("");
-		gtk_misc_set_alignment (GTK_MISC (part->error_widget), 0., 0.);
+		gtk_widget_set_halign (part->error_widget, GTK_ALIGN_START);
 		part->error_widget_page = gtk_notebook_append_page (part->nb, part->error_widget,
 								    NULL);
 		gtk_widget_show (part->error_widget);
@@ -625,7 +621,7 @@ static gboolean
 source_exec_started_cb_timeout (DataPart *part)
 {
 	gtk_notebook_set_current_page (part->nb, 0);
-	browser_spinner_start (part->spinner);
+	gtk_spinner_start (GTK_SPINNER (part->spinner));
 	part->spinner_show_timer_id = 0;
 	return FALSE; /* remove timer */
 }
@@ -700,7 +696,7 @@ source_exec_finished_cb (G_GNUC_UNUSED DataSource *source, GError *error, DataPa
 		part->spinner_show_timer_id = 0;
 	}
 	else
-		browser_spinner_stop (part->spinner);
+		gtk_spinner_stop (GTK_SPINNER (part->spinner));
 	
 #ifdef GDA_DEBUG_NO
 	g_print ("==== Execution of source [%s] finished\n", data_source_get_title (part->source));
@@ -712,10 +708,10 @@ source_exec_finished_cb (G_GNUC_UNUSED DataSource *source, GError *error, DataPa
 	
 	if (! part->data_widget) {
 		GtkWidget *cwid, *wid;
-		BrowserConnection *bcnc;
-		bcnc = browser_window_get_connection ((BrowserWindow*) gtk_widget_get_toplevel ((GtkWidget*) part->dwid));
+		TConnection *tcnc;
+		tcnc = browser_window_get_connection ((BrowserWindow*) gtk_widget_get_toplevel ((GtkWidget*) part->dwid));
 		cwid = (GtkWidget*) data_source_create_grid (part->source);
-		ui_formgrid_handle_user_prefs (UI_FORMGRID (cwid), bcnc,
+		ui_formgrid_handle_user_prefs (UI_FORMGRID (cwid), tcnc,
 					       data_source_get_statement (part->source));
 		g_signal_connect (cwid, "data-set-changed",
 				  G_CALLBACK (formgrid_data_set_changed_cb), part);
@@ -859,7 +855,7 @@ compute_sources_dependencies (DataPart *part, GError **error)
 		return TRUE;
 
 	GSList *holders;
-	for (holders = import->holders; holders; holders = holders->next) {
+	for (holders = gda_set_get_holders (import); holders; holders = holders->next) {
 		GdaHolder *holder = (GdaHolder*) holders->data;
 		const gchar *hid = gda_holder_get_id (holder);
 		GSList *list;
