@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2011 - 2016 Vivien Malerba <malerba@gnome-db.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,10 +24,11 @@
 #include "marshal.h"
 #include <time.h>
 #include <libgda-ui/libgda-ui.h>
+#include "../ui-support.h"
 #include "../text-search.h"
 
 struct _EntryPropertiesPrivate {
-	BrowserConnection *bcnc;
+	TConnection *tcnc;
 
 	GtkTextView *view;
 	GtkTextBuffer *text;
@@ -104,8 +105,8 @@ entry_properties_dispose (GObject *object)
 
 	/* free memory */
 	if (eprop->priv) {
-		if (eprop->priv->bcnc) {
-			g_object_unref (eprop->priv->bcnc);
+		if (eprop->priv->tcnc) {
+			g_object_unref (eprop->priv->tcnc);
 		}
 
 		g_free (eprop->priv);
@@ -152,13 +153,13 @@ static void show_search_bar (EntryProperties *eprop);
  * Returns: a new #GtkWidget
  */
 GtkWidget *
-entry_properties_new (BrowserConnection *bcnc)
+entry_properties_new (TConnection *tcnc)
 {
 	EntryProperties *eprop;
-	g_return_val_if_fail (BROWSER_IS_CONNECTION (bcnc), NULL);
+	g_return_val_if_fail (T_IS_CONNECTION (tcnc), NULL);
 
 	eprop = ENTRY_PROPERTIES (g_object_new (ENTRY_PROPERTIES_TYPE, NULL));
-	eprop->priv->bcnc = g_object_ref (bcnc);
+	eprop->priv->tcnc = g_object_ref (tcnc);
 	
 	GtkWidget *sw;
         sw = gtk_scrolled_window_new (NULL, NULL);
@@ -222,8 +223,8 @@ data_save_cb (GtkWidget *mitem, EntryProperties *eprop)
 	dialog = gtk_file_chooser_dialog_new (_("Select the file to save data to"),
 					      (GtkWindow*) gtk_widget_get_toplevel (GTK_WIDGET (eprop)),
 					      GTK_FILE_CHOOSER_ACTION_SAVE,
-					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+					      _("_Cancel"), GTK_RESPONSE_CANCEL,
+					      _("_Save"), GTK_RESPONSE_ACCEPT,
 					      NULL);
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog),
 					     gdaui_get_default_path ());
@@ -231,15 +232,15 @@ data_save_cb (GtkWidget *mitem, EntryProperties *eprop)
 		char *filename;
 		GValue *binvalue;
 		GError *lerror = NULL;
-		const GdaBinary *bin = NULL;
+		GdaBinary *bin = NULL;
 
 		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
 		binvalue = g_object_get_data (G_OBJECT (mitem), "binvalue");
 		if (binvalue)
 			bin = gda_value_get_binary (binvalue);
-		if (!bin || !g_file_set_contents (filename, (gchar*) bin->data,
-						  bin->binary_length, &lerror)) {
-			browser_show_error ((GtkWindow*) gtk_widget_get_toplevel (GTK_WIDGET (eprop)),
+		if (!bin || !g_file_set_contents (filename, (gchar*) gda_binary_get_data (bin),
+						  gda_binary_get_size (bin), &lerror)) {
+			ui_show_error ((GtkWindow*) gtk_widget_get_toplevel (GTK_WIDGET (eprop)),
 					    _("Could not save data: %s"),
 					    lerror && lerror->message ? lerror->message : _("No detail"));
 			g_clear_error (&lerror);
@@ -486,15 +487,15 @@ data_to_pixbuf (const GValue *cvalue)
 	GdkPixbuf *retpixbuf = NULL;
 
 	if (G_VALUE_TYPE (cvalue) == GDA_TYPE_BINARY) {
-		const GdaBinary *bin;
+		GdaBinary *bin;
 		GdkPixbufLoader *loader;
-
-		bin = gda_value_get_binary (cvalue);
-		if (!bin->data)
+		
+		bin = gda_value_get_binary ((GValue*) cvalue);
+		if (!gda_binary_get_data (bin))
 			goto out;
 
 		loader = gdk_pixbuf_loader_new ();
-		if (gdk_pixbuf_loader_write (loader, bin->data, bin->binary_length, NULL)) {
+		if (gdk_pixbuf_loader_write (loader, gda_binary_get_data (bin), gda_binary_get_size (bin), NULL)) {
 			if (gdk_pixbuf_loader_close (loader, NULL)) {
 				retpixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
 				g_object_ref (retpixbuf);
@@ -596,7 +597,7 @@ ad_1601_timestamp_to_string (const gchar *value, const gchar *attname)
 	GdaDataHandler *dh;
 	struct tm *stm;
 	GValue tvalue;
-	GdaTimestamp ts;
+	GDateTime *ts;
 	time_t nsec = (time_t) i64;
 	gchar *str;
 #ifdef HAVE_LOCALTIME_R
@@ -615,20 +616,20 @@ ad_1601_timestamp_to_string (const gchar *value, const gchar *attname)
 	if (!stm)
 		return NULL;
 
-	memset (&ts, 0, sizeof (GdaTimestamp));
-	ts.year = stm->tm_year + 1900;
-	ts.month = stm->tm_mon + 1;
-	ts.day = stm->tm_mday;
-	ts.hour = stm->tm_hour;
-	ts.minute = stm->tm_min;
-	ts.second = stm->tm_sec;
-	ts.timezone = GDA_TIMEZONE_INVALID;
+	GTimeZone *tz = g_time_zone_new ("Z");
+	ts = g_date_time_new (tz,
+												stm->tm_year + 1900,
+												stm->tm_mon + 1,
+												stm->tm_mday,
+												stm->tm_hour,
+												stm->tm_min,
+												stm->tm_sec);
 	memset (&tvalue, 0, sizeof (GValue));
-	gda_value_set_timestamp (&tvalue, &ts);
-	dh = gda_data_handler_get_default (GDA_TYPE_TIMESTAMP);
+	g_value_set_boxed (&tvalue, ts);
+	dh = gda_data_handler_get_default (G_TYPE_DATE_TIME);
 	str = gda_data_handler_get_str_from_value (dh, &tvalue);
 	g_value_reset (&tvalue);
-
+	g_date_time_unref (ts);
 	return str;
 }
 
@@ -743,96 +744,74 @@ ad_sam_account_type_to_string (const gchar *value)
 }
 
 static void
-info_fetch_cb (BrowserConnection *bcnc, gpointer out_result, EntryProperties *eprop, G_GNUC_UNUSED GError *error)
+entry_info_fetched_done (EntryProperties *eprop, GdaLdapEntry *entry)
 {
-	if (out_result) {
-		GtkTextBuffer *tbuffer;
-		GtkTextIter start, end;
-		
-		tbuffer = eprop->priv->text;
-		gtk_text_buffer_get_start_iter (tbuffer, &start);
-		gtk_text_buffer_get_end_iter (tbuffer, &end);
-		gtk_text_buffer_delete (tbuffer, &start, &end);
+	GtkTextBuffer *tbuffer;
+	GtkTextIter start, end;
+	TConnection *tcnc = eprop->priv->tcnc;
+	
+	tbuffer = eprop->priv->text;
+	gtk_text_buffer_get_start_iter (tbuffer, &start);
+	gtk_text_buffer_get_end_iter (tbuffer, &end);
+	gtk_text_buffer_delete (tbuffer, &start, &end);
 
-		GdaLdapEntry *entry = (GdaLdapEntry*) out_result;
-		guint i;
-		GtkTextIter current;
+	guint i;
+	GtkTextIter current;
 
-		gtk_text_buffer_get_start_iter (tbuffer, &current);
+	gtk_text_buffer_get_start_iter (tbuffer, &current);
 
-		/* DN */
-		gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current, _("Distinguished Name:"), -1,
-							  "section", NULL);
-		gtk_text_buffer_insert (tbuffer, &current, "\n", -1);
-		gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current, " ", -1, "starter", NULL);
-		gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current, entry->dn, -1,
-							  "data", NULL);
+	/* DN */
+	gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current, _("Distinguished Name:"), -1,
+						  "section", NULL);
+	gtk_text_buffer_insert (tbuffer, &current, "\n", -1);
+	gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current, " ", -1, "starter", NULL);
+	gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current, entry->dn, -1,
+						  "data", NULL);
+	gtk_text_buffer_insert (tbuffer, &current, "\n", -1);
+
+	/* other attributes */
+	const gchar *basedn;
+	GdaDataHandler *ts_dh = NULL;
+	basedn = t_connection_ldap_get_base_dn (tcnc);
+
+	for (i = 0; i < entry->nb_attributes; i++) {
+		GdaLdapAttribute *attr;
+		gchar *tmp;
+		attr = entry->attributes [i];
+		tmp = g_strdup_printf ("%s:", attr->attr_name);
+		gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current, tmp, -1, "section", NULL);
+		g_free (tmp);
 		gtk_text_buffer_insert (tbuffer, &current, "\n", -1);
 
-		/* other attributes */
-		const gchar *basedn;
-		GdaDataHandler *ts_dh = NULL;
-		basedn = browser_connection_ldap_get_base_dn (bcnc);
+		guint j;
+		for (j = 0; j < attr->nb_values; j++) {
+			const GValue *cvalue;
+			cvalue = attr->values [j];
 
-		for (i = 0; i < entry->nb_attributes; i++) {
-			GdaLdapAttribute *attr;
-			gchar *tmp;
-			attr = entry->attributes [i];
-			tmp = g_strdup_printf ("%s:", attr->attr_name);
-			gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current, tmp, -1, "section", NULL);
-			g_free (tmp);
-			gtk_text_buffer_insert (tbuffer, &current, "\n", -1);
+			gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current, " ", -1,
+								  "starter", NULL);
 
-			guint j;
-			for (j = 0; j < attr->nb_values; j++) {
-				const GValue *cvalue;
-				cvalue = attr->values [j];
+			if (G_VALUE_TYPE (cvalue) == GDA_TYPE_BINARY) {
+				GValue *copyvalue;
+				GtkTextTagTable *table;
+				GtkTextTag *tag;
+				GtkTextMark *mark;
 
-				gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current, " ", -1,
-									  "starter", NULL);
+				copyvalue = gda_value_copy (cvalue);
+				table = gtk_text_buffer_get_tag_table (tbuffer);
+				tag = gtk_text_tag_new (NULL);
+				gtk_text_tag_table_add (table, tag);
+				g_object_set_data_full ((GObject*) tag, "binvalue",
+							copyvalue, (GDestroyNotify) gda_value_free);
+				g_object_unref ((GObject*) tag);
 
-				if (G_VALUE_TYPE (cvalue) == GDA_TYPE_BINARY) {
-					GValue *copyvalue;
-					GtkTextTagTable *table;
-					GtkTextTag *tag;
-					GtkTextMark *mark;
+				mark = gtk_text_buffer_create_mark (tbuffer, NULL, &current, TRUE);
 
-					copyvalue = gda_value_copy (cvalue);
-					table = gtk_text_buffer_get_tag_table (tbuffer);
-					tag = gtk_text_tag_new (NULL);
-					gtk_text_tag_table_add (table, tag);
-					g_object_set_data_full ((GObject*) tag, "binvalue",
-								copyvalue, (GDestroyNotify) gda_value_free);
-					g_object_unref ((GObject*) tag);
-
-					mark = gtk_text_buffer_create_mark (tbuffer, NULL, &current, TRUE);
-
-					GdkPixbuf *pixbuf;
-					pixbuf = data_to_pixbuf (cvalue);
-					if (pixbuf) {
-						gtk_text_buffer_insert_pixbuf (tbuffer, &current, pixbuf); 
-						g_object_unref (pixbuf);
-											}
-					else {
-						GdaDataHandler *dh;
-						dh = gda_data_handler_get_default (G_VALUE_TYPE (cvalue));
-						if (dh)
-							tmp = gda_data_handler_get_str_from_value (dh, cvalue);
-						else
-							tmp = gda_value_stringify (cvalue);
-						gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current,
-											  tmp, -1,
-											  "data", NULL);
-						g_free (tmp);
-					}
-					GtkTextIter before;
-					gtk_text_buffer_get_iter_at_mark (tbuffer, &before, mark);
-					gtk_text_buffer_apply_tag (tbuffer, tag, &before, &current);
-					gtk_text_buffer_delete_mark (tbuffer, mark);
-					
-					gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current,
-										  "\n", 1,
-										  "data", NULL);
+				GdkPixbuf *pixbuf;
+				pixbuf = data_to_pixbuf (cvalue);
+				if (pixbuf) {
+					gtk_text_buffer_insert_pixbuf (tbuffer, &current, pixbuf); 
+					g_object_unref (pixbuf);
 				}
 				else {
 					GdaDataHandler *dh;
@@ -841,94 +820,110 @@ info_fetch_cb (BrowserConnection *bcnc, gpointer out_result, EntryProperties *ep
 						tmp = gda_data_handler_get_str_from_value (dh, cvalue);
 					else
 						tmp = gda_value_stringify (cvalue);
-					if (tmp) {
-						if (*tmp &&
-						    ((basedn && g_str_has_suffix (tmp, basedn)) || !basedn) &&
-						    gda_ldap_is_dn (tmp)) {
-							/* we have a DN */
-							GtkTextTag *tag;
-							tag = gtk_text_buffer_create_tag (tbuffer, NULL,
-											  "foreground", "blue",
-											  "weight", PANGO_WEIGHT_NORMAL,
-											  "underline", PANGO_UNDERLINE_SINGLE,
-											  NULL);
-							g_object_set_data_full (G_OBJECT (tag), "dn",
-										g_strdup (tmp), g_free);
-							gtk_text_buffer_insert_with_tags (tbuffer, &current,
-											  tmp, -1,
-											  tag, NULL);
-						}
-						else if (attr->attr_name &&
-							 !g_ascii_strcasecmp (attr->attr_name, "objectClass")) {
-							GtkTextTag *tag;
-							tag = gtk_text_buffer_create_tag (tbuffer, NULL,
-											  "foreground", "blue",
-											  "weight", PANGO_WEIGHT_NORMAL,
-											  "underline", PANGO_UNDERLINE_SINGLE,
-											  NULL);
-							g_object_set_data_full (G_OBJECT (tag), "class",
-										g_strdup (tmp), g_free);
-							gtk_text_buffer_insert_with_tags (tbuffer, &current,
-											  tmp, -1,
-											  tag, NULL);
-						}
-						else
-							gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current, tmp, -1,
-												  "data", NULL);
-
-						gchar *extrainfo = NULL;
-						if (!strncmp (attr->attr_name, "shadow", 6) &&
-						    (!strcmp (attr->attr_name, "shadowLastChange") ||
-						     !strcmp (attr->attr_name, "shadowMax") ||
-						     !strcmp (attr->attr_name, "shadowMin") ||
-						     !strcmp (attr->attr_name, "shadowInactive") ||
-						     !strcmp (attr->attr_name, "shadowExpire")))
-							extrainfo = unix_shadow_to_string (tmp, attr->attr_name);
-						else if (!strcmp (attr->attr_name, "badPasswordTime") ||
-							 !strcmp (attr->attr_name, "lastLogon") ||
-							 !strcmp (attr->attr_name, "pwdLastSet") ||
-							 !strcmp (attr->attr_name, "accountExpires") ||
-							 !strcmp (attr->attr_name, "lockoutTime") ||
-							 !strcmp (attr->attr_name, "lastLogonTimestamp"))
-							extrainfo = ad_1601_timestamp_to_string (tmp, attr->attr_name);
-						else if (!strcmp (attr->attr_name, "userAccountControl"))
-							extrainfo = ad_1601_uac_to_string (tmp);
-						else if (!strcmp (attr->attr_name, "sAMAccountType"))
-							extrainfo = ad_sam_account_type_to_string (tmp);
-
-						if (extrainfo) {
-							gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current,
-												  " ", 1,
-												  "data", NULL);
-							gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current,
-												  extrainfo, -1,
-												  "convdata", NULL);
-							g_free (extrainfo);
-						}
-						g_free (tmp);
-					}
-					else {
-						gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current, _("Can't display attribute value"), -1,
-											  "error", NULL);
-					}
 					gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current,
-										  "\n", 1,
+										  tmp, -1,
 										  "data", NULL);
+					g_free (tmp);
 				}
+				GtkTextIter before;
+				gtk_text_buffer_get_iter_at_mark (tbuffer, &before, mark);
+				gtk_text_buffer_apply_tag (tbuffer, tag, &before, &current);
+				gtk_text_buffer_delete_mark (tbuffer, mark);
+					
+				gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current,
+									  "\n", 1,
+									  "data", NULL);
+			}
+			else {
+				GdaDataHandler *dh;
+				dh = gda_data_handler_get_default (G_VALUE_TYPE (cvalue));
+				if (dh)
+					tmp = gda_data_handler_get_str_from_value (dh, cvalue);
+				else
+					tmp = gda_value_stringify (cvalue);
+				if (tmp) {
+					if (*tmp &&
+					    ((basedn && g_str_has_suffix (tmp, basedn)) || !basedn) &&
+					    gda_ldap_is_dn (tmp)) {
+						/* we have a DN */
+						GtkTextTag *tag;
+						tag = gtk_text_buffer_create_tag (tbuffer, NULL,
+										  "foreground", "blue",
+										  "weight", PANGO_WEIGHT_NORMAL,
+										  "underline", PANGO_UNDERLINE_SINGLE,
+										  NULL);
+						g_object_set_data_full (G_OBJECT (tag), "dn",
+									g_strdup (tmp), g_free);
+						gtk_text_buffer_insert_with_tags (tbuffer, &current,
+										  tmp, -1,
+										  tag, NULL);
+					}
+					else if (attr->attr_name &&
+						 !g_ascii_strcasecmp (attr->attr_name, "objectClass")) {
+						GtkTextTag *tag;
+						tag = gtk_text_buffer_create_tag (tbuffer, NULL,
+										  "foreground", "blue",
+										  "weight", PANGO_WEIGHT_NORMAL,
+										  "underline", PANGO_UNDERLINE_SINGLE,
+										  NULL);
+						g_object_set_data_full (G_OBJECT (tag), "class",
+									g_strdup (tmp), g_free);
+						gtk_text_buffer_insert_with_tags (tbuffer, &current,
+										  tmp, -1,
+										  tag, NULL);
+					}
+					else
+						gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current, tmp, -1,
+											  "data", NULL);
+
+					gchar *extrainfo = NULL;
+					if (!strncmp (attr->attr_name, "shadow", 6) &&
+					    (!strcmp (attr->attr_name, "shadowLastChange") ||
+					     !strcmp (attr->attr_name, "shadowMax") ||
+					     !strcmp (attr->attr_name, "shadowMin") ||
+					     !strcmp (attr->attr_name, "shadowInactive") ||
+					     !strcmp (attr->attr_name, "shadowExpire")))
+						extrainfo = unix_shadow_to_string (tmp, attr->attr_name);
+					else if (!strcmp (attr->attr_name, "badPasswordTime") ||
+						 !strcmp (attr->attr_name, "lastLogon") ||
+						 !strcmp (attr->attr_name, "pwdLastSet") ||
+						 !strcmp (attr->attr_name, "accountExpires") ||
+						 !strcmp (attr->attr_name, "lockoutTime") ||
+						 !strcmp (attr->attr_name, "lastLogonTimestamp"))
+						extrainfo = ad_1601_timestamp_to_string (tmp, attr->attr_name);
+					else if (!strcmp (attr->attr_name, "userAccountControl"))
+						extrainfo = ad_1601_uac_to_string (tmp);
+					else if (!strcmp (attr->attr_name, "sAMAccountType"))
+						extrainfo = ad_sam_account_type_to_string (tmp);
+
+					if (extrainfo) {
+						gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current,
+											  " ", 1,
+											  "data", NULL);
+						gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current,
+											  extrainfo, -1,
+											  "convdata", NULL);
+						g_free (extrainfo);
+					}
+					g_free (tmp);
+				}
+				else {
+					gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current, _("Can't display attribute value"), -1,
+										  "error", NULL);
+				}
+				gtk_text_buffer_insert_with_tags_by_name (tbuffer, &current,
+									  "\n", 1,
+									  "data", NULL);
 			}
 		}
-		if (ts_dh)
-			g_object_unref (ts_dh);
-		gda_ldap_entry_free (entry);
 	}
-	else
-		browser_show_message (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) eprop)),
-				      "%s", _("Could not get information about LDAP entry"));
+	if (ts_dh)
+		g_object_unref (ts_dh);
+	gda_ldap_entry_free (entry);
+
 
 	if (eprop->priv->text_search && gtk_widget_get_visible (eprop->priv->text_search))
 		text_search_rerun (TEXT_SEARCH (eprop->priv->text_search));
-
-	g_object_unref (eprop);
 }
 
 /**
@@ -952,12 +947,12 @@ entry_properties_set_dn (EntryProperties *eprop, const gchar *dn)
         gtk_text_buffer_delete (tbuffer, &start, &end);
 
 	if (dn && *dn) {
-		guint id;
-		id = browser_connection_ldap_describe_entry (eprop->priv->bcnc, dn,
-							     (BrowserConnectionJobCallback) info_fetch_cb,
-							     g_object_ref (eprop), NULL);
-		if (id == 0)
-			browser_show_message (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) eprop)),
+		GdaLdapEntry *entry;
+		entry = t_connection_ldap_describe_entry (eprop->priv->tcnc, dn, NULL);
+		if (entry)
+			entry_info_fetched_done (eprop, entry);
+		else 
+			ui_show_message (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) eprop)),
 					      "%s", _("Could not get information about LDAP entry"));
 	}
 }
