@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2011 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2011 - 2015 Vivien Malerba <malerba@gnome-db.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,7 +24,7 @@
 #include <libgda-ui/gdaui-data-selector.h>
 
 struct _FilterEditorPrivate {
-	BrowserConnection *bcnc;
+	TConnection *tcnc;
 	GtkWidget *base_dn;
 	GtkWidget *filter;
 	GtkWidget *attributes;
@@ -73,7 +73,7 @@ static void
 filter_editor_init (FilterEditor *feditor, G_GNUC_UNUSED FilterEditorClass *klass)
 {
 	feditor->priv = g_new0 (FilterEditorPrivate, 1);
-	feditor->priv->bcnc = NULL;
+	feditor->priv->tcnc = NULL;
 	feditor->priv->default_scope = GDA_LDAP_SEARCH_SUBTREE;
 
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (feditor), GTK_ORIENTATION_VERTICAL);
@@ -86,8 +86,8 @@ filter_editor_dispose (GObject *object)
 
 	/* free memory */
 	if (feditor->priv) {
-		if (feditor->priv->bcnc)
-			g_object_unref (feditor->priv->bcnc);
+		if (feditor->priv->tcnc)
+			g_object_unref (feditor->priv->tcnc);
 		g_free (feditor->priv);
 		feditor->priv = NULL;
 	}
@@ -131,54 +131,52 @@ activated_cb (G_GNUC_UNUSED GtkEntry *entry, FilterEditor *feditor)
  * Returns: a new #GtkWidget
  */
 GtkWidget *
-filter_editor_new (BrowserConnection *bcnc)
+filter_editor_new (TConnection *tcnc)
 {
 	FilterEditor *feditor;
 	GtkWidget *grid, *label, *entry;
 	GdaDataModel *model;
 	GList *values;
 	GValue *v1, *v2;
-	gfloat ya;
 
-	g_return_val_if_fail (BROWSER_IS_CONNECTION (bcnc), NULL);
+	g_return_val_if_fail (T_IS_CONNECTION (tcnc), NULL);
 
 	feditor = FILTER_EDITOR (g_object_new (FILTER_EDITOR_TYPE, NULL));
-	feditor->priv->bcnc = g_object_ref ((GObject*) bcnc);
+	feditor->priv->tcnc = g_object_ref ((GObject*) tcnc);
 
 	grid = gtk_grid_new ();
 	gtk_grid_set_column_spacing (GTK_GRID (grid), 5);
 	gtk_box_pack_start (GTK_BOX (feditor), grid, TRUE, TRUE, 0);
 	
 	label = gtk_label_new (_("Base DN:"));
-	gtk_misc_get_alignment (GTK_MISC (label), NULL, &ya);
-	gtk_misc_set_alignment (GTK_MISC (label), 0., ya);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
 	gtk_grid_attach (GTK_GRID (grid), label, 0, 0, 1, 1);
 	label = gtk_label_new (_("Filter expression:"));
-	gtk_misc_get_alignment (GTK_MISC (label), NULL, &ya);
-	gtk_misc_set_alignment (GTK_MISC (label), 0., ya);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
 	gtk_grid_attach (GTK_GRID (grid), label, 0, 1, 1, 1);
 	label = gtk_label_new (_("Attributes to fetch:"));
-	gtk_misc_get_alignment (GTK_MISC (label), NULL, &ya);
-	gtk_misc_set_alignment (GTK_MISC (label), 0., ya);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
 	gtk_grid_attach (GTK_GRID (grid), label, 0, 2, 1, 1);
 	label = gtk_label_new (_("Search scope:"));
-	gtk_misc_get_alignment (GTK_MISC (label), NULL, &ya);
-	gtk_misc_set_alignment (GTK_MISC (label), 0., ya);
+	gtk_widget_set_halign (label, GTK_ALIGN_START);
 	gtk_grid_attach (GTK_GRID (grid), label, 0, 3, 1, 1);
 	
 	entry = gtk_entry_new ();
+	gtk_widget_set_hexpand (entry, TRUE);
 	gtk_grid_attach (GTK_GRID (grid), entry, 1, 0, 1, 1);
 	feditor->priv->base_dn = entry;
 	g_signal_connect (entry, "activate",
 			  G_CALLBACK (activated_cb), feditor);
 
 	entry = gtk_entry_new ();
+	gtk_widget_set_hexpand (entry, TRUE);
 	gtk_grid_attach (GTK_GRID (grid), entry, 1, 1, 1, 1);
 	feditor->priv->filter = entry;
 	g_signal_connect (entry, "activate",
 			  G_CALLBACK (activated_cb), feditor);
 
 	entry = gtk_entry_new ();
+	gtk_widget_set_hexpand (entry, TRUE);
 	gtk_grid_attach (GTK_GRID (grid), entry, 1, 2, 1, 1);
 	feditor->priv->attributes = entry;
 	g_signal_connect (entry, "activate",
@@ -211,6 +209,7 @@ filter_editor_new (BrowserConnection *bcnc)
 
 	gint cols[] = {1};
 	entry = gdaui_combo_new_with_model (model, 1, cols);
+	gtk_widget_set_hexpand (entry, TRUE);
 	g_object_unref (model);
 	gtk_grid_attach (GTK_GRID (grid), entry, 1, 3, 1, 1);
 	feditor->priv->scope = entry;
