@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
- * Copyright (C) 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2011 - 2014 Vivien Malerba <malerba@gnome-db.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,9 +21,10 @@
 #include <libgda/libgda.h>
 #include "mgr-ldap-entries.h"
 #include <libgda/gda-tree-node.h>
+#include "../ui-support.h"
 
 struct _MgrLdapEntriesPriv {
-	BrowserConnection *bcnc;
+	TConnection *tcnc;
 	gchar             *dn;
 };
 
@@ -71,8 +72,8 @@ mgr_ldap_entries_dispose (GObject *object)
 	g_return_if_fail (MGR_IS_LDAP_ENTRIES (mgr));
 
 	if (mgr->priv) {
-		if (mgr->priv->bcnc)
-			g_object_unref (mgr->priv->bcnc);
+		if (mgr->priv->tcnc)
+			g_object_unref (mgr->priv->tcnc);
 		g_free (mgr->priv->dn);
 		g_free (mgr->priv);
 		mgr->priv = NULL;
@@ -117,7 +118,7 @@ mgr_ldap_entries_get_type (void)
 
 /**
  * mgr_ldap_entries_new:
- * @cnc: a #BrowserConnection object
+ * @cnc: a #TConnection object
  * @dn: (allow-none): a schema name or %NULL
  *
  * Creates a new #BrowserTreeManager object which will list the children of the LDAP entry which Distinguished name
@@ -127,36 +128,20 @@ mgr_ldap_entries_get_type (void)
  * Returns: (transfer full): a new #BrowserTreeManager object
  */
 GdaTreeManager*
-mgr_ldap_entries_new (BrowserConnection *bcnc, const gchar *dn)
+mgr_ldap_entries_new (TConnection *tcnc, const gchar *dn)
 {
 	MgrLdapEntries *mgr;
-	g_return_val_if_fail (BROWSER_IS_CONNECTION (bcnc), NULL);
+	g_return_val_if_fail (T_IS_CONNECTION (tcnc), NULL);
 
 	mgr = (MgrLdapEntries*) g_object_new (MGR_TYPE_LDAP_ENTRIES, NULL);
 
-	mgr->priv->bcnc = g_object_ref (bcnc);
+	mgr->priv->tcnc = g_object_ref (tcnc);
 	if (dn)
 		mgr->priv->dn = g_strdup (dn);
 
 	return (GdaTreeManager*) mgr;
 }
 
-typedef struct {
-	GMainLoop     *loop;
-	GdaLdapEntry **entries;
-	GError        *error;
-} AsyncExecData;
-
-static void
-update_children_cb (G_GNUC_UNUSED BrowserConnection *bcnc,
-		    gpointer out_result, AsyncExecData *data, GError *error)
-{
-	data->entries = (GdaLdapEntry **) out_result;
-	if (! data->entries && error)
-		data->error = g_error_copy (error);
-	g_main_loop_quit (data->loop);
-}
-
 static gint
 lentry_array_sort_func (gconstpointer a, gconstpointer b)
 {
@@ -182,13 +167,13 @@ lentry_array_sort_func (gconstpointer a, gconstpointer b)
 
 static GSList *
 mgr_ldap_entries_update_children (GdaTreeManager *manager, GdaTreeNode *node,
-				       G_GNUC_UNUSED const GSList *children_nodes, gboolean *out_error,
-				       GError **error)
+				  G_GNUC_UNUSED const GSList *children_nodes, gboolean *out_error,
+				  GError **error)
 {
 	MgrLdapEntries *mgr = MGR_LDAP_ENTRIES (manager);
 	gchar *real_dn = NULL;
 
-	g_return_val_if_fail (mgr->priv->bcnc, NULL);
+	g_return_val_if_fail (mgr->priv->tcnc, NULL);
 
 	if (mgr->priv->dn)
 		real_dn = g_strdup (mgr->priv->dn);
@@ -200,37 +185,21 @@ mgr_ldap_entries_update_children (GdaTreeManager *manager, GdaTreeNode *node,
 			real_dn = g_value_dup_string (cvalue);
 	}
 
-	AsyncExecData data;
-	guint id;
-	data.loop = NULL;
-	data.entries = NULL;
-	data.error = NULL;
+	GdaLdapEntry **entries;
 	gchar *attrs[] = {"objectClass", "cn", NULL};
+	entries = t_connection_ldap_get_entry_children (mgr->priv->tcnc, real_dn, attrs, error);
 	
-	id = browser_connection_ldap_get_entry_children (mgr->priv->bcnc, real_dn, attrs,
-							 BROWSER_CONNECTION_JOB_CALLBACK (update_children_cb),
-							 &data, error);
-	g_free (real_dn);
-	if (id == 0) {
-		if (out_error)
-			*out_error = TRUE;
-		return NULL;
-	}
-	data.loop = g_main_loop_new (NULL, FALSE);
-	g_main_loop_run (data.loop);
-	g_main_loop_unref (data.loop);
-	
-	if (data.entries) {
+	if (entries) {
 		guint i;
 		GSList *list = NULL;
 		GArray *sorted_array;
 		sorted_array = g_array_new (FALSE, FALSE, sizeof (GdaLdapEntry*));
-		for (i = 0; data.entries [i]; i++) {
+		for (i = 0; entries [i]; i++) {
 			GdaLdapEntry *lentry;
-			lentry = data.entries [i];
+			lentry = entries [i];
 			g_array_prepend_val (sorted_array, lentry);
 		}
-		g_free (data.entries);
+		g_free (entries);
 
 		g_array_sort (sorted_array, (GCompareFunc) lentry_array_sort_func);
 
@@ -270,7 +239,7 @@ mgr_ldap_entries_update_children (GdaTreeManager *manager, GdaTreeNode *node,
 			/* icon */
 			GdkPixbuf *pixbuf;
 			attr = g_hash_table_lookup (lentry->attributes_hash, "objectClass");
-			pixbuf = browser_connection_ldap_icon_for_class (attr);
+			pixbuf = ui_connection_ldap_icon_for_class (attr);
 
 			dnv = gda_value_new (G_TYPE_OBJECT);
 			g_value_set_object (dnv, pixbuf);
@@ -297,10 +266,6 @@ mgr_ldap_entries_update_children (GdaTreeManager *manager, GdaTreeNode *node,
 
 		return list;
 	}
-	else {
-		g_propagate_error (error, data.error);
-		if (out_error)
-			*out_error = TRUE;
+	else
 		return NULL;
-	}
 }
