@@ -1,7 +1,8 @@
 /*
- * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2015 Gergely Polonkai <gergely@polonkai.eu>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,23 +23,15 @@
 #include <string.h>
 #include <gdk/gdkkeysyms.h>
 #ifdef HAVE_GTKSOURCEVIEW
-  #ifdef GTK_DISABLE_SINGLE_INCLUDES
-    #undef GTK_DISABLE_SINGLE_INCLUDES
-  #endif
-
-  #include <gtksourceview/gtksourceview.h>
-  #include <gtksourceview/gtksourcelanguagemanager.h>
-  #include <gtksourceview/gtksourcebuffer.h>
-  #include <gtksourceview/gtksourcestyleschememanager.h>
-  #include <gtksourceview/gtksourcestylescheme.h>
+  #include <gtksourceview/gtksource.h>
 #endif
 #include "query-editor.h"
 #include <binreloc/gda-binreloc.h>
-#include "../browser-connection.h"
+#include "../common/t-connection.h"
 #include "../browser-window.h"
 #include <libgda-ui/internal/popup-container.h>
-#include "../support.h"
-#include "../common/widget-overlay.h"
+#include "../ui-support.h"
+#include "../widget-overlay.h"
 
 #define QUERY_EDITOR_LANGUAGE_SQL "gda-sql"
 #define COLOR_ALTER_FACTOR 1.8
@@ -93,10 +86,6 @@ struct _QueryEditorPrivate {
 	GtkTreeView *completion_treeview;
 	GtkCellRenderer *completion_renderer;
 	GtkWidget *completion_sw;
-
-	/* tooltip */
-	GtkWidget *ovl;
-	GtkWidget *tooltip_widget;
 };
 
 static void query_editor_class_init (QueryEditorClass *klass);
@@ -360,11 +349,11 @@ display_completions (QueryEditor *editor)
 	if (!str)
 		return;
 
-	BrowserConnection *bcnc;
+	TConnection *tcnc;
 	gchar **compl;
 
-	bcnc = browser_window_get_connection ((BrowserWindow*) gtk_widget_get_toplevel ((GtkWidget*) editor));
-	compl = browser_connection_get_completions (bcnc, str, ptr - str, strlen (str));
+	tcnc = browser_window_get_connection ((BrowserWindow*) gtk_widget_get_toplevel ((GtkWidget*) editor));
+	compl = t_connection_get_completions (tcnc, str, ptr - str, strlen (str));
 	g_free (str);
 
 	if (compl) {
@@ -376,7 +365,7 @@ display_completions (QueryEditor *editor)
 			GtkWidget *popup, *sw;
 			
 			model = gtk_list_store_new (1, G_TYPE_STRING);
-			treeview = browser_make_tree_view (GTK_TREE_MODEL (model));
+			treeview = ui_make_tree_view (GTK_TREE_MODEL (model));
 			gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
 			gtk_tree_view_set_grid_lines (GTK_TREE_VIEW (treeview), GTK_TREE_VIEW_GRID_LINES_NONE);
 			gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview)),
@@ -417,7 +406,7 @@ display_completions (QueryEditor *editor)
 		
 		/* fill in the model */
 		GtkTreeIter iter;
-		gint w, width = 0, h, height = 0;
+		gint width = 0, height = 0;
 		gint i;
 		
 		for (i = 0; ; i++) {
@@ -692,14 +681,18 @@ copy_all_in_signle_line_cb (GtkMenuItem *mitem, QueryEditor *editor)
 }
 
 static void
-text_view_populate_popup_cb (GtkTextView *entry, GtkMenu *menu, QueryEditor *editor)
+text_view_populate_popup_cb (GtkTextView *entry, GtkWidget *menu, QueryEditor *editor)
 {
-	GtkWidget *mitem;
-	mitem = gtk_menu_item_new_with_label (_("Copy all in a single line"));
-	g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (copy_all_in_signle_line_cb),
-			  editor);
-        gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), mitem);
-        gtk_widget_show (mitem);
+	if (GTK_IS_MENU (menu)) {
+		if (editor->priv->mode != QUERY_EDITOR_HISTORY) {
+			GtkWidget *mitem;
+			mitem = gtk_menu_item_new_with_label (_("Copy all in a single line"));
+			g_signal_connect (G_OBJECT (mitem), "activate",
+					  G_CALLBACK (copy_all_in_signle_line_cb), editor);
+			gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), mitem);
+			gtk_widget_show (mitem);
+		}
+	}
 }
 
 static void
@@ -708,7 +701,6 @@ query_editor_init (QueryEditor *editor, G_GNUC_UNUSED QueryEditorClass *klass)
 	int tab = 8;
 	gboolean highlight = TRUE;
 	gboolean showlinesno = FALSE;
-	GtkWidget *ovl, *wid;
 
 	g_return_if_fail (QUERY_IS_EDITOR (editor));
 
@@ -728,32 +720,12 @@ query_editor_init (QueryEditor *editor, G_GNUC_UNUSED QueryEditorClass *klass)
 	editor->priv->completion_popup = NULL;
 
 	/* set up widgets */
-	ovl = widget_overlay_new ();
-	editor->priv->ovl = ovl;
-	gtk_box_pack_start (GTK_BOX (editor), ovl, TRUE, TRUE, 2);
-
 	editor->priv->scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+	gtk_box_pack_start (GTK_BOX (editor), editor->priv->scrolled_window, TRUE, TRUE, 2);
         gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (editor->priv->scrolled_window),
-					     GTK_SHADOW_ETCHED_OUT);
+					     GTK_SHADOW_NONE);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (editor->priv->scrolled_window),
 					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_container_add (GTK_CONTAINER (ovl), editor->priv->scrolled_window);
-	widget_overlay_set_child_props (WIDGET_OVERLAY (ovl), editor->priv->scrolled_window,
-					WIDGET_OVERLAY_CHILD_HALIGN, WIDGET_OVERLAY_ALIGN_FILL,
-					WIDGET_OVERLAY_CHILD_VALIGN, WIDGET_OVERLAY_ALIGN_FILL,
-					-1);
-
-	wid = gtk_label_new ("");
-	editor->priv->tooltip_widget = wid;
-	gtk_label_set_markup (GTK_LABEL (wid), QUERY_EDITOR_TOOLTIP);
-	gtk_container_add (GTK_CONTAINER (ovl), wid);
-	widget_overlay_set_child_props (WIDGET_OVERLAY (ovl), wid,
-					WIDGET_OVERLAY_CHILD_HALIGN, WIDGET_OVERLAY_ALIGN_CENTER,
-					WIDGET_OVERLAY_CHILD_VALIGN, WIDGET_OVERLAY_ALIGN_CENTER,
-					WIDGET_OVERLAY_CHILD_ALPHA, .8,
-					WIDGET_OVERLAY_CHILD_TOOLTIP, TRUE,
-					-1);
-
 #ifdef HAVE_GTKSOURCEVIEW
 	editor->priv->text = gtk_source_view_new ();
 	gtk_source_buffer_set_highlight_syntax (GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (editor->priv->text))),
@@ -763,6 +735,7 @@ query_editor_init (QueryEditor *editor, G_GNUC_UNUSED QueryEditorClass *klass)
 #else
 	editor->priv->text = gtk_text_view_new ();
 #endif
+	gtk_widget_set_tooltip_markup (editor->priv->text, QUERY_EDITOR_TOOLTIP);
 
 	gtk_container_add (GTK_CONTAINER (editor->priv->scrolled_window), editor->priv->text);
 	g_signal_connect (editor->priv->text, "event", 
@@ -803,10 +776,10 @@ query_editor_init (QueryEditor *editor, G_GNUC_UNUSED QueryEditorClass *klass)
 
 	create_tags_for_sql (editor, QUERY_EDITOR_LANGUAGE_SQL);
 
-	gtk_widget_show_all (ovl);
-
 	/* timeout function to update timestamps */
 	editor->priv->ts_timeout_id = 0;
+
+	gtk_widget_show_all (GTK_WIDGET (editor));
 }
 
 static void
@@ -999,8 +972,6 @@ query_editor_set_mode (QueryEditor *editor, QueryEditorMode mode)
 		gtk_widget_set_tooltip_markup (editor->priv->text, NULL);
 		gtk_text_view_set_editable (GTK_TEXT_VIEW (editor->priv->text), FALSE);
 		gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (editor->priv->text), FALSE);
-		gtk_widget_destroy (editor->priv->tooltip_widget);
-		editor->priv->tooltip_widget = NULL;
 		break;
 	default:
 		g_assert_not_reached ();
@@ -1516,7 +1487,7 @@ query_editor_get_current_history_batch (QueryEditor *editor)
  * query_editor_history_is_empty
  * @editor: a #QueryEditor widget.
  *
- * Returns: %TRUE if @editor doe snot have any history item
+ * Returns: %TRUE if @editor does not have any history item
  */
 gboolean
 query_editor_history_is_empty (QueryEditor *editor)
@@ -1900,7 +1871,7 @@ query_editor_history_batch_new (GTimeVal run_time, GdaSet *params)
 		qib->params = gda_set_copy (params);
 
 #ifdef QUERY_EDITOR_DEBUG
-	g_print ("\t%s() => %p\n", __FUNCTION__, qib);
+	g_print ("%s (%p)\n", __FUNCTION__, qib);
 #endif
 	return qib;
 }
@@ -1922,7 +1893,7 @@ query_editor_history_batch_unref (QueryEditorHistoryBatch *qib)
 	g_return_if_fail (qib);
 	qib->ref_count--;
 #ifdef QUERY_EDITOR_DEBUG
-	g_print ("%s (%p) ref count:%d\n", __FUNCTION__, qib, qib->ref_count);
+	g_print ("%s (%p)\n", __FUNCTION__, qib);
 #endif
 	if (qib->ref_count <= 0) {
 		if (qib->hist_items) {
@@ -1932,15 +1903,11 @@ query_editor_history_batch_unref (QueryEditorHistoryBatch *qib)
 		if (qib->params)
 			g_object_unref (qib->params);
 
+#ifdef QUERY_EDITOR_DEBUG
+		g_print ("Freeing QueryEditorHistoryBatch (%p)\n", qib);
+#endif
 		g_free (qib);
-#ifdef QUERY_EDITOR_DEBUG
-		g_print ("\t%s() => %p\n", __FUNCTION__, qib);
-#endif
 	}
-#ifdef QUERY_EDITOR_DEBUG
-	else
-		g_print ("\tFAILED %s() => %p:%d\n", __FUNCTION__, qib, qib->ref_count);
-#endif
 }
 
 static void
@@ -1981,7 +1948,7 @@ query_editor_history_item_new (const gchar *sql, GObject *result, GError *error)
 		qih->exec_error = g_error_copy (error);
 
 #ifdef QUERY_EDITOR_DEBUG
-	g_print ("\t%s() => %p\n", __FUNCTION__, qih);
+	g_print ("%s() => %p\n", __FUNCTION__, qih);
 #endif
 	return qih;
 }
@@ -2003,7 +1970,7 @@ query_editor_history_item_unref (QueryEditorHistoryItem *qih)
 	g_return_if_fail (qih);
 	qih->ref_count--;
 #ifdef QUERY_EDITOR_DEBUG
-	g_print ("%s (%p) ref count:%d\n", __FUNCTION__, qih, qih->ref_count);
+	g_print ("%s (%p)\n", __FUNCTION__, qih);
 #endif
 	if (qih->ref_count <= 0) {
 		g_free (qih->sql);
@@ -2011,15 +1978,12 @@ query_editor_history_item_unref (QueryEditorHistoryItem *qih)
 			g_object_unref (qih->result);
 		if (qih->exec_error)
 			g_error_free (qih->exec_error);
+#ifdef QUERY_EDITOR_DEBUG
+		g_print ("Freeing QueryEditorHistoryItem %p\n", qih);
+#endif
+
 		g_free (qih);
-#ifdef QUERY_EDITOR_DEBUG
-		g_print ("\t%s() => %p\n", __FUNCTION__, qih);
-#endif
 	}
-#ifdef QUERY_EDITOR_DEBUG
-	else
-		g_print ("\tFAILED %s() => %p:%d\n", __FUNCTION__, qih, qih->ref_count);
-#endif
 }
 
 /*
@@ -2061,5 +2025,8 @@ hist_item_data_unref (HistItemData *hdata)
 		if (hdata->tag)
 			g_object_unref (hdata->tag);
 		g_free (hdata);
+#ifdef QUERY_EDITOR_DEBUG
+		g_print ("HistItemData %p freed\n", hdata);
+#endif
 	}
 }
