@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 - 2012 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 - 2015 Vivien Malerba <malerba@gnome-db.org>
  * Copyright (C) 2010 David King <davidk@openismus.com>
  * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
@@ -22,7 +22,8 @@
 #include <string.h>
 #include "table-info.h"
 #include "../dnd.h"
-#include "../support.h"
+#include "../ui-support.h"
+#include "../ui-customize.h"
 #include "../gdaui-bar.h"
 #include "table-columns.h"
 #include "table-preferences.h"
@@ -31,27 +32,29 @@
 #endif
 #include "schema-browser-perspective.h"
 #include "../browser-page.h"
-#include "../browser-stock-icons.h"
 #include "../browser-window.h"
 #include "../data-manager/data-manager-perspective.h"
 #include <libgda-ui/gdaui-enums.h>
 #include <libgda-ui/gdaui-basic-form.h>
-#include <libgda-ui/internal/popup-container.h>
 #include <libgda/gda-data-model-extra.h>
-#include "../common/fk-declare.h"
+#include "../fk-declare.h"
 #include <libgda/gda-debug-macros.h>
+#include "../perspectives-names.h"
+
+#define PAGE_COLUMNS "C"
+#define PAGE_RELATIONS "R"
+#define PAGE_PREFS "P"
 
 struct _TableInfoPrivate {
-	BrowserConnection *bcnc;
+	TConnection *tcnc;
 
 	gchar *schema;
 	gchar *table_name;
 	gchar *table_short_name;
 
 	GdauiBar *header;
-	GtkWidget *contents; /* notebook with pageO <=> @unknown_table_notice, page1 <=> @pages */
+	GtkWidget *contents; /* notebook with pageO <=> @unknown_table_notice, page1 <=> pages stack */
 	GtkWidget *unknown_table_notice;
-	GtkWidget *pages; /* notebook to store individual pages */
 
 	GtkWidget *insert_popup;
 	GHashTable *insert_columns_hash; /* key = column index as a pointer, value = GdaHolder in the
@@ -71,11 +74,10 @@ static void table_info_get_property (GObject *object,
 				     GParamSpec *pspec);
 /* BrowserPage interface */
 static void                 table_info_page_init (BrowserPageIface *iface);
-static GtkActionGroup      *table_info_page_get_actions_group (BrowserPage *page);
-static const gchar         *table_info_page_get_actions_ui (BrowserPage *page);
-static GtkWidget           *table_info_page_get_tab_label (BrowserPage *page, GtkWidget **out_close_button);
+static void                 table_info_customize (BrowserPage *page, GtkToolbar *toolbar, GtkHeaderBar *header);
+static GtkWidget           *table_info_get_tab_label (BrowserPage *page, GtkWidget **out_close_button);
 
-static void meta_changed_cb (BrowserConnection *bcnc, GdaMetaStruct *mstruct, TableInfo *tinfo);
+static void meta_changed_cb (TConnection *tcnc, GdaMetaStruct *mstruct, TableInfo *tinfo);
 
 /* properties */
 enum {
@@ -106,9 +108,9 @@ table_info_class_init (TableInfoClass *klass)
 static void
 table_info_page_init (BrowserPageIface *iface)
 {
-	iface->i_get_actions_group = table_info_page_get_actions_group;
-	iface->i_get_actions_ui = table_info_page_get_actions_ui;
-	iface->i_get_tab_label = table_info_page_get_tab_label;
+	iface->i_customize = table_info_customize;
+	iface->i_uncustomize = NULL;
+	iface->i_get_tab_label = table_info_get_tab_label;
 }
 
 static void
@@ -133,10 +135,10 @@ table_info_dispose (GObject *object)
 		g_free (tinfo->priv->schema);
 		g_free (tinfo->priv->table_name);
 		g_free (tinfo->priv->table_short_name);
-		if (tinfo->priv->bcnc) {
-			g_signal_handlers_disconnect_by_func (tinfo->priv->bcnc,
+		if (tinfo->priv->tcnc) {
+			g_signal_handlers_disconnect_by_func (tinfo->priv->tcnc,
 							      G_CALLBACK (meta_changed_cb), tinfo);
-			g_object_unref (tinfo->priv->bcnc);
+			g_object_unref (tinfo->priv->tcnc);
 		}
 
 		g_free (tinfo->priv);
@@ -259,12 +261,46 @@ source_drag_data_get_cb (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED GdkDragC
 	}
 }
 
-static void
-meta_changed_cb (G_GNUC_UNUSED BrowserConnection *bcnc, GdaMetaStruct *mstruct, TableInfo *tinfo)
+static GdaMetaDbObject *
+get_table_meta_data (TableInfo *tinfo)
 {
-	GdaMetaDbObject *dbo;
-	GValue *schema_v = NULL, *name_v;
+	gtk_widget_set_sensitive (GTK_WIDGET (tinfo), FALSE);
+	GdaMetaStruct *mstruct;
+	mstruct = t_connection_get_meta_struct (tinfo->priv->tcnc);
+	if (!mstruct) {
+		BrowserWindow *bwin;
+		bwin = (BrowserWindow*) gtk_widget_get_toplevel ((GtkWidget*) tinfo);
+		gtk_widget_set_sensitive (GTK_WIDGET (tinfo), TRUE);
+		ui_show_error (GTK_WINDOW (bwin), _("Meta data not yet available"));
+		return NULL;
+	}
 
+	/* get table's information */
+	GdaMetaDbObject *dbo;
+	GValue *v1, *v2;
+	g_value_set_string ((v1 = gda_value_new (G_TYPE_STRING)),
+			    tinfo->priv->schema);
+	g_value_set_string ((v2 = gda_value_new (G_TYPE_STRING)),
+			    tinfo->priv->table_name);
+	dbo = gda_meta_struct_complement (mstruct,
+					  GDA_META_DB_TABLE, NULL, v1, v2, NULL);
+	gda_value_free (v1);
+	gda_value_free (v2);
+
+	if (! dbo) {
+		BrowserWindow *bwin;
+		bwin = (BrowserWindow*) gtk_widget_get_toplevel ((GtkWidget*) tinfo);
+		gtk_widget_set_sensitive (GTK_WIDGET (tinfo), TRUE);
+		ui_show_error (GTK_WINDOW (bwin), _("Can't find information about table"));
+	}
+
+	gtk_widget_set_sensitive (GTK_WIDGET (tinfo), TRUE);
+	return dbo;
+}
+
+static void
+meta_changed_cb (G_GNUC_UNUSED TConnection *tcnc, GdaMetaStruct *mstruct, TableInfo *tinfo)
+{
 	if (tinfo->priv->insert_columns_hash) {
 		g_hash_table_destroy (tinfo->priv->insert_columns_hash);
 		tinfo->priv->insert_columns_hash = NULL;
@@ -274,13 +310,6 @@ meta_changed_cb (G_GNUC_UNUSED BrowserConnection *bcnc, GdaMetaStruct *mstruct,
 		tinfo->priv->insert_popup = NULL;
 	}
 
-	g_value_set_string ((schema_v = gda_value_new (G_TYPE_STRING)), tinfo->priv->schema);
-	g_value_set_string ((name_v = gda_value_new (G_TYPE_STRING)), tinfo->priv->table_name);
-	dbo = gda_meta_struct_get_db_object (mstruct, NULL, schema_v, name_v);
-	if (schema_v)
-		gda_value_free (schema_v);
-	gda_value_free (name_v);
-
 	if (tinfo->priv->table_short_name) {
 		g_free (tinfo->priv->table_short_name);
 		tinfo->priv->table_short_name = NULL;
@@ -289,12 +318,15 @@ meta_changed_cb (G_GNUC_UNUSED BrowserConnection *bcnc, GdaMetaStruct *mstruct,
 		g_signal_handlers_disconnect_by_func (tinfo->priv->header,
 						      G_CALLBACK (source_drag_data_get_cb), tinfo);
 	}
+
+	GdaMetaDbObject *dbo;
+	dbo = get_table_meta_data (tinfo);
 	if (dbo) {
 		tinfo->priv->table_short_name = g_strdup (dbo->obj_short_name);
 		gtk_drag_source_set ((GtkWidget *) tinfo->priv->header, GDK_BUTTON1_MASK | GDK_BUTTON3_MASK,
 				     dbo_table, G_N_ELEMENTS (dbo_table), GDK_ACTION_COPY);
 		gtk_drag_source_set_icon_pixbuf ((GtkWidget *) tinfo->priv->header,
-						 browser_get_pixbuf_icon (BROWSER_ICON_TABLE));
+						 ui_get_pixbuf_icon (UI_ICON_TABLE));
 		g_signal_connect (tinfo->priv->header, "drag-data-get",
 				  G_CALLBACK (source_drag_data_get_cb), tinfo);
 	}
@@ -308,30 +340,48 @@ meta_changed_cb (G_GNUC_UNUSED BrowserConnection *bcnc, GdaMetaStruct *mstruct,
  * Returns: a new #GtkWidget
  */
 GtkWidget *
-table_info_new (BrowserConnection *bcnc,
+table_info_new (TConnection *tcnc,
 		const gchar *schema, const gchar *table)
 {
 	TableInfo *tinfo;
 
-	g_return_val_if_fail (BROWSER_IS_CONNECTION (bcnc), NULL);
+	g_return_val_if_fail (T_IS_CONNECTION (tcnc), NULL);
 	g_return_val_if_fail (schema, NULL);
 	g_return_val_if_fail (table, NULL);
 
 	tinfo = TABLE_INFO (g_object_new (TABLE_INFO_TYPE, NULL));
 
-	tinfo->priv->bcnc = g_object_ref (bcnc);
-	g_signal_connect (tinfo->priv->bcnc, "meta-changed",
+	tinfo->priv->tcnc = g_object_ref (tcnc);
+	g_signal_connect (tinfo->priv->tcnc, "meta-changed",
 			  G_CALLBACK (meta_changed_cb), tinfo);
 	tinfo->priv->schema = g_strdup (schema);
 	tinfo->priv->table_name = g_strdup (table);
-	
+
+	GdaMetaDbObject *dbo;
+	dbo = get_table_meta_data (tinfo);
+
 	/* header */
         GtkWidget *label;
 	gchar *str, *tmp;
 
+	/* To translators: "Object" is used when we can't determine if @table is a table or a view */
+	const gchar *otype = _("Object");
+	if (dbo) {
+		switch (dbo->obj_type) {
+		case GDA_META_DB_TABLE:
+			otype = _("Table");
+			break;
+		case GDA_META_DB_VIEW:
+			otype = _("View");
+			break;
+		default:
+			break;
+		}
+	}
+
 	/* To translators: "In schema" refers to the database schema an object is in */
 	tmp = g_strdup_printf (_("In schema '%s'"), schema);
-	str = g_strdup_printf ("<b>%s</b>\n%s", table, tmp);
+	str = g_markup_printf_escaped ("%s <b>%s</b>\n%s", otype, table, tmp);
 	g_free (tmp);
 	label = gdaui_bar_new (str);
 	g_free (str);
@@ -353,7 +403,7 @@ table_info_new (BrowserConnection *bcnc,
 
 	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (""), TRUE, TRUE, 0);
-	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_DIALOG);
+	image = gtk_image_new_from_icon_name ("dialog-warning", GTK_ICON_SIZE_DIALOG);
 	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 10);
 	label = gtk_label_new (_("Table not found. If you think this is an error,\n"
 				 "please refresh the meta data from the database\n"
@@ -365,53 +415,50 @@ table_info_new (BrowserConnection *bcnc,
 	tinfo->priv->unknown_table_notice = label;
 
 	/* notebook for the pages */
-	GtkWidget *sub_nb;
-	sub_nb = gtk_notebook_new ();
-	tinfo->priv->pages = sub_nb;
-	gtk_notebook_append_page (GTK_NOTEBOOK (top_nb), sub_nb, NULL);
-	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (sub_nb), GTK_POS_BOTTOM);
+	GtkWidget *sub_stack;
+	sub_stack = gtk_stack_new ();
+	gtk_stack_set_transition_type (GTK_STACK (sub_stack), GTK_STACK_TRANSITION_TYPE_CROSSFADE);
+	GtkWidget *vbox;
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), sub_stack, TRUE, TRUE, 0);
+	GtkWidget *ss;
+	ss = gtk_stack_switcher_new ();
+	gdaui_bar_add_widget (tinfo->priv->header, ss);
+	gtk_stack_switcher_set_stack (GTK_STACK_SWITCHER (ss), GTK_STACK (sub_stack));
+	gtk_widget_show (ss);
+	gtk_notebook_append_page (GTK_NOTEBOOK (top_nb), vbox, NULL);
 
 	/* append pages */
 	GtkWidget *page;
 	page = table_columns_new (tinfo);
 	if (page) {
-		label = gtk_label_new ("");
-		str = g_strdup_printf ("<small>%s</small>", _("Columns"));
-		gtk_label_set_markup (GTK_LABEL (label), str);
-		g_free (str);
 		gtk_widget_show (page);
-		gtk_notebook_append_page (GTK_NOTEBOOK (sub_nb), page, label);
+		gtk_stack_add_titled (GTK_STACK (sub_stack), page, PAGE_COLUMNS, _("Columns"));
 	}
 #ifdef HAVE_GOOCANVAS
 	page = table_relations_new (tinfo);
 	if (page) {
-		label = gtk_label_new ("");
-		str = g_strdup_printf ("<small>%s</small>", _("Relations"));
-		gtk_label_set_markup (GTK_LABEL (label), str);
-		g_free (str);
 		gtk_widget_show (page);
-		gtk_notebook_append_page (GTK_NOTEBOOK (sub_nb), page, label);
+		gtk_stack_add_titled (GTK_STACK (sub_stack), page, PAGE_RELATIONS, _("Relations"));
 	}
 #endif
 	page = table_preferences_new (tinfo);
 	if (page) {
-		label = gtk_label_new ("");
-		str = g_strdup_printf ("<small>%s</small>", _("Preferences"));
-		gtk_label_set_markup (GTK_LABEL (label), str);
-		g_free (str);
 		gtk_widget_show (page);
-		gtk_notebook_append_page (GTK_NOTEBOOK (sub_nb), page, label);
+		gtk_stack_add_titled (GTK_STACK (sub_stack), page, PAGE_PREFS, _("Preferences"));
 	}
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (sub_nb), 0);
+	gtk_stack_set_visible_child_name (GTK_STACK (sub_stack), PAGE_COLUMNS);
 
 	/* show everything */
         gtk_widget_show_all (top_nb);
 	display_table_not_found_error (tinfo, TRUE);
 	
 	GdaMetaStruct *mstruct;
-	mstruct = browser_connection_get_meta_struct (tinfo->priv->bcnc);
+	mstruct = t_connection_get_meta_struct (tinfo->priv->tcnc);
 	if (mstruct)
-		meta_changed_cb (tinfo->priv->bcnc, mstruct, tinfo);
+		meta_changed_cb (tinfo->priv->tcnc, mstruct, tinfo);
+	else
+		display_table_not_found_error (tinfo, dbo ? FALSE : TRUE);
 
 	return (GtkWidget*) tinfo;
 }
@@ -439,33 +486,36 @@ table_info_get_table_name (TableInfo *tinfo)
 /**
  * table_info_get_connection
  */
-BrowserConnection *
+TConnection *
 table_info_get_connection (TableInfo *tinfo)
 {
 	g_return_val_if_fail (IS_TABLE_INFO (tinfo), NULL);
-	return tinfo->priv->bcnc;
+	return tinfo->priv->tcnc;
 }
 
 /*
  * UI actions
  */
 static void
-action_add_to_fav_cb (G_GNUC_UNUSED GtkAction *action, TableInfo *tinfo)
+action_add_to_fav_cb (G_GNUC_UNUSED GSimpleAction *action, G_GNUC_UNUSED GVariant *state, gpointer data)
 {
-	ToolsFavorites *bfav;
-        ToolsFavoritesAttributes fav;
+	TableInfo *tinfo;
+	tinfo = TABLE_INFO (data);
+
+	TFavorites *bfav;
+        TFavoritesAttributes fav;
         GError *error = NULL;
 
-        memset (&fav, 0, sizeof (ToolsFavoritesAttributes));
+        memset (&fav, 0, sizeof (TFavoritesAttributes));
         fav.id = -1;
-        fav.type = GDA_TOOLS_FAVORITES_TABLES;
+        fav.type = T_FAVORITES_TABLES;
         fav.name = NULL;
         fav.descr = NULL;
         fav.contents = table_info_to_selection (tinfo);
 
-        bfav = browser_connection_get_favorites (tinfo->priv->bcnc);
-        if (! gda_tools_favorites_add (bfav, 0, &fav, ORDER_KEY_SCHEMA, G_MAXINT, &error)) {
-                browser_show_error ((GtkWindow*) gtk_widget_get_toplevel ((GtkWidget*) tinfo),
+        bfav = t_connection_get_favorites (tinfo->priv->tcnc);
+        if (! t_favorites_add (bfav, 0, &fav, ORDER_KEY_SCHEMA, G_MAXINT, &error)) {
+                ui_show_error ((GtkWindow*) gtk_widget_get_toplevel ((GtkWidget*) tinfo),
                                     _("Could not add favorite: %s"),
                                     error && error->message ? error->message : _("No detail"));
                 if (error)
@@ -475,15 +525,17 @@ action_add_to_fav_cb (G_GNUC_UNUSED GtkAction *action, TableInfo *tinfo)
 }
 
 static void
-action_view_contents_cb (G_GNUC_UNUSED GtkAction *action, TableInfo *tinfo)
+action_view_contents_cb (G_GNUC_UNUSED GSimpleAction *action, G_GNUC_UNUSED GVariant *state, gpointer data)
 {
+	TableInfo *tinfo;
+	tinfo = TABLE_INFO (data);
 	if (! tinfo->priv->table_short_name)
 		return;
 
 	BrowserWindow *bwin;
 	BrowserPerspective *pers;
 	bwin = (BrowserWindow*) gtk_widget_get_toplevel ((GtkWidget*) tinfo);
-	pers = browser_window_change_perspective (bwin, _("Data manager"));
+	pers = browser_window_change_perspective (bwin, DATA_MANAGER_PERSPECTIVE_NAME);
 
 	xmlDocPtr doc;
 	xmlNodePtr node, topnode;
@@ -510,24 +562,6 @@ insert_form_params_changed_cb (GdauiBasicForm *form, G_GNUC_UNUSED GdaHolder *pa
 					   gdaui_basic_form_is_valid (form));
 }
 
-static void statement_executed_cb (G_GNUC_UNUSED BrowserConnection *bcnc,
-				   G_GNUC_UNUSED guint exec_id,
-				   G_GNUC_UNUSED GObject *out_result,
-				   G_GNUC_UNUSED GdaSet *out_last_inserted_row, GError *error,
-				   TableInfo *tinfo)
-{
-	if (error)
-		browser_show_error (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tinfo)),
-				    _("Error executing query:\n%s"),
-				    error->message ?
-				    error->message : _("No detail"));
-	else
-		browser_window_show_notice_printf (BROWSER_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tinfo)),
-						   GTK_MESSAGE_INFO,
-						   "DataInsertQuery",
-						   "%s", _("Data successfully inserted"));
-}
-
 static void
 insert_response_cb (GtkWidget *dialog, gint response_id, TableInfo *tinfo)
 {
@@ -539,13 +573,19 @@ insert_response_cb (GtkWidget *dialog, gint response_id, TableInfo *tinfo)
 		stmt = g_object_get_data (G_OBJECT (dialog), "stmt");
 		params = g_object_get_data (G_OBJECT (dialog), "params");
 
-		if (! browser_connection_execute_statement_cb (tinfo->priv->bcnc,
+		GObject *result;
+		result = t_connection_execute_statement (tinfo->priv->tcnc,
 							       stmt, params,
-							       GDA_STATEMENT_MODEL_RANDOM_ACCESS,
-							       FALSE, 
-							       (BrowserConnectionExecuteCallback) statement_executed_cb,
-							       tinfo, &lerror)) {
-			browser_show_error (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tinfo)),
+							       GDA_STATEMENT_MODEL_RANDOM_ACCESS, NULL, &lerror);
+		if (result) {
+			browser_window_show_notice_printf (BROWSER_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tinfo)),
+							   GTK_MESSAGE_INFO,
+							   "DataInsertQuery",
+							   "%s", _("Data successfully inserted"));
+			g_object_unref (result);
+		}
+		else {
+			ui_show_error (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tinfo)),
 					    _("Error executing query: %s"),
 					    lerror && lerror->message ? lerror->message : _("No detail"));
 			g_clear_error (&lerror);
@@ -554,7 +594,7 @@ insert_response_cb (GtkWidget *dialog, gint response_id, TableInfo *tinfo)
 	}
 #ifdef HAVE_GDU
 	else if (response_id == GTK_RESPONSE_HELP) {
-		browser_show_help ((GtkWindow*) gtk_widget_get_toplevel ((GtkWidget*) tinfo),
+		ui_show_help ((GtkWindow*) gtk_widget_get_toplevel ((GtkWidget*) tinfo),
 				   "table-insert-data");
 	}
 #endif
@@ -572,22 +612,21 @@ typedef struct {
 	gboolean      model_rerunning;
 } FKBindData;
 
+/*
+ * @result: (transfer full)
+ */
 static void
-fk_bind_select_executed_cb (G_GNUC_UNUSED BrowserConnection *bcnc,
-			    G_GNUC_UNUSED guint exec_id,
-			    GObject *out_result,
-			    G_GNUC_UNUSED GdaSet *out_last_inserted_row, G_GNUC_UNUSED GError *error,
-			    FKBindData *fkdata)
+fk_bind_select_executed_cb (GObject *result, FKBindData *fkdata)
 {
 	gint i;
 	GdaDataModel *model;
-	if (! out_result)
+	if (! result)
 		return;
 
 	if (fkdata->model)
 		g_object_unref (fkdata->model);
 
-	model = GDA_DATA_MODEL (out_result);
+	model = GDA_DATA_MODEL (result);
 	for (i = 0; i < fkdata->cols_nb; i++) {
 		GdaHolder *h;
 		GdaSetSource *source;
@@ -621,7 +660,7 @@ fk_bind_select_executed_cb (G_GNUC_UNUSED BrowserConnection *bcnc,
 			}
 		}
 	}
-	fkdata->model = g_object_ref (out_result);
+	fkdata->model = GDA_DATA_MODEL (result);
 	fkdata->model_rerunning = FALSE;
 }
 
@@ -642,8 +681,11 @@ fkdata_list_free (GSList *fkdata_list)
 }
 
 static void
-action_insert_cb (G_GNUC_UNUSED GtkAction *action, TableInfo *tinfo)
+action_insert_cb (G_GNUC_UNUSED GSimpleAction *action, G_GNUC_UNUSED GVariant *state, gpointer data)
 {
+	TableInfo *tinfo;
+	tinfo = TABLE_INFO (data);
+
 	/* init */
 	if (! tinfo->priv->table_short_name)
 		return;
@@ -656,43 +698,23 @@ action_insert_cb (G_GNUC_UNUSED GtkAction *action, TableInfo *tinfo)
 		     fkdata_list; fkdata_list = fkdata_list->next) {
 			FKBindData *fkdata = (FKBindData *) fkdata_list->data;
 			if (fkdata->model && !fkdata->model_rerunning) {
-				if (browser_connection_execute_statement_cb (tinfo->priv->bcnc,
-									     fkdata->stmt, NULL,
-									     GDA_STATEMENT_MODEL_RANDOM_ACCESS,
-									     FALSE,
-									     (BrowserConnectionExecuteCallback) fk_bind_select_executed_cb,
-									     fkdata, NULL) != 0)
-					fkdata->model_rerunning = TRUE;
+				GObject *result;
+				fkdata->model_rerunning = TRUE;
+				result = t_connection_execute_statement (tinfo->priv->tcnc,
+									       fkdata->stmt, NULL,
+									       GDA_STATEMENT_MODEL_RANDOM_ACCESS, NULL,
+									       NULL);
+				fk_bind_select_executed_cb (result, fkdata);
 			}
 		}
 		return;
 	}
 
-	BrowserWindow *bwin;
-	GdaMetaStruct *mstruct;
-	bwin = (BrowserWindow*) gtk_widget_get_toplevel ((GtkWidget*) tinfo);
-	mstruct = browser_connection_get_meta_struct (tinfo->priv->bcnc);
-	if (!mstruct) {
-		browser_show_error (GTK_WINDOW (bwin), _("Meta data not yet available"));
-		return;
-	}
-
 	/* get table's information */
 	GdaMetaDbObject *dbo;
-	GValue *v1, *v2;
-	g_value_set_string ((v1 = gda_value_new (G_TYPE_STRING)),
-			    tinfo->priv->schema);
-	g_value_set_string ((v2 = gda_value_new (G_TYPE_STRING)),
-			    tinfo->priv->table_name);
-	dbo = gda_meta_struct_complement (mstruct,
-					  GDA_META_DB_TABLE, NULL, v1, v2, NULL);
-	gda_value_free (v1);
-	gda_value_free (v2);
-
-	if (! dbo) {
-		browser_show_error (GTK_WINDOW (bwin), _("Can't find information about table"));
+	dbo = get_table_meta_data (tinfo);
+	if (! dbo)
 		return;
-	}
 
 	/* build statement */
 	GdaSqlBuilder *b;
@@ -719,14 +741,24 @@ action_insert_cb (G_GNUC_UNUSED GtkAction *action, TableInfo *tinfo)
 	g_free (sql);
 #endif
 
+	/* common data required further down */
+	BrowserWindow *bwin;
+	GdaMetaStruct *mstruct;
+	bwin = (BrowserWindow*) gtk_widget_get_toplevel ((GtkWidget*) tinfo);
+	mstruct = t_connection_get_meta_struct (tinfo->priv->tcnc);
+	if (!mstruct) {
+		ui_show_error (GTK_WINDOW (bwin), _("Meta data not yet available"));
+		return;
+	}
+
 	/* handle user preferences */
 	GdaSet *params;
 	if (! gda_statement_get_parameters (stmt, &params, NULL)) {
 		gchar *sql;
 		sql = gda_statement_to_sql (stmt, NULL, NULL);
 
-		browser_show_error (GTK_WINDOW (bwin),
-				    _("Internal error while building INSERT statement:\n%s"), sql);
+		ui_show_error (GTK_WINDOW (bwin),
+			       _("Internal error while building INSERT statement:\n%s"), sql);
 		g_free (sql);
 		g_object_unref (stmt);
 		return;
@@ -742,9 +774,9 @@ action_insert_cb (G_GNUC_UNUSED GtkAction *action, TableInfo *tinfo)
 		const GValue *autoinc;
 		GdaHolder *holder;
 
-		plugin = browser_connection_get_table_column_attribute (tinfo->priv->bcnc,
+		plugin = t_connection_get_table_column_attribute (tinfo->priv->tcnc,
 									mtable,	col,
-									BROWSER_CONNECTION_COLUMN_PLUGIN,
+									T_CONNECTION_COLUMN_PLUGIN,
 									NULL);
 		holder = gda_set_get_holder (params, col->column_name);
 		if (!holder)
@@ -805,7 +837,7 @@ action_insert_cb (G_GNUC_UNUSED GtkAction *action, TableInfo *tinfo)
 		g_value_set_string ((name_v = gda_value_new (G_TYPE_STRING)), rdbo->obj_name);
 		dbo = gda_meta_struct_get_db_object (mstruct, NULL, schema_v, name_v);
 
-		cmodel = gda_meta_store_extract (browser_connection_get_meta_store (tinfo->priv->bcnc),
+		cmodel = gda_meta_store_extract (t_connection_get_meta_store (tinfo->priv->tcnc),
 						"SELECT tc.constraint_name, k.column_name FROM _key_column_usage k INNER JOIN _table_constraints tc ON (k.table_catalog=tc.table_catalog AND k.table_schema=tc.table_schema AND k.table_name=tc.table_name AND k.constraint_name=tc.constraint_name) WHERE tc.constraint_type='UNIQUE' AND k.table_catalog = ##catalog::string AND k.table_schema = ##schema::string AND k.table_name = ##tname::string ORDER by k.ordinal_position", &lerror,
 						"catalog", catalog_v,
 						"schema", schema_v,
@@ -866,28 +898,24 @@ action_insert_cb (G_GNUC_UNUSED GtkAction *action, TableInfo *tinfo)
 #endif
 
 				FKBindData *fkdata;
-				guint eid;
 				fkdata = g_new0 (FKBindData, 1);
 				fkdata->cols_nb = fk->cols_nb;
 				fkdata->fk_cols_array = g_new (gint, fk->cols_nb);
 				memcpy (fkdata->fk_cols_array, fk->fk_cols_array, sizeof (gint) * fk->cols_nb);
 				fkdata->chash = tinfo->priv->insert_columns_hash;
 				fkdata->stmt = stmt;
-				eid = browser_connection_execute_statement_cb (tinfo->priv->bcnc, stmt, NULL,
+
+				GObject *result;
+				result = t_connection_execute_statement (tinfo->priv->tcnc, stmt, NULL,
 									       GDA_STATEMENT_MODEL_RANDOM_ACCESS,
-									       FALSE,
-									       (BrowserConnectionExecuteCallback) fk_bind_select_executed_cb,
-									       fkdata, NULL);
-				if (! eid) {
-					g_free (fkdata->fk_cols_array);
-					g_object_unref (fkdata->stmt);
-					g_free (fkdata);
-				}
+									       NULL, NULL);
 				fkdata->insert_params = g_object_ref (params);
 
 				/* attach the kfdata to @popup to be able to re-run the SELECT
 				 * everytime the window is shown */
 				fkdata_list = g_slist_prepend (fkdata_list, fkdata);
+
+				fk_bind_select_executed_cb (result, fkdata);
 			}
 
 			g_object_unref (b);
@@ -899,10 +927,10 @@ action_insert_cb (G_GNUC_UNUSED GtkAction *action, TableInfo *tinfo)
 	popup = gtk_dialog_new_with_buttons (_("Values to insert into table"), GTK_WINDOW (bwin),
 					     0,
 #ifdef HAVE_GDU
-					     GTK_STOCK_HELP, GTK_RESPONSE_HELP,
+					     _("Help"), GTK_RESPONSE_HELP,
 #endif
-					     GTK_STOCK_EXECUTE, GTK_RESPONSE_ACCEPT,
-					     GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+					     _("Execute"), GTK_RESPONSE_ACCEPT,
+					     _("Cancel"), GTK_RESPONSE_REJECT,
 					     NULL);
 	tinfo->priv->insert_popup = popup;
 	g_object_set_data_full (G_OBJECT (popup), "stmt", stmt, g_object_unref);
@@ -928,7 +956,6 @@ action_insert_cb (G_GNUC_UNUSED GtkAction *action, TableInfo *tinfo)
 			    label, FALSE, FALSE, 0);
 	
 	form = gdaui_basic_form_new (params);
-	g_object_set ((GObject*) form, "show-actions", TRUE, NULL);
 	g_signal_connect (form, "holder-changed",
 			  G_CALLBACK (insert_form_params_changed_cb), popup);
 	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (popup))),
@@ -941,23 +968,21 @@ action_insert_cb (G_GNUC_UNUSED GtkAction *action, TableInfo *tinfo)
 }
 
 static void
-action_declarefk_cb (G_GNUC_UNUSED GtkAction *action, TableInfo *tinfo)
+action_declarefk_cb (G_GNUC_UNUSED GSimpleAction *action, G_GNUC_UNUSED GVariant *state, gpointer data)
 {
+	TableInfo *tinfo;
+	tinfo = TABLE_INFO (data);
+
 	GtkWidget *dlg, *parent;
 	GdaMetaStruct *mstruct;
 	GdaMetaDbObject *dbo;
 	gint response;
-	GValue *v1, *v2;
 
 	parent = (GtkWidget*) gtk_widget_get_toplevel ((GtkWidget*) tinfo);
-	mstruct = browser_connection_get_meta_struct (tinfo->priv->bcnc);
-	g_value_set_string ((v1 = gda_value_new (G_TYPE_STRING)), tinfo->priv->schema);
-	g_value_set_string ((v2 = gda_value_new (G_TYPE_STRING)), tinfo->priv->table_name);
-	dbo = gda_meta_struct_get_db_object (mstruct, NULL, v1, v2);
-	gda_value_free (v1);
-	gda_value_free (v2);
+	mstruct = t_connection_get_meta_struct (tinfo->priv->tcnc);
+	dbo = get_table_meta_data (tinfo);
 	if (!dbo || (dbo->obj_type != GDA_META_DB_TABLE)) {
-		browser_show_error ((GtkWindow *) parent, _("Can't find information about table '%s'"),
+		ui_show_error ((GtkWindow *) parent, _("Can't find information about table '%s'"),
 				    tinfo->priv->table_short_name);
 		return;
 	}
@@ -969,7 +994,7 @@ action_declarefk_cb (G_GNUC_UNUSED GtkAction *action, TableInfo *tinfo)
 		if (! fk_declare_write (FK_DECLARE (dlg),
 					BROWSER_IS_WINDOW (parent) ? BROWSER_WINDOW (parent) : NULL,
 					&error)) {
-			browser_show_error ((GtkWindow *) parent, _("Failed to declare foreign key: %s"),
+			ui_show_error ((GtkWindow *) parent, _("Failed to declare foreign key: %s"),
 					    error && error->message ? error->message : _("No detail"));
 			g_clear_error (&error);
 		}
@@ -978,73 +1003,76 @@ action_declarefk_cb (G_GNUC_UNUSED GtkAction *action, TableInfo *tinfo)
 						    GTK_MESSAGE_INFO, "fkdeclare",
 						    _("Successfully declared foreign key"));
 		else
-			browser_show_message ((GtkWindow *) parent, "%s",
+			ui_show_message ((GtkWindow *) parent, "%s",
 					      _("Successfully declared foreign key"));
 	}
 	
 	gtk_widget_destroy (dlg);
 }
 
-static GtkActionEntry ui_actions[] = {
-	{ "Table", NULL, N_("_Table"), NULL, N_("Table"), NULL },
-	{ "AddToFav", STOCK_ADD_BOOKMARK, N_("Add to _Favorites"), NULL, N_("Add table to favorites"),
-	  G_CALLBACK (action_add_to_fav_cb)},
-	{ "ViewContents", GTK_STOCK_EDIT, N_("_Contents"), NULL, N_("View table's contents"),
-	  G_CALLBACK (action_view_contents_cb)},
-	{ "InsertData", GTK_STOCK_ADD, N_("_Insert Data"), NULL, N_("Insert data into table"),
-	  G_CALLBACK (action_insert_cb)},
-	{ "KfDeclare", NULL, N_("_Declare Foreign Key"), NULL, N_("Declare a foreign key for table"),
-	  G_CALLBACK (action_declarefk_cb)},
+static GActionEntry win_entries[] = {
+        { "AddToFav", action_add_to_fav_cb, NULL, NULL, NULL },
+	{ "ViewContents", action_view_contents_cb, NULL, NULL, NULL },
+	{ "InsertData", action_insert_cb, NULL, NULL, NULL },
+	{ "KfDeclare", action_declarefk_cb, NULL, NULL, NULL }
 };
-static const gchar *ui_actions_info =
-	"<ui>"
-	"  <menubar name='MenuBar'>"
-	"    <placeholder name='MenuExtension'>"
-        "      <menu name='Table' action='Table'>"
-        "        <menuitem name='AddToFav' action= 'AddToFav'/>"
-        "        <menuitem name='InsertData' action= 'InsertData'/>"
-        "        <menuitem name='ViewContents' action= 'ViewContents'/>"
-        "        <separator/>"
-        "        <menuitem name='KfDeclare' action= 'KfDeclare'/>"
-        "      </menu>"
-        "    </placeholder>"
-	"  </menubar>"
-	"  <toolbar name='ToolBar'>"
-	"    <separator/>"
-	"    <toolitem action='AddToFav'/>"
-	"    <toolitem action='ViewContents'/>"
-	"    <toolitem action='InsertData'/>"
-	"  </toolbar>"
-	"</ui>";
 
-static GtkActionGroup *
-table_info_page_get_actions_group (BrowserPage *page)
+static void
+table_info_customize (BrowserPage *page, GtkToolbar *toolbar, GtkHeaderBar *header)
 {
-	GtkActionGroup *agroup;
-	agroup = gtk_action_group_new ("SchemaBrowserTableInfoActions");
-	gtk_action_group_set_translation_domain (agroup, GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (agroup, ui_actions, G_N_ELEMENTS (ui_actions), page);
-	
-	return agroup;
-}
+	g_print ("%s ()\n", __FUNCTION__);
 
-static const gchar *
-table_info_page_get_actions_ui (G_GNUC_UNUSED BrowserPage *page)
-{
-	return ui_actions_info;
+	customization_data_init (G_OBJECT (page), toolbar, header);
+
+	/* add perspective's actions */
+	customization_data_add_actions (G_OBJECT (page), win_entries, G_N_ELEMENTS (win_entries));
+
+	/* add to toolbar */
+	GtkToolItem *titem;
+	titem = gtk_tool_button_new (NULL, NULL);
+	gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON (titem), "bookmark-new-symbolic");
+	gtk_widget_set_tooltip_text (GTK_WIDGET (titem), _("Add to Favorites"));
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), titem, -1);
+	gtk_actionable_set_action_name (GTK_ACTIONABLE (titem), "win.AddToFav");
+	gtk_widget_show (GTK_WIDGET (titem));
+	customization_data_add_part (G_OBJECT (page), G_OBJECT (titem));
+
+	titem = gtk_tool_button_new (NULL, NULL);
+	gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON (titem), "go-jump-symbolic");
+	gtk_widget_set_tooltip_text (GTK_WIDGET (titem), _("View table's contents"));
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), titem, -1);
+	gtk_actionable_set_action_name (GTK_ACTIONABLE (titem), "win.ViewContents");
+	gtk_widget_show (GTK_WIDGET (titem));
+	customization_data_add_part (G_OBJECT (page), G_OBJECT (titem));
+
+	titem = gtk_tool_button_new (NULL, NULL);
+	gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON (titem), "document-new-symbolic");
+	gtk_widget_set_tooltip_text (GTK_WIDGET (titem), _("Insert data into table"));
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), titem, -1);
+	gtk_actionable_set_action_name (GTK_ACTIONABLE (titem), "win.InsertData");
+	gtk_widget_show (GTK_WIDGET (titem));
+	customization_data_add_part (G_OBJECT (page), G_OBJECT (titem));
+
+	titem = gtk_tool_button_new (NULL, NULL);
+	gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON (titem), "insert-link-symbolic");
+	gtk_widget_set_tooltip_text (GTK_WIDGET (titem), _("Declare Foreign Key"));
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), titem, -1);
+	gtk_actionable_set_action_name (GTK_ACTIONABLE (titem), "win.KfDeclare");
+	gtk_widget_show (GTK_WIDGET (titem));
+	customization_data_add_part (G_OBJECT (page), G_OBJECT (titem));
 }
 
 static GtkWidget *
-table_info_page_get_tab_label (BrowserPage *page, GtkWidget **out_close_button)
+table_info_get_tab_label (BrowserPage *page, GtkWidget **out_close_button)
 {
 	TableInfo *tinfo;
 	const gchar *tab_name;
 	GdkPixbuf *table_pixbuf;
 
 	tinfo = TABLE_INFO (page);
-	table_pixbuf = browser_get_pixbuf_icon (BROWSER_ICON_TABLE);
+	table_pixbuf = ui_get_pixbuf_icon (UI_ICON_TABLE);
 	tab_name = tinfo->priv->table_short_name ? tinfo->priv->table_short_name : tinfo->priv->table_name;
-	return browser_make_tab_label_with_pixbuf (tab_name,
-						   table_pixbuf,
-						   out_close_button ? TRUE : FALSE, out_close_button);
+	return ui_make_tab_label_with_pixbuf (tab_name,
+					      table_pixbuf,
+					      out_close_button ? TRUE : FALSE, out_close_button);
 }
